#ifndef EVENT_INCLUDE
#define EVENT_INCLUDE

#include "common.h"
#include <SDL/SDL.h>


enum {
	EVT_NONE = 0,

	EVT_INIT,
	EVT_IDLE,
	EVT_CREATE,
	EVT_UPDATE,
	EVT_COLLIDE,
	EVT_DAMAGE,
	EVT_DESTROY,
	EVT_KILL,
	EVT_RAISE,
	EVT_ACTION,
	EVT_COMMAND,
	EVT_CALL,

	EVT_SELECT,
	EVT_DISSELECT,
	EVT_BUTTON,
	EVT_MOTION,
	EVT_KEY,
	EVT_SYSTEM,

	// ȉʐMpPbg 
	EVT_PING = 1000,	// none // T[o[ɑłEVT_PONGԂ 
	EVT_PONG,			// none
	EVT_QUERY_VERSION,	// none // T[o[̃o[W𓾂 
	EVT_VERSION,		// string version
	EVT_QUERY_PLAYERS,	// none // Q҈ꗗ𓾂 
	EVT_PLAYERS,		// string version

	EVT_LOGIN = 1050,	// string name // Q[ւ̎Qv 
	EVT_LOGIN_ACCEPT,	// size_t id // Q[ւ̎QFꂽ 
	EVT_LOGOUT,			// none

	EVT_SYNC = 1100,	// size_t frame // T[o[Frameڂ` 
	EVT_SAY,			// string message // `bg 
	EVT_DISCONNECT,		// none // ؒf 
};

enum {
	RCV_UPDATE  =1,
	RCV_COLLIDE =2,
	RCV_SELECT  =4,
	RCV_BUTTON=8,
	RCV_MOTION=16,
	RCV_KEY=32,
};



enum {
	FACTION,
	UNIT,
	BUILDING,
	MATERIAL,
	UNITGROUP,
};


class Event
{
public:
	class RefCount
	{
	public:
		RefCount() : ref_count(0) {}
		virtual ~RefCount() {}
		void addRef() { ++ref_count; }
		void release() { if(--ref_count==0) delete this; }
	private:
		int ref_count;
	};

	class CreateInfo : public RefCount
	{
	public:
		typedef std::vector<gobj_id> id_cont;

		CreateInfo() : type(0), id(0), faction(0) {}
		CreateInfo(int t, const std::string& cn, gobj_id i, gobj_id f=0) :
			type(t), id(i), classname(cn), faction(f) {}
		CreateInfo(int t, gobj_id i, const std::string& cn, gobj_id f=0) :
			type(t), id(i), classname(cn), faction(f) {}
		int type;
		gobj_id id;
		std::string classname;
		gobj_id faction;
		id_cont group;
	};

	class DamageInfo : public RefCount
	{
	public:
		DamageInfo(float bld, float lsd, float bmd, float idd=0, bullet_ptr b=bullet_ptr()) :
			bullet_damage(bld),
			laser_damage(lsd),
			bomb_damage(bmd),
			ignore_defence_damage(idd),
			bullet(b)
		{}
		float bullet_damage;
		float laser_damage;
		float bomb_damage;
		float ignore_defence_damage;
		bullet_ptr bullet;
	};

	class CollideInfo : public RefCount
	{
	public:
		CollideInfo(const ist::vector4& p, const ist::vector4& a, float d) : pos(p), aim(a), distance(d) {}
		ist::vector4 pos;
		ist::vector4 aim;
		float distance;
	};

	class ActionInfo : public RefCount
	{
	public:
		enum {
			APPEND=1,
			APPENDF=2,
			INSIST=4,
			ACTIVE=8,
			RANK=16,
			PATH=32,
			LINK=64,
		};

		ActionInfo(const ist::vector4& a, selectable_ptr t, int f=0, gobj_id tid=0) :
			target_pos(a),
			target_obj(t),
			target_id(tid),
			speed(0),
			flag(f)
		{}
		int getFlag() { return flag; }
		bool flagAppend() { return (flag&APPEND)!=0; }
		bool flagAppendF(){ return (flag&APPENDF)!=0; }
		bool flagInsist() { return (flag&INSIST)!=0; }
		bool flagActive() { return (flag&ACTIVE)!=0; }
		bool flagRank()   { return (flag&RANK)!=0; }
		bool flagPath()   { return (flag&PATH)!=0; }
		bool flagLink()   { return (flag&LINK)!=0; }

		selectable_ptr& getObj();

		ist::vector4 target_pos;
		selectable_ptr target_obj;
		gobj_id target_id;
		float speed;
		int flag;
	};

	class CommandInfo : public RefCount
	{
	public:
		CommandInfo(const std::string& n, const boost::any& v) :
			name(n), value(v) {}

		std::string name;
		boost::any value;
	};

	class ButtonInfo : public RefCount
	{
	public:
		ButtonInfo(int s, int b, float x_, float y_) :
			state(s), button(b), x(x_), y(y_) {}

		bool isButtonDown() { return state==SDL_MOUSEBUTTONDOWN; }
		bool isButtonUp() { return state==SDL_MOUSEBUTTONUP; }

		int state;
		int button;
		float x;
		float y;
	};

	class MotionInfo : public RefCount
	{
	public:
		MotionInfo(bool s, float x_, float y_, float dx=0, float dy=0) :
			is_dragging(s), x(x_), y(y_), drag_x(dx), drag_y(dy) {}

		bool isDragging() { return is_dragging; }

		bool is_dragging;
		float x;
		float y;
		float drag_x;
		float drag_y;
		ist::FrustumCollisionModel dragging_frustom;
	};

	class KeyInfo : public RefCount
	{
	public:
		KeyInfo(int s, int k) : state(s), key(k) {}

		bool isKeyDown() { return state==SDL_KEYDOWN; }
		bool isKeyUp() { return state==SDL_KEYUP; }

		int state;
		int key;
	};

	typedef CommandInfo SystemInfo;
	typedef CommandInfo CallInfo;

	typedef boost::intrusive_ptr<RefCount>    info_ptr;
	typedef boost::intrusive_ptr<CreateInfo>  createinfo_ptr;
	typedef boost::intrusive_ptr<DamageInfo>  damageinfo_ptr;
	typedef boost::intrusive_ptr<CollideInfo> collideinfo_ptr;
	typedef boost::intrusive_ptr<ActionInfo>  actioninfo_ptr;
	typedef boost::intrusive_ptr<CommandInfo> commandinfo_ptr;
	typedef boost::intrusive_ptr<CallInfo>    callinfo_ptr;
	typedef boost::intrusive_ptr<SystemInfo>  systeminfo_ptr;
	typedef boost::intrusive_ptr<ButtonInfo>  buttoninfo_ptr;
	typedef boost::intrusive_ptr<MotionInfo>  motioninfo_ptr;
	typedef boost::intrusive_ptr<KeyInfo>     keyinfo_ptr;


	Event() : type(EVT_NONE), frame(0), from(0), to(0), delay(0) {}
	Event(int type_, gobj_id from_, gobj_id to_, info_ptr ip=info_ptr(), int d=0) :
		type(type_), from(from_), to(to_), delay(d), info(ip) {}

	int getType() const { return type; }
	CreateInfo& getCreateInfo()   const { return static_cast<CreateInfo&>(*info); }
	CollideInfo& getCollideInfo() const { return static_cast<CollideInfo&>(*info); }
	DamageInfo& getDamageInfo()   const { return static_cast<DamageInfo&>(*info); }
	ActionInfo& getActionInfo()   const { return static_cast<ActionInfo&>(*info); }
	CommandInfo& getCommandInfo() const { return static_cast<CommandInfo&>(*info); }
	CallInfo& getCallInfo()       const { return static_cast<CallInfo&>(*info); }
	SystemInfo& getSystemInfo()   const { return static_cast<SystemInfo&>(*info); }
	ButtonInfo& getButtonInfo()   const { return static_cast<ButtonInfo&>(*info); }
	MotionInfo& getMotionInfo()   const { return static_cast<MotionInfo&>(*info); }
	KeyInfo& getKeyInfo()         const { return static_cast<KeyInfo&>(*info); }

	int type;
	size_t frame;
	gobj_id from;
	gobj_id to;
	int delay;
	mutable info_ptr info;
};




inline void intrusive_ptr_add_ref(Event::RefCount *p) { p->addRef(); }
inline void intrusive_ptr_release(Event::RefCount *p) { p->release(); }



#endif

