#ifndef INLINES_INCLUDE
#define INLINES_INCLUDE

#include "interface.h"
#include "macro.h"





inline bool ToBool(const boost::any& v) {
	if(v.type()==typeid(bool))
		return boost::any_cast<bool>(v);
	return false;
}

inline int ToInt(const boost::any& v) {
	if(v.type()==typeid(int))
		return boost::any_cast<int>(v);
	return 0;
}

inline int ToSizeT(const boost::any& v) {
	if(v.type()==typeid(size_t))
		return boost::any_cast<size_t>(v);
	return 0;
}

inline float ToFloat(const boost::any& v) {
	if(v.type()==typeid(float))
		return boost::any_cast<float>(v);
	return 0;
}

inline const ist::vector4& ToVector4(const boost::any& v) {
	if(v.type()==typeid(ist::vector4))
		return *boost::any_cast<ist::vector4>(&v);
	static ist::vector4 dummy;
	return dummy;
}

inline const std::string& ToString(const boost::any& v) {
	if(v.type()==typeid(std::string))
		return *boost::any_cast<std::string>(&v);
	printf("ToString: warning\n");
	static std::string dummy;
	return dummy;
}



inline bool IsTargetable(selectable_ptr p) {
	boost::any v;
	if(p->query("isTargetable", v) && !ToBool(v))
		return false;
	return true;
}



template<class T>
struct releaser
{
	void operator()(T* p) {
		p->release();
	}
};

template<class T>
struct gobj_is_dead
{
	bool operator()(const T& u) const {
		return (!u || u->isDead());
	}
};

template <class T>
struct less_id
{
	bool operator()(const T& lhs, const T& rhs) const {
		return lhs->getID() < rhs->getID();
	}
};

template<class T>
class gobj_is_unit : public std::unary_function<T, bool>
{
public:
	bool operator()(const T& p) const {
		return p->isUnit();
	}
};








inline selectable_ptr ToSelectable(gobj_ptr p)
{
	return (!!p && p->isSelectable()) ? boost::static_pointer_cast<ISelectable>(p) : selectable_ptr();
}

inline breakable_ptr ToBreakable(gobj_ptr p)
{
	return (!!p && p->isBreakable()) ?  boost::static_pointer_cast<IBreakable>(p) : breakable_ptr();
}



inline unit_ptr ToUnit(gobj_ptr p)
{
	return (!!p && p->isUnit()) ? boost::static_pointer_cast<IUnit>(p) : unit_ptr();
}

inline building_ptr ToBuilding(gobj_ptr p)
{
	return (!!p && p->isBuilding()) ? boost::static_pointer_cast<IBuilding>(p) : building_ptr();
}

inline material_ptr ToMaterial(gobj_ptr p)
{
	return (!!p && p->isMaterial()) ? boost::static_pointer_cast<IMaterial>(p) : material_ptr();
}

inline bullet_ptr ToBullet(gobj_ptr p)
{
	return (!!p && p->isBullet()) ? boost::static_pointer_cast<IBullet>(p) : bullet_ptr();
}

inline menu_ptr ToMenu(gobj_ptr p)
{
	return (!!p && p->isMenu()) ? boost::static_pointer_cast<IMenu>(p) : menu_ptr();
}

inline faction_ptr ToFaction(gobj_ptr p)
{
	return (!!p && p->isFaction()) ? boost::static_pointer_cast<IFaction>(p) : faction_ptr();
}

inline group_ptr ToGroup(gobj_ptr p)
{
	return (!!p && p->isGroup()) ? boost::static_pointer_cast<IGroup>(p) : group_ptr();
}




inline Event CreateEvent(Event::createinfo_ptr info)
{
	return Event(EVT_CREATE, 0, 0, info);
}

inline Event UpdateEvent(gobj_id from, gobj_id to)
{
	return Event(EVT_UPDATE, from, to);
}


inline Event CollideEvent(gobj_id from, gobj_id to, Event::collideinfo_ptr info)
{
	return Event(EVT_COLLIDE, from, to, info);
}

inline Event DamageEvent(gobj_id from, gobj_id to, Event::damageinfo_ptr info)
{
	return Event(EVT_DAMAGE, from, to, info);
}

inline Event DestroyEvent(gobj_id from, gobj_id to)
{
	return Event(EVT_DESTROY, from, to);
}

inline Event KillEvent(gobj_id from, gobj_id to)
{
	return Event(EVT_KILL, from, to);
}

inline Event ActionEvent(gobj_id from, gobj_id to, Event::actioninfo_ptr info, int delay=0)
{
	return Event(EVT_ACTION, from, to, info, delay);
}

inline Event CommandEvent(gobj_id from, gobj_id to, Event::commandinfo_ptr info, int delay=0)
{
	return Event(EVT_COMMAND, from, to, info, delay);
}

inline Event CallEvent(gobj_id from, gobj_id to, Event::callinfo_ptr info, int delay=0)
{
	return Event(EVT_CALL, from, to, info, delay);
}

inline Event SystemEvent(gobj_id from, gobj_id to, Event::systeminfo_ptr info, int delay=0)
{
	return Event(EVT_SYSTEM, from, to, info, delay);
}

inline Event SelectEvent(gobj_id from, gobj_id to)
{
	return Event(EVT_SELECT, from, to);
}

inline Event DisselectEvent(gobj_id from, gobj_id to)
{
	return Event(EVT_DISSELECT, from, to);
}

inline Event ButtonEvent(gobj_id from, gobj_id to, Event::buttoninfo_ptr info)
{
	return Event(EVT_BUTTON, from, to, info);
}

inline Event MotionEvent(gobj_id from, gobj_id to, Event::motioninfo_ptr info)
{
	return Event(EVT_MOTION, from, to, info);
}

inline Event KeyEvent(gobj_id from, gobj_id to, Event::keyinfo_ptr info)
{
	return Event(EVT_KEY, from, to, info);
}


inline void SendCollideEvent(const gobj_ptr& from, const gobj_ptr& to, Event::collideinfo_ptr info) {
	gobj_id f = (!!from) ? from->getID() : 0;
	to->sendEvent(Event(EVT_COLLIDE, f, to->getID(), info));
}

inline void SendDamageEvent(const gobj_ptr& from, const gobj_ptr& to, Event::damageinfo_ptr info) {
	gobj_id f = (!!from) ? from->getID() : 0;
	to->sendEvent(Event(EVT_DAMAGE, f, to->getID(), info));
}

inline void SendCommandEvent(const gobj_ptr& from, const gobj_ptr& to, Event::commandinfo_ptr info, int d=0) {
	gobj_id f = (!!from) ? from->getID() : 0;
	to->sendEvent(Event(EVT_COMMAND, f, to->getID(), info, d));
}

inline void SendCallEvent(const gobj_ptr& from, const gobj_ptr& to, Event::commandinfo_ptr info, int d=0) {
	gobj_id f = (!!from) ? from->getID() : 0;
	to->sendEvent(Event(EVT_CALL, f, to->getID(), info, d));
}

inline void SendActionEvent(const gobj_ptr& from, const gobj_ptr& to, Event::actioninfo_ptr info, int d=0) {
	gobj_id f = (!!from) ? from->getID() : 0;
	to->sendEvent(Event(EVT_ACTION, f, to->getID(), info, d));
}

inline void SendDestroyEvent(const gobj_ptr& from, const gobj_ptr& to) {
	gobj_id f = (!!from) ? from->getID() : 0;
	to->sendEvent(Event(EVT_DESTROY, f, to->getID()));
}

inline void SendKillEvent(const gobj_ptr& from, const gobj_ptr& to) {
	gobj_id f = (!!from) ? from->getID() : 0;
	to->sendEvent(Event(EVT_KILL, f, to->getID()));
}

inline void SendSelectEvent(const gobj_ptr& from, const gobj_ptr& to) {
	gobj_id f = (!!from) ? from->getID() : 0;
	to->sendEvent(Event(EVT_SELECT, f, to->getID()));
}

inline void SendDisselectEvent(const gobj_ptr& from, const gobj_ptr& to) {
	gobj_id f = (!!from) ? from->getID() : 0;
	to->sendEvent(Event(EVT_DISSELECT, f, to->getID()));
}

inline void SendKeyEvent(const gobj_ptr& from, const gobj_ptr& to, Event::keyinfo_ptr info) {
	gobj_id f = (!!from) ? from->getID() : 0;
	to->sendEvent(Event(EVT_KEY, f, to->getID(), info));
}

inline void SendButtonEvent(const gobj_ptr& from, const gobj_ptr& to, Event::buttoninfo_ptr info) {
	gobj_id f = (!!from) ? from->getID() : 0;
	to->sendEvent(Event(EVT_BUTTON, f, to->getID(), info));
}

inline void SendMotionEvent(const gobj_ptr& from, const gobj_ptr& to, Event::motioninfo_ptr info) {
	gobj_id f = (!!from) ? from->getID() : 0;
	to->sendEvent(Event(EVT_MOTION, f, to->getID(), info));
}








template <class T>
class IteratorAdapter : public IGObjIterator
{
public:
	typedef typename T::const_iterator Iter;

	IteratorAdapter() {}
	IteratorAdapter(Iter b, Iter e) { set(b, e); }
	void set(Iter b, Iter e) { begin=b; end=e; reset(); }
	void reset() { it=begin; }
	bool hasNext() {
		if(it!=end)
			return true;
		reset();
		return false;
	}
	gobj_ptr iterate() {
		gobj_ptr ret = *it;
		++it;
		return ret;
	}

private:
	Iter begin;
	Iter end;
	Iter it;
};



template<class T>
struct not_in_group
{
	not_in_group(group_ptr g) : group(g) {}
	bool operator()(const T& u) const { return (!u || u->isDead() || u->getGroup()!=group); }

	group_ptr group;
};









#endif

#ifdef GetIGame
#ifndef INLINES_IG_INCLUDE
#define INLINES_IG_INCLUDE

inline selectable_ptr& Event::ActionInfo::getObj() {
	if(target_id!=0) {
		if(selectable_ptr s = ToSelectable(GetGObj(target_id)))
			target_obj = s;
		target_id = 0;
	}
	return target_obj;
}


inline ist::vector4 RandomVector() {
	return ist::vector4(GetRand()*2.0f-1.0f, GetRand()*2.0f-1.0f, GetRand()*2.0f-1.0f);
}

inline gobj_ptr ToGObj(const boost::any& v) {
	gobj_ptr g;
	if(v.type()==typeid(int))
		g = GetGObj(gobj_id(boost::any_cast<int>(v)));
	else if(v.type()==typeid(size_t))
		g = GetGObj(boost::any_cast<gobj_id>(v));
	else if(v.type()==typeid(gobj_ptr))
		g = boost::any_cast<gobj_ptr>(v);
	return g;
}

inline bool IsInCameraSight(const ist::vector4& pos, float radius) {
	return GetCamera().view_frustum.test(pos, radius)!=0;
}


inline bool IsAlly(faction_ptr p1, faction_ptr p2)
{
	return (!!p1 && !!p2 && (p1->isAlly(p2) || p2->isMaterial()));
}

inline bool IsAlly(selectable_ptr p1, selectable_ptr p2)
{
	return IsAlly(p1->getFaction(), p2->getFaction());
}

template<class T1, class T2>
inline bool IsEnemy(T1 p1, T2 p2)
{
	if(!!p2 && p2->isMaterial())
		return false;
	return IsEnemy<faction_ptr, faction_ptr>(p1->getFaction(), p2->getFaction());
}

template<>
inline bool IsEnemy(faction_ptr p1, faction_ptr p2)
{
	return !IsAlly(p1, p2);
}



inline void RouteCreateEvent(Event::createinfo_ptr info) {
	RouteEvent(Event(EVT_CREATE, 0, 0, info));
}

inline void RouteSystemEvent(Event::systeminfo_ptr info) {
	RouteEvent(Event(EVT_SYSTEM, 0, 0, info));
}


inline void RouteDestroyEvent(const gobj_ptr& from, const gobj_ptr& to, int d=0) {
	gobj_id f = (!!from) ? from->getID() : 0;
	RouteEvent(Event(EVT_DESTROY, f, to->getID(), 0, d));
}

inline void RouteCommandEvent(const gobj_ptr& from, const gobj_ptr& to, Event::commandinfo_ptr info, int d=0) {
	gobj_id f = (!!from) ? from->getID() : 0;
	RouteEvent(Event(EVT_COMMAND, f, to->getID(), info, d));
}

inline void RouteCallEvent(const gobj_ptr& from, const gobj_ptr& to, Event::commandinfo_ptr info, int d=0) {
	gobj_id f = (!!from) ? from->getID() : 0;
	RouteEvent(Event(EVT_CALL, f, to->getID(), info, d));
}

inline void RouteActionEvent(const gobj_ptr& from, const gobj_ptr& to, Event::actioninfo_ptr info, int d=0) {
	gobj_id f = (!!from) ? from->getID() : 0;
	RouteEvent(Event(EVT_ACTION, f, to->getID(), info, d));
}


inline void RouteDestroyEvent(gobj_id from, gobj_id to, int d=0) {
	RouteEvent(Event(EVT_DESTROY, from, to, 0, d));
}

inline void RouteCommandEvent(gobj_id from, gobj_id to, Event::commandinfo_ptr info, int d=0) {
	RouteEvent(Event(EVT_COMMAND, from, to, info, d));
}

inline void RouteCallEvent(gobj_id from, gobj_id to, Event::commandinfo_ptr info, int d=0) {
	RouteEvent(Event(EVT_CALL, from, to, info, d));
}

inline void RouteActionEvent(gobj_id from, gobj_id to, Event::actioninfo_ptr info, int d=0) {
	RouteEvent(Event(EVT_ACTION, from, to, info, d));
}

inline texture_ptr GetTexture(const std::string& name) {
	texture_ptr t = CreateTexture(name);
	if(!(*t))
		t->load(name.c_str());
	return t;
}


template<class Iter>
void SetGroup(group_ptr group, Iter begin, Iter end) {
	for(; begin!=end; ++begin) {
		if(u = ToUnit(*begin)) {
			u->setGroup(group);
			group->add(u);
		}
	}
}

template <class T>
class IDIteratorAdapter : public IGObjIterator
{
public:
	typedef typename T::iterator Iter;

	IDIteratorAdapter() {}
	IDIteratorAdapter(Iter b, Iter e) { set(b, e); }
	void set(Iter b, Iter e) { begin=b; end=e; reset(); }
	void reset() { it=begin; }
	bool hasNext() {
		if(it!=end)
			return true;
		reset();
		return false;
	}
	gobj_ptr iterate() {
		gobj_ptr ret = GetGObj(*it);
		++it;
		return ret;
	}

private:
	Iter begin;
	Iter end;
	Iter it;
};


inline faction_ptr CheckedCreateFaction(const std::string& name)
{
	faction_ptr p = CreateFaction(name);
	assert(!!p);
	return p;
}

inline unit_ptr CheckedCreateUnit(const std::string& name, faction_ptr f)
{
	unit_ptr p = CreateUnit(name, f);
	assert(!!p);
	return p;
}

inline building_ptr CheckedCreateBuilding(const std::string& name, faction_ptr f)
{
	building_ptr p = CreateBuilding(name, f);
	assert(!!p);
	return p;
}

inline material_ptr CheckedCreateMaterial(const std::string& name)
{
	material_ptr p = CreateMaterial(name);
	assert(!!p);
	return p;
}

inline gobj_ptr CreateGObj(const std::string& name, faction_ptr f) {
	gobj_ptr p = CreateUnit(name, f);
	if(!p)   p = CreateBuilding(name, f);
	if(!p)   p = CreateMaterial(name);
	return p;
}

inline gobj_ptr CheckedCreateGObj(const std::string& name, faction_ptr f) {
	gobj_ptr p = CreateGObj(name, f);
	assert(!!p);
	return p;
}

#endif
#endif



