#ifndef INTERFACE_INCLUDE
#define INTERFACE_INCLUDE

#include "event.h"








class Resource
{
public:
	Resource() : timber(0), silicon(0), iron(0), energy(0) {}
	Resource(float t, float s, float i, float e) : timber(t), silicon(s), iron(i), energy(e) {}

	float timber;
	float silicon;
	float iron;
	float energy;

	void operator+=(const Resource& r) {
		timber+=r.timber;
		silicon+=r.silicon;
		iron+=r.iron;
		energy+=r.energy;
	}
	void operator-=(const Resource& r) {
		timber-=r.timber;
		silicon-=r.silicon;
		iron-=r.iron;
		energy-=r.energy;
	}
	void operator*=(float r) {
		timber*=r;
		silicon*=r;
		iron*=r;
		energy*=r;
	}
	bool operator>(Resource& r) {
		return (r.timber<=0.0f || timber>=r.timber)
			&& (r.silicon<=0.0f || silicon>=r.silicon)
			&& (r.iron<=0.0f || iron>=r.iron)
			&& (r.energy<=0.0f || energy>=r.energy);
	}
	bool operator<(Resource& r) {
		return timber<r.timber
			|| silicon<r.silicon
			|| iron<r.iron
			|| energy<r.energy;
	}
};


class IGObjIterator
{
public:
	virtual ~IGObjIterator() {}
	virtual void reset()=0;
	virtual bool hasNext()=0;
	virtual gobj_ptr iterate()=0;
};

class IKeyObserver : public ist::Object
{
public:
	virtual ~IKeyObserver() {}
	virtual bool isExclusive()=0;
	virtual void exec(int state)=0;
};
typedef boost::intrusive_ptr<IKeyObserver> keyobserv_ptr;



class ISoundPlayer : public ist::Object
{
public:
	virtual ~ISoundPlayer() {}
	virtual bool loadSE(const std::string& filename)=0;
	virtual bool playSE(const std::string& filename, int channnel=-1)=0;

	virtual bool loadBGM(const std::string& fiename)=0;
	virtual bool playBGM(const std::string& fiename)=0;
	virtual void haltBGM()=0;
};


class IResourceManager : public ist::Object
{
public:
	virtual ~IResourceManager() {}
	virtual texture_ptr getTexture(const std::string& name)=0;
	virtual pmodel_ptr getPModel(const std::string& name)=0;
	virtual amodel_ptr getAModel(const std::string& name)=0;
	virtual cmodel_ptr getCModel(const std::string& name)=0;
};

class IPathFinder : public ist::Object
{
public:
	virtual ~IPathFinder() {}
	virtual bool findPath(const ist::vector4& base, const ist::vector4& t_pos, selectable_ptr t_obj)=0;
	virtual size_t getPathSize()=0;
	virtual ist::vector4& getPath(int s)=0;
};


enum {
	SIMPLE,
	SINGLE,
	SERVER,
	CLIENT,
	REPLAY,
};

class IWorld : public ist::Object
{
public:
	virtual ~IWorld() {}

	virtual void setMode(int mode)=0;
	virtual int getMode()=0;
	virtual void setServerAddress(const std::string& address)=0;
	virtual void setServerPort(unsigned short port)=0;
	virtual const std::string& getServerDatabase()=0;

	virtual bool loadReplay(const std::string& path)=0;

	virtual void registKeyObserver(int key, keyobserv_ptr observer)=0;
	virtual void deregistKeyObserver(int key, keyobserv_ptr observer)=0;

	virtual gobj_id createID()=0;
	virtual faction_ptr  createFaction(const std::string& name)=0;
	virtual unit_ptr     createUnit(const std::string& name, faction_ptr)=0;
	virtual building_ptr createBuilding(const std::string& name, faction_ptr)=0;
	virtual material_ptr createMaterial(const std::string& name)=0;
	virtual group_ptr    createGroup(IGObjIterator& i)=0;

	virtual gobj_ptr getGObj(gobj_id id)=0;
	virtual bool addGObj(gobj_ptr p)=0;

	virtual void sendEvent(const Event& e)=0;
	virtual void stopSendingUpdateEvent()=0;
	virtual void startSendingUpdateEvent()=0;

	virtual void loadGround(const std::string& filename)=0;
	virtual void loadGroundTexture(const std::string& filename)=0;
	virtual float getHeight(const ist::vector4& pos)=0;
	virtual const ist::vector2& getMapMin()=0;
	virtual const ist::vector2& getMapMax()=0;
	virtual ist::Light& getGroundLight()=0;
	virtual void enableGroundLighting(bool)=0;
	virtual void enableGroundFog(bool)=0;
	virtual void enableSight(bool)=0;
	virtual bool isSighted(faction_ptr f, const ist::vector4& pos)=0;
	virtual bool isExplored(faction_ptr f, const ist::vector4& pos)=0;
	virtual void getMapTexture(ist::Bitmap& bmp)=0;
	virtual IPathFinder& getPathFinder()=0;

	virtual IGObjIterator& getBlocksByRadius(const ist::vector4& pos, float radius)=0;
	virtual IGObjIterator& getGObjsInSight()=0;
	virtual IGObjIterator& getFactions()=0;
	virtual IGObjIterator& getMenus()=0;

	virtual void insertToCharacterGrid(const ist::vector4& pos, gobj_ptr p)=0;
	virtual void eraseFromCharacterGrid(const ist::vector4& pos, gobj_ptr p)=0;
	virtual void moveCharacterGrid(const ist::vector4& before, const ist::vector4& after, gobj_ptr p)=0;
	virtual void addCircleCollision(selectable_ptr p, const ist::vector4& pos, float radius)=0;
	virtual void removeCircleCollision(selectable_ptr p, const ist::vector4& pos, float radius)=0;

	virtual faction_ptr getPlayerFaction()=0;
	virtual faction_ptr getNatureFaction()=0;
	virtual void setPlayerFaction(faction_ptr p)=0;
	virtual void setNatureFaction(faction_ptr p)=0;

	virtual ist::PerspectiveCamera& getCamera()=0;
	virtual const ist::vector4& getCameraPos()=0;
	virtual void setCameraPos(const ist::vector4&)=0;
	virtual const ist::matrix44& getAimCameraMatrix()=0;
	virtual const ist::vector4& getScreenRay()=0;
	virtual const ist::vector4& getMousePointerPos3D()=0;
	virtual const ist::vector2& getMousePointerPos2D()=0;

	virtual frame_t getCurrentFrame()=0;
};


class IGame : public ist::Object
{
public:
	virtual ~IGame() {}

	virtual void addFactionCreator(const std::string& name, faction_creator)=0;
	virtual void addUnitCreator(const std::string& name, unit_creator)=0;
	virtual void addBuildingCreator(const std::string& name, building_creator)=0;
	virtual void addMaterialCreator(const std::string& name, material_creator)=0;

	virtual faction_ptr  createFaction(const std::string& name)=0;
	virtual unit_ptr     createUnit(const std::string& name, faction_ptr)=0;
	virtual building_ptr createBuilding(const std::string& name, faction_ptr)=0;
	virtual material_ptr createMaterial(const std::string& name)=0;
	virtual group_ptr    createGroup(IGObjIterator& i)=0;

	virtual IWorld& getWorld(const std::string& name)=0;
	virtual IWorld& getCurrentWorld()=0;
	virtual void setCurrentWorld(const std::string& name)=0;
	virtual void requestSetCurrentWorld(const std::string& name)=0;

	virtual bool getAxis(int i)=0;
	virtual bool getButton(int i)=0;

	virtual int getWindowWidth()=0;
	virtual int getWindowHeight()=0;
	virtual void drawQuad(const ist::vector2& v1, const ist::vector2& v2, const ist::vector2& v3, const ist::vector2& v4)=0;
	virtual void drawRect(const ist::vector2& min, const ist::vector2& max)=0;
	virtual void drawFont(int x, int y, const char *message)=0;
	virtual ist::Kanji& getFont()=0;
	virtual float getRand()=0;

	virtual bool isShiftKeyDown()=0;
	virtual bool isAltKeyDown()=0;
	virtual bool isCtrlKeyDown()=0;

	virtual bool loadXMLDocument(const std::string& filename)=0;
	virtual bool loadDSO(const std::string& filename)=0;

	virtual IResourceManager& getResourceManager()=0;
	virtual ISoundPlayer& getSoundPlayer()=0;
	virtual void setSoundPlayer(boost::intrusive_ptr<ISoundPlayer>)=0;

	virtual void enableNoWait(bool)=0;
	virtual void exit()=0;
};





class IGameObject : public ist::Object
{
public:
	virtual ~IGameObject() {}

	virtual void update()=0;
	virtual void draw() const=0;

	virtual category_t getRecieveEvents() const=0;
	virtual gobj_id getID() const=0;
	virtual int getDrawPriority() const=0;
	virtual bool isDead() const=0;

	virtual void sendEvent(const Event&)=0;

	virtual bool category(const std::string&) const { return false; }
	virtual bool query(const std::string&, boost::any&) const  { return false; }
	virtual bool call(const std::string&, boost::any&) { return false; }

	virtual void onAdded()   {}
	virtual void onRemoved() {}

	virtual bool isVisible() const   { return false; }
	virtual bool isSelectable() const{ return false; }
	virtual bool isBreakable() const { return false; }

	virtual bool isGroup() const     { return false; }
	virtual bool isUnit() const      { return false; }
	virtual bool isBuilding() const  { return false; }
	virtual bool isMaterial() const  { return false; }
	virtual bool isBullet() const    { return false; }
	virtual bool isParticle() const  { return false; }
	virtual bool isMenu() const      { return false; }
	virtual bool isFaction() const   { return false; }
};



class IVisible : public IGameObject
{
public:
	virtual bool isVisible() const { return true; }

	virtual bool isInCameraSight() const=0;
	virtual bool isInSight(faction_ptr f) const=0;

	virtual const ist::vector4& getPos() const=0;
	virtual const ist::vector4& getRot() const=0;
	virtual const ist::vector4& getScl() const=0;
	virtual void setPos(const ist::vector4& pos)=0;
	virtual void setRot(const ist::vector4& pos)=0;
	virtual void setScl(const ist::vector4& pos)=0;
	virtual void calcMatrix(ist::matrix44&) const=0;
};

class ISelectable : public IVisible
{
public:
	virtual bool isSelectable() const{ return true; }

	virtual bool isMouseOver() const=0;
	virtual bool isSelected() const=0;

	virtual float getSight() const=0;
	virtual const ist::ICollisionModel& getHitModel() const=0;
	virtual const ist::ICollisionModel& getSelectionModel() const=0;
	virtual menu_ptr getInfomationMenu()=0;
	virtual menu_ptr getCommandMenu()=0;
	virtual menu_ptr getExtraMenu()=0;
	virtual texture_ptr getUnitImage() const=0;
	virtual const std::string& getUnitName() const=0;
	virtual const std::string& getClassName() const=0;
	virtual const std::string& getDescription() const=0;
	virtual faction_ptr getFaction() const=0;
};

class IBreakable : public ISelectable
{
public:
	virtual bool isBreakable() const { return true; }

	virtual float getBaseLife() const=0;
	virtual float getBaseMaxLife() const=0;
	virtual float getBaseFirePower() const=0;
	virtual float getBaseFireRange() const=0;
	virtual float getBaseAntiBullet() const=0;
	virtual float getBaseAntiLaser() const=0;
	virtual float getBaseAntiBomb() const=0;
	virtual float getLife() const=0;
	virtual float getMaxLife() const=0;
	virtual float getFirePower() const=0;
	virtual float getFireRange() const=0;
	virtual float getAntiBullet() const=0;
	virtual float getAntiLaser() const=0;
	virtual float getAntiBomb() const=0;
};



class IGroup : public IGameObject
{
public:
	bool isGroup() const      { return true; }

	virtual IGObjIterator& getUnits() const=0;
	virtual size_t size() const=0;
	virtual void erase(unit_ptr u)=0;
	virtual void add(unit_ptr u)=0;
	virtual bool find(unit_ptr u) const=0;
	virtual unit_ptr getLeader() const=0;
};

class IUnit : public IBreakable
{
public:
	bool isUnit() const      { return true; }

	virtual Event::actioninfo_ptr getCurrentAction() const=0;
	virtual void pushbackAction(Event::actioninfo_ptr a)=0;
	virtual void pushfrontAction(Event::actioninfo_ptr a)=0;
	virtual void clearAction()=0;
	virtual float getMoveSpeed() const=0;

	virtual group_ptr getGroup() const=0;
	virtual void setGroup(group_ptr g)=0;
};

class IBuilding : public IBreakable
{
public:
	bool isBuilding() const  { return true; }

	virtual void build(float b)=0;
	virtual float getBuildStatus() const=0;
	virtual bool isBuilt() const=0;
	virtual bool isSettlement() const=0;
};

class IMaterial : public ISelectable
{
public:
	bool isMaterial() const  { return true; }

	virtual float getQuantity() const=0;
	virtual void decreaseQuantity(float q)=0;

	virtual bool isTimber() const=0;
	virtual bool isSilicon() const=0;
	virtual bool isIron() const=0;
};



class IBullet : public IVisible
{
public:
	bool isBullet() const  { return true; }
};


class IParticle : public IVisible
{
public:
	bool isParticle() const  { return true; }
};


class IMenu : public IGameObject
{
public:
	virtual bool isMouseOver() const=0;
	virtual const std::string& getDescription() const=0;

	bool isMenu() const      { return true; }
};




class IRequest : public ist::Object
{
public:
	virtual ~IRequest() {}

	virtual Resource& getUsedResource()=0;
	virtual bool isComplete() const=0;
	virtual bool isCanceled() const=0;
	virtual const std::string getName() const=0;
	virtual void addProgress(float)=0;
};


class IFaction : public IGameObject
{
public:
	virtual Resource& getResource()=0;
	virtual int getPop() const=0;

	virtual bool pushRequest(request_ptr)=0;
	virtual void incrementPop(selectable_ptr)=0;
	virtual void decrementPop(selectable_ptr)=0;
	virtual IGObjIterator& getUnitTable(const std::string& classname) const=0;
	virtual IGObjIterator& getSelection() const=0;
	virtual gobj_ptr getFocused() const=0;

	virtual const ist::Material& getFactionMaterial() const=0;
	virtual const ist::Color& getFactionColor() const=0;

	virtual gobj_id getFactionID() const=0;
	virtual const std::string& getFactionName() const=0;
	virtual const std::string& getPlayerName() const=0;
	virtual const std::string& getAgeName() const=0;
	virtual void setFactionID(gobj_id)=0;
	virtual void setPlayerName(const std::string&)=0;
	virtual void pushMessage(const std::string&)=0;
	virtual const std::string& getMessage(size_t) const=0;
	virtual size_t getNumMessage() const=0;
	virtual menu_ptr createCommandMenu(const std::string&)=0;

	virtual bool addReserch(const std::string&)=0;
	virtual unit_ptr createUnit(const std::string&)=0;
	virtual building_ptr createBuilding(const std::string&)=0;

	virtual bool isAlly(faction_ptr f) const=0;
	virtual void addAlly(faction_ptr f)=0;
	virtual IGObjIterator& getAlly() const=0;

	bool isFaction() const  { return true; }
};




#endif
