#ifndef IST_BITMAP_H
#define IST_BITMAP_H

#include <vector>
#include <string>
#include "ist_conf.h"
#include "array2.h"

/*
#ifdef _WIN32
#	pragma comment(lib,"zdll.lib")
#	pragma comment(lib,"libpng.lib")
#	ifdef WITH_JPEG
#		pragma comment(lib,"libjpeg.lib")
#	endif
#endif
*/

namespace ist {


template <typename T>
struct RGBA {
	union {
		struct { T r,g,b,a; };
		T v[4];
	};

	RGBA<T> operator + (RGBA<T> &t) const { return RGBA<T>(r+t.r, g+t.g, b+t.b, a+t.a ); }
	RGBA<T> operator - (RGBA<T> &t) const { return RGBA<T>(r-t.r, g-t.g, b-t.b, a-t.a ); }
	RGBA<T> operator * (RGBA<T> &t) const { return RGBA<T>(r*t.r, g*t.g, b*t.b, a*t.a ); }
	RGBA<T> operator / (RGBA<T> &t) const { return RGBA<T>(r/t.r, g/t.g, b/t.b, a/t.a ); }
	RGBA<T>& operator +=(RGBA<T> &t) { *this=*this+t; return *this; }
	RGBA<T>& operator -=(RGBA<T> &t) { *this=*this-t; return *this; }
	RGBA<T>& operator *=(RGBA<T> &t) { *this=*this*t; return *this; }
	RGBA<T>& operator /=(RGBA<T> &t) { *this=*this/t; return *this; }
	bool operator ==(const RGBA<T> &t) const { return (r==t.r && g==t.g && b==t.b && a==t.a); }
	bool operator !=(const RGBA<T> &t) const { return !(*this==t); }
	T& operator [](int i) { return v[i]; }
	const T& operator [](int i) const { return v[i]; }

	RGBA<T>() : r(0), g(0), b(0), a(0) {}
	RGBA<T>(T _r, T _g, T _b, T _a) : r(_r), g(_g), b(_b), a(_a) {}

	T shin() const { return (r*0.299f + g*0.587f + b*0.114f); }
};

typedef RGBA<unsigned char> bRGBA;
typedef RGBA<float> fRGBA;
inline fRGBA bRGBA_to_fRGBA(const bRGBA& b) { return fRGBA(float(b.r)/255.0f, float(b.g)/255.0f, float(b.b)/255.0f, float(b.a)/255.0f ); }



class DSO_CLASS Bitmap : public array2<bRGBA>, public ist::Object
{
public:
	Bitmap() {}
	explicit Bitmap(const char *filename);
	Bitmap(size_t w, size_t h) { resize(w, h); }
	Bitmap(Bitmap& v, size_t x, size_t y, size_t width, size_t height) { assign(v, x, y, width, height); }

	bool load(const char *filename);
	bool save(const char *filename) const;

	bool loadBMP(const char *filename);
	bool loadTGA(const char *filename);
	bool saveBMP(const char *filename) const;
	bool saveTGA(const char *filename) const;

#ifdef WITH_PNG
	bool loadPNG(const char *filename);
	bool savePNG(const char *filename) const;
#endif
#ifdef WITH_JPEG
	bool loadJPG(const char *filename);
	bool saveJPG(const char *filename, int quality=75) const;
#endif
};


class DSO_CLASS fBitmap1 : public array2<float>, public ist::Object
{
public:
	fBitmap1() { clear(); }
	explicit fBitmap1(const char *filename) { clear(); load(filename); }
	fBitmap1(size_t w, size_t h) { clear(); resize(w, h); }

	bool load(const char *filename);
	bool save(const char *filename) const;
};



} //ist

#endif
