#include "ist_i3d.h"


namespace ist {


/*
	Diagram
*/

AABox AABox::getAABB(const ist::matrix44 &trans) const
{
	AABox aabb;

	for(int i=0; i<8; ++i) {
		ist::vector4 pos;

		switch(i) {
		case 0: pos=min; break;
		case 1: pos.x=max.x; pos.y=min.y; pos.z=min.z; break;
		case 2: pos.x=min.x; pos.y=max.y; pos.z=min.z; break;
		case 3: pos.x=min.x; pos.y=min.y; pos.z=max.z; break;
		case 4: pos=max; break;
		case 5: pos.x=min.x; pos.y=max.y; pos.z=max.z; break;
		case 6: pos.x=max.x; pos.y=min.y; pos.z=max.z; break;
		case 7: pos.x=max.x; pos.y=max.y; pos.z=min.z; break;
		default: break;
		}

		pos = trans*pos;

		if(i==0)
			aabb.max=aabb.min=pos;

		if		(pos.x>aabb.max.x) aabb.max.x=pos.x;
		else if	(pos.x<aabb.min.x) aabb.min.x=pos.x;

		if		(pos.y>aabb.max.y) aabb.max.y=pos.y;
		else if	(pos.y<aabb.min.y) aabb.min.y=pos.y;

		if		(pos.z>aabb.max.z) aabb.max.z=pos.z;
		else if	(pos.z<aabb.min.z) aabb.min.z=pos.z;
	}

	return aabb;
}

bool AABox::overlap(const AABox& bb) const
{
	if(		(max.x < bb.min.x || min.x < bb.max.x)
		&&	(max.y < bb.min.y || min.y < bb.max.y)
		&&	(max.z < bb.min.z || min.z < bb.max.z) )
	{
		return true;
	}
	else {
		return false;
	}
}

bool AABox::innerPoint(const Sphere& sphere) const
{
	return innerPoint(sphere.pos, sphere.radius);
}

bool AABox::innerPoint(const ist::vector4& pos, float radius) const
{
	if(		pos.x < max.x + radius
		&&	pos.x > min.x - radius
		&&	pos.y < max.y + radius
		&&	pos.y > min.y - radius
		&&	pos.z < max.z + radius
		&&	pos.z > min.z - radius )
	{
		return true;
	}
	else {
		return false;
	}
}

int AABox::innerTriangle(const ist::vector4& v1, const ist::vector4& v2, const ist::vector4& v3) const
{
	int result=0;

	if(		min.x < v1.x && max.x > v1.x
		&&	min.y < v1.y && max.y > v1.y
		&&	min.z < v1.z && max.z > v1.z ) ++result;

	if(		min.x < v2.x && max.x > v2.x
		&&	min.y < v2.y && max.y > v2.y
		&&	min.z < v2.z && max.z > v2.z ) ++result;

	if(		min.x < v3.x && max.x > v3.x
		&&	min.y < v3.y && max.y > v3.y
		&&	min.z < v3.z && max.z > v3.z ) ++result;

	return result;
}








/*
	CollisionDetector
*/


ICollisionModel::~ICollisionModel() {}

CollisionDetector::CollisionDetector() :
	result(false), distance(0)
{}

//|S-|S
template <>
bool CollisionDetector::detect_<IPolygonCollisionModel, IPolygonCollisionModel>
	(const IPolygonCollisionModel& base, const IPolygonCollisionModel& target)
{
	throw std::runtime_error("not implemented");

/*	if(base.chunc==0 || target.chunc==0
	|| !base.bb.overlap(target.bb) )
		return false;

	const BasePolygonChunc::line_cont &line_array = base.chunc->line_array;
	for(BasePolygonChunc::line_cont::const_iterator p=line_array.begin(); p!=line_array.end(); ++p) {
		const vector4& head = base.t_vertex[ p->getIndex(0) ];
		const vector4& tail = base.t_vertex[ p->getIndex(1) ];

		for(unsigned int i=0; i<target.chunc->getFaceNum(); ++i) {
			const vector4& normal = target.t_normal[i];
			const vector4 *vtx[3] = {
				&target.t_vertex[ target.chunc->idx_array[i*3+0] ],
				&target.t_vertex[ target.chunc->idx_array[i*3+1] ],
				&target.t_vertex[ target.chunc->idx_array[i*3+2] ],
			};
		
			float dist_tri = normal.dot( *vtx[0] );
			float dist_tail= normal.dot(tail);
			if( (normal.dot( head )-dist_tri)*(dist_tail-dist_tri)>0)
				continue;

			vector4 direction=(head-tail).normalize();
			float t=-(normal.dot(tail)-normal.dot(*vtx[0]))/normal.dot(direction);
			vector4 proj_pos=tail+direction*t;

			float inside=0;
			for(unsigned int j=0; j<3; ++j)
				inside+=(*vtx[j]-proj_pos).cos( *vtx[(j+1)%3]-proj_pos );

			if( inside>358.0f*radian ) {
				distance = base.chunc->bounding_sphere.radius;
				collide_pos=proj_pos;
				collide_normal=normal;
				return true;
			}
		}
	}
	
	return false;
*/
}

//|S-_
template <>
bool CollisionDetector::detect_<IPolygonCollisionModel, IPointCollisionModel>
	(const IPolygonCollisionModel& base, const IPointCollisionModel& target)
{
	const IPolygonCollisionModel::vertex_cont &b_vtx = base.getVertex();
	const IPolygonCollisionModel::normal_cont &b_nml = base.getNormal();
	const IPolygonCollisionModel::index_cont  &b_idx = base.getIndex();
	const vector4& t_pos = target.getPos();
	float t_radius = target.getRadius();

	if(!base.getBoundingBox().innerPoint(t_pos, t_radius))
		return false;


	bool is_inside = false;
	float min_distance = 0;
	vector4 last_collide_pos;
	vector4 last_collide_normal;

	for(unsigned int i=0; i<(b_idx.size()/3); ++i) {
		const int *triangle_index = &b_idx[i*3];
		const vector4& normal = b_nml[i];
		const vector4 *vtx[3] = {
			&b_vtx[ triangle_index[0] ],
			&b_vtx[ triangle_index[1] ],
			&b_vtx[ triangle_index[2] ],
		};

		for(int i=0; i<3; ++i) { // _ƏՓ˂Ă邩
			float d = (*vtx[i]-t_pos).norm();
			if(d<t_radius) {
				distance = d;
				collide_pos = *vtx[i];
				collide_normal = normal;
				return true;
			}
		}

		float dist1 = normal.dot( t_pos );	//_-_ԋ
		float dist2 = normal.dot( *vtx[0] );		//_-ʊԋ
		float dist3 = dist1-dist2;				//-_ԋAꂪʂ̔Αɂ

		if( fabsf(dist3)<fabsf(min_distance) || !is_inside ) {	//_Ɉԋ߂ʂT
			vector4 proj_pos = t_pos-normal*dist3;			//ʂɓeꂽ_̈ʒu

			float inside = 0;
			for(unsigned int j=0; j<3; ++j)		//Op`Ƃ̓OBe_Ɛp360Ȃɂ͂B
				inside+=(*vtx[j]-proj_pos).cos( *vtx[(j+1)%3]-proj_pos );

			if(inside>358.0f*radian) {	//2̌덷F߂
				is_inside = true;
				min_distance = dist3;

				last_collide_pos = proj_pos;
				last_collide_normal = normal;
			}
		}
	}

	if(min_distance<t_radius && is_inside) {
		distance = t_radius;
		collide_pos = last_collide_pos;
		collide_normal = last_collide_normal;
		return true;
	}
	else
		return false;
}

//_-|S
template <>
bool CollisionDetector::detect_<IPointCollisionModel, IPolygonCollisionModel>
	(const IPointCollisionModel& base, const IPolygonCollisionModel& target)
{
	return detect_(target, base);
}

//|S-
template<>
bool CollisionDetector::detect_<IPolygonCollisionModel, ILineCollisionModel>
	(const IPolygonCollisionModel& base, const ILineCollisionModel& target)
{
	if( !base.getBoundingBox().overlap(target.getBoundingBox()) )
		return false;

	const IPolygonCollisionModel::vertex_cont &b_vtx = base.getVertex();
	const IPolygonCollisionModel::normal_cont &b_nml = base.getNormal();
	const IPolygonCollisionModel::index_cont  &b_idx = base.getIndex();
	const vector4& t_head = target.getHead();
	const vector4& t_tail = target.getTail();

	bool is_inside = false;
	float min_distance = 0;
	vector4 last_collide_pos;
	vector4 last_collide_normal;

	for(unsigned int i=0; i<(b_idx.size()/3); ++i) {
		const vector4& normal = b_nml[i];
		const vector4 *vtx[3] = {
			&b_vtx[ b_idx[i*3+0] ],
			&b_vtx[ b_idx[i*3+1] ],
			&b_vtx[ b_idx[i*3+2] ],
		};
	
		float dist_tri  = normal.dot(*vtx[0]);
		float dist_tail = normal.dot(t_tail);
		if( (normal.dot(t_head)-dist_tri)*(dist_tail-dist_tri)>0)
			continue;

		vector4 direction = (t_head-t_tail).normalize();
		float t = -(normal.dot(t_tail)-normal.dot(*vtx[0]))/normal.dot(direction);
		vector4 proj_pos = t_tail+direction*t;

		float inside=0;
		for(unsigned int j=0; j<3; ++j) {
			if((proj_pos-*vtx[j]).square()<0.01) {
				inside=360.0f*radian;
				break;
			}
			inside+=(*vtx[j]-proj_pos).cos( *vtx[(j+1)%3]-proj_pos );
		}

		if( (inside>358.0f*radian)
		&&  (fabsf(dist_tail)<fabsf(min_distance) || !is_inside) ) {
			is_inside = true;
			min_distance = dist_tail;
			last_collide_pos = proj_pos;
			last_collide_normal = normal;
		}
	}

	if(is_inside) {
		distance = target.getRadius()+((t_tail-last_collide_pos).norm()+min_distance);
		collide_pos = last_collide_pos;
		collide_normal = last_collide_normal;
		return true;
	}
	else
		return false;
}

//-|S
template<>
bool CollisionDetector::detect_<ILineCollisionModel, IPolygonCollisionModel>
	(const ILineCollisionModel& base, const IPolygonCollisionModel& target)
{
	return detect_(target, base);
}



//_-_
template <>
bool CollisionDetector::detect_<IPointCollisionModel, IPointCollisionModel>
	(const IPointCollisionModel& base, const IPointCollisionModel& target)
{
	if( !base.getBoundingBox().overlap(target.getBoundingBox()) )
		return false;

	const vector4 &b_pos = base.getPos();
	const vector4 &t_pos = target.getPos();

	vector4 pos_gap = t_pos-b_pos;
	float union_radius = base.getRadius() + target.getRadius();

	if(pos_gap.norm() < union_radius) {
		distance = union_radius;
		collide_normal = vector4(pos_gap).normalize();
		collide_pos = b_pos;
		return true;
	}

	return false;
}


//-_
template <>
bool CollisionDetector::detect_<ILineCollisionModel, IPointCollisionModel>
	(const ILineCollisionModel& base, const IPointCollisionModel& target)
{
	if( !base.getBoundingBox().overlap(target.getBoundingBox()) )
		return false;

	const vector4 &b_head = base.getHead();
	const vector4 &b_tail = base.getTail();
	float b_radius = base.getRadius();
	const vector4 &t_pos = target.getPos();
	float t_radius = target.getRadius();

	bool _result = false;
	vector4 segment = b_head-b_tail;
	float union_radius = b_radius+t_radius;

	float u = 0;
	if(b_tail!=b_head)
		u = (t_pos-b_tail).dot(segment) / segment.square();

	if(u>0 && u<1) {
		vector4 nearest = b_tail+(segment*u);
		float distance = (t_pos-nearest).norm();

		if( distance < union_radius ) {
			collide_normal = (t_pos-nearest).normalize();
			collide_pos = nearest;
			_result=true;
		}
	}
	else if((t_pos-b_head).norm()<union_radius) {
		collide_normal = (t_pos-b_head).normalize();
		collide_pos = b_head;
		_result=true;
	}
	else if((t_pos-b_tail).norm()<union_radius) {
		collide_normal = (t_pos-b_tail).normalize();
		collide_pos = b_tail;
		_result=true;
	}

	if(_result)
		distance = union_radius;

	return _result;
}


//_-
template <>
bool CollisionDetector::detect_<IPointCollisionModel, ILineCollisionModel>
	(const IPointCollisionModel& base, const ILineCollisionModel& target)
{
	return detect_(target, base);
}


//-
template <>
bool CollisionDetector::detect_<ILineCollisionModel, ILineCollisionModel>
	(const ILineCollisionModel& base, const ILineCollisionModel& target)
{
	if( !base.getBoundingBox().overlap(target.getBoundingBox()) )
		return false;

	return false;
}


//_-2D|
template <>
bool CollisionDetector::detect_<IPointCollisionModel, IPolygon2DCollisionModel>
	(const IPointCollisionModel& base, const IPolygon2DCollisionModel& target)
{
	vector2 point_pos(base.getPos().x, base.getPos().z);
	float ang = 0;
	for(size_t i=0; i<target.getNumVertex(); ++i) {
		ang+=(target.getVertex(i)-point_pos).cos(target.getVertex((i+1)%target.getNumVertex())-point_pos);
	}
	if(ang>358.0f*radian) {
		return true;
	}

	return false;
}

//2D|-_
template <>
bool CollisionDetector::detect_<IPolygon2DCollisionModel, IPointCollisionModel>
	(const IPolygon2DCollisionModel& base, const IPointCollisionModel& target)
{
	return detect_(target, base);
}

//2D|-2D|
template <>
bool CollisionDetector::detect_<IPolygon2DCollisionModel, IPolygon2DCollisionModel>
	(const IPolygon2DCollisionModel& base, const IPolygon2DCollisionModel& target)
{
	return false;
}


//NX^
bool CollisionDetector::detect(const ICollisionModel &base, const ICollisionModel &target)
{
	result = false;

	if(base.getType()==CM_TRUE || target.getType()==CM_TRUE) {
		result = true;
	}
	else if( base.getType()==CM_POINT ) {
		if     ( target.getType()==CM_POINT )
			result=detect_( (static_cast<const IPointCollisionModel&>(base)),
							(static_cast<const IPointCollisionModel&>(target)) );
		else if( target.getType()==CM_LINE )
			result=detect_( (static_cast<const IPointCollisionModel&>(base)),
							(static_cast<const ILineCollisionModel&>(target)) );
		else if( target.getType()==CM_POLYGON )
			result=detect_( (static_cast<const IPointCollisionModel&>(base)),
							(static_cast<const IPolygonCollisionModel&>(target)) );
		else if( target.getType()==CM_POLYGON2D )
			result=detect_( (static_cast<const IPointCollisionModel&>(base)),
							(static_cast<const IPolygon2DCollisionModel&>(target)) );
	}
	else if( base.getType()==CM_LINE ) {
		if     ( target.getType()==CM_POINT )
			result=detect_( (static_cast<const ILineCollisionModel&>(base)),
							(static_cast<const IPointCollisionModel&>(target)) );
		else if( target.getType()==CM_LINE )
			result=detect_( (static_cast<const ILineCollisionModel&>(base)),
							(static_cast<const ILineCollisionModel&>(target)) );
		else if( target.getType()==CM_POLYGON )
			result=detect_( (static_cast<const ILineCollisionModel&>(base)),
							(static_cast<const IPolygonCollisionModel&>(target)) );
	}
	else if( base.getType()==CM_POLYGON ) {
		if( target.getType()==CM_POINT )
			result=detect_( (static_cast<const IPolygonCollisionModel&>(base)),
							(static_cast<const IPointCollisionModel&>(target)) );
		else if( target.getType()==CM_LINE )
			result=detect_( (static_cast<const IPolygonCollisionModel&>(base)),
							(static_cast<const ILineCollisionModel&>(target)) );
		else if( target.getType()==CM_POLYGON )
			result=detect_( (static_cast<const IPolygonCollisionModel&>(base)),
							(static_cast<const IPolygonCollisionModel&>(target)) );
	}
	else if( base.getType()==CM_POLYGON2D ) {
		if( target.getType()==CM_POINT )
			result=detect_( (static_cast<const IPolygon2DCollisionModel&>(base)),
							(static_cast<const IPointCollisionModel&>(target)) );
		else if( target.getType()==CM_POLYGON2D )
			result=detect_( (static_cast<const IPolygon2DCollisionModel&>(base)),
							(static_cast<const IPolygon2DCollisionModel&>(target)) );
	}
	return result;
}




/*
	PolygonModel
*/


BPolygonChunc::BPolygonChunc() :
	facet(0.0f),
	face_num(0)
{}

BPolygonChunc::~BPolygonChunc()
{}

void BPolygonChunc::clear()
{
	index.clear();
	vertex.clear();
	vtexcoord.clear();
	normal.clear();
	vnormal.clear();
	bbox = AABox();
	bsphere = Sphere();
	facet = 0.0f;
}

void BPolygonChunc::draw() const
{
	assert(0);
}


void BPolygonChunc::setNormal()
{
	if(normal.empty() && vnormal.empty()) {
		for(size_t i=0; i<face_num; ++i) {
			ist::vector4 vec1 = vertex[index[i*3+1]].position - vertex[index[i*3]].position;
			ist::vector4 vec2 = vertex[index[i*3+2]].position - vertex[index[i*3]].position;

			ist::vector4 cur_normal = ist::vector4().cross(vec1, vec2);
			normal.push_back(cur_normal.v3());
		}
	}

	if(vnormal.empty()) {
		vnormal.reserve(index.size());
		for(size_t i=0; i<index.size(); ++i) {
			int union_num = 0;
			ist::vector3 std_normal = normal[i/3];
			ist::vector3 cur_normal = std_normal;

			for(size_t j=0; j<face_num; ++j) {
				if( index[i]==index[j*3+0] || index[i]==index[j*3+1] || index[i]==index[j*3+2] ) {
					if( std_normal!=normal[j] && std_normal.cos(normal[j]) <= facet*radian) {
						cur_normal+=normal[j];
						union_num++;
					}
				}
			}

			cur_normal.normalize();
			vnormal.push_back(cur_normal);
		}
	}

	for(unsigned int i=0; i<normal.size(); ++i) {
		normal[i].normalize();
	}
}

void BPolygonChunc::setBoundingVolume()
{
	if(!vertex.empty()) {
		bbox.min = vertex.front().position;
		bbox.max = vertex.front().position;
		for(vertex_cont::iterator p=vertex.begin(); p!=vertex.end(); ++p) {
			const ist::vector4& v = p->position;
			if(v.x > bbox.max.x)      bbox.max.x = v.x;
			else if(v.x < bbox.min.x) bbox.min.x = v.x;
			if(v.y > bbox.max.y)      bbox.max.y = v.y;
			else if(v.y < bbox.min.y) bbox.min.y = v.y;
			if(v.z > bbox.max.z)      bbox.max.z = v.z;
			else if(v.z < bbox.min.z) bbox.min.z = v.z;
		}
	}
	else if(!vvertex.empty()) {
		bbox.min = vvertex.front().v4();
		bbox.max = vvertex.front().v4();
		for(position_cont::iterator p=vvertex.begin(); p!=vvertex.end(); ++p) {
			if(p->x > bbox.max.x)      bbox.max.x = p->x;
			else if(p->x < bbox.min.x) bbox.min.x = p->x;
			if(p->y > bbox.max.y)      bbox.max.y = p->y;
			else if(p->y < bbox.min.y) bbox.min.y = p->y;
			if(p->z > bbox.max.z)      bbox.max.z = p->z;
			else if(p->z < bbox.min.z) bbox.min.z = p->z;
		}
	}

	vector4 center = (bbox.max+bbox.min)/2.0f;
	bsphere.pos = center;
	bsphere.radius = (bbox.max-center).norm();
}


void BPolygonChunc::setup()
{
	face_num = index.size()/3;

	if(vvertex.empty()) {
		vvertex.resize(index.size());
		for(size_t i=0; i<index.size(); ++i) {
			vvertex[i] = vertex[index[i]].position.v3();
		}
	}

	setNormal();
	setBoundingVolume();
}





BPolygonModel::chunc_ptr BPolygonModel::getChunc(int index)
{
	chunc_cont::iterator p = chuncs.begin();
	advance(p, index);
	return (*p).get();
}

BPolygonModel::chunc_ptr BPolygonModel::getChunc(const std::string &label)
{
	int i = 0;
	for(label_cont::iterator p=labels.begin(); p!=labels.end(); ++p) {
		if(*p==label)
			return getChunc(i);
		++i;
	}

	return chunc_ptr();
}



void BPolygonModel::draw() const
{
	for(chunc_cont::const_iterator p=chuncs.begin(); p!=chuncs.end(); ++p)
		(*p)->draw();
}


bool BPolygonModel::load(const std::string& filename)
{
	chuncs.clear();
	labels.clear();

	if(    (strncmp(&filename[filename.size()]-4, ".mqo", 4)==0 && loadMQO(filename))
		|| (strncmp(&filename[filename.size()]-4, ".obj", 4)==0 && loadOBJ(filename)) ) {

		setup();
		return true;
	}

	return false;
}

void BPolygonModel::setup()
{
	for(chunc_cont::iterator p=chuncs.begin(); p!=chuncs.end(); ) {
		(*p)->setup();
		++p;
	}
}



// objǂݍ 

class OBJLoader
{
public:
	typedef std::vector<ist::vector3> position_cont;
	typedef std::vector<ist::vector3> normal_cont;
	typedef std::vector<ist::vector2> texcoord_cont;
	typedef std::vector<size_t>  index_cont;

	struct chunc
	{
		std::string label;
		index_cont    vindex;
		index_cont    tindex;
		index_cont    nindex;
		float facet;

		chunc() : facet(60.0f) {}
	};
	typedef std::list<chunc> chunc_cont;

	chunc_cont chuncs;
	position_cont vertex;
	normal_cont   normal;
	texcoord_cont texcoord;
	int status;
	std::iostream& in;



	OBJLoader(std::iostream& ios) : status(0), in(ios) {
		char buf[1024];

		while(true) {
			in.getline(buf, 1024);
			if(strncmp(buf, "g ", 2)==0 || chuncs.empty()) {
				chuncs.push_back(chunc());
				char label[128];
				if(sscanf(buf, "g %s", label)==1)
					chuncs.back().label = label;

			   	continue;
			}

			int s = loadVertex(buf);
			if(s!=0)
				status|=s;
			else
				loadFace(buf);
		}

	//	for(chunc_cont::iterator p=chuncs.begin(); p!=chuncs.end(); ++p)
	//		assignVertexArray();

		for(chunc_cont::iterator p=chuncs.begin(); p!=chuncs.end(); ) {
			if(p->vindex.empty())
				chuncs.erase(p++);
			else
				++p;
		}
	}

	void assignVertexArray(chunc& c, position_cont& varray, index_cont& vindex)
	{
		if(c.vindex.empty())
			return;

		size_t max_index = *max_element( c.vindex.begin(), c.vindex.end() )+1;
		size_t min_index = *min_element( c.vindex.begin(), c.vindex.end() );

		varray.assign( vertex.begin()+min_index, vertex.begin()+max_index );	// KvȒ_Rs[
		for(index_cont::iterator p=vindex.begin(); p!=vindex.end(); ++p)
			*p-=min_index;
	}

	int loadVertex(char *buf)
	{
		if(buf[0]!='v')
			return 0;

		ist::vector3 vtx;
		ist::vector3 nml;
		ist::vector2 tex;

		if(sscanf(buf, "v %f %f %f", &vtx.x, &vtx.y, &vtx.z)==3) {
			vertex.push_back(vtx);
			return 1;
		}
		else if( sscanf(buf, "vt %f %f", &tex.x, &tex.y)==2 ) {
			texcoord.push_back(tex);
			return 2;
		}
		else if( sscanf(buf, "vn %f %f %f", &nml.x, &nml.y, &nml.z)==3 ) {
			normal.push_back(nml);
			return 4;
		}
		else {
			return 0;
		}
	}

	void loadFace(char *buf)
	{
		if(strncmp(buf, "f ", 2)!=0)
			return;
		if(chuncs.empty())
			chuncs.push_back(chunc());

		chunc& c = chuncs.back();
		std::vector<size_t> vindex, tindex, nindex;
		int ngon = loadIndex(buf, vindex, tindex, nindex);

		for(int i=0; i<ngon-2; i++) {	// S3p|Sɕϊ
			int index[3];
			index[0] = 0;
			index[1] = i+1;
			index[2] = i+2;

			for(int j=0; j<3; ++j) {
				if(status&1)
					c.vindex.push_back(vindex[index[j]]);
				if(status&2)
					c.tindex.push_back(tindex[index[j]]);
				if(status&4)
					c.nindex.push_back(nindex[index[j]]);
			}
		}
	}

	int loadIndex(char *buf, std::vector<size_t>& vindex, std::vector<size_t>& tindex, std::vector<size_t>& nindex)
	{
		int tvi, tti, tni;
		int ngon = 0;

		char *token;
		token = strtok(buf, " "); // v. 
		token = strtok(0, " ");

		while(token) {
			switch(status) {
			case(1):
				sscanf(token, "%d", &tvi);
				vindex.push_back(tvi-1);
				break;

			case(3):
				sscanf(token, "%d/%d", &tvi, &tti);
				vindex.push_back(tvi-1);
				tindex.push_back(tti-1);
				break;

			case(5):
				sscanf(token, "%d/%d", &tvi, &tni);
				vindex.push_back(tvi-1);
				nindex.push_back(tni-1);
				break;

			case(7):
				sscanf(token, "%d/%d/%d", &tvi, &tti, &tni);
				vindex.push_back(tvi-1);
				tindex.push_back(tti-1);
				nindex.push_back(tni-1);
				break;

			default:
				break;
			}

			++ngon;
			token = strtok(0, " ");
		}

		return ngon;
	}
};

bool BPolygonModel::loadOBJ(const std::string& filename) {
	std::fstream in(filename.c_str(), std::ios::in);
	if(!in)
		return false;
	
	OBJLoader loader(in);
	for(OBJLoader::chunc_cont::iterator p=loader.chuncs.begin(); p!=loader.chuncs.end(); ++p) {
		cchunc_ptr c = createChunc();
		for(size_t i=0; i<loader.vertex.size(); ++i) {
			vertex_t v;
			v.position = loader.vertex[i].v4();
			c->vertex.push_back(v);
		}
		c->index = p->vindex;
		for(size_t i=0; i<p->tindex.size(); ++i) {
			c->vtexcoord.push_back(loader.texcoord[p->tindex[i]]);
		}
		// normaldl 

		c->setup();
		addChunc(c, p->label);
	}
	return true;
}


// mqoǂݍ 

class MQOLoader
{
public:
	typedef std::vector<ist::vector3> position_cont;
	typedef std::vector<ist::vector2> texcoord_cont;
	typedef std::vector<size_t>  index_cont;

	struct chunc
	{
		std::string label;
		position_cont vertex;
		texcoord_cont vtexcoord;
		index_cont    index;
		float facet;

		chunc() : facet(60.0f) {}
	};
	typedef std::list<chunc> chunc_cont;

	chunc_cont chuncs;
	std::iostream& in;

	MQOLoader(std::iostream& ios) : in(ios) {
		char buf[1024];
		char label[128];
		while(!in.eof()) {
			in.getline(buf, 1024);
			if(strcmp(buf, "Eof")==0)
				break;
			if(sscanf(buf, "Object \"%[^\"]\" {", label)==1) {
				chuncs.push_back(chunc());
				chuncs.back().label = label;

				while(true) {
					in.getline(buf, 1024);
					if(buf[0]=='}')
						break;
					loadAttrib(buf);
					loadVertex(buf);
					loadFace(buf);
				}
			}
		}
	}

	bool loadFace(char *buf) {
		size_t num_face;
		if(sscanf(buf, "\tface %d", &num_face)==0)
			return false;

		chunc& c = chuncs.back();
		for(size_t i=0; i<num_face; ++i) {
			int ngon = 0;
			int index[4];
			vector2 tex[4];
	
			in.getline(buf, 1024);
			char *token = strtok(buf, " ");
			sscanf(token, " %d", &ngon);

			while(token) {
				if(sscanf(token, " V(%d %d %d %d", &index[0], &index[1], &index[2], &index[3])==ngon ) {
					for(int i=0; i<ngon-2; ++i) {
						c.index.push_back(index[0]);
						c.index.push_back(index[i+2]);
						c.index.push_back(index[i+1]);
					}
				}
				else if(sscanf(token, " UV(%f %f %f %f %f %f %f %f",
						 &tex[0].x,&tex[0].y, &tex[1].x,&tex[1].y, &tex[2].x,&tex[2].y, &tex[3].x,&tex[3].y)==ngon*2 ) {
					for(int i=0; i<ngon-2; ++i) {
						c.vtexcoord.push_back(tex[0]);
						c.vtexcoord.push_back(tex[i+2]);
						c.vtexcoord.push_back(tex[i+1]);
					}
				}
				token = strtok(0, ")");
			}
		}
		in.getline(buf, 1024); // "	}"
		return true;
	}

	bool loadVertex(char *buf)
	{
		size_t num_vertex;
		if(sscanf(buf, "\tvertex %d", &num_vertex)==0)
			return false;

		chunc& c = chuncs.back();
		for(size_t i=0; i<num_vertex; ++i) {
			ist::vector3 vtx;
			in.getline(buf, 1024);
			if(sscanf(buf, "\t\t%f %f %f", &vtx.x, &vtx.y, &vtx.z)==3)
				c.vertex.push_back(vtx);
		}
		in.getline(buf, 1024); // "	}"
		return true;
	}

	bool loadAttrib(char *buf)
	{
		float facet;
		if(sscanf(buf, "\tfacet %f", &facet)==1) {
			chunc& c = chuncs.back();
			c.facet = facet;
			return true;
		}
		return false;
	}

};



bool BPolygonModel::loadMQO(const std::string& filename)
{
	std::fstream in(filename.c_str(), std::ios::in);
	if(!in)
		return false;
	
	MQOLoader loader(in);
	for(MQOLoader::chunc_cont::iterator p=loader.chuncs.begin(); p!=loader.chuncs.end(); ++p) {
		cchunc_ptr c = createChunc();
		for(size_t i=0; i<p->vertex.size(); ++i) {
			vertex_t v;
			v.position = p->vertex[i].v4();
			c->vertex.push_back(v);
		}
		c->index = p->index;
		c->vtexcoord = p->vtexcoord;
		c->facet = p->facet;
		c->setup();
		addChunc(c, p->label);
	}
	return true;
}






/*
	MorphModel
*/

struct AVTXHeader {
	unsigned int version_number;
	unsigned int index_count;
	unsigned int vertex_count;
	unsigned int vertex_clip_count;
};

struct AVTXClipHeader {
	unsigned int frame_count;
	unsigned int label_length;
};


BMorphModel::BMorphModel() :
	num_vertex(0)
{}

BMorphModel::~BMorphModel()
{}

void BMorphModel::draw(const std::string& clipname, float t) const
{
	draw(vtx_clip.find(clipname), t);
}

void BMorphModel::draw(iterator iter, float t) const
{}

void BMorphModel::setup()
{
	if(vtx_clip.empty() || vtx_clip.begin()->second.empty())
		return;

	vertex_cont& vc = vtx_clip.begin()->second.begin()->second;
	bbox.min = vc.front().v4();
	bbox.max = vc.front().v4();
	for(vertex_cont::iterator p=vc.begin(); p!=vc.end(); ++p) {
		if(p->x > bbox.max.x)      bbox.max.x = p->x;
		else if(p->x < bbox.min.x) bbox.min.x = p->x;
		if(p->y > bbox.max.y)      bbox.max.y = p->y;
		else if(p->y < bbox.min.y) bbox.min.y = p->y;
		if(p->z > bbox.max.z)      bbox.max.z = p->z;
		else if(p->z < bbox.min.z) bbox.min.z = p->z;
	}

	ist::vector4 center = (bbox.max+bbox.min)/2;
	bsphere.pos = center;
	bsphere.radius = (bbox.max-center).norm();
}



bool BMorphModel::load(const std::string& filename) {
	clear();

	if(strncmp(&filename[filename.size()]-4, ".avx", 5)!=0)
		return false;

	std::fstream f(filename.c_str(), std::ios::in | std::ios::binary);
	ist::biostream bf(f);
	if(!f)
		return false;

	AVTXHeader header;
	bf	>> header.version_number
		>> header.index_count
		>> header.vertex_count
		>> header.vertex_clip_count;
	if(header.version_number!=100)
		return false;

	idx_array.resize(header.index_count);
	tex_array.resize(header.index_count);
	num_vertex = header.vertex_count;


	for(unsigned int i=0; i<idx_array.size(); ++i)
		bf >> idx_array[i];

	for(unsigned int i=0; i<tex_array.size(); ++i) {
		bf >> tex_array[i].x;
		bf >> tex_array[i].y;
	}

	for(unsigned int i=0; i<header.vertex_clip_count; ++i) {
		AVTXClipHeader clip_header;
		std::vector<char> temp_string(256, '\0');
		bf >> clip_header.frame_count;
		bf >> clip_header.label_length;
		bf.read(&temp_string[0], clip_header.label_length);

		vertex_sequence& seq = vtx_clip[&temp_string[0]];

		for(unsigned int j=0; j<clip_header.frame_count; ++j) {
			float t;
			bf >> t;

			vertex_cont& vcont = seq[t];
			vcont.resize(header.vertex_count);

			for(unsigned int k=0; k<vcont.size(); ++k) {
				bf >> vcont[k].x >> vcont[k].y >> vcont[k].z;
			}
		}
	}
	setup();

	return true;
}

bool BMorphModel::save(const std::string& filename) {
	std::fstream f(filename.c_str(), std::ios::out | std::ios::binary);
	ist::biostream bf(f);
	if(!f)
		return false;

	AVTXHeader header={
		100,
		idx_array.size(),
		num_vertex,
		vtx_clip.size()
	};

	bf	<< header.version_number
		<< header.index_count
		<< header.vertex_count
		<< header.vertex_clip_count;

	for(unsigned int i=0; i<idx_array.size(); ++i)
		bf << idx_array[i];

	for(unsigned int i=0; i<tex_array.size(); ++i) {
		bf << tex_array[i].x;
		bf << tex_array[i].y;
	}

	for(vertex_clip::iterator p=vtx_clip.begin(); p!=vtx_clip.end(); ++p) {
		vertex_sequence& seq = p->second;
		AVTXClipHeader clip_header={
			seq.size(),
			p->first.size(),
		};

		bf	<< clip_header.frame_count
			<< clip_header.label_length;
		bf.write(p->first.c_str(), p->first.size());

		for(vertex_sequence::iterator q=seq.begin(); q!=seq.end(); ++q) {
			bf << q->first;

			vertex_cont& vcont = q->second;
			for(unsigned int i=0; i<vcont.size(); ++i) {
				bf << vcont[i].x << vcont[i].y << vcont[i].z;
			}
		}
	}

	return true;
}

void BMorphModel::clear() {
	vtx_clip.clear();
	tex_array.clear();
	idx_array.clear();
	num_vertex = 0;
}

void BMorphModel::vertex_sequence::morph(float t, vertex_cont& t_vertex) const
{
	if(t<=beginFrame()) {
		t_vertex = begin()->second;
	}
	else if(t>=endFrame()) {
		t_vertex = (--end())->second;
	}
	else {
		vertex_sequence::range_t range = equal_range(t);
		if(range.first->first==t) {
			t_vertex = range.first->second;
		}
		else {
			--range.first;
			const float& keyframe1 = range.first->first;
			const float& keyframe2 = range.second->first;
			const vertex_cont& vcont1 = range.first->second;
			const vertex_cont& vcont2 = range.second->second;
			t_vertex.resize(vcont1.size());
			for(size_t i=0; i<vcont1.size(); ++i) {
				vector3 dir = (vcont2[i]-vcont1[i])/(keyframe2-keyframe1);
				t_vertex[i] = vcont1[i] + dir*(t-keyframe1);
			}
		}
	}
}




BCachedMorphModel::BCachedMorphModel() :
	num_vertex(0),
	facet(60.0f),
	enable_normal(true)
{}

BCachedMorphModel::BCachedMorphModel(const std::string& filename, bool en) :
	num_vertex(0),
	facet(60.0f),
	enable_normal(true)
{
	enableNormal(en);
	load(filename);
}

BCachedMorphModel::~BCachedMorphModel()
{}

bool BCachedMorphModel::load(const std::string& filename) {
	BMorphModel tpolygon;
	if(!tpolygon.load(filename))
		return false;

	tex_array = tpolygon.tex_array;
	idx_array = tpolygon.idx_array;
	num_vertex = tpolygon.num_vertex;

	vertex_cont t_vertex;
	typedef BMorphModel::vertex_clip::iterator clip_iterator;
	for(clip_iterator p=tpolygon.vtx_clip.begin(); p!=tpolygon.vtx_clip.end(); ++p) {
		vertex_sequence& seq = vtx_clip[p->first];
		seq.begin_frame = int(tpolygon.beginFrame(p->first));
		seq.end_frame = int(tpolygon.endFrame(p->first));
		int frame_length = seq.endFrame()-seq.beginFrame();
		seq.vertex.resize(std::max<int>(1, frame_length));
		seq.normal.resize(std::max<int>(1, frame_length));
		int t=0;
		do {
			p->second.morph(t+seq.beginFrame(), t_vertex);
			setVertex(t_vertex, seq.vertex[t]);
			if(enable_normal)
				setNormal(t_vertex, seq.normal[t]);
			++t;
		} while(t<frame_length);
	}

	setup();
	return true;
}

void BCachedMorphModel::setVertex(vertex_cont& vtx_array, vertex_cont& vvtx_array)
{
	vvtx_array.resize(idx_array.size());
	for(size_t i=0; i<idx_array.size(); ++i) {
		vvtx_array[i] = vtx_array[idx_array[i]];
	}
}

void BCachedMorphModel::setNormal(vertex_cont& vtx_array, normal_cont& vnml_array)
{
	normal_cont nml_array;

	for(size_t i=0; i<idx_array.size()/3; ++i) {
		vector3 vec1 = vtx_array[ idx_array[i*3+1] ] - vtx_array[ idx_array[i*3] ];
		vector3 vec2 = vtx_array[ idx_array[i*3+2] ] - vtx_array[ idx_array[i*3] ];
		nml_array.push_back(vector3().cross(vec1, vec2));
	}

	vnml_array.reserve(idx_array.size());
	for(size_t i=0; i<idx_array.size(); ++i) {
		int union_num = 0;
		vector3 std_normal = nml_array[i/3];
		vector3 cur_normal = std_normal;

		for(size_t j=0; j<idx_array.size()/3; ++j) {
			if(idx_array[i]==idx_array[j*3+0] || idx_array[i]==idx_array[j*3+1] || idx_array[i]==idx_array[j*3+2]) {
				if(std_normal!=nml_array[j] && std_normal.cos(nml_array[j]) <= facet*radian) {
					cur_normal+=nml_array[j];
					union_num++;
				}
			}
		}

		cur_normal.normalize();
		vnml_array.push_back(cur_normal);
	}
}

void BCachedMorphModel::draw(const std::string& clipname, int t) const
{
	draw(vtx_clip.find(clipname), t);
}

void BCachedMorphModel::draw(iterator p, int t) const
{}

void BCachedMorphModel::setup()
{
	if(vtx_clip.empty())
		return;

	vertex_cont& vtx_array = vtx_clip.begin()->second.vertex.front();
	bbox.min = vtx_array.front().v4();
	bbox.max = vtx_array.front().v4();
	for(vertex_cont::iterator p=vtx_array.begin(); p!=vtx_array.end(); ++p) {
		if(p->x > bbox.max.x)      bbox.max.x = p->x;
		else if(p->x < bbox.min.x) bbox.min.x = p->x;
		if(p->y > bbox.max.y)      bbox.max.y = p->y;
		else if(p->y < bbox.min.y) bbox.min.y = p->y;
		if(p->z > bbox.max.z)      bbox.max.z = p->z;
		else if(p->z < bbox.min.z) bbox.min.z = p->z;
	}

	ist::vector4 center = (bbox.max+bbox.min)/2;
	bsphere.pos = center;
	bsphere.radius = (bbox.max-center).norm();
}

void BCachedMorphModel::clear() {
	vtx_clip.clear();
	tex_array.clear();
	idx_array.clear();
	num_vertex=0;
}


}

