#include "ist_i3d.h"

#define BUFFER_OFFSET(i) ((char*)0 + (i))



namespace ist {



/*
	GLInfo
*/

void GLInfo::print(std::iostream& out) {
	out << "GL_VERSION:  " << glGetString(GL_VERSION) << std::endl;
	out << "GL_RENDERER: " << glGetString(GL_RENDERER) << std::endl;
	out << "GL_VENDOR:   " << glGetString(GL_VENDOR) << std::endl;

	out << std::endl;
	out << "GL_EXTENSIONS:" << std::endl;
	for(const GLubyte *ext = glGetString(GL_EXTENSIONS); *ext!='\0'; ++ext) {
		if(*ext==' ')
			out << std::endl;
		else
			out << *ext;
	}

	out << std::endl;
	int v[2];
	float fv[2];
	glGetIntegerv(GL_MAX_ATTRIB_STACK_DEPTH, v);
	out << "GL_MAX_ATTRIB_STACK_DEPTH:     " << v[0] << std::endl;

	glGetIntegerv(GL_MAX_CLIP_PLANES, v);
	out << "GL_MAX_CLIP_PLANES:            " << v[0] << std::endl;

	glGetIntegerv(GL_MAX_EVAL_ORDER, v);
	out << "GL_MAX_EVAL_ORDER:             " << v[0] << std::endl;

	glGetIntegerv(GL_MAX_LIGHTS, v);
	out << "GL_MAX_LIGHTS:                 " << v[0] << std::endl;

	glGetIntegerv(GL_MAX_LIST_NESTING, v);
	out << "GL_MAX_LIST_NESTING:           " << v[0] << std::endl;

	glGetIntegerv(GL_MAX_MODELVIEW_STACK_DEPTH, v);
	out << "GL_MAX_MODELVIEW_STACK_DEPTH:  " << v[0] << std::endl;

	glGetIntegerv(GL_MAX_NAME_STACK_DEPTH, v);
	out << "GL_MAX_NAME_STACK_DEPTH:       " << v[0] << std::endl;

	glGetIntegerv(GL_MAX_PIXEL_MAP_TABLE, v);
	out << "GL_MAX_PIXEL_MAP_TABLE:        " << v[0] << std::endl;

	glGetIntegerv(GL_MAX_PROJECTION_STACK_DEPTH, v);
	out << "GL_MAX_PROJECTION_STACK_DEPTH: " << v[0] << std::endl;

	glGetIntegerv(GL_MAX_TEXTURE_SIZE, v);
	out << "GL_MAX_TEXTURE_SIZE:           " << v[0] << std::endl;

	glGetIntegerv(GL_MAX_TEXTURE_STACK_DEPTH, v);
	out << "GL_MAX_TEXTURE_STACK_DEPTH:    " << v[0] << std::endl;

	glGetIntegerv(GL_MAX_VIEWPORT_DIMS, v);
	out << "GL_MAX_VIEWPORT_DIMS:          " << v[0] << " " << v[1] << std::endl;


	glGetIntegerv(GL_SUBPIXEL_BITS, v);
	out << "GL_SUBPIXEL_BITS:              " << v[0] << std::endl;

	glGetFloatv(GL_POINT_SIZE_RANGE, fv);
	out << "GL_POINT_SIZE_RANGE:           " << fv[0] << " " << fv[1] << std::endl;

	glGetFloatv(GL_POINT_SIZE_GRANULARITY, fv);
	out << "GL_POINT_GRANULARITY:          " << fv[0] << std::endl;

	glGetFloatv(GL_LINE_WIDTH_RANGE, fv);
	out << "GL_LINE_WIDTH_RANGE:           " << fv[0] << " " << fv[1] << std::endl;

	glGetFloatv(GL_LINE_WIDTH_GRANULARITY, fv);
	out << "GL_LINE_WIDTH_GRANULARITY:     " << fv[0] << std::endl;
	out << std::endl;
	out << std::endl;
}








/*
	Matrix
*/

void ScreenMatrix()
{
	int viewport[4];

	glGetIntegerv(GL_VIEWPORT, viewport);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0,viewport[2], 0,viewport[3], -1,5000);
//	gluOrtho2D(0,viewport[2], 0,viewport[3]);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void ScreenMatrixIV()
{
	int viewport[4];

	glGetIntegerv(GL_VIEWPORT, viewport);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0,viewport[2], viewport[3],0, -1,5000);
//	gluOrtho2D(0,viewport[2], viewport[3],0);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

MatrixSaver::MatrixSaver(int matrix_type) {
	glGetFloatv(matrix_type, matrix.v);

	switch(matrix_type) {
	case GL_MODELVIEW_MATRIX:
		matrix_mode = GL_MODELVIEW;
		break;
	case GL_PROJECTION_MATRIX:
		matrix_mode = GL_PROJECTION;
		break;
	case GL_TEXTURE_MATRIX:
		matrix_mode = GL_TEXTURE;
		break;
	default:
		matrix_mode = GL_MODELVIEW;
		break;
	}
}

MatrixSaver::~MatrixSaver() {
	int current_matrix_mode;
	glGetIntegerv(GL_MATRIX_MODE, &current_matrix_mode);

	glMatrixMode(matrix_mode);
	glLoadMatrixf(matrix.v);
	glMatrixMode(current_matrix_mode);
}

const ist::matrix44& MatrixSaver::getMatrix() const { return matrix; }









/*
	Material
*/

Material::Material() :
	ambient(0.2f, 0.2f, 0.2f, 1.0f),
	diffuse(0.8f, 0.8f, 0.8f, 1.0f),
	specular(0.0f, 0.0f, 0.0f, 0.0f),
	emission(0.0f, 0.0f, 0.0f, 0.0f),
	shininess(60.0f)
{};

void Material::assign() const
{
	glMaterialfv(GL_FRONT, GL_AMBIENT,  ambient.v);
	glMaterialfv(GL_FRONT, GL_DIFFUSE,  diffuse.v);
	glMaterialfv(GL_FRONT, GL_SPECULAR, specular.v);
	glMaterialfv(GL_FRONT, GL_EMISSION, emission.v);
	glMaterialfv(GL_FRONT, GL_SHININESS,&shininess);
}


Color::Color() :
	color(1.0f, 1.0f, 1.0f, 1.0f)
{}

Color::Color(float r, float g, float b, float a) :
	color(r, g, b, a)
{}

void Color::assign() const
{
	glColor4fv(color.v);
}



/*
	Light
*/

namespace light {
	std::map<int, const Light*> light_list;
	int light_table[8]={GL_LIGHT0, GL_LIGHT1, GL_LIGHT2, GL_LIGHT3, GL_LIGHT4, GL_LIGHT5, GL_LIGHT6, GL_LIGHT7};

	bool assign_id(const Light *lp) {
		for(int i=0; i<8; ++i) {
			if(light_list[ light_table[i] ]==0) {
				light_list[ light_table[i] ]=lp;
				return true;
			}
		}
		return false;
	}

	bool remove_id(const Light *lp) {
		for(int i=0; i<8; ++i) {
			if(light_list[ light_table[i] ]==lp) {
				light_list[ light_table[i] ]=0;
				return true;
			}
		}
		return false;
	}

	int search_id(const Light *lp) {
		static const Light *last_lp = 0;
		static int last_id = 0;

		if(lp==last_lp)
			return last_id;
		for(int i=0; i<8; ++i) {
			if(light_list[ light_table[i] ]==lp) {
				last_lp = lp;
				last_id = light_table[i];
				return light_table[i];
			}
		}
		return 0;
	}
}

Light::Light()
	: ambient(0.2, 0.2, 0.2, 1.0),
	  diffuse(0.8, 0.8, 0.8, 1.0),
	  specular(1.0, 1.0, 1.0, 1.0)
{
	light::assign_id(this);
}
	
Light::~Light()
{
	light::remove_id(this);
}

void Light::enable() const
{
	int id = light::search_id(this);
	if(id==0)
		return;
	glEnable(id);
	glLightfv(id, GL_POSITION, position.v);
	glLightfv(id, GL_AMBIENT,  ambient.v);
	glLightfv(id, GL_DIFFUSE,  diffuse.v);
	glLightfv(id, GL_SPECULAR, specular.v);
}

void Light::disable() const
{
	int id = light::search_id(this);
	if(id==0)
		return;
	glDisable(id);
}

bool Light::isValid()  const {
	return light::search_id(this)!=0;
}



/*
	Fog
*/

Fog::Fog() : start(0), end(100.0f) {}
Fog::~Fog() { disable(); }

void Fog::enable() const
{
	glEnable(GL_FOG);
	glFogi(GL_FOG_MODE, GL_LINEAR);
	glFogf(GL_FOG_START, start);
	glFogf(GL_FOG_END, end);
	glFogfv(GL_FOG_COLOR, color.v);
}

void Fog::disable() const
{
	glDisable(GL_FOG);
}



/*
	Camera
*/

ViewFrustum::ViewFrustum() : z_near(1.0f), z_far(2000.0f) {
	vertex.resize(5);
	normal.resize(6);
	index.resize(18);

	index[0]=0; index[1]=1; index[2]=2;
	index[3]=0; index[4]=2; index[5]=3;
	index[6]=0; index[7]=3; index[8]=4;
	index[9]=0; index[10]=4; index[11]=1;
	index[12]=1; index[13]=4; index[14]=3;
	index[15]=3; index[16]=2; index[17]=1;
}

void ViewFrustum::setup(const vector4& pos_, const vector4& target4, const vector3& up, float z_near_, float z_far_, float fovy, float aspect) {
	float fovyx = fovy*aspect;
	front = ist::vector3((target4-pos).v).normalize();
	pos = pos_;
	z_near = z_near_;
	z_far = z_far_;

	ist::vector3 vx;
	ist::vector3 vy;
	vx.cross(front, up);
	vx.normalize();
	vy.cross(vx, front);
	vy.normalize();

	matrix33 rot_t = ist::matrix33().rotateA(vx, -90-fovy/2.0f);
	matrix33 rot_b = ist::matrix33().rotateA(vx,  90+fovy/2.0f);
	matrix33 rot_r = ist::matrix33().rotateA(vy,  90+fovyx/2.0f);
	matrix33 rot_l = ist::matrix33().rotateA(vy, -90-fovyx/2.0f);
	top   = rot_t*front;
	bottom= rot_b*front;
	left  = rot_l*front;
	right = rot_r*front;
	vertex[0] = pos;
	vertex[1] = pos - (top+right).v4()*z_far;
	vertex[2] = pos - (top+left).v4()*z_far;
	vertex[3] = pos - (bottom+left).v4()*z_far;
	vertex[4] = pos - (bottom+right).v4()*z_far;
	for(size_t i=0; i<index.size()/3; ++i) {
		normal[i] = vector4().cross(
			vertex[index[i*3+1]]-vertex[index[i*3]], vertex[index[i*3+2]]-vertex[index[i*3]]).normalize();
	}

	setBoundingBox();
}

int ViewFrustum::test(const vector4& center, float r) const {
	vector4 rel_pos = center-pos;
	float dist = front.dot(rel_pos);

	if(dist<z_near-r || dist>z_far+r)
		return 0;
	float dot_top = top.dot(rel_pos);
	float dot_bottom = bottom.dot(rel_pos);
	float dot_right = right.dot(rel_pos);
	float dot_left = left.dot(rel_pos);
	if(dot_top>r || dot_bottom>r || dot_right>r || dot_left>r)
		return 0;
	else if(dot_top>-r || dot_bottom>-r || dot_right>-r || dot_left>-r)
		return 1;
	return 2;
}

int ViewFrustum::test(const ICollisionModel& cm) const {
	if(cm.getType()==CM_TRUE) {
		return 2;
	}
	else if(cm.getType()==CM_POINT) {
		const IPointCollisionModel& pcm = static_cast<const IPointCollisionModel&>(cm);
		return test(pcm.getPos(), pcm.getRadius());
	}
	else if(cm.getType()==CM_LINE) {
		const ILineCollisionModel& lcm = static_cast<const ILineCollisionModel&>(cm);
		return test(lcm.getHead(), lcm.getRadius());
	}

	return false;
}

Camera::Camera() :
	position(0.0f, 0.0f, 100.0f),
	target(0.0f, 0.0f, 0.0f),
	direction(0.0f, 1.0f, 0.0f),
	z_near(1.0f), z_far(1000.0f)
{}

Camera::~Camera() {}


OrthographicCamera::OrthographicCamera() :
	left(0.0f),
	right(100.0f),
	bottom(100.0f),
	top(0.0f)
{
	z_near = -1.0f;
}

void OrthographicCamera::look() {
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glOrtho(left, right, bottom, top, z_near, z_far);
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	gluLookAt(
		position.x, position.y, position.z,
		target.x, target.y, target.z,
		direction.x, direction.y, direction.z);
}

void OrthographicCamera::setScreen(float l, float r, float b, float t) {
	left = l;
	right = r;
	bottom = b;
	top = t;
}


PerspectiveCamera::PerspectiveCamera() :
	fovy(60.0f), aspect(1.3333333f)
{}

void PerspectiveCamera::look() {
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective(fovy, aspect, z_near, z_far);
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	gluLookAt(
		position.x, position.y, position.z,
		target.x, target.y, target.z,
		direction.x, direction.y, direction.z);
}

void PerspectiveCamera::setViewFrustum() {
	view_frustum.setup(position, target, direction, z_near, z_far, fovy, aspect);
}

const ViewFrustum& PerspectiveCamera::getViewFrustum() const {
	return view_frustum;
}




/*
	Texture
*/

Texture::~Texture() { clear(); }
Texture::Texture() { setDefaultParam(); }

Texture::Texture(const char *filename, size_t cell_width, size_t cell_height) {
	setDefaultParam();
	load(Bitmap(filename), cell_width, cell_height);
}

Texture::Texture(const Bitmap& src, size_t cell_width, size_t cell_height) {
	setDefaultParam();
	load(src, cell_width, cell_height);
}

void Texture::setDefaultParam() {
	setDimension(GL_TEXTURE_2D);
	setType(GL_RGBA);
	setMagFilter(GL_LINEAR);	// GL_LINEAR | GL_NEAREST
	setMinFilter(GL_LINEAR);
	setWarpS(GL_CLAMP);			// GL_CLAMP | GL_REPEAT
	setWarpT(GL_CLAMP);
	setEnvMode(GL_MODULATE);	// GL_MODULATE | GL_DECAL | GL_BLEND | GL_ADD
}

void Texture::clear() {
	if(!_tex_name.empty()) {
		glDeleteTextures(_tex_name.size(), &_tex_name[0]);
		_tex_name.clear();
	}
}

void Texture::setParam(unsigned int name) const {
	glTexParameteri(_dimension, GL_TEXTURE_WRAP_S, _warp_s);
	glTexParameteri(_dimension, GL_TEXTURE_WRAP_T, _warp_t);
	glTexParameteri(_dimension, GL_TEXTURE_MAG_FILTER, _mag_filter);
	glTexParameteri(_dimension, GL_TEXTURE_MIN_FILTER, _min_filter);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, _env_mode);
	
	float ecolor[] = {1.0f, 1.0f, 1.0f, 1.0f};
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, _env_mode);
	glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, ecolor);
}

void Texture::generate(unsigned int& name, const Bitmap& src) {
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glGenTextures(1, &name);
	glBindTexture(_dimension, name);
	setParam(name);

	unsigned int width = src.getWidth();
	unsigned int height = src.getHeight();

	if(_type==GL_RGBA) {
		glTexImage2D(_dimension, 0, 4, width, height, 0, _type, GL_UNSIGNED_BYTE, src[0][0].v);
	}
	else if(_type==GL_RGB) {
		std::vector<unsigned char> data(width*height*3);
		for(int i=0; i<height; ++i) {
			for(int j=0; j<width; ++j) {
				data[i*width*3 +j*3 +0] = src[i*width][j].r;
				data[i*width*3 +j*3 +1] = src[i*width][j].g;
				data[i*width*3 +j*3 +2] = src[i*width][j].b;
			}
		}
		glTexImage2D(_dimension, 0, 3, width, height, 0, _type, GL_UNSIGNED_BYTE, &data[0]);
	}
	else if(_type==GL_ALPHA || _type==GL_LUMINANCE) {
		std::vector<unsigned char> data(width*height);
		for(int i=0; i<height; ++i) {
			for(int j=0; j<width; ++j) {
				data[i*width +j] = src[i*width][j].a;
			}
		}
		glTexImage2D(_dimension, 0, 1, width, height, 0, _type, GL_UNSIGNED_BYTE, &data[0]);
	}
}

bool Texture::load(const char *filename, size_t cell_width, size_t cell_height) {
	Bitmap bmp;
	if(!bmp.load(filename))
		return false;
	return load(bmp, cell_width, cell_height);
}

bool Texture::load(const Bitmap& src, size_t cell_width, size_t cell_height) {
	clear();
	if(src.getWidth()<16)
		return false;

	if(cell_width==0 && cell_height==0) {
		_tex_name.push_back(0);
		generate(_tex_name.back(), src);
	}
	else if(cell_width>=16 && cell_height>=16){
		Bitmap cell(cell_width, cell_height);

		unsigned int repeat_x = src.getWidth()/cell_width;
		unsigned int repeat_y = src.getHeight()/cell_height;
		_tex_name.resize(repeat_x*repeat_y);

		for(unsigned int i=0; i<_tex_name.size(); i++) {
			for(unsigned int y=0; y<cell_height; y++) {
				for(unsigned int x=0; x<cell_width; x++) {
					cell[y][x] = src[(i/repeat_x)*cell_height+y][(i%repeat_x)*cell_width+x];
				}
			}
			generate(_tex_name[i], cell);
		}
	}
	else {
		return false;
	}

	return true;
}


void Texture::assign(int index) const {
	if(!_tex_name.empty() && index<_tex_name.size() ) {
		glBindTexture(_dimension, _tex_name[index]);
		setParam(_tex_name[index]);
	}
	else {
		glBindTexture(_dimension, 0);
	}
}





/*
	PolygonModel
*/

PolygonChunc::PolygonChunc()
{}

PolygonChunc::~PolygonChunc()
{}

void PolygonChunc::setup() {
	BPolygonChunc::setup();

	vertex_array.resize(vvertex.size()*3+vnormal.size()*3+vtexcoord.size()*2);

	size_t offset = 0;
	for(size_t i=0; i<vvertex.size(); ++i) {
		vertex_array[i*3+0] = vvertex[i].x;
		vertex_array[i*3+1] = vvertex[i].y;
		vertex_array[i*3+2] = vvertex[i].z;
	}
	offset+=vvertex.size()*3;

	for(size_t i=0; i<vnormal.size(); ++i) {
		vertex_array[offset+i*3+0] = vnormal[i].x;
		vertex_array[offset+i*3+1] = vnormal[i].y;
		vertex_array[offset+i*3+2] = vnormal[i].z;
	}
	offset+=vnormal.size()*3;

	if(!vtexcoord.empty()) {
		for(size_t i=0; i<vtexcoord.size(); ++i) {
			vertex_array[offset+i*2+0] = vtexcoord[i].x;
			vertex_array[offset+i*2+1] = vtexcoord[i].y;
		}
	}
	else {
		for(size_t i=0; i<vtexcoord.size(); ++i) {
			vertex_array[offset+i*2+0] = 0;
			vertex_array[offset+i*2+1] = 0;
		}
	}
//	vvertex().swap(position_cont());
}


void PolygonChunc::draw() const {
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_NORMAL_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

	const float *vertex_pointer = &vertex_array[0];
	glVertexPointer(3, GL_FLOAT, 0, vertex_pointer);
	glNormalPointer(GL_FLOAT, 0, vertex_pointer+face_num*3*3 );
	glTexCoordPointer(2, GL_FLOAT, 0, vertex_pointer+face_num*3*6 );

	glDrawArrays(GL_TRIANGLES, 0, face_num*3);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	glDisableClientState(GL_NORMAL_ARRAY);
	glDisableClientState(GL_VERTEX_ARRAY);
}



VBOPolygonChunc::VBOPolygonChunc() : vbo_id(0)
{}

VBOPolygonChunc::~VBOPolygonChunc() {
	if(vbo_id)
		glDeleteBuffersARB(1, &vbo_id);
}
void VBOPolygonChunc::setup() {
	PolygonChunc::setup();

	glGenBuffersARB(1, &vbo_id);
	glBindBufferARB(GL_ARRAY_BUFFER_ARB, vbo_id);
	glBufferDataARB(GL_ARRAY_BUFFER_ARB, index.size()*8*sizeof(float), &vertex_array[0], GL_STATIC_DRAW_ARB);
}


void VBOPolygonChunc::draw() const {
	if(!vbo_id)
		return;

	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_NORMAL_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

	glBindBufferARB(GL_ARRAY_BUFFER_ARB, vbo_id);
	glVertexPointer(3, GL_FLOAT, 0, 0);
	glNormalPointer(GL_FLOAT, 0, BUFFER_OFFSET(index.size()*3*sizeof(float)) );
	glTexCoordPointer(2, GL_FLOAT, 0, BUFFER_OFFSET(index.size()*6*sizeof(float)) );

	glDrawArrays(GL_TRIANGLES, 0, face_num*3);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	glDisableClientState(GL_NORMAL_ARRAY);
	glDisableClientState(GL_VERTEX_ARRAY);
}


void MorphModel::draw(iterator p, float t) const
{
	if(p==vtx_clip.end() || p->second.empty())
		return;

	static vertex_cont t_vertex;
	static vertex_cont vertex_array;
	p->second.morph(t, t_vertex);

	vertex_array.resize(idx_array.size()*3);
	for(size_t i=0; i<idx_array.size(); ++i) {
		vertex_array[i] = t_vertex[idx_array[i]];
	}

	glVertexPointer(3, GL_FLOAT, 0, vertex_array.front().v);
	glTexCoordPointer(2, GL_FLOAT, 0, tex_array.front().v);
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glDrawArrays(GL_TRIANGLES, 0, idx_array.size());
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	glDisableClientState(GL_VERTEX_ARRAY);
}


void CachedMorphModel::draw(iterator p, int t) const
{
	if(p==vtx_clip.end())
		return;

	const vertex_sequence& seq = p->second;
	if(seq.vertex.empty())
		return;

	const vertex_cont *vertex_array;
	if(t<=seq.beginFrame())
		vertex_array = &seq.vertex.front();
	else if(t>=seq.endFrame())
		vertex_array = &seq.vertex.back();
	else
		vertex_array = &seq.vertex[t-seq.beginFrame()];

	if(enable_normal) {
		const normal_cont *normal_array;
		if(t<seq.beginFrame())
			normal_array = &seq.normal.front();
		else if(t>=seq.endFrame())
			normal_array = &seq.normal.back();
		else
			normal_array = &seq.normal[t-seq.beginFrame()];

		glVertexPointer(3, GL_FLOAT, 0, vertex_array->front().v);
		glNormalPointer(GL_FLOAT, 0, normal_array->front().v);
		glTexCoordPointer(2, GL_FLOAT, 0, tex_array.front().v);
		glEnableClientState(GL_VERTEX_ARRAY);
		glEnableClientState(GL_NORMAL_ARRAY);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		glDrawArrays(GL_TRIANGLES, 0, idx_array.size());
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		glDisableClientState(GL_NORMAL_ARRAY);
		glDisableClientState(GL_VERTEX_ARRAY);
	}
	else {
		glVertexPointer(3, GL_FLOAT, 0, vertex_array->front().v);
		glTexCoordPointer(2, GL_FLOAT, 0, tex_array.front().v);
		glEnableClientState(GL_VERTEX_ARRAY);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		glDrawArrays(GL_TRIANGLES, 0, idx_array.size());
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		glDisableClientState(GL_VERTEX_ARRAY);
	}
}




/*
	Kanji
*/

static void sjis2jis(unsigned char &c1, unsigned char &c2)
{
	if (c1>=0xe0) { c1 = c1-0x40; }
	if (c2>=0x80) { c2 = c2-1; }
	if (c2>=0x9e) {
		c1 = (c1-0x70) * 2;
		c2 = c2-0x7d;
	} else {
		c1 = ((c1-0x70)*2)-1;
		c2 = c2-0x1f;
	}
}

static void euc2jis(unsigned char &c1, unsigned char &c2)
{
	c1-=0x80;
	c2-=0x80;
}




Kanji::Kanji(int _code)
	: code(_code)
{
	init();
}

Kanji::Kanji(const std::string& filename, int _code)
	: code(_code)
{
	init();
	load(filename);
}

void Kanji::setCode(int _code) { code=_code; }
int Kanji::getFontSize() { return font_size; }

void Kanji::init()
{
	font_size = 0;
	char_byte = 0;
	max_index = 0;
}

void Kanji::print(int x, int y, const char *_str)
{
	if(_str==0 || data.empty()) return;

	const unsigned char *str = reinterpret_cast<const unsigned char*>(_str);

	MatrixSaver mmsaver(GL_MODELVIEW_MATRIX);	// ϊsޔ
	MatrixSaver pmsaver(GL_PROJECTION_MATRIX);
	ScreenMatrixIV();		// WnXN[ɂ킹

	glRasterPos2i(x, y+font_size);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	int down = 0;

	while(*str!=0) {
		if(*str=='\n') {
			down+=font_size+1;
			++str;
			glRasterPos2i(x, y+font_size+down);
			continue;
		}
		if(isprint(*str)) {	// asciȉꍇ
			if(*str < max_index)
				glBitmap(font_size, font_size, font_size/2, 0, font_size/2, 0, getChar(*str) );
			++str;
		}
		else {	// {ꕶ̏ꍇ
			unsigned char c1=str[0];
			unsigned char c2=str[1];

			switch(code) {
				case SJIS: sjis2jis(c1,c2); break;
				case EUC:  euc2jis (c1,c2); break;
			}

			int index=(c1-0x20)*96 + c2-0x20 +0xff;
			if(index<max_index && index>=0)
				glBitmap(font_size, font_size, 0, 0, font_size, 0, getChar(index) );
			str+=2;
		}
	}
}


bool Kanji::load(const std::string& filename)
{
	data.clear();
	font_size = 0;
	char_byte = 0;

	std::fstream f(filename.c_str(), std::ios::in | std::ios::binary);
	ist::biostream bf(f);
	if(!f)
		return false;

	bf >> font_size;

	int n = 0;
	while(n<font_size) { n+=8; }
	int char_bit = n*font_size;
	char_byte = char_bit/8;


	unsigned char length, temp;
	while(!f.eof()) {
		bf >> length;
		if(length<0x80) {
			for(int i=0; i<length+1; ++i) {
				bf >> temp;
				data.push_back(temp);
			}
		}
		else {
			bf >> temp;

			for(int i=0x80; i<length+1; ++i)
				data.push_back(temp);
		}
	}

	max_index = data.size()/char_byte;
	return true;
}

const unsigned char* Kanji::getChar(int index) { return &data[index*char_byte]; }



}// ist
