#include "ist_net.h"
#include <sstream>
#include <algorithm>

namespace ist {

HTTP::HTTP() : respons(0), clength(0), recieved(0), stream(0)
{
	stream = &content;
}

bool HTTP::get(const std::string& url, unsigned short port) {
	clear();
	splitURL(url);

	TCPSocket sst(host.c_str(), port);
	if(!sst)
		return false;

	std::stringstream ss;
	ss	<< "GET " << path << " HTTP/1.1\r\n"
		<< "Accept: */*\r\n"
		<< "Host: " << host << "\r\n"
		<< "\r\n";
	sst << ss.str();
	analyzeHead(sst);
	getContent(sst);
	return !!(*this);
}

bool HTTP::head(const std::string& url, unsigned short port) {
	clear();
	splitURL(url);

	TCPSocket sst(host.c_str(), port);
	if(!sst)
		return false;

	std::stringstream ss;
	ss	<< "HEAD " << path << " HTTP/1.1\r\n"
		<< "Accept: */*\r\n"
		<< "Host: " << host << "\r\n"
		<< "\r\n";
	sst << ss.str();

	analyzeHead(sst);
	return !!(*this);
}

bool HTTP::post(const std::string& url, const std::string& postdata, unsigned short port) {
	clear();
	splitURL(url);

	TCPSocket sst(host.c_str(), port);
	if(!sst)
		return false;

	std::stringstream ss;
	ss	<< "POST " << path << " HTTP/1.1\r\n"
		<< "Content-Length: " << postdata.size() << "\r\n"
		<< "Host: " << host << "\r\n"
		<< "\r\n"
		<< postdata << "\r\n";
	sst << ss.str();
	analyzeHead(sst);
	getContent(sst);
	return !!(*this);
}

bool HTTP::operator!() const {
	return respons!=200;
}

ist::bbuffer& HTTP::getContent() {
	return content;
}

void HTTP::setStream(ist::bstream& s) {
	stream = &s;
}


void HTTP::clear() {
	content.clear();
	respons = 0;
	clength = 0;
	recieved = 0;
}

void HTTP::splitURL(const std::string& _url) {
	std::string url = _url;
	if(strncmp(url.c_str(), "http://", 7)==0)
		url.erase(url.begin(), url.begin()+7);
	if(!url.empty() && url[url.size()-1]=='\n')
		url.erase(url.end()-1);
	if(!url.empty() && url[url.size()-1]=='\r')
		url.erase(url.end()-1);

	if(url.find('/')==std::string::npos) {
		host = url;
		path = "/";
	}
	else {
		host = std::string(url.begin(), url.begin()+url.find_first_of('/'));
		path = std::string(url.begin()+url.find_first_of('/'), url.begin()+url.size());
		filename = std::string(url.begin()+url.find_last_of('/')+1, url.begin()+url.size());
	}
}

void HTTP::analyzeHead(TCPSocket& sst) {
	bool clength_catched = false;
	std::string l;
	while(!!sst && std::getline(sst, l)) {	// wb_ 
		for(size_t i=0; i<l.size(); ++i)
			l[i] = tolower(l[i]);

		if(l[0]=='\r') {
			break;
		}
		else if(sscanf(l.c_str(), "content-length: %d", &clength)==1) {
			clength_catched = true;
		}
		else {
			sscanf(l.c_str(), "http/1.1 %d", &respons);
		}
	}
	if(!!sst && !clength_catched) {
		std::getline(sst, l);
		sscanf(l.c_str(), "%x", &clength);
	}
}

void HTTP::getContent(TCPSocket& sst) {
	char buf[1024];
	while(!!sst && recieved<clength) {
		size_t num = std::min<size_t>(1024, clength-recieved);
		sst.read(buf, num);
		stream->write(buf, num);
		recieved+=num;
	}
}

} // ist
