#ifndef IST_SYS_H
#define IST_SYS_H

#ifdef _WIN32
	#include <windows.h>
#endif
#include <string>
#include <vector>
#include <map>
#include "ist_conf.h"
#include "bstream.h"


namespace ist {

/*
	Directory
*/
DSO_EXPORT bool IsDir(const char *path);
DSO_EXPORT bool MakeDir(const char *path);
DSO_EXPORT size_t MakeDeepDir(const char *path); // "hoge/hage/hige" Ƃw肵hoge/hage/܂ō쐬B쐬ԂB 
DSO_EXPORT bool RemoveDir(const char *path); // ̃fBNĝ݂폜B 
DSO_EXPORT bool Remove(const char *path); // t@C͋̃fBNg폜B 
DSO_EXPORT bool RemoveRecursive(const char *path); // fBNg/t@CċAIɊSBނ݂Ɏg_B 


class DSO_CLASS Dir : public ist::Object
{
public:
	typedef std::vector<std::string> path_cont;
	typedef path_cont::iterator iterator;
	typedef path_cont::const_iterator const_iterator;

	Dir();
	explicit Dir(const char *path);
	bool open(const char *path);
	bool openRecursive(const char *path);

	size_t size() const;
	const std::string& operator[](size_t i) const;
	const std::string& getPath() const;

	iterator begin() { return m_files.begin(); }
	iterator end()   { return m_files.end(); }
	const_iterator begin() const { return m_files.begin(); }
	const_iterator end() const   { return m_files.end(); }

private:
	std::string m_path;
	path_cont m_files;
};


/*
	DynamicSharedObject
*/
class DSO_CLASS DSO : public ist::Object
{
public:
	DSO();
	DSO(const std::string& filename);
	~DSO();
	bool load(const std::string& filename);
	void unload();
	void* getFunction(const std::string& funcname) const;
	const std::string& getFilename() const;
	bool operator!() const;

private:
	std::string filename;
#ifdef _WIN32
	HMODULE handle;
#else
	void *handle;
#endif
};



/*
	IGZ
*/
class DSO_CLASS IGZCompresser : public ist::Object
{
public:
	IGZCompresser(ist::bstream& s);
	virtual ~IGZCompresser();

	virtual size_t add(const char *path); // w肵fBNg/t@CċAIɎ擾ĈkBkt@C̐ԂB 
	virtual bool addBuffer(const char *path, const ist::bbuffer& out);

private:
	ist::bstream& bf;
};

class DSO_CLASS IGZExtracter : public ist::Object
{
public:
	IGZExtracter(ist::bstream& s);
	virtual ~IGZExtracter();

	virtual bool extractBuffer(const std::string& filename, ist::bbuffer& out);
	virtual bool extractFile(const std::string& filename);
	virtual size_t extractAllFiles();

private:
	typedef std::map<std::string, std::streampos> entry_cont;
	entry_cont entry;
	ist::bstream& bf;
};


} // namespace ist

#endif
