#ifndef CONFIG_INCLUDE
#define CONFIG_INCLUDE

#include <string>
#include <list>

class FrameConfig
{
public:
	static FrameConfig& instance() {
		static FrameConfig c;
		return c;
	}
	int width;
	int height;
	int ex_flags;

private:
	FrameConfig() : width(800), height(600), ex_flags(0) {}
};


class GameConfig
{
public:
	static GameConfig& instance() {
		static GameConfig c;
		return c;
	}

	bool noeffect;
	bool nowait;
	bool frameskip;
	unsigned short port;
	unsigned short server_port;
	std::string player_name;
	std::string server_database;
	std::string server_address;
	std::string system_font;
	std::list<std::string> dso_list;

private:
	GameConfig() {
		player_name = "hoge";

		noeffect = false;
		nowait = false;
		frameskip = false;
		system_font = "resource/font16.f1b";

		port = 10031;

		server_database = "http://i-saint.skr.jp/empire_tohu/serverdb.cgi";
		server_address = "192.168.0.2";
		server_port = 10031;
	}
};


#define GetFrameConfig() FrameConfig::instance()
#define GetGameConfig()  GameConfig::instance()


#include <SDL/SDL.h>
inline void write_config(const std::string& fn) {
	const FrameConfig& cf = GetFrameConfig();
	const GameConfig& gf = GetGameConfig();

	FILE *f = fopen(fn.c_str(), "wb");
	fprintf(f, "set_player_name(\"%s\")\n", gf.player_name.c_str());
	fprintf(f, "set_system_font(\"%s\")\n", gf.system_font.c_str());
	fprintf(f, "set_window_size(%d, %d)\n", cf.width, cf.height);

	if((cf.ex_flags & SDL_FULLSCREEN)!=0)
		fprintf(f, "enable_fullscreen(true)\n");
	if(gf.frameskip)
		fprintf(f, "enable_frameskip(true)\n");
	if(gf.noeffect)
		fprintf(f, "enable_noeffect(true)\n");
	fprintf(f, "set_port(\"%d\")\n", gf.port);
	fprintf(f, "set_server_database(\"%s\")\n", gf.server_database.c_str());
	fprintf(f, "set_server_address(\"%s\")\n", gf.server_address.c_str());
	fprintf(f, "set_server_port(\"%d\")\n", gf.server_port);
	fclose(f);
}




#endif
