#include "ground.h"


BGround::BGround()
{
	grid_size=20;
	clear();
}


void BGround::draw() const
{
}

void BGround::draw(const ist::PerspectiveCamera& camera) const
{
}


bool BGround::load(const std::string& fname) {
	std::fstream f(fname.c_str(), std::ios::in | std::ios::binary);
	ist::biostream bf(f);
	if(!f)
		return false;

	int version;
	bf >> version;
	if(version!=101)
		return false;

	ist::vector2 t_mn, t_mx;
	float gs;
	bf >> t_mn.x >> t_mn.y >> t_mx.x >> t_mx.y >> gs;
	resize(t_mn, t_mx, gs);

	for(size_t i=0; i<vertex.size(); ++i) {
		bf >> vertex[i].pos.y >> vertex[i].tex.x >> vertex[i].tex.y >> vertex[i].attr;
	}

	setNormal();
	return true;
}


void BGround::clear() {
	x_div = 1;
	z_div = 1;
	vertex.resize(1);
	index.clear();
}

void BGround::resize(const ist::vector2& new_min, const ist::vector2& new_max, float gs) {
	BaseClass::resize(new_min, new_max, gs);

	vertex.clear();
	vertex.resize(x_div*z_div);

	initVertex();
	setIndex();
}

void BGround::resizePreserved(const ist::vector2& new_min, const ist::vector2& new_max, float gs) {
	BGround old=*this;
	resize(new_min, new_max, gs);

	for(int i=0; i<z_div; ++i) {
		for(int j=0; j<x_div; ++j) {
			ist::vector3& v=(*this)[i][j].pos;
			v.y = old.getHeight(ist::vector2(v.x, v.z));
		}
	}
	setNormal();
}

float BGround::getHeight(const ist::vector4& pos) const
{
	return getHeight(ist::vector2(pos.x, pos.z));
}

float BGround::getHeight(const ist::vector2& _target_pos) const
{
	if(vertex.empty())
		return 0.0f;
	else if(vertex.size()==1)
		return vertex[0].pos.y;

	ist::vector2 target_pos = _target_pos;
	coord_t coord = getCoord(target_pos);
	if(coord.x<0 || coord.z<0 || coord.x>=x_div-1 || coord.z>=z_div-1) {
		float res = 0.0f;
		if(coord.x<0 && coord.z<0)
			res = (*this)[0][0].pos.y;
		else if(coord.x>=x_div-1 && coord.z>=z_div-1)
			res = (*this)[z_div-1][x_div-1].pos.y;
		else if(coord.x<0 && coord.z>=z_div-1)
			res = (*this)[z_div-1][0].pos.y;
		else if(coord.x>=x_div-1 && coord.z<0)
			res = (*this)[0][x_div-1].pos.y;
		else if(coord.x<0)
			res = (*this)[coord.z][0].pos.y;
		else if(coord.x>=x_div-1)
			res = (*this)[coord.z][x_div-1].pos.y;
		else if(coord.z<0)
			res = (*this)[0][coord.x].pos.y;
		else if(coord.z>=z_div-1)
			res = (*this)[z_div-1][coord.x].pos.y;
		return res;
	}


	float h1 = (*this)[coord.z][coord.x].pos.y;
	float h2 = (*this)[coord.z][coord.x+1].pos.y;
	float h3 = (*this)[coord.z+1][coord.x].pos.y;
	float h4 = (*this)[coord.z+1][coord.x+1].pos.y;

	target_pos-=min_edge;
	ist::vector2 near_grid(grid_size*coord.x, grid_size*coord.z);
	float h5 = h1 + (h2-h1)/grid_size * (target_pos.x-near_grid.x);
	float h6 = h3 + (h4-h3)/grid_size * (target_pos.x-near_grid.x);
	return h5 + (h6-h5)/grid_size * (target_pos.y-near_grid.y);
}

ist::vector4 BGround::getRayCross(const ist::vector4 &eye, const ist::vector4 &ray) const
{
	ist::vector4 c_pos = eye;
	for(int i=0; i<50; ++i) {
		c_pos+=ray*50.0f;
		if(c_pos.y<getHeight(c_pos)) {
			for(int j=0; i<50; ++j) {
				c_pos-=ray;
				if(c_pos.y>getHeight(c_pos)) {
					return c_pos;
				}
			}
		}
	}
	return ist::vector4(0, -10000.0f, 0);
}

void BGround::getHeightRange(float &low, float &high) const
{
	for(size_t i=0; i<vertex.size(); ++i) {
		float h = vertex[i].pos.y;
		if(i==0)
			low=high=h;
		if(h>high)
			high=h;
		if(h<low)
			low=h;
	}
}




void BGround::setNormal()
{
	setNormal(coord_t(0, 0), coord_t(getXDiv(), getZDiv()));
}

void BGround::setNormal(const coord_t& mn, const coord_t& mx)
{
	for(int i=mn.z; i<mx.z; ++i) {
		for(int j=mn.x; j<mx.x; ++j) {
			ist::vector3 c1;
			ist::vector3 c2;
			if(i!=0 && i!=z_div-1 && j!=0 && j!=x_div-1) {
				c1 = (*this)[i][j+1].pos - (*this)[i][j-1].pos;
				c2 = (*this)[i+1][j].pos - (*this)[i-1][j].pos;
				(*this)[i][j].nml = ist::vector3().cross(c2, c1).normalize();
			}
			else {
				(*this)[i][j].nml = ist::vector3(0.0f, 1.0f, 0.0f);
			}
		}
	}
}

void BGround::setIndex()
{
	index.resize(std::max((x_div-1)*(z_div-1)*4, 0));
	for(int i=0; i<z_div-1; ++i) {
		for(int j=0; j<x_div-1; ++j) {
			int bi=(x_div-1)*i*4+j*4;
			index[bi+0]=x_div*i + j;
			index[bi+1]=x_div*(i+1) + j;
			index[bi+2]=x_div*(i+1) + j+1;
			index[bi+3]=x_div*i + j+1;
		}
	}
}

void BGround::initVertex() {
	for(int i=0; i<z_div; ++i) {
		for(int j=0; j<x_div; ++j) {
			vertex_t &v = (*this)[i][j];
			v.pos = ist::vector3(min_edge.x+grid_size*j, 0, min_edge.y+grid_size*i);
			v.tex.x = 0.01f;
			v.tex.y = 0.01f;
			v.nml.y = 1.0f;
			v.col = ist::vector4(1.0f, 1.0f, 1.0f, 1.0f);
		}
	}
}

