#include "game.h"
#include "templates/gobj_parts.h"


typedef LinearInherit5(
	HaveActionQueue,
	HaveDeathFlag,
	HaveID,
	HaveEventQueue,
	IGroup) GroupRoot;

class Group : public GroupRoot
{
public:
	typedef std::set<unit_ptr, less_id<unit_ptr> > unit_cont;
	typedef unit_cont::iterator iterator;

	Group() : speed(0.0f) {
	}

	int getDrawPriority() const { return 500; }
	category_t getRecieveEvents() const { return RCV_UPDATE; }

	void draw() const {
		drawPath();
	}

	virtual void drawPath() const {
		glDisable(GL_DEPTH_TEST);
		ist::Color(1.0f, 1.0f, 1.0f, 0.6f).assign();
		glBegin(GL_LINE_STRIP);
		Event::actioninfo_ptr ap;
		for(size_t i=0; i<sizeAction(); ++i) {
			ap = this->getAction(i);
			if(!!ap->getObj() && !ap->flagPath())
				glVertex3fv(ap->getObj()->getPos().v);
			else
				glVertex3fv(ap->target_pos.v);
		}
		glEnd();

		ist::Color().assign();
		glEnable(GL_DEPTH_TEST);
	}


	IGObjIterator& getUnits() const { gp.set(unit.begin(), unit.end()); return gp; }
	size_t size() const { return unit.size(); }
	void add(unit_ptr u) {
		if(!!u) {
			unit.insert(u);
			u->setGroup(this);
			if(speed==0.0f)
				speed = u->getMoveSpeed();
			else
				speed = std::min<float>(speed, u->getMoveSpeed());
		}
	}
	void erase(unit_ptr u) {
		unit.erase(u);
		if(unit.empty())
			SendKillEvent(this, this);
	}
	bool find(unit_ptr u) const { return unit.find(u)!=unit.end(); }
	unit_ptr getLeader() const { return unit.empty() ? unit_ptr() : *unit.begin(); }

	void move(Event::ActionInfo& info) { action(info); }
	void action(Event::ActionInfo& info) {
		unit_ptr leader = getLeader();
		if(!leader)
			clearAction();
		else if(!leader->getCurrentAction())
			popAction();
		if(sizeAction()==0)
			SendKillEvent(this, this);
	}

	void onChangeCurrentAction(Event::actioninfo_ptr a) {
		BaseClass::onChangeCurrentAction(a);
		if(!a)
			return;

		unit_ptr leader;
		float dist = 0.0f;
		for(unit_cont::iterator p=unit.begin(); p!=unit.end(); ++p) {
			if(!leader) {
				leader = *p;
				dist = (a->target_pos-leader->getPos()).norm();
			}
			else {
				float d = (a->target_pos-(*p)->getPos()).norm();
				if(d<dist) {
					dist = d;
					leader = *p;
				}
			}
		}
		if(!leader)
			return;

		a->speed = speed;
		a->flag|=Event::ActionInfo::PATH;
		a->flag|=Event::ActionInfo::RANK;

		pc.construct(leader->getPos(), a);
		for(unit_cont::iterator p=unit.begin(); p!=unit.end(); ++p) {
			(*p)->clearAction();
			for(int i=int(pc.action.size())-1; i>=0; --i)
				(*p)->pushbackAction(pc.action[i]);
			(*p)->pushbackAction(a);
		}
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		for(unit_cont::iterator p=unit.begin(); p!=unit.end();) {
			if((*p)->isDead() || (*p)->getGroup().get()!=this)
				unit.erase(p++);
			else
				++p;
		}
		if(unit.empty()) {
			SendKillEvent(this, this);
		}
	}

	void onKill(Event& e) {
		for(unit_cont::iterator p=unit.begin(); p!=unit.end(); ++p) {
			(*p)->setGroup(group_ptr());
		}
		unit.clear();
		BaseClass::onKill(e);
	}

private:
	float speed;
	unit_cont unit;
	PathConstructor pc;
	mutable IteratorAdapter<unit_cont> gp;
};


group_ptr create_group(IGObjIterator& i) {
	Group *g = new Group();
	while(i.hasNext()) {
		if(unit_ptr u = ToUnit(i.iterate()))
			g->add(u);
	}
	return g;
}
