#include "ground.h"



class CollisionMap : public TMapBase<ICollisonMap>
{
public:
	struct data_t {
		unsigned char count;
		selectable_ptr obj[4];

		data_t() : count(0) {}
		void addObj(selectable_ptr s) {
			if(!s)
				return;
			for(int i=0; i<4; ++i) {
				if(!obj[i]) {
					obj[i]=s;
					break;
				}
			}
		}
		void removeObj(selectable_ptr s) {
			if(!s)
				return;
			for(int i=0; i<4; ++i) {
				if(obj[i]==s) {
					obj[i]=selectable_ptr();
					break;
				}
			}
		}
		bool findObj(selectable_ptr s) {
			if(!s)
				return false;
			for(int i=0; i<4; ++i) {
				if(obj[i]==s)
					return true;
			}
			return false;
		}
	};

	bool empty() { return data.empty(); }

	void resize(const ist::vector2& new_min, const ist::vector2& new_max, float gs) {
		BaseClass::resize(new_min, new_max, gs);
		data.resize(getXDiv()*getZDiv());
	}

	void clearSight() {
		for(size_t i=0; i<data.size(); ++i)
			data[i]=data_t();
	}

	void addCollision(selectable_ptr s, const ist::vector4& pos, float radius) {
		coord_t mn = getClampedCoord(ist::vector2(pos.x-radius, pos.z-radius));
		coord_t mx = getClampedCoord(ist::vector2(pos.x+radius, pos.z+radius));
		ist::vector2 pos2(pos.x, pos.z);
		for(int i=mn.z; i<mx.z; ++i) {
			for(int j=mn.x; j<mx.x; ++j) {
				float gs = getGridSize();
				float radius2 = radius*radius;
				ist::vector2 vpos = getMin()+ist::vector2(gs*(float(j)+0.5f), gs*(float(i)+0.5f));
				if((vpos-pos2).square()<radius2) {
					(*this)[i][j].count++;
					(*this)[i][j].addObj(s);
				}
			}
		}
	}

	void removeCollision(selectable_ptr s, const ist::vector4& pos, float radius) {
		coord_t mn = getClampedCoord(ist::vector2(pos.x-radius, pos.z-radius));
		coord_t mx = getClampedCoord(ist::vector2(pos.x+radius, pos.z+radius));
		ist::vector2 pos2(pos.x, pos.z);
		for(int i=mn.z; i<mx.z; ++i) {
			for(int j=mn.x; j<mx.x; ++j) {
				float gs = getGridSize();
				float radius2 = radius*radius;
				ist::vector2 vpos = getMin()+ist::vector2(gs*(float(j)+0.5f), gs*(float(i)+0.5f));
				if((*this)[i][j].count>0
				&&((vpos-pos2).square()<radius2)) {
					(*this)[i][j].count--;
					(*this)[i][j].removeObj(s);
				}
			}
		}
	}

	bool isCollide(const ist::vector4& pos) const {
		coord_t c = getSafeCoord(ist::vector2(pos.x, pos.z));
		return ((*this)[c.z][c.x].count!=0);
	}

	data_t* operator[](int i) { return &data[i*getXDiv()]; }
	const data_t* operator[](int i) const { return &data[i*getXDiv()]; }


	void draw(ist::Bitmap& bmp) {
		bmp.resize(getXDiv(), getZDiv());
		for(int i=0; i<getXDiv(); ++i) {
			for(int j=0; j<getZDiv(); ++j) {
				bmp[i][j].r = ((*this)[i][j].count!=0)?0:255 ;
				bmp[i][j].g = bmp[i][j].r;
				bmp[i][j].b = bmp[i][j].r;
				bmp[i][j].a = 255;
			}
		}
	}

private:
	std::vector<data_t> data;
};









class QTree
{
public:
	struct coord2D
	{
		coord2D() : x(0), y(0) {}
		coord2D(int _x, int _y) : x(_x), y(_y) {}
		bool operator==(const coord2D& c) const { return (x==c.x && y==c.y); }
		bool operator!=(const coord2D& c) const { return !((*this)==c); }
		coord2D operator+(const coord2D& c) const { return coord2D(x+c.x, y+c.y); }
		coord2D operator/(int d) const { return coord2D(x/d, y/d); }

		float distance(const coord2D& c) {
		//	return abs((c.x-x))+abs((c.y-y));
			float x2=float(x-c.x);
			float y2=float(y-c.y);
			return sqrtf(x2*x2+y2*y2);
		}
	
		int x,y;
	};

	class Node
	{
	public:
		typedef boost::intrusive_ptr<Node> node_ptr;
		Node(coord2D _dl, coord2D _ur) :
			ref_count(0), dl(_dl), ur(_ur),
			weight(1.0f), length(0),
			depth(0), grid_info(0),
			id(0) {
			length=float((ur.x-dl.x)+(ur.y-dl.y))*0.5f;
			center=(dl+ur)/2;
		}

		void clear() {
			next.clear();
		}

		void addRef() { ++ref_count; }
		void release() { if(--ref_count==0) delete this; }
		int ref_count;

		coord2D dl, ur, center;
		float weight;
		float length;
		char depth;
		char grid_info;
		int id;
		std::vector<node_ptr> next;
	};
	typedef Node::node_ptr node_ptr;
	typedef std::set<node_ptr> node_set;

	QTree() {}
	~QTree() {
		clearBottom();
	}

	void clearBottom() {
		for(node_set::iterator p=bottoms.begin(); p!=bottoms.end(); ++p)
			(*p)->clear();
		bottoms.clear();
	}

	void construct(CollisionMap& cm) {
		if(cm.empty())
			return;
		clearBottom();
		if(cm.getXDiv()!=int(passed.getWidth()) || cm.getZDiv()!=int(passed.getHeight())) {
			passed.clear();
			passed.resize(cm.getXDiv(), cm.getZDiv());
		}
		node = new Node(coord2D(0,0), coord2D(cm.getXDiv(), cm.getZDiv()));
		construct(cm, node);
		constructNext();
	}

	void construct(CollisionMap& cm, node_ptr n) {
		if(n->dl==n->ur)
			return;

		const coord2D& dl = n->dl;
		const coord2D& ur = n->ur;
		coord2D center = (n->dl+n->ur)/2;
		coord2D c_pos[8] = {
			dl, center,
			coord2D(dl.x, center.y), coord2D(center.x, ur.y),
			coord2D(center.x, dl.y), coord2D(ur.x, center.y),
			center, ur,
		};
		bool same[4] = {false, false, false, false};
		const CollisionMap::data_t& first = cm[dl.y][dl.x];

		for(int k=0; k<4; ++k) {
			const coord2D& c_dl = c_pos[k*2+0];
			const coord2D& c_ur = c_pos[k*2+1];
			bool s = true;
			for(int i=c_dl.y; i<c_ur.y; ++i) {
				for(int j=c_dl.x; j<c_ur.x; ++j) {
					if((first.count==0 && cm[i][j].count!=0)
					|| (first.count!=0 && cm[i][j].count==0)) {
						s = false;
						goto loop_end;
					}
				}
			}
			loop_end:
			if(s)
				same[k] = true;
			else
				break;
		}

		if(same[0]==true && same[1]==true && same[2]==true && same[3]==true) {
			n->grid_info = gridInfo(first);
			bottoms.insert(n);
		}
		else {
			for(int k=0; k<4; ++k) {
				node_ptr child = new Node(c_pos[k*2+0], c_pos[k*2+1]);
				child->depth = n->depth+1;
				if(same[k]) {
					child->grid_info = gridInfo(first);
					bottoms.insert(child);
				}
				else {
					construct(cm, child);
				}
			}
		}
	}

	char gridInfo(const CollisionMap::data_t& c) {
		if(c.count!=0)
			return 1;
		return 0;
	}

	void insertChild(node_set& next, node_ptr n, node_ptr child) {
		if(child->grid_info!=0)
			return;
		if(next.insert(child).second)
			n->next.push_back(child);
	}

	void constructNext() { // אڃm[hTo
		int id = 0;
		for(node_set::iterator p=bottoms.begin(); p!=bottoms.end(); ++p) {
			node_ptr n = *p;
			n->id = id++;
			for(int i=n->dl.y; i<n->ur.y; ++i) {
				for(int j=n->dl.x; j<n->ur.x; ++j) {
					passed[i][j]=n;
				}
			}
		}

		for(node_set::iterator p=bottoms.begin(); p!=bottoms.end(); ++p) {
			node_ptr n = *p;

			node_set next;
			int x[2] = {n->dl.x-1, n->ur.x};
			int y[2] = {n->dl.y-1, n->ur.y};
			for(int j=0; j<2; ++j) {
				if(x[j]<0 || x[j]>=int(passed.getWidth()))
					continue;
				for(int i=n->dl.y; i<n->ur.y; ++i)
					insertChild(next, n, passed[i][x[j]]);
			}
			for(int j=0; j<2; ++j) {
				if(y[j]<0 || y[j]>=int(passed.getHeight()))
					continue;
				for(int i=n->dl.x; i<n->ur.x; ++i)
					insertChild(next, n, passed[y[j]][i]);
			}

			coord2D edge[4] = {
				coord2D(n->dl.x-1, n->ur.y),
				coord2D(n->dl.x-1, n->dl.y-1),
				coord2D(n->ur.x, n->dl.y-1),
				coord2D(n->ur.x, n->ur.y),
			};
			for(int i=0; i<4; ++i) {
				if(edge[i].y<0 || edge[i].y>=int(passed.getHeight())
				|| edge[i].x<0 || edge[i].x>=int(passed.getWidth()))
					continue;
				insertChild(next, n, passed[edge[i].y][edge[i].x]);
			}
		}
	}

	static void fill(ist::Bitmap& bmp, node_ptr n, ist::bRGBA col) {
		for(int i=n->dl.y; i<n->ur.y; ++i ) {
			for(int j=n->dl.x; j<n->ur.x; ++j ) {
				bmp[i][j]=col;
			}
		}
	}

	void draw(ist::Bitmap& bmp) {
		if(!node)
			return;

		bmp.resize(node->ur.x, node->ur.y);
		for(node_set::iterator p=bottoms.begin(); p!=bottoms.end(); ++p) {
			node_ptr n = *p;
			ist::bRGBA col;
			int d=255-n->depth*15;
			if(n->grid_info==1)
				col = ist::bRGBA(0,0,0,255);
			else
				col = ist::bRGBA(d,d,d,255);
			fill(bmp, n, col);
		}
	}

	size_t size() { return bottoms.size(); }
	node_ptr* operator[](int i) { return passed[i]; }

	node_ptr node;
	node_set bottoms;
	ist::array2<node_ptr> passed;
};

inline void intrusive_ptr_add_ref(QTree::Node *p) { p->addRef(); }
inline void intrusive_ptr_release(QTree::Node *p) { p->release(); }



class AStar
{
public:
	typedef QTree::coord2D coord2D;
	typedef QTree::node_ptr qnode_ptr;

	class Node
	{
	public:
	//	typedef boost::intrusive_ptr<Node> node_ptr;
		typedef Node* node_ptr;

		Node() : weight(0), root(0) {}
		Node(qnode_ptr q) : weight(0), qnode(q), root(0) {}
		void init() {
			root=node_ptr();
			children.clear();
		}

		void setRoot(node_ptr r) {
			root = r;
			weight = root->weight + root->center().distance(center());
		}

		int id() { return qnode->id; }
		size_t size() { return qnode->next.size(); }
		qnode_ptr at(size_t i) { return qnode->next[i]; }
		coord2D& center() { return qnode->center; }
		coord2D& dl() { return qnode->dl; }
		coord2D& ur() { return qnode->ur; }

		float weight;
		qnode_ptr qnode;
		node_ptr root;
		std::vector<node_ptr> children;
	};
	typedef Node::node_ptr node_ptr;


	AStar() : nearest(0), nearest_distance(0) {}
	~AStar() {
		clear();
	}


	void clear() {
		start = 0;
		goal = 0;
		nearest_distance = 0;
		nearest = 0;
		most_short.clear();
		closed.clear();
		for(size_t i=0; i<passed.size(); ++i)
			passed[i].init();
	}

	bool findPath(QTree& qt, int sx, int sy, int gx, int gy, selectable_ptr sp) {
		clear();
		{
			passed.resize(qt.size());
			int i=0;
			for(QTree::node_set::iterator p=qt.bottoms.begin(); p!=qt.bottoms.end(); ++p) {
				passed[i].qnode = *p;
				++i;
			}
		}

		start = qt[sy][sx]->id;
		goal = qt[gy][gx]->id;
		if(start==goal)
			return true;

		Node s(qt[sy][sx]);
		passed[s.id()] = s;
		node = &passed[s.id()];

		bool res = findPath(qt, node);
		if(!res && !!nearest) {
			saveShort(nearest);
		}
		return res;
	}

	bool findPath(QTree& qt, node_ptr n) {
		std::vector<node_ptr> roots;
		std::vector<node_ptr> added;

		roots.push_back(n);
		float w=0;
		while(!roots.empty()) {
			for(size_t i=0; i<roots.size(); ++i) {
				if(roots[i]->weight>w) {
					added.push_back(roots[i]);
					continue;
				}

				createChildren(qt, roots[i]);
				bool cls = false;
				for(size_t j=0; j<roots[i]->children.size(); ++j) {
					node_ptr child = roots[i]->children[j];
					if(child->root==roots[i]) {
						if(child->id()==goal) { // S[
							saveShort(child);
							return true;
						}
						added.push_back(child);
						cls = true;
					}
				}
				if(!cls) {
					closed.push_back(roots[i]);
					float d = passed[goal].center().distance(roots[i]->center());
					if(!nearest || d < nearest_distance) {
						nearest_distance = d;
						nearest = roots[i];
					}
				}
			}
			roots=added;
			added.clear();
			w+=1.0f;
		}
		return false;
	}

	void createChildren(QTree& qt, node_ptr n) {
		if(n->children.empty()) {
			for(size_t i=0; i<n->size(); ++i) {
				node_ptr child = &passed[n->at(i)->id];
				if(!child->root && child->id()!=start)
					child->setRoot(n);
				n->children.push_back(child);
			}
		}
	}

	void saveShort(node_ptr n) {
		most_short.clear();
		while(!!n) {
			most_short.push_back(n);
			n = n->root;
		}
	}

	void drawExplored(ist::Bitmap& bmp) {
		for(size_t i=0; i<passed.size(); ++i) {
			if(!!passed[i].root)
				continue;
			ist::bRGBA col(200, 200, 255, 255);
			QTree::fill(bmp, passed[i].qnode, col);
		}
	}
	void drawPath(ist::Bitmap& bmp) {
		for(size_t i=0; i<most_short.size(); ++i) {
			ist::bRGBA col(255, 0, 0, 255);
			QTree::fill(bmp, most_short[i]->qnode, col);
		}
	}


	int start, goal;
	node_ptr node;
	std::vector<Node> passed;
	std::vector<node_ptr> closed;
	std::vector<node_ptr> most_short;
	node_ptr nearest;
	float nearest_distance;
};












class PathFinder : public IPathFinder
{
public:
	PathFinder() : update_count(0) {}
	~PathFinder() {}

	void update() {
		if(--update_count==0)
			construct();
	}

	void construct() {
		qt.construct(cmap);
	}

	void addCollision(selectable_ptr s, const ist::vector4& pos, float radius) {
		cmap.addCollision(s, pos, radius);
		if(update_count<=0)
			update_count = 1800;
	}

	void removeCollision(selectable_ptr s, const ist::vector4& pos, float radius) {
		cmap.removeCollision(s, pos, radius);
		if(update_count<=0)
			update_count = 1800;
	}


	bool findPath(const ist::vector4& base, const ist::vector4& t_pos, selectable_ptr t_obj) {
		if(qt.bottoms.size()==0)
			construct();

		path.clear();
		if(findStraightPath(base, t_pos)) {
			return true;
		}
		else {
			CollisionMap::coord_t start = cmap.getClampedCoord(ToVector2(base));
			CollisionMap::coord_t goal = cmap.getClampedCoord(ToVector2(t_pos));
			bool res=as.findPath(qt, start.x, start.z, goal.x, goal.z, t_obj);
			createASPath(AStar::coord2D(goal.x, goal.z));
			return res;
		}
	}

	bool findStraightPath(const ist::vector4& base, const ist::vector4& target) {
		CollisionMap::coord_t start = cmap.getClampedCoord(ToVector2(base));
		CollisionMap::coord_t goal = cmap.getClampedCoord(ToVector2(target));
		CollisionMap::coord_t d = goal-start;
		ist::vector2 dv(float(d.x), float(d.z));
		float dist = dv.norm();
		ist::vector2 aim = (dv/dist);
		ist::vector2 sv = ist::vector2(float(start.x), float(start.z));
		for(float i=0; i<dist; i+=1.0f) {
			sv+=aim;
			if(cmap[int(sv.y)][int(sv.x)].count!=0)
				return false;
		}
		return true;
	}


	bool save(const std::string& fn) {
		ist::Bitmap bmp;
		qt.draw(bmp);
		return bmp.save(fn.c_str());
	}


	size_t getPathSize() { return path.size(); }
	ist::vector4& getPath(int s) { return path[s]; }

	void createASPath(const AStar::coord2D& goal) {
		AStar::coord2D prev;
		for(size_t i=0; i<as.most_short.size(); ++i) {
			AStar::node_ptr n = as.most_short[i];
			AStar::coord2D coord;
			if(i==0) {
				coord = goal;
			}
			else {
				AStar::coord2D edge[8] = {
					n->dl(),
					AStar::coord2D(n->dl().x, n->ur().y),
					AStar::coord2D(n->ur().x, n->dl().y),
					n->ur(),
					AStar::coord2D(n->dl().x, n->center().y),
					AStar::coord2D(n->center().x, n->dl().y),
					AStar::coord2D(n->center().x, n->ur().y),
					AStar::coord2D(n->ur().x, n->center().y),
				};
				int dist = 0;
				for(int j=0; j<8; ++j) {
					int dt = abs(edge[j].x-prev.x)+abs(edge[j].y-prev.y);
					if(dt < dist || j==0) {
						coord = edge[j];
						dist = dt;
					}
				}
			}

			float gs = cmap.getGridSize();
			ist::vector4 tmp;
			tmp.x = cmap.getMin().x+coord.x*gs+gs*0.5f;
			tmp.z = cmap.getMin().y+coord.y*gs+gs*0.5f;
			path.push_back(tmp);
			prev = coord;
		}
	}

	void createStraightPath() {
	}

	int update_count;
	std::vector<ist::vector4> path;
	CollisionMap cmap;
	QTree qt;
	AStar as;
};


