#include "game.h"
#include "world.h"
#include <algorithm>
#include <fstream>

using namespace std;
using namespace boost;
using namespace ist;




namespace {
	bool is_dragging=false;
	vector2 d_begin;
	vector2 d_end;
	Event::motioninfo_ptr mouse_motion;
}






template <class T>
struct less_draw_priority
{
	bool operator()(const T& lhs, const T& rhs) {
		return lhs->getDrawPriority() < rhs->getDrawPriority();
	}
};

template<class T>
struct _greater_dist {
	_greater_dist(const ist::vector4& f) : from(f) {}
	bool operator()(const T& lhs, const T& rhs) {
		return (lhs.getContainer().getPos()-from).square()>(rhs.getContainer().getPos()-from).square();
	}

	ist::vector4 from;
};







namespace {
	bool top_view = false;
}


World::World() :
	id_seed(0),
	frame(0), frame_limit(0),
	enable_sight(true),
	is_update_stoped(false)
{
	event_router = CreateEventRouter();

	camera_dist = ist::vector4(450, 900, 450);
	setCameraPos(vector4(0,0,0));
	camera.fovy = 30.0f;
	camera.z_far = 1750.0f;
}

World::~World()
{
	killAll();
}


void World::setMode(int mode) {
	event_router = CreateEventRouter(mode);
}

int World::getMode() {
	return event_router->getType();
}

bool World::loadReplay(const std::string& filename) {
	return event_router->load(filename);
}

void World::setServerAddress(const std::string& address) {
	GetGameConfig().server_address = address;
}

void World::setServerPort(unsigned short port) {
	GetGameConfig().server_port = port;
}

const std::string& World::getServerDatabase() {
	return GetGameConfig().server_database;
}


void World::registKeyObserver(int key, keyobserv_ptr observer) {
	keymap[key].push_back(observer);
}

void World::deregistKeyObserver(int key, keyobserv_ptr observer) {
	keyob_list& l = keymap[key];
	l.erase(std::find(l.begin(), l.end(), observer));
}


gobj_id World::createID()  {
	return ++id_seed;
}


faction_ptr World::createFaction(const std::string& name)
{
	faction_ptr p = GetIGame().createFaction(name);
	return p;
}

unit_ptr World::createUnit(const std::string& name, faction_ptr f)
{
	unit_ptr p = GetIGame().createUnit(name, f);
	return p;
}

building_ptr World::createBuilding(const std::string& name, faction_ptr f)
{
	building_ptr p = GetIGame().createBuilding(name, f);
	return p;
}

material_ptr World::createMaterial(const std::string& name)
{
	material_ptr p = GetIGame().createMaterial(name);
	return p;
}

group_ptr World::createGroup(IGObjIterator& i)
{
	group_ptr p = GetIGame().createGroup(i);
	return p;
}



void World::setPlayerFaction(faction_ptr p) {
	if(!!player_faction)
		SendDisselectEvent(0, player_faction);
	player_faction = p;
	SendSelectEvent(0, player_faction);
	RouteCallEvent(0, player_faction, new Event::CallInfo("setPlayerName", GetGameConfig().player_name));
}
void World::setNatureFaction(faction_ptr p) { nature_faction=p; }

faction_ptr World::getPlayerFaction() {
	if(!player_faction) {
		if(!factions.empty())
			setPlayerFaction(*factions.begin());
	}
	assert(!!player_faction);
	return player_faction;
}
faction_ptr World::getNatureFaction() {
	assert(!!nature_faction);
	return nature_faction;
}



void World::setCameraPos(const vector4& p)
{
	camera.target = p;
	camera.position = camera.target+camera_dist;
	camera.setViewFrustum();
}







void World::loadGround(const string& filename) {
	ground.load(filename);
	pfinder.cmap.resize(ground.getMin(), ground.getMax(), 10.0f);
	if(character_grid.empty()) {
		ist::vector2 gmin = ground.getMin()-ist::vector2(100.0f, 100.0f);
		ist::vector2 gmax = ground.getMax()+ist::vector2(100.0f, 100.0f);
		character_grid.resize(gmin, gmax, int((gmax.x-gmin.x)/100.0f), int((gmax.y-gmin.y)/100.0f));
	}
}

void World::loadGroundTexture(const string& filename) {
	ground.loadTexture(filename);
}

float World::getHeight(const vector4& pos) {
	return ground.getHeight(pos);
}

const ist::vector2& World::getMapMin() {
	return ground.getMin();
}

const ist::vector2& World::getMapMax() {
	return ground.getMax();
}

ist::Light& World::getGroundLight() {
	return ground.getLight();
}

void World::enableGroundLighting(bool f) {
	ground.enableLighting(f);
}

void World::enableGroundFog(bool f) {
	ground.enableFog(f);
}

void World::getMapTexture(ist::Bitmap& bmp) {
	ground.genBitmap(bmp);
}

bool World::isSighted(faction_ptr f, const ist::vector4& pos) {
	Ground::sight_ptr& s = sights[f];
	if(!s)
		return false;
	return s->isSighted(pos);
}

bool World::isExplored(faction_ptr f, const ist::vector4& pos) {
	Ground::sight_ptr& s = sights[f];
	if(!s)
		return false;
	return s->isExplored(pos);
}

void World::enableSight(bool sw) {
	enable_sight = sw;
}





IGObjIterator& World::getBlocksByRadius(const ist::vector4& pos, float radius) {
	blocks.set(character_grid.getBlocksByRadius(pos, radius));
	return blocks;
}

IGObjIterator& World::getBlocksByBox(const ist::vector4& min, const ist::vector4& max) {
	blocks.set(character_grid.getBlocksByBox(min, max));
	return blocks;
}

IGObjIterator& World::getGObjsInSight() {
	static IteratorAdapter<gobj_vector> gobjs_insight_iter;
	gobjs_insight_iter.set(gobjs_insight_selectable.begin(), gobjs_insight_selectable.end());
	return gobjs_insight_iter; 
}

IGObjIterator& World::getFactions() {
	static IteratorAdapter<faction_set> faction_iter;
	faction_iter.set(factions.begin(), factions.end());
	return faction_iter; 
}

IGObjIterator& World::getMenus() {
	static IteratorAdapter<menu_set> menu_iter;
	menu_iter.set(menus.begin(), menus.end());
	return menu_iter; 
}




void World::insertToCharacterGrid(const ist::vector4& pos, gobj_ptr p) {
	character_grid.insert(pos, p);
}

void World::eraseFromCharacterGrid(const ist::vector4& pos, gobj_ptr p) {
	character_grid.erase(pos, p);
}

void World::moveCharacterGrid(const ist::vector4& before, const ist::vector4& after, gobj_ptr p) {
	character_grid.move(before, after, p);
}

void World::addCircleCollision(selectable_ptr p, const ist::vector4& pos, float radius) {
	pfinder.addCollision(p, pos, radius);
}

void World::removeCircleCollision(selectable_ptr p, const ist::vector4& pos, float radius) {
	pfinder.removeCollision(p, pos, radius);
}



gobj_ptr World::getGObj(gobj_id id)
{
	gobj_cont::iterator it = gobjs.find(id);
	if(it==gobjs.end())
		return gobj_ptr();
	return it->second;
}



bool World::addGObj(gobj_ptr p)
{
	if(!p || p->getID()==0)
		return false;
	gobj_ptr& ins = gobjs[p->getID()];
	if(!!ins) {
		assert(0);
		return false;
	}

	ins = p;
	update_list.push_back(p);
	insertReciever(p);

	if(p->isMenu()) {
		menus.insert(ToMenu(p));
	}
	else if(p->isFaction()) {
		faction_ptr fp = ToFaction(p);
		if(factions.size()==0) {
			setNatureFaction(fp);
		}
		fp->setFactionID(factions.size());
		factions.insert(fp);
		sights[fp] = ground.createSightMap();
	}
	p->onAdded();
	return true;
}

void World::eraseGObj(gobj_ptr p)
{
	p->onRemoved();
	gobjs.erase(p->getID());
	update_list.erase(std::find(update_list.begin(), update_list.end(), p));
	eraseReciever(p);

	if(p->isMenu()) {
		menus.erase(ToMenu(p));
	}
	else if(p->isFaction()) {
		faction_ptr f = ToFaction(p);
		factions.erase(f);
		sights.erase(f);
	}
}

void World::insertReciever(gobj_ptr p)
{
	unsigned int rcv = p->getRecieveEvents();
	if(rcv & RCV_UPDATE) rcv_update.insert(p);
	if(rcv & RCV_KEY)    rcv_keyboard.insert(p);
}

void World::eraseReciever(gobj_ptr p)
{
	unsigned int rcv = p->getRecieveEvents();
	if(rcv & RCV_UPDATE) rcv_update.erase(p);
	if(rcv & RCV_KEY)    rcv_keyboard.erase(p);
}



void World::update()
{
	event_router->update();
	if(!event_router->conditionToStart() ||
		(event_router->getType()==CLIENT && frame+1>=frame_limit))
		return;

	sendUpdateEvent();

	pfinder.update();

	camera.setViewFrustum();
	aim_camera = matrix44().aimVector(camera.position-camera.target);

	for(gobj_list::iterator p=update_list.begin(); p!=update_list.end();) {
		gobj_ptr& gp = *p;
		gp->update();

		++p;
		if(gp->isDead())
			eraseGObj(gp);
	}

	if(frame%30==0) {
		for(sight_cont::iterator p=sights.begin(); p!=sights.end(); ++p) {
			if(!!p->second)
				p->second->clearSight();
		}
		for(gobj_cont::iterator p=gobjs.begin(); p!=gobjs.end(); ++p) {
			gobj_ptr& gp = p->second;
			if(!gp->isVisible())
				continue;
			if(gp->isUnit() || gp->isBuilding()) {
				selectable_ptr sp = boost::static_pointer_cast<ISelectable>(gp);
				Ground::sight_ptr& f = sights[sp->getFaction()];
				if(!!f)
					f->addSight(sp->getPos(), sp->getSight());
			}
		}
		if(!!player_faction) {
			if(!enable_sight)
				sights[player_faction]->fillSight();
			else {
				IGObjIterator& ally = player_faction->getAlly();
				while(ally.hasNext()) {
					if(faction_ptr f = ToFaction(ally.iterate())) {
						sights[player_faction]->addSight(*sights[f]);
					}
				}
			}
			ground.assignSightMap(sights[player_faction]);
		}
	}

	camera.setViewFrustum();

	gobjs_insight.clear();
	const ist::ViewFrustum& vf = GetCamera().getViewFrustum();
	const ist::AABox& bb = vf.getBoundingBox();

	character_cont::block_iter& block = character_grid.getBlocksByBox(bb.min, bb.max);

	std::sort(block.begin(), block.end(), _greater_dist<character_cont::block_iter::pos_t>(getCamera().position));
//	block.sort(getCamera().position);

	for(size_t i=0; i<block.size(); ++i) {
		character_cont::block_elem& be = block[i];
		ist::vector4 center = ist::vector4(be.center).setY(getHeight(be.center));
		int vftest = vf.test(center, be.radius);
		if(vftest==0)
			continue;

		typedef character_cont::block_elem::iterator iterator;
		for(iterator q = be.begin(); q!=be.end(); ++q) {
			gobj_ptr& p = *q;

			if(GetGameConfig().noeffect && (p->isBullet() || p->isParticle()))
				continue;

			if(!p->isVisible())
				continue;
			if(vftest!=0) {
				visible_ptr vp = boost::static_pointer_cast<IVisible>(p);
				if( vp->isInCameraSight()) {
					if(vp->isInSight(player_faction))
						gobjs_insight.push_back(p);
				}
			}
		}
	}

	gobjs_insight_selectable.clear();
	for(gobj_vector::iterator p=gobjs_insight.begin(); p!=gobjs_insight.end(); ++p) {
		if(!!ToSelectable(*p))
			gobjs_insight_selectable.push_back(*p);
	}
	std::sort(gobjs_insight.begin(), gobjs_insight.end(), less_id<gobj_ptr>());

	++frame;



	// ʐMΐ30tȏxƒǂƎ݂ 
	static bool skipped;
	if(frame_limit>30 && frame<frame_limit-30 && !skipped) {
		skipped = true;
		update();
	}
	else {
		skipped = false;
	}
}

void World::killAll()
{
	for(gobj_list::iterator p=update_list.begin(); p!=update_list.end();) {
		gobj_ptr& gp = *p;
		Event e(EVT_KILL, 0, gp->getID());
		gp->sendEvent(e);
		gp->update();

		++p;
		if(gp->isDead())
			eraseGObj(gp);
	}

	assert(update_list.empty());
	assert(gobjs.empty());
}


void World::draw()
{
	camera.setViewFrustum();
	camera.look();
	if(top_view) {
		PerspectiveCamera top_camera;
		top_camera.direction = ist::vector3(0, 0, -1.0f);
		top_camera.position = ist::vector4(0, 3000.0f, 0);
		top_camera.z_far =4000.0f;
		top_camera.look();
	}

	ground.draw(camera);

	for(faction_set::iterator p=factions.begin(); p!=factions.end(); ++p)
		(*p)->draw();

	for(menu_set::iterator p=menus.begin(); p!=menus.end(); ++p)
		gobjs_insight.push_back(*p);

	std::stable_sort(gobjs_insight.begin(), gobjs_insight.end(), less_draw_priority<gobj_ptr>());
	for(gobj_vector::iterator p=gobjs_insight.begin(); p!=gobjs_insight.end(); ++p)
		(*p)->draw();

	if(is_dragging)
		GetIGame().drawEdge(d_begin, d_end);

	event_router->draw();

/*
	{
		glPointSize(10);
		glBegin(GL_POINTS);
		glVertex3fv(getMousePointerPos3D().v);
		glEnd();
		glPointSize(1);
	}
	const ViewFrustum& vf = GetCamera().getViewFrustum();
	{
		glBegin(GL_LINES);
		glVertex3fv(vf.vertex[0].v); glVertex3fv(vf.vertex[1].v);
		glVertex3fv(vf.vertex[0].v); glVertex3fv(vf.vertex[2].v);
		glVertex3fv(vf.vertex[0].v); glVertex3fv(vf.vertex[3].v);
		glVertex3fv(vf.vertex[0].v); glVertex3fv(vf.vertex[4].v);
		glVertex3fv(vf.vertex[1].v); glVertex3fv(vf.vertex[2].v);
		glVertex3fv(vf.vertex[2].v); glVertex3fv(vf.vertex[3].v);
		glVertex3fv(vf.vertex[3].v); glVertex3fv(vf.vertex[4].v);
		glVertex3fv(vf.vertex[4].v); glVertex3fv(vf.vertex[1].v);
		glEnd();
	}
*/
}





void World::onButton(Event::buttoninfo_ptr info)
{
	float x_rate = (float)GetFrameConfig().width/640.0f;
	float y_rate = (float)GetFrameConfig().height/480.0f;
	info->x/=x_rate;
	info->y/=y_rate;

	for(menu_set::iterator p=menus.begin(); p!=menus.end(); ++p) {
		if((*p)->getRecieveEvents() & RCV_BUTTON)
			SendButtonEvent(0, *p, info);
	}

	if(!!player_faction)
		SendButtonEvent(0, player_faction, info);

	if(info->isButtonUp() && info->button==1)
		is_dragging = false;
}

void World::onMotion(Event::motioninfo_ptr info)
{
	mouse_motion = info;
	if(info->is_dragging) {
		is_dragging = true;
		d_begin = ist::vector2(std::min(info->drag_x, info->x), std::min(info->drag_y, info->y));
		d_end = ist::vector2(std::max(info->drag_x, info->x), std::max(info->drag_y, info->y));
	}

	screen_ray = getScreenRay(ist::vector2(info->x, info->y));
	mouse_pointer_pos3 = _getMousePointerPos3D();
	float x_rate = (float)GetFrameConfig().width/640.0f;
	float y_rate = (float)GetFrameConfig().height/480.0f;
	info->x/=x_rate;
	info->y/=y_rate;

	mouse_pointer_pos2.x = info->x;
	mouse_pointer_pos2.y = info->y;

	for(menu_set::iterator p=menus.begin(); p!=menus.end(); ++p) {
		if((*p)->getRecieveEvents() & RCV_MOTION)
			SendMotionEvent(0, *p, info);
	}

	if(!!player_faction)
		SendMotionEvent(0, player_faction, info);
}

void World::onKey(Event::keyinfo_ptr info)
{
	if(info->state==SDL_KEYDOWN && info->key==SDLK_t)
		top_view = !top_view;
	if(info->state==SDL_KEYDOWN && info->key==SDLK_F9)
		pfinder.save("cmap.png");
	if(info->state==SDL_KEYDOWN && info->key==SDLK_F10)
		RouteSystemEvent(new Event::SystemInfo("enableSight", !enable_sight));
	if(info->state==SDL_KEYDOWN && info->key==SDLK_F8) {
		for(gobj_list::iterator p=update_list.begin(); p!=update_list.end(); ++p) {
			printf("%u ", (*p)->getID());
		}
	}

	keymap_cont::iterator p = keymap.find(info->key);
	if(p!=keymap.end()) {
		keyob_list& l = p->second;
		for(keyob_list::reverse_iterator p=l.rbegin(); p!=l.rend();) {
			keyobserv_ptr k = *(p++);
			k->exec(info->state);
			if(k->isExclusive())
				break;
		}
	}

	for(menu_set::iterator p=menus.begin(); p!=menus.end(); ++p) {
		if((*p)->getRecieveEvents() & RCV_KEY)
			SendKeyEvent(0, *p, info);
	}
//	for(gobj_set::iterator p=rcv_keyboard.begin(); p!=rcv_keyboard.end(); ++p)

	if(!!player_faction)
		SendKeyEvent(0, player_faction, info);
}

void World::sendUpdateEvent()
{
	if(!is_update_stoped) {
		Event e(EVT_UPDATE, 0, 0);
		for(gobj_set::iterator p=rcv_update.begin(); p!=rcv_update.end(); ++p) {
			e.to = (*p)->getID();
			(*p)->sendEvent(e);
		}
	}
}


ist::vector4 World::getScreenRay(const ist::vector2& screen)
{
	int viewport[4];
	glGetIntegerv(GL_VIEWPORT, viewport);
	ist::vector4 mouse_vec;
	float window_size = float(std::max<int>(viewport[2]-viewport[0], viewport[3]-viewport[1]));
	float cx = float(viewport[0]+viewport[2])/2.0f;
	float cy = float(viewport[1]+viewport[3])/2.0f;
	mouse_vec.x = (screen.x-cx)/window_size;
	mouse_vec.y = -(screen.y-cy)/window_size;
//	mouse_vec.z = -0.65f; // 60fovy
	mouse_vec.z = -1.4f; // 30fovy

	ist::matrix44 modelview;
	GetCamera().look();
	glGetFloatv(GL_MODELVIEW_MATRIX, modelview.v);
	modelview.invert();

	ist::matrix44 aim_camera;
	aim_camera.aimVector(modelview*mouse_vec-GetCamera().position);

	return aim_camera*ist::vector4(0, 1, 0);
}

vector4 World::_getMousePointerPos3D()
{
	ist::vector4 c_pos = GetCamera().position;
	ist::vector4 ray = getScreenRay()*50;
	for(int i=0; i<50; ++i) {
		c_pos+=ray;
		if(c_pos.y<GetHeight(c_pos)) {
			for(int j=0; i<50; ++j) {
				c_pos-=GetScreenRay();
				if(c_pos.y>GetHeight(c_pos)) {
					return c_pos;
				}
			}
		}
	}
	return ist::vector4(0, -10000.0f, 0);
}


void World::moveCamera()
{
	static ist::vector4 pre_camera_pos;
	static int pre_x, pre_y, pre_drag_state;
	int x,y, drag_state;

	drag_state = SDL_GetMouseState(&x, &y);

	float move_x = static_cast<float>(x-pre_x);
	float move_y = static_cast<float>(y-pre_y);

	if(drag_state&1 && pre_drag_state&1) {
		static const matrix33 turn90degree=matrix33().rotateY(90);
		float rotate_speed=25;
		float ang_xz = move_y*rotate_speed*radian;
		float ang_y = -move_x*rotate_speed*radian;
		ist::vector3 axis=(turn90degree*vector3(camera.position.x, 0, camera.position.z)).normalize();

		ist::matrix33 camera_rot;
		camera_rot.rotateA(axis, ang_xz);
		camera_rot.rotateY(ang_y);

		camera.position=camera_rot*camera.position;
	}

	if(drag_state&4 && pre_drag_state&4) {
		camera.position*=(1+(move_x-move_y)*0.003f);
	}


	pre_x = x;
	pre_y = y;
	pre_drag_state = drag_state;

	pre_camera_pos = camera.position;
}



