#ifdef GetIGame 

#ifndef XML
#define XML

#include "aos/aos.h"


#ifdef USE_TINYXML
#include <tinyxml.h>


inline bool ToBool(TiXmlElement *el) {
	bool v = false;
	if(!!el && !!el->FirstChild())
		v = std::string((el->FirstChild()->ToText()->Value()))=="true";
	return v;
}

inline float ToFloat(TiXmlElement *el) {
	float v = 0;
	if(!!el && !!el->FirstChild())
		sscanf(el->FirstChild()->ToText()->Value(), "%f", &v);
	return v;
}

inline std::string ToString(TiXmlElement *el) {
	std::string v;
	if(!!el && !!el->FirstChild())
		v = el->FirstChild()->ToText()->Value();
	return v;
}

inline ist::vector4 ToVector4(TiXmlElement *el) {
	ist::vector4 v;
	if(!!el && !!el->FirstChild())
		sscanf(el->FirstChild()->ToText()->Value(), "%f %f %f", &v.x, &v.y, &v.z);
	return v;
}


inline boost::any ToAny(TiXmlElement *el) {
	boost::any v;
	if(!el) {
		std::string type = el->Value();
		if(type=="bool")
			v = ToBool(el);
		else if(type=="float")
			v = ToFloat(el);
		else if(type=="vector4")
			v = ToVector4(el);
	}
	return v;
}


class XMLLoader
{
public:
	bool load(const std::string& fn) {
		TiXmlDocument doc;
		doc.LoadFile(fn.c_str());
		TiXmlElement *root = doc.FirstChildElement("empire_tohu");
		if(!root)
			return false;
		ground(root->FirstChildElement("ground"));
		faction(root->FirstChildElement("faction"));
		return true;
	}

	void ground(TiXmlElement *el) {
		if(!el || !el->Attribute("src"))
			return;
		LoadGround(el->Attribute("src"));
		for(TiXmlElement *n=el->FirstChildElement(); !!n; n=n->NextSiblingElement()) {
			
		}
	}

	void faction(TiXmlElement *el) {
		if(!el || !el->Attribute("class"))
			return;
		faction_ptr f = CheckedCreateFaction(el->Attribute("class"));
		AddGObj(f);
		for(TiXmlElement *n=el->FirstChildElement("object"); !!n; n=n->NextSiblingElement("object")) {
			object(n, f);
		}
	}

	void object(TiXmlElement *el, faction_ptr f) {
		if(!el || !el->Attribute("class"))
			return;
		gobj_ptr obj = CheckedCreateGObj(el->Attribute("class"), f);
		for(TiXmlElement *n=el->FirstChildElement("call"); !!n; n=n->NextSiblingElement("call")) {
			if(!n->Attribute("target"))
				continue;
			obj->call(n->Attribute("target"), ToAny(n->FirstChildElement()));
		}
		AddGObj(obj);
	}
};
#else


class XMLLoader
{
public:
	bool load(const std::string& fn) { return false; }
};

#endif



#endif
#endif
