#ifndef T_EFFECT
#define T_EFFECT



class CameraShaker : public LinearInherit2(HaveFrameCounter, Particle)
{
public:
	CameraShaker(int t, float l) : shake_time(t), shake_length(l) {}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		GetCamera().position-=pre_moved;
		GetCamera().target-=pre_moved;
		pre_moved = ist::vector4(GetRand()*shake_length, GetRand()*shake_length, GetRand()*shake_length)*
			(1.0f-((float)getFrame()/(float)shake_time));
		GetCamera().position+=pre_moved;
		GetCamera().target+=pre_moved;

		if(getFrame()>=shake_time)
			SendKillEvent(this, this);
	}

private:
	int shake_time;
	float shake_length;
	ist::vector4 pre_moved;
};


class ExplodeParticle : public LinearInherit4(HaveBlendAddAttrib, BillboardAttrib, HavePolygonModel, Particle)
{
public:
	const ist::vector4& getVel() { return vel; }
	void setVel(const ist::vector4& v) { vel=v; }

	ExplodeParticle() {
		setPModel("resource/plane.mqo");
		setTexture("resource/explode2.png");
		setVel( ist::vector4((GetRand()-0.5f)*0.5f, GetRand()*0.7f, (GetRand()-0.5f))*0.5f );
		setScl( ist::vector4(40.0f, 40.0f, 40.0f) );
		setRot( ist::vector4(0, GetRand()*360, 0) );
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		setPos(getPos()+getVel());

		size_t ft = getFrameTexture()+1;
		setFrameTexture(ft);
		if(ft>=getTexture()->getNumCell())
			SendKillEvent(this, this);
	}

private:
	ist::vector4 vel;
};


class Flasher : public LinearInherit4(HaveBlendAddAttrib, BillboardAttrib, HavePolygonModel, Particle)
{
public:
	Flasher() {
		setPModel("resource/plane.mqo");
		setTexture("resource/flare.png");
		setScl( ist::vector4(100.0f, 100.0f, 100.0f) );
		setRot( ist::vector4(0, GetRand()*360, 0) );
	}
	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		ist::vector4 scl = getScl()-ist::vector4(5, 5, 5);
		setScl(scl);
		if(scl.x<15)
			SendKillEvent(this, this);
	}
};

class Ring : public LinearInherit4(HaveBlendAddAttrib, BillboardAttrib, HavePolygonModel, Particle)
{
public:
	Ring() : fade(0.04f) {
		setPModel("resource/plane.mqo");
		setTexture("resource/ring.png");
		setScl( ist::vector4(0, 0, 0) );
		setRot( ist::vector4(0, GetRand()*360, 0) );
		setExpand(ist::vector4(1.5f, 1.5f, 1.5f));
	}

	void setFade(float f) { fade=f; }
	void setExpand(const ist::vector4& ex) { expand=ex; }

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		setScl(getScl()+expand);
		col.color.w-=fade;
		if(col.color.w<0)
			SendKillEvent(this, this);
	}

	void draw() const {
		col.assign();
		BaseClass::draw();
		ist::Color().assign();
	}

	float fade;
	ist::vector4 expand;
	ist::Color col;
};


class Pillar : public LinearInherit3(HaveBlendAddAttrib, HavePolygonModel, Particle)
{
public:
	Pillar() : fade(1.0f/40.0f) {
		setPModel("resource/pillar.mqo");
		setTexture("resource/heal.png");
		setScl( ist::vector4(0, 0, 0) );
		setRot( ist::vector4(0, GetRand()*360, 0) );
		setExpand(ist::vector4(0.0f, 300.0f/(1.0f/getFade()), 0.0f));
	}

	void setFade(float f) { fade=f; }
	void setColor(const ist::Color& c) { col=c; }
	void setExpand(const ist::vector4& ex) { expand=ex; }
	float getFade() { return fade; }
	const ist::vector4& getExpand() { return expand; }

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		setScl(getScl()+expand);
		col.color.w-=fade;
		if(col.color.w<0)
			SendKillEvent(this, this);
	}

	void draw() const {
		col.assign();
		BaseClass::draw();
		ist::Color().assign();
	}

	float fade;
	ist::vector4 expand;
	ist::Color col;
};








class SimplePModel
{
public:
	virtual ~SimplePModel() {}

	virtual void calcMatrix(ist::matrix44& mat) const=0;
	virtual void setLight(ist::Light& light) const {
		glEnable(GL_LIGHTING);
		light.position.y = 10000.0f;
		light.position.x = 10000.0f;
		light.position.z = -10000.0f;
		light.enable();
	}
	virtual void drawModel() const {
		if(!pmodel)
			return;
		ist::Light light;
		setLight(light);
		{
			ist::MatrixSaver msaver(GL_MODELVIEW_MATRIX);
			ist::matrix44 mat;
			calcMatrix(mat);
			glMultMatrixf(mat.v);
			pmodel->draw();
		}
		glDisable(GL_LIGHTING);
	}

	pmodel_ptr pmodel;
};



template <class T>
class Fragments : public LinearInherit2(HaveFrameCounter, Particle)
{
public:
	std::vector<T> particles;

	void add(T p) { particles.push_back(p); }
	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);

		bool alive=false;
		for(size_t i=0; i<particles.size(); ++i) {
			particles[i].update();
			if(!particles[i].isDead())
				alive=true;
		}
		if(!particles.empty() && !alive) {
			SendKillEvent(this, this);
		}
	}

	void draw() const{
		for(size_t i=0; i<particles.size(); ++i) {
			if(!particles[i].isDead())
				particles[i].draw();
		}
	}
};




class Blur : public LinearInherit2(HaveFrameCounter, Particle)
{
public:
	Blur() : fb(1024, 1024), w(2), h(1.5), opa(0.9f), fade(false) {
		setDrawPriority(250);
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);

		if(fade) {
			opa-=0.01f;
			if(opa<0)
				SendKillEvent(this, this);
		}
	}

	void draw() const {
		ist::MatrixSaver msaver(GL_MODELVIEW_MATRIX);
		ist::MatrixSaver psaver(GL_PROJECTION_MATRIX);
		ist::ScreenMatrix();

		glDepthMask(GL_FALSE);
		glEnable(GL_TEXTURE_2D);
		fb.copy();
		fb.assign();

		float tx=float(GetWindowWidth())/1024.0f;
		float ty=float(GetWindowHeight())/1024.0f;
		glColor4f(1,1,1,opa);
		glBegin(GL_QUADS);
		glTexCoord2f(0, ty);
		glVertex2f(0-w, GetWindowHeight()+h);

		glTexCoord2f(0, 0);
		glVertex2f(0-w, 0-h);

		glTexCoord2f(tx, 0);
		glVertex2f(GetWindowWidth()+w, -h);

		glTexCoord2f(tx, ty);
		glVertex2f(GetWindowWidth()+w, GetWindowHeight()+h);
		glEnd();

		glDisable(GL_TEXTURE_2D);
		glDepthMask(GL_TRUE);
		glColor4f(1,1,1,1);
	}

	mutable FrameBuffer fb;
	float w, h, opa;
	bool fade;
};



#endif
