#ifndef GAME_OBJECT_INCLUDE
#define GAME_OBJECT_INCLUDE

#include <fstream>
#include "gobj_parts.h"
#include "menu.h"








class ProductRequest : public IRequest
{
public:
	float progress;
	bool is_canceled;
	std::string name;
	Resource resource;

	ProductRequest(const std::string& n) :
		progress(0),
		is_canceled(false),
		name(n)
	{}

	Resource& getUsedResource() { return resource; }
	bool isComplete() const { return progress>100.0f; }
	bool isCanceled() const { return is_canceled; }
	const std::string getName() const { return name; }
	void addProgress(float p) { progress+=p; }
	float getProgress() const { return progress; }

	void cancel() { is_canceled = true; }

};
typedef boost::intrusive_ptr<ProductRequest> productr_ptr;




class IProductInfo : public ist::Object
{
public:
	virtual ~IProductInfo() {}

	virtual bool isDead() const { return false; }
	virtual gobj_id getID() const { return 0; }

	virtual bool query(const std::string&, boost::any&) const { return false; }
	virtual bool call(const std::string&, boost::any&) { return false; }
};

class ProductInfo : public LinearInherit2(HaveEventQueue, IProductInfo)
{
public:
	typedef LinearInherit2(HaveEventQueue, IProductInfo) RootClass;

	ProductInfo() : needs_time(1.0f), is_reserch(false), limit(0) {}
	ProductInfo(faction_ptr f, const std::string& pn, const Resource& c, float nt=1.0f) :
		needs_time(1.0f), is_reserch(false), limit(0) {
		setFaction(f);
		setProductName(pn);
		setCost(c, nt);
	}
	~ProductInfo() {
		if(!!button)
			button->disable();
	}

	void sendEvent(const Event& e) {
		RootClass::sendEvent(e);
		this->update();
	}

	bool isValidRequest(request_ptr r) {
		Resource& fr = getFaction()->getResource();
		if(fr>cost) {
			fr-=cost;
			Resource& used = r->getUsedResource();
			used = cost;
			return true;
		}
		return false;
	}

	void processRequest(request_ptr r) {
		r->addProgress(100.0f/(needs_time*60.0f));
	}


	void setCost(const Resource& c, float t=1.0f) { cost=c; needs_time=t; }
	void setFaction(faction_ptr f) { assert(!!f); faction =f; }
	void setProductName(const std::string& pn) { product_name=pn; }
	void setLimit(int l) { limit=l; }

	Resource& getCost() { return cost; }
	const std::string& getProductName() { return product_name; }
	faction_ptr getFaction() { assert(!!faction); return faction; }
	size_t getLimit() { return limit; }

	bool isExclusive() { return getLimit()>0; }
	bool isReserch() { return is_reserch; }

	ProductInfo& toReserch() {
		is_reserch = true;
		limit = 1;
		return *this;
	}
	ProductInfo& toLimited(size_t i) {
		limit = i;
		return *this;
	}

	void setButton(button_ptr b) { button=b; }

private:
	std::string product_name;
	faction_ptr faction;
	Resource cost;
	float needs_time;
	bool is_reserch;
	size_t limit;
	button_ptr button;
};

typedef boost::intrusive_ptr<ProductInfo> productinfo_ptr;






class DummyGObjIterator : public IGObjIterator
{
public:
	void reset() {}
	bool hasNext() { return false; }
	gobj_ptr iterate() { return gobj_ptr(); }
};




class SimpleFaction : public LinearInherit5(
	HaveFrameCounter,
	HaveDeathFlag,
	HaveID,
	HaveEventQueue,
	IFaction)
{
public:
	typedef std::set<selectable_ptr, less_id<selectable_ptr> > unit_set;
	typedef std::set<faction_ptr, less_id<faction_ptr> > faction_set;
	typedef std::map<std::string, productinfo_ptr> product_table;
	typedef std::map<std::string, unit_set> unit_table;
	typedef std::set<std::string> uginfo_cont;

	SimpleFaction() :
		faction_id(0),
		pop(0),
		resource(200, 200, 200, 200)
	{}

	int getDrawPriority() const { return 100000; }
	void draw() const {}
	category_t getRecieveEvents() const { return RCV_UPDATE | RCV_BUTTON | RCV_MOTION; }
	Resource& getResource() { return resource; }
	gobj_id getFactionID() const { return faction_id; }
	void setFactionID(gobj_id i) { faction_id=i; }

	bool pushRequest(request_ptr) { return false; }
	gobj_ptr getFocused() const { return gobj_ptr(); }
	void pushMessage(const std::string&) {}
	const std::string& getMessage(size_t) const { static std::string dummy; return dummy; }
	size_t getNumMessage() const { return 0; }

	const std::string& getPlayerName() const { return player_name; }
	void setPlayerName(const std::string& pn) { player_name=pn; }


	size_t getNumUnit(const std::string& name) const {
		unit_table::const_iterator p = unittable.find(name);
		return p==unittable.end() ? 0 : p->second.size();
	}

	int getPop() const { return pop; }
	void incrementPop(selectable_ptr p) {
		const std::string& cn = p->getClassName();
		unittable[cn].insert(p);
		sendUGInfo(p);
		if(prod_table.find(cn)!=prod_table.end())
			SendCommandEvent(this, p, new Event::CommandInfo("setCost", prod_table[cn]->getCost()));
		if(p->isUnit())
			++pop;
	}
	void decrementPop(selectable_ptr p) {
		unittable[p->getClassName()].erase(p);
		if(p->isUnit())
			--pop;
	}

	unit_table& getUnitTable() { return unittable; }
	IGObjIterator& getUnitTable(const std::string& classname) const {
		unit_table::const_iterator p = unittable.find(classname);
		if(p!=unittable.end()) {
			unittable_iter.set(p->second.begin(), p->second.end());
			return unittable_iter;
		}
		static DummyGObjIterator dummy;
		return dummy;
	}

	IGObjIterator& getSelection() const {
		static DummyGObjIterator dummy;
		return dummy;
	}

	const ist::Material& getFactionMaterial() const {
		static std::map<gobj_id, ist::Material> mat;
		if(mat.empty()) {
			mat[0].diffuse = ist::vector4(0.9f, 0.7f, 0.5f);
			mat[1].diffuse = ist::vector4(0.5f, 0.5f, 1.0f);
			mat[2].diffuse = ist::vector4(1.0f, 0.5f, 0.5f);
			mat[3].diffuse = ist::vector4(0.5f, 1.0f, 0.5f);
			mat[4].diffuse = ist::vector4(1.0f, 1.0f, 0.2f);
			mat[5].diffuse = ist::vector4(1.0f, 0.2f, 1.0f);
			mat[6].diffuse = ist::vector4(0.2f, 1.0f, 1.0f);
			mat[7].diffuse = ist::vector4(1.0f, 1.0f, 1.0f);
			mat[8].diffuse = ist::vector4(0.3f, 0.3f, 0.3f);
		}
		return mat[getFactionID()];
	}

	const ist::Color& getFactionColor() const {
		static std::map<gobj_id, ist::Color> mat;
		if(mat.empty()) {
			mat[0].color = ist::vector4(0.9f, 0.7f, 0.5f);
			mat[1].color = ist::vector4(0.3f, 0.3f, 1.0f);
			mat[2].color = ist::vector4(1.0f, 0.3f, 0.3f);
			mat[3].color = ist::vector4(0.3f, 0.8f, 0.3f);
			mat[4].color = ist::vector4(1.0f, 1.0f, 0.2f);
			mat[5].color = ist::vector4(1.0f, 0.2f, 1.0f);
			mat[6].color = ist::vector4(0.2f, 1.0f, 1.0f);
			mat[7].color = ist::vector4(1.0f, 1.0f, 1.0f);
			mat[8].color = ist::vector4(0.3f, 0.3f, 0.3f);
		}
		return mat[getFactionID()];
	}

	void onKill(Event& e) {
		unittable.clear();
		BaseClass::onKill(e);
	}

	virtual bool addReserch(const std::string& ugname) { return false; }
	virtual bool upgrade(const std::string& ug) {
		if(uginfo.insert(ug).second) {
			Event e = CommandEvent(getID(), 0, new Event::CommandInfo("upgrade", ug));
			for(unit_table::iterator p=getUnitTable().begin(); p!=getUnitTable().end(); ++p) {
				for(unit_set::iterator q=p->second.begin(); q!=p->second.end(); ++q) {
					e.to = (*q)->getID();
					(*q)->sendEvent(e);
				}
			}
			return true;
		}
		return false;
	}

	virtual bool isUpgraded(const std::string& ug) const {
		return uginfo.find(ug)!=uginfo.end();
	}

	void setTimber(float q) { resource.timber = q; }
	void setSilicon(float q){ resource.silicon = q; }
	void setIron(float q)   { resource.iron = q; }
	void setEnergy(float q) { resource.energy = q; }

	bool query(const std::string& name, boost::any& value) const {
		if(name=="isUpgraded")         value = isUpgraded(ToString(value));
		else if(name=="getPlayerName") value = getPlayerName();
		else return BaseClass::query(name, value);
		return true;
	}
	bool call(const std::string& name, boost::any& value) {
		if(name=="upgrade")        upgrade(ToString(value));
		else if(name=="setTimber") setTimber(ToFloat(value));
		else if(name=="setSilicon")setSilicon(ToFloat(value));
		else if(name=="setIron")   setIron(ToFloat(value));
		else if(name=="setEnergy") setEnergy(ToFloat(value));
		else if(name=="setPlayerName") setPlayerName(ToString(value));
		else if(name=="addAlly")   addAlly(ToFaction(ToGObj(value)));
		else return BaseClass::call(name, value);
		return true;
	}

	void sendUGInfo(gobj_ptr u) {
		for(uginfo_cont::iterator p=uginfo.begin(); p!=uginfo.end(); ++p)
			SendCommandEvent(this, u, new Event::CommandInfo("upgrade", *p));
		u->update();
	}

	void addAlly(faction_ptr f) { if(!!f && f.get()!=this) ally.insert(f); }
	bool isAlly(faction_ptr f) const { return (f.get()==this || ally.find(f)!=ally.end()); }
	IGObjIterator& getAlly() const { ally_iter.set(ally.begin(), ally.end()); return ally_iter; }

	unit_ptr createUnit(const std::string& n) { return CheckedCreateUnit(n, this); }
	building_ptr createBuilding(const std::string& n) { return CheckedCreateBuilding(n, this); }

protected:
	faction_set ally;
	product_table prod_table;
	unit_table unittable;
	uginfo_cont reserchinfo;
	uginfo_cont uginfo;
	std::string player_name;
	gobj_id faction_id;
	size_t pop;
	Resource resource;
	mutable IteratorAdapter<faction_set> ally_iter;
	mutable IteratorAdapter<unit_set> unittable_iter;
};



class Faction : public SimpleFaction
{
public:
	typedef SimpleFaction BaseClass;
	typedef std::set<selectable_ptr, less_id<selectable_ptr> > selection_cont;
	typedef std::deque< std::pair<frame_t, std::string> > message_queue;
	typedef std::map<int, selection_cont> numbering_table;
	typedef std::list<request_ptr> request_cont;

	Faction() :
		group(0),
		is_button_down(false),
		is_area_selecting(false),
		is_player_faction(false)
	{}


	void draw() const {
		BaseClass::draw();

		if(GetPlayerFaction().get()!=this)
			return;
	/*	Event e(EVT_CALL, 0, 0, new Event::CommandInfo("drawPath", boost::any()));
		for(selection_cont::const_iterator p=selection.begin(); p!=selection.end(); ++p) {
			e.to = (*p)->getID();
			(*p)->sendEvent(e);
		}
	*/
	}


	void setFrameTexture(const std::string& path) {
		if(!!screen_frame)
			SendCommandEvent(this, screen_frame, new Event::CommandInfo("setTexture", path));
	}

	gobj_ptr getFocused() const { return focused; }

	IGObjIterator& getSelection() const {
		selection_iter.set(selection.begin(), selection.end());
		return selection_iter;
	}

	void pushMessage(const std::string& mes) {
		mes_queue.push_back(std::pair<frame_t, std::string>(getFrame(), mes));
		if(mes_queue.size()>15)
			mes_queue.pop_front();
	}
	const std::string& getMessage(size_t i) const { return mes_queue[i].second; }
	size_t getNumMessage() const { return mes_queue.size(); }

	virtual void refreshMenu() {
		setMenu(selection_front);
	}

	virtual void setMenu(selectable_ptr p) {
		removeMenu();
		if(!p)
			return;

		selection_menu = p->getInfomationMenu();
		if(!!selection_menu && !GetGObj(selection_menu->getID()))
			AddGObj(selection_menu);
	//	if(IsAlly(p->getFaction(), this)) {
		if(p->getFaction().get()==this) {
			command_menu = p->getCommandMenu();
			if(!!command_menu && !GetGObj(command_menu->getID()))
				AddGObj(command_menu);
		}
	//	if(IsAlly(p->getFaction(), this)) {
		if(p->getFaction().get()==this) {
			exinfo_menu = p->getExtraMenu();
			if(!!exinfo_menu && !GetGObj(exinfo_menu->getID()))
				AddGObj(exinfo_menu);
		}
	}

	void removeMenu() {
		if(!!selection_menu) {
			SendKillEvent(this, selection_menu);
			selection_menu = menu_ptr();
		}
		if(!!command_menu) {
			SendKillEvent(this, command_menu);
			command_menu = menu_ptr();
		}
		if(!!exinfo_menu) {
			SendKillEvent(this, exinfo_menu);
			exinfo_menu = menu_ptr();
		}
	}



//	virtual bool isMouseOverlapedUI() {
//	}

	virtual void executeCommand(Event::ButtonInfo&, Event::CommandInfo&) {}
	virtual void onCommandChange() {}
	virtual void changeCommand(Event::commandinfo_ptr new_command) {
		command = new_command;
		onCommandChange();
	}
	virtual void clearCommand() {
		command = Event::commandinfo_ptr();
	}

	void onButton1(Event::ButtonInfo& info) {
		if(info.isButtonDown()) {
			if(!minimap->isMouseOver() && info.y<380)
				is_button_down = true;
		}
		else if(info.isButtonUp()) {
			if(minimap->isMouseOver() || info.y>380) {
				is_area_selecting = false;
				is_button_down = false;
				return;
			}
			if(!!command) {
				executeCommand(info, *command);
				Event::commandinfo_ptr old = getCurrentCommand();
				clearCommand();
				if(IsShiftKeyDown())
					changeCommand(old);
				return;
			}

			bool is_multi_selected = selection.size()>1;
			if(!IsShiftKeyDown() && !is_area_selecting)
				disselectUnit();
			selectable_ptr s = ToSelectable(focused);
			if(!!s && s->isMouseOver()) {
				if(IsAlly(s->getFaction(), this) && !is_multi_selected && s->isSelected())
					selectKindOf(s);
				else
					selectUnit(s);
			}
			else if(!!s && !!selection_front && s->getClassName()==selection_front->getClassName()) {
				selectKindOf(s);
			}
			is_area_selecting = false;
			is_button_down = false;
		}
	}

	Event::actioninfo_ptr createActionInfo(const ist::vector4& pos, selectable_ptr tgt) {
		Event::actioninfo_ptr action_info = new Event::ActionInfo(pos, tgt);
		action_info->flag|=Event::ActionInfo::INSIST;
		if(IsShiftKeyDown())
			action_info->flag|=Event::ActionInfo::APPEND;
		if(IsAltKeyDown())
			action_info->flag|=Event::ActionInfo::ACTIVE;
		return action_info;
	}

	void onButton3(Event::ButtonInfo& info) {
		if(info.isButtonUp()) {
			if(!!command) {
				clearCommand();
				return;
			}

			ist::vector4 mouse;
			selectable_ptr target;
			if(minimap->isMouseOver())
				mouse = minimap->getMousePointerPos();
			else {
				mouse = GetMousePointerPos3D();
				if(mouse.y<-9999)
					return;
				target = ToSelectable(focused);
			}

			updateSelection();

			if(selection.size()>1) { // IĂO[v쐬 
				group_ptr g = ToGroup(GetGObj(group));
				if(IsShiftKeyDown() && !!g && selection.find(g->getLeader())!=selection.end()) {
				//	SetGroup(group, selection.begin(), selection.end());
				}
				else {
					group = CreateID();
					Event::createinfo_ptr info = new Event::CreateInfo(UNITGROUP, group, std::string());
					for(selection_cont::iterator p=selection.begin(); p!=selection.end(); ++p) {
						if(unit_ptr u = ToUnit(*p))
							info->group.push_back(u->getID());
					}
					RouteCreateEvent(info);
				}
				RouteActionEvent(this->getID(), group, createActionInfo(mouse, target));

				for(selection_cont::iterator p=selection.begin(); p!=selection.end(); ++p) {
					if((*p)->isBuilding())
						RouteActionEvent(this, *p, createActionInfo(mouse, target));
				}
			}
			else if(selection.size()==1){
				selectable_ptr p = *selection.begin();
				if(isAlly(p->getFaction()))
					RouteActionEvent(this, p, createActionInfo(mouse, target));
			}

		}
	}

	void selectKindOf(selectable_ptr u) {
		IGObjIterator& gobjs_insight = GetGObjsInSight();
		while(gobjs_insight.hasNext()) {
			selectable_ptr p = ToSelectable(gobjs_insight.iterate());
			if(!p || IsEnemy(faction_ptr(this), p->getFaction()) || p->getClassName()!=u->getClassName())
				continue;
			if(!selectUnit(p))
				break;
		}
	}

	bool selectUnit(selectable_ptr p) {
		if((selection.size()>100)
		|| (IsEnemy(faction_ptr(this), p->getFaction()) && !selection.empty())
		|| (IsCtrlKeyDown() && (!selection.empty() && (*selection.begin())->getClassName()!=p->getClassName()))
		|| (!p->isUnit() && !IsShiftKeyDown() && !selection.empty() && (*selection.begin())->isUnit()) ) {
			return false;
		}

		if(!selection.empty() && IsEnemy(faction_ptr(this), (*selection.begin())->getFaction())) {
			disselectUnit();
		}
		if(p->isUnit() && !IsShiftKeyDown()) {
			for(selection_cont::iterator p=selection.begin(); p!=selection.end();) {
				if(!(*p)->isUnit()) {
					disselectUnit(*p);
					selection.erase(p++);
				}
				else {
					 ++p;
				}
			}
		}

		if(selection.find(p)==selection.end()) {
			SendSelectEvent(this, p);
			selection.insert(p);
			return true;
		}
		return false;
	}

	void disselectUnit(selectable_ptr p) {
		SendDisselectEvent(this, p);
	}

	void disselectUnit() {
		for(selection_cont::iterator p=selection.begin(); p!=selection.end(); ++p)
			disselectUnit(*p);
		selection.clear();
	}

	void onButton(Event& e) {
		Event::ButtonInfo& info = e.getButtonInfo();
		if(minimap->isMouseOver())
			SendButtonEvent(this, minimap, &info);

		if(info.button==1)
			onButton1(info);
		else if(info.button==3)
			onButton3(info);
	}

	void onMotion(Event& e) {
		Event::MotionInfo& info = e.getMotionInfo();
		if(minimap->isMouseOver())
			SendMotionEvent(this, minimap, &info);

		if(is_button_down && !command) {
			is_area_selecting = true;
			if(!IsShiftKeyDown() && !IsCtrlKeyDown())
				disselectUnit();
			IGObjIterator& gobjs_insight = GetGObjsInSight();
			while(gobjs_insight.hasNext()) {
				selectable_ptr p = ToSelectable(gobjs_insight.iterate());
				if(!!p && info.dragging_frustom.test(p->getSelectionModel())) {
					selectUnit(p);
				}
			}
		}
	}




	virtual void processMessages() {
		if(!mes_queue.empty() && getFrame()-mes_queue.front().first>420) {
			mes_queue.pop_front();
		}
	}

	virtual bool pushRequest(request_ptr r) {
		if(prod_table.find(r->getName())==prod_table.end())
			return false;
		productinfo_ptr pi = prod_table[r->getName()];
		if(!pi->isReserch() && pi->isExclusive() && getNumUnit(r->getName())+exclusive_requests[r->getName()]>=pi->getLimit()) {
			pushMessage("ȏセ̃jbg͍쐬ł܂B");
			return false;
		}
		else if(pi->isReserch() && exclusive_requests[r->getName()]) {
			pushMessage("̏ꏊŌ̂߁ALZ܂B");
			return false;
		}
		else if(pi->isValidRequest(r)) {
			requests.push_back(r);
			if(pi->isExclusive())
				exclusive_requests[r->getName()]++;
			return true;
		}
		return false;
	}

	virtual void processRequests() {
		for(request_cont::iterator p=requests.begin(); p!=requests.end();) {
			request_ptr r = *p;
			productinfo_ptr pi = prod_table[r->getName()];
			bool flag_erase = false;

			if(!pi || r->isCanceled()) {
				resource+=r->getUsedResource();
				flag_erase = true;
			}
			else {
				pi->processRequest(r);
				if(r->isComplete())
					flag_erase = true;
			}

			if(flag_erase) {
				requests.erase(p++);
				if(!!pi && (pi->isExclusive() && exclusive_requests[r->getName()]>0) && !pi->isReserch())
					exclusive_requests[r->getName()]--;
			}
			else
				++p;
		}
	}

	virtual void searchFocusedObject() {
		if(GetPlayerFaction().get()!=this)
			return;

		focused = gobj_ptr();
		IGObjIterator& menus = GetMenus();
		// J[\ڂĂ郁j[΂tH[JX 
		while(menus.hasNext()) {
			menu_ptr p = ToMenu(menus.iterate());
			if(p->isMouseOver()) {
				focused = p;
			}
		}
		// J[\ڂĂ郆jbg΂tH[JX 
		if(!focused) {
			selectable_ptr s;
			const ist::vector4& cpos = GetCamera().position;
			IGObjIterator& gobjs_insight = GetGObjsInSight();
			while(gobjs_insight.hasNext()) {
				selectable_ptr p = ToSelectable(gobjs_insight.iterate());
				if(!!p && p->isMouseOver() &&
					(!s || (p->getPos()-cpos).square()<(s->getPos()-cpos).square())) {
					s = p;
				}
			}
			if(!!s)
				focused = s;
		}
	}

	virtual void updateSelection() {
		for(selection_cont::iterator p=selection.begin(); p!=selection.end();) {
			if((*p)->isDead())
				selection.erase(p++);
			else
				++p;
		}

		if(selection.empty()) {
			selection_front = selectable_ptr();
			removeMenu();
		}
		else if((*selection.begin())!=selection_front) {
			selection_front = *selection.begin();
			setMenu(selection_front);
		}
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		processMessages();
		processRequests();
		searchFocusedObject();
		updateSelection();
	}


	void registNumbering(int k) {
		numbering[k] = selection;
	}
	void selectNumbering(int k) {
		if(!IsShiftKeyDown())
			disselectUnit();
		selection_cont& c = numbering[k];
		for(selection_cont::iterator p=c.begin(); p!=c.end();) {
			if((*p)->isDead())
				c.erase(p++);
			else
				++p;
		}
		for(selection_cont::iterator p=c.begin(); p!=c.end(); ++p)
			selectUnit(*p);
	}

	void onKey(Event& e) {
		Event::KeyInfo& info = e.getKeyInfo();
		if(info.state==SDL_KEYUP) {
			if(info.key==SDLK_DELETE) {
				if(!selection.empty() && (*selection.begin())->getFaction().get()==this)
					RouteEvent(DestroyEvent(getID(), (*selection.begin())->getID()));
			}
			else if(info.key>=SDLK_1 && info.key<=SDLK_9) {
				if(IsCtrlKeyDown())
					registNumbering(info.key);
				else
					selectNumbering(info.key);
			}
			else if(info.key==SDLK_SPACE) {
				if(!selection.empty())
					SetCameraPos((*selection.begin())->getPos());
			}
		}
	}

	void onSelect(Event& e) {
		BaseClass::onSelect(e);

		screen_frame = new BottomFrame();
		AddGObj(screen_frame);

		factioninfo_menu = new FactionInfoMenu(this);
		AddGObj(factioninfo_menu);

		minimap = new MiniMap(this);
		minimap->setPos(ist::vector2(485, 330));
		AddGObj(minimap);

		description_menu = new DescriptionMenu(this);
		AddGObj(description_menu);
	}

	void killMenus() {
		if(!!factioninfo_menu) {
			SendKillEvent(this, factioninfo_menu);
			factioninfo_menu = menu_ptr();
		}
		if(!!minimap) {
			SendKillEvent(this, minimap);
			minimap = boost::intrusive_ptr<MiniMap>();
		}
		if(!!description_menu) {
			SendKillEvent(this, description_menu);
			description_menu = menu_ptr();
		}
		if(!!screen_frame) {
			SendKillEvent(this, screen_frame);
			screen_frame = menu_ptr();
		}
	}

	void onDisselect(Event& e) {
		BaseClass::onDisselect(e);
		removeMenu();
		killMenus();
	}

	void onKill(Event& e) {
		prod_table.clear();
		selection.clear();
		numbering.clear();
		minimap = boost::intrusive_ptr<MiniMap>();
		focused = gobj_ptr();
		selection_front = selectable_ptr();
		description_menu = menu_ptr();
		factioninfo_menu = menu_ptr();
		selection_menu = menu_ptr();
		command_menu = menu_ptr();
		exinfo_menu = menu_ptr();
		screen_frame = menu_ptr();
		BaseClass::onKill(e);
	}


	bool call(const std::string& name, boost::any& value) {
		if(name=="refreshMenu") refreshMenu();
		else if(name=="upgrade") upgrade(ToString(value));
		else if(name=="pushMessage") pushMessage(ToString(value));
		else return BaseClass::call(name, value);
		return true;
	}

	void onCommand(Event& e) {
		Event::CommandInfo& info = e.getCommandInfo();
		if(info.name=="refreshMenu")  refreshMenu();
		else if(info.name=="upgrade") upgrade(ToString(info.value));
		else changeCommand(&e.getCommandInfo());
	}


protected:
	boost::intrusive_ptr<MiniMap> minimap;

	int isExclusiveRequest(const std::string& name) const {
		std::map<std::string, size_t>::const_iterator p = exclusive_requests.find(name);
		if(p!=exclusive_requests.end())
			return p->second;
		return 0;
	}

	Event::commandinfo_ptr getCurrentCommand() const { return command; }


private:
	Event::commandinfo_ptr command;
	selection_cont selection;
	gobj_id group;
	numbering_table numbering;
	request_cont requests;
	std::map<std::string, size_t> exclusive_requests;
	gobj_ptr focused;
	selectable_ptr selection_front;
	menu_ptr description_menu;
	menu_ptr factioninfo_menu;
	menu_ptr selection_menu;
	menu_ptr command_menu;
	menu_ptr exinfo_menu;
	menu_ptr screen_frame;
	message_queue mes_queue;
	bool is_button_down;
	bool is_area_selecting;
	bool is_player_faction;
	mutable IteratorAdapter<selection_cont> selection_iter;
};





template <class T>
class _BreakableRoot : public LinearInherit5(
	BelongFaction,
	HaveCharacterGridInserter,
	HaveSelectChecker,
	HaveFrameCounter,
	LinearInherit9(
		HaveActionQueue,
		HavePointCollisionModel,
		HaveBreakableInfo,
		HaveSelectableInfo,
		HaveTransformInfo,
		HaveDeathFlag,
		HaveID,
		HaveEventQueue,
		T ))
{};


template <class T>
class Breakable : public _BreakableRoot<T>
{
private:
	float floating;

public:
	typedef _BreakableRoot<T> RootClass;
	typedef Breakable BaseClass;

	typedef std::vector<std::string> info_type;

	Breakable() : floating(0) {}

	int getDrawPriority() const { return 0; }
	category_t getRecieveEvents() const { return RCV_UPDATE; }

	virtual void landing(ist::vector4& dst) const {
		dst.y = GetHeight(dst)+floating;
	}

	float getFloat() const { return floating; }
	void setFloat(float f) { floating = f; }
	void setPos(const ist::vector4& v) {
		const ist::vector2& mmin = GetMapMin();
		const ist::vector2& mmax = GetMapMax();
		ist::vector4 dst;
		if(v.x<mmin.x)
			dst.x = mmin.x;
		else if(v.x>mmax.x)
			dst.x = mmax.x;
		else
			dst.x = v.x;
		if(v.z<mmin.y)
			dst.z = mmin.y;
		else if(v.z>mmax.y)
			dst.z = mmax.y;
		else
			dst.z = v.z;
		dst.y = v.y;
		landing(dst);
		this->RootClass::setPos(dst);
	}

	void onAdded() {
		this->RootClass::onAdded();
		this->getFaction()->incrementPop(this);
	}

	virtual const info_type& getExtInfo() const {
		static info_type info;
		info.clear();
		return info;
	}

	virtual void drawLifeGage() const {
		float ratio = this->getLife()/this->getMaxLife();
		float up = this->getLifeGageFloating();
		const ist::vector4& pos = this->getPos();
		ist::matrix44 mat;
		mat.translate(pos.v);
		mat*=GetAimCameraMatrix();

		ist::MatrixSaver msaver(GL_MODELVIEW_MATRIX);
		glMultMatrixf(mat.v);
		glDisable(GL_DEPTH_TEST);
		glLineWidth(3);
		glBegin(GL_LINES);
		ist::Color(0, 0, 0, 1).assign();
		glVertex3f(-10.0f, 0, -up);
		glVertex3f(10.0f, 0, -up);
		this->getFaction()->getFactionColor().assign();
		glVertex3f(-10.0f, 0, -up);
		glVertex3f(-10.0f+(20.0f*ratio), 0, -up);
		glEnd();
		glLineWidth(1);
		glEnable(GL_DEPTH_TEST);
		ist::Color().assign();
	}

	virtual void drawPath() const {
		glDisable(GL_DEPTH_TEST);
		ist::Color(1.0f, 1.0f, 1.0f, 0.4f).assign();

		glBegin(GL_LINE_STRIP);
		glVertex3fv(this->getPos().v);
		Event::actioninfo_ptr ap;
		for(size_t i=0; i<this->sizeAction(); ++i) {
			ap = this->getAction(i);
			selectable_ptr s = ap->getObj();
			if(!!s && !ap->flagPath())
				glVertex3fv(s->getPos().v);
			else
				glVertex3fv(ap->target_pos.v);
		}
		glEnd();

		ist::Color().assign();
		glEnable(GL_DEPTH_TEST);
	}

	virtual void assignMaterial(const ist::Material& _mat) const {
		bool highlight = this->isAdded() && (this->isSelected() || (GetPlayerFaction()->getFocused().get()==this));
		if(highlight) {
			ist::Material mat = _mat;
			mat.emission = ist::vector4(0.15f, 0.15f, 0.15f);
			mat.assign();
		}
		else {
			_mat.assign();
		}
	}

	virtual void drawModel() const {}

	void draw() const {
		assignMaterial(this->getFaction()->getFactionMaterial());
		this->drawModel();
		ist::Material().assign();

		bool highlight = this->isAdded() && (this->isSelected() || (GetPlayerFaction()->getFocused().get()==this));
		if(highlight)
			drawLifeGage();
	}

	virtual float getDamage(Event::DamageInfo& info) const {
		float ret = 0;
		ret+= std::max<float>(0, info.bullet_damage*(1.0f-this->getAntiBullet()));
		ret+= std::max<float>(0, info.laser_damage*(1.0f-this->getAntiLaser()));
		ret+= std::max<float>(0, info.bomb_damage*(1.0f-this->getAntiBomb()));
		return ret;
	}

	virtual void upgrade(const std::string& ugname) {}

	void onDamage(Event& e) {
		RootClass::onDamage(e);
		this->setLife(this->getLife()-getDamage(e.getDamageInfo()));
		if(this->getLife()<=0) {
			SendDestroyEvent(GetGObj(e.from), this);
		}
	}

	void onKill(Event& e) {
		this->getFaction()->decrementPop(this);
		this->RootClass::onKill(e);
	}

	bool query(const std::string& name, boost::any& value) const {
		if(name=="getExtInfo") value = &getExtInfo();
		else return this->RootClass::query(name, value);
		return true;
	}

	bool call(const std::string& name, boost::any& value) {
		if(name=="drawPath")     drawPath();
		else if(name=="upgrade") upgrade( ToString(value) );
		else return RootClass::call(name, value);
		return true;
	}

	menu_ptr getInfomationMenu() { return new SelectionMenu(this); }
	menu_ptr getExtraMenu() { return new ExtInfoMenu(this); }
};




typedef LinearInherit3(HaveAttackAction, Breakable, IBuilding) _BuildingRoot;

class Building : public _BuildingRoot
{
public:
	typedef _BuildingRoot RootClass;
	typedef Building BaseClass;

	Building() : build_speed(1.0f), build_status(1.0f) {
		setDrawPriority(100);
	}

	bool category(const std::string& c) const {
		return c=="Building" ? true : RootClass::category(c);
	}

	bool isSettlement() const { return false; }

	void assignMaterial(const ist::Material& _mat) const {
		bool highlight = this->isAdded() && (this->isSelected() || (GetPlayerFaction()->getFocused().get()==this));
		if(highlight) {
			ist::Material mat = _mat;
			mat.emission = ist::vector4(0.15f, 0.15f, 0.15f);
			mat.assign();
		}
		else if(!GetCurrentWorld().isSighted(GetPlayerFaction(), getPos())) {
			ist::Material mat = _mat;
			mat.diffuse.x*=0.4f;
			mat.diffuse.y*=0.4f;
			mat.diffuse.z*=0.4f;
			mat.assign();
		}
		else {
			_mat.assign();
		}
	}

	bool isInSight(faction_ptr f) const {
		if(!f)
			return true;
		return GetCurrentWorld().isExplored(f, this->getPos());
	}

	virtual const Resource& getCost() const { return cost; }
	virtual void setCost(const Resource& c) { cost=c; }


	void build(float b) {
		float l = isBuilt() ? b*0.5f : b*build_speed;
		setLife(getLife()+l);
		if(!isBuilt()) {
			build_status+=l/getMaxLife();
			if(getLife()==getMaxLife())
				build_status = 1.0f;
			if(isBuilt())
				onBuildComplete();
		}
	}

	virtual bool isBuilt() const { return getBuildStatus()>=1.0f; }
	virtual void onBuildComplete() {
		SendCommandEvent(this, getFaction(), new Event::CommandInfo("refreshMenu", boost::any()));
	}

	float getSight() const {
		if(!isBuilt())
			return 100;
		return RootClass::getSight();
	}

	float getAntiBullet() const{
		if(!isBuilt())
			return 0;
		return RootClass::getAntiBullet();
	}

	float getAntiLaser() const {
		if(!isBuilt())
			return 0;
		return RootClass::getAntiLaser();
	}

	float getAntiBomb() const  {
		if(!isBuilt())
			return 0;
		return RootClass::getAntiBomb();
	}

	virtual void initBuildStatus() {
		setLife(1);
		build_status = 0;
	}

	float getBuildSpeed() const  { return build_speed; }
	float getBuildStatus() const { return build_status; }
	void turnToTarget(const ist::vector4&) {}

	void processActionQueue() {
		if(isBuilt())
			RootClass::processActionQueue();
	}


	bool call(const std::string& name, boost::any& value) {
		if(name=="build")               build( ToFloat(value) );
		else if(name=="setCost")        setCost(boost::any_cast<Resource>(value));
		else if(name=="initBuildStatus")initBuildStatus();
		else return RootClass::call(name, value);
		return true;
	}

	void onKill(Event& e) {
		if(!isBuilt()) {
			Resource ret = getCost();
			ret*=(1.0f-getBuildStatus());
			getFaction()->getResource()+=ret;
		}
		RootClass::onKill(e);
	}

protected:
	void setBuildSpeed(float bs) { build_speed=bs; }

private:
	Resource cost;
	float build_speed;
	float build_status;
};



typedef LinearInherit5(HaveMoveAction, HaveAttackAction, HaveCollideChecker, Breakable, IUnit) _UnitRoot;

class Unit : public _UnitRoot
{
public:
	typedef _UnitRoot RootClass;
	typedef Unit BaseClass;

	Unit() : col_num(0), move_speed(1), energy(0), max_energy(0) {
		setDrawPriority(200);
	}

	bool category(const std::string& c) const {
		return c=="Unit" ? true : RootClass::category(c);
	}

	void draw() const {
		if(isSelected())
			drawPath();
		RootClass::draw();
	}

	void drawPath() const {
		RootClass::drawPath();
		if(!!group)
			group->draw();
	}

	virtual float getMoveSpeed() const { return move_speed; }
	virtual void setMoveSpeed(float s) { move_speed=s; }
	virtual void setEnergy(float a)    { energy=std::max<float>(std::min<float>(a, getMaxEnergy()), 0); }
	virtual void setMaxEnergy(float a) {
		float old_max_energy = max_energy;
		max_energy=a;
		if(energy>max_energy)
			energy=max_energy;
		else if(max_energy>old_max_energy)
			energy+=max_energy-old_max_energy;
	}
	virtual float getEnergy() const    { return energy; }
	virtual float getMaxEnergy() const { return max_energy; }
	virtual bool conditionEnergyRegen() const { return true; }

	virtual void recover(float f) {
		if(sizeAction()!=0)
			RootClass::recover(f*0.3f);
		else
			RootClass::recover(f);
	}

	virtual group_ptr getGroup() const { return group; }
	virtual void setGroup(group_ptr g) { group = g; }


	virtual float getDamage(Event::DamageInfo& info) {
		float ret = 0;
		ret+=std::max<float>(0, info.bullet_damage*(std::min<float>(1.0f, 1.0f-this->getAntiBullet())));
		ret+=std::max<float>(0, info.laser_damage*(std::min<float>(1.0f, 1.0f-this->getAntiLaser())));
		ret+=std::max<float>(0, info.bomb_damage*(1.0f-this->getAntiBomb()));
		ret+=info.ignore_defence_damage;
		return ret;
	}

	bool isNeedsAStar(Event::actioninfo_ptr a) {
		if(a->flagPath() || a->flagLink() || a->flagRank())
			return false;
		float min_dist = std::max(getSearchRange(), std::max(getFireRange(), getFloat()));
		selectable_ptr s = a->getObj();
		if(!!s) {
			a->target_pos = s->getPos();
			if((a->target_pos-getPos()).setY(0).norm()<min_dist)
				return false;
		}
		return true;
	}

	void onChangeCurrentAction(Event::actioninfo_ptr a) {
		this->RootClass::onChangeCurrentAction(a);

		if(isNeedsAStar(a)) {
			PathConstructor pc;
			a->flag|=Event::ActionInfo::PATH;
			pc.construct(getPos(), a);
			for(size_t i=0; i<pc.action.size(); ++i)
				pushfrontAction(pc.action[i]);
		}
	}

	void onUpdate(Event& e) {
		this->RootClass::onUpdate(e);
		if(col_num>0) {
			col_aim.normalize();
			setPos(getPos()+col_aim*(getMoveSpeed()*(1.0f+0.1f*col_num)));
			col_aim = ist::vector4();
			col_num = 0;
		}
	}

	void onCollide(Event& e) {
		this->RootClass::onCollide(e);
		Event::CollideInfo& info = e.getCollideInfo();
		col_aim+=info.aim;
		++col_num;

		unit_ptr u = ToUnit(GetGObj(e.from));
		if(!!u && IsAlly(this, u) && !u->getCurrentAction()) {
			SendActionEvent(this, u,
				new Event::ActionInfo(u->getPos()+info.aim*-10.0f, selectable_ptr(), Event::ActionInfo::PATH));
		}
	}

	virtual void commandAttack(selectable_ptr) {}
	virtual void commandSpecialAttack(selectable_ptr) {}

	virtual void explode() {}
	void onDestroy(Event& e) {
		RootClass::onDestroy(e);
		if(isInCameraSight())
			explode();
	}

	void onKill(Event& e) {
		group = group_ptr();
		RootClass::onKill(e);
	}

	bool query(const std::string& name, boost::any& value) const {
		if     (name=="getMoveSpeed") value = getMoveSpeed();
		else if(name=="getEnergy")    value = getEnergy();
		else if(name=="getMaxEnergy") value = getMaxEnergy();
		else return this->RootClass::query(name, value);
		return true;
	}

	bool call(const std::string& name, boost::any& value) {
		if     (name=="setMoveSpeed") setMoveSpeed( ToFloat(value) );
		else if(name=="setEnergy")    setEnergy( ToFloat(value) );
		else if(name=="setMaxEnergy") setMaxEnergy( ToFloat(value) );
		else if(name=="attack")       commandAttack( boost::any_cast<selectable_ptr>(value) );
		else if(name=="specialAttack")commandSpecialAttack( boost::any_cast<selectable_ptr>(value) );
		else return RootClass::call(name, value);
		return true;
	}

protected:
	group_ptr group;
	ist::vector4 col_aim;
	int col_num;
	float move_speed;
	float energy;
	float max_energy;
};




template<class T>
class _Particle : public LinearInherit6(
	HaveCharacterGridInserter,
	HaveTransformInfo,
	HaveDeathFlag,
	HaveEventQueue,
	HaveID,
	T )
{
public:
	virtual void drawModel() const {}
	void draw() const {
		drawModel();
	}
};

typedef LinearInherit2(_Particle, IParticle) _ParticleRoot;

class Particle : public _ParticleRoot
{
public:
	typedef _ParticleRoot RootClass;
	typedef Particle BaseClass;

	Particle() {
		setDrawPriority(400);
	}

	category_t getRecieveEvents() const { return RCV_UPDATE; }
};


typedef LinearInherit6(
	BelongFaction,
	HaveCollideChecker,
	HavePointCollisionModel,
	HaveFrameCounter,
	_Particle,
	IBullet) _BulletRoot;

class BBullet : public _BulletRoot
{
public:
	typedef BBullet BaseClass;

	BBullet() {
		setDrawPriority(300);
	}
	category_t getRecieveEvents() const { return RCV_UPDATE; }

	virtual Event::damageinfo_ptr getDamage() { assert(!!damage); return damage; }
	virtual breakable_ptr getRoot() { return root; }
	virtual void setDamage(Event::damageinfo_ptr d) { damage=d; }
	virtual void setRoot(breakable_ptr r) {
		root = r;
		setFaction(getRoot()->getFaction());
	}

	void onKill(Event& e) {
		root = breakable_ptr();
		_BulletRoot::onKill(e);
	}

private:
	breakable_ptr root;
	Event::damageinfo_ptr damage;
};

class Bullet : public BBullet
{
public:
	typedef BBullet RootClass;
	typedef Bullet BaseClass;

	Bullet() : life_time(100) {}

	virtual const ist::vector4& getVel() const { return vel; }
	virtual void setLifeTime(int lt)     { life_time=lt; }
	virtual void setVel(const ist::vector4& v) { vel=v; }

	bool conditionCollide(selectable_ptr p) {
		if(IsEnemy(getFaction(), p->getFaction()))
			return true;
		return false;
	}

	void onUpdate(Event& e) {
		RootClass::onUpdate(e);
		if(getFrame()>1)
			setPos(getPos()+getVel());
		if(getFrame()>life_time)
			SendKillEvent(this, this);
	}

private:
	ist::vector4 vel;
	int life_time;
};





class Fader : public LinearInherit3(HavePolygonModel, HaveFrameCounter, Particle)
{
public:
	ist::vector4 diffuse;
	float fade;
	bool stoped;

	Fader() : fade(0.02f) {
		diffuse = ist::vector4(1.0f, 1.0f, 1.0f);
	}

	void draw() const {
		glMaterialfv(GL_FRONT, GL_DIFFUSE,  diffuse.v);
		drawModel();
		ist::Material().assign();
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);

		diffuse.w-=fade;
		if(diffuse.w<=0)
			sendEvent(KillEvent(getID(), getID()));
	}
};



#endif
