#ifndef gobj_parts
#define gobj_parts


#include <deque>
#include <set>


#ifndef GetIGame
class IGame;
extern IGame& _GetIGame();
#define GetIGame() _GetIGame()
#include "aos/aos.h"
#endif




enum {
	SE_VOICE=0,
	SE_DESTROY=1,
};


#define LinearInherit2(C2, C1)  C2< C1 >
#define LinearInherit3(C3, C2, C1)  C3< LinearInherit2(C2, C1) >
#define LinearInherit4(C4, C3, C2, C1)  C4< LinearInherit3(C3, C2, C1) >
#define LinearInherit5(C5, C4, C3, C2, C1)  C5< LinearInherit4(C4, C3, C2, C1) >
#define LinearInherit6(C6, C5, C4, C3, C2, C1)  C6< LinearInherit5(C5, C4, C3, C2, C1) >
#define LinearInherit7(C7, C6, C5, C4, C3, C2, C1)  C7< LinearInherit6(C6, C5, C4, C3, C2, C1) >
#define LinearInherit8(C8, C7, C6, C5, C4, C3, C2, C1)  C8< LinearInherit7(C7, C6, C5, C4, C3, C2, C1) >
#define LinearInherit9(C9, C8, C7, C6, C5, C4, C3, C2, C1)  C9< LinearInherit8(C8, C7, C6, C5, C4, C3, C2, C1) >

#define DEC_GETCLASSNAME(name) \
	const std::string& getClassName() { static std::string cn(name); return cn; }





class EmptyClass
{};


template<class T>
class HaveTransformInfo : public T
{
public:
	typedef HaveTransformInfo BaseClass;

	HaveTransformInfo() : scl(1.0f, 1.0f, 1.0f), draw_priority(0) {}

	virtual const ist::vector4& getPos() const { return pos; }
	virtual const ist::vector4& getRot() const { return rot; }
	virtual const ist::vector4& getScl() const { return scl; }
	virtual int getDrawPriority() const        { return draw_priority; }
	virtual void setPos(const ist::vector4& v) { pos=v; }
	virtual void setRot(const ist::vector4& v) { rot=v; }
	virtual void setScl(const ist::vector4& v) { scl=v; }
	virtual void setDrawPriority(int p)        { draw_priority=p; }
	virtual void calcMatrix(ist::matrix44& mat) const {
		mat.translate(this->getPos().v);
		mat.rotate(this->getRot().v);
		mat.scale(this->getScl().v);
	}

	virtual bool isInCameraSight() const { return GetCamera().view_frustum.test(getPos(), 0)!=0; }
	virtual bool isInSight(faction_ptr f) const {
		if(!f)
			return true;
		return GetCurrentWorld().isSighted(f, this->getPos());
	}


	bool query(const std::string& name, boost::any& value) const {
		if(name=="getPos")       value = getPos();
		else if(name=="getRot")  value = getRot();
		else if(name=="getScl")  value = getScl();
		else return this->T::query(name, value);
		return true;
	}

	bool call(const std::string& name, boost::any& value) {
		if     (name=="setPos")  setPos( ToVector4(value) );
		else if(name=="setRot")  setRot( ToVector4(value) );
		else if(name=="setScl")  setScl( ToVector4(value) );
		else return this->T::call(name, value);
		return true;
	}

private:
	ist::vector4 pos;
	ist::vector4 rot;
	ist::vector4 scl;
	int draw_priority;
};


template<class T>
class HaveSelectableInfo : public T
{
public:
	typedef HaveSelectableInfo BaseClass;

	HaveSelectableInfo() {}
	virtual menu_ptr getInfomationMenu() { return menu_ptr(); }
	virtual menu_ptr getCommandMenu() { return menu_ptr(); }
	virtual menu_ptr getExtraMenu() { return menu_ptr(); }
};






template<class T>
class HaveBreakableInfo : public T
{
public:
	typedef HaveBreakableInfo BaseClass;

	HaveBreakableInfo() :
		life(1.0f),
		max_life(1.0f),
		anti_bullet(0.0f),
		anti_laser(0.0f),
		anti_bomb(0.0f),
		fire_power(0.0f),
		fire_range(0.0f),
		search_range(0.0f),
		sight(20.0f)
	{}

	float getBaseLife() const      { return life; }
	float getBaseMaxLife() const   { return max_life; }
	float getBaseAntiBullet() const{ return anti_bullet; }
	float getBaseAntiLaser() const { return anti_laser; }
	float getBaseAntiBomb() const  { return anti_bomb; }
	float getBaseFirePower() const  { return fire_power; }
	float getBaseFireRange() const  { return fire_range; }
	float getBaseSearchRange() const{ return search_range; }
	float getBaseSight() const      { return sight; }

	virtual float getLife() const      { return life; }
	virtual float getMaxLife() const   { return max_life; }
	virtual float getAntiBullet() const{ return anti_bullet; }
	virtual float getAntiLaser() const { return anti_laser; }
	virtual float getAntiBomb() const  { return anti_bomb; }
	virtual float getFirePower() const  { return fire_power; }
	virtual float getFireRange() const  { return fire_range; }
	virtual float getSearchRange() const{ return search_range; }
	virtual float getSight() const      { return sight; }


	virtual void setLife(float f)      { life=std::min<float>(getMaxLife(), f); }
	virtual void setMaxLife(float f)   {
		float old_max_life = max_life;
		max_life=f;
		if(life>max_life)
			life=max_life;
		else if(max_life>old_max_life)
			life+=max_life-old_max_life;
	}
	virtual void setAntiBullet(float f){ anti_bullet=f; }
	virtual void setAntiLaser(float f) { anti_laser=f; }
	virtual void setAntiBomb(float f)  { anti_bomb=f; }
	virtual void setFirePower(float f)  { fire_power=f; }
	virtual void setFireRange(float f)  { fire_range=f; }
	virtual void setSearchRange(float f){ search_range=std::min(sight, f); }
	virtual void setSight(float f)      { sight=f; }


	virtual float getLifeGageFloating() const { return 10; }

	virtual void recover(float f) {
		setLife(getLife()+f);
	}

	bool query(const std::string& name, boost::any& value) const {
		if     (name=="getLife")      value = getLife();
		else if(name=="getMaxLife")   value = getMaxLife();
		else if(name=="getAntiBullet")value = getAntiBullet();
		else if(name=="getAntiLaser") value = getAntiLaser();
		else if(name=="getAntiBomb")  value = getAntiBomb();

		else if(name=="getFirePower")  value = getFirePower();
		else if(name=="getFireRange")  value = getFireRange();
		else if(name=="getSearchRange")value = getSearchRange();
		else if(name=="getSight")      value = getSight();
		else return this->T::query(name, value);
		return true;
	}
	bool call(const std::string& name, boost::any& value) {
		if     (name=="recover")      recover( ToFloat(value) );
		else if(name=="setLife")      setLife( ToFloat(value) );
		else if(name=="setMaxLife")   setMaxLife( ToFloat(value) );
		else if(name=="setAntiBullet")setAntiBullet( ToFloat(value) );
		else if(name=="setAntiLaser") setAntiLaser( ToFloat(value) );
		else if(name=="setAntiBomb")  setAntiBomb( ToFloat(value) );

		else if(name=="setFirePower")  setFirePower( ToFloat(value) );
		else if(name=="setFireRange")  setFireRange( ToFloat(value) );
		else if(name=="setSearchRange")setSearchRange( ToFloat(value) );
		else if(name=="setSight")      setSight( ToFloat(value) );
		else return this->T::call(name, value);
		return true;
	}

private:
	float life;
	float max_life;
	float anti_bullet;
	float anti_laser;
	float anti_bomb;

	float fire_power;
	float fire_range;
	float search_range;
	float sight;
};






template<class T>
class HaveEventQueue : public T
{
public:
	typedef HaveEventQueue BaseClass;
	typedef std::list<Event> EventCont;


	HaveEventQueue() : current_event(0) {
		this->sendEvent(Event(EVT_INIT, 0, 0));
	}

	void sendEvent(const Event& e) {
		if(e.getType()==EVT_CALL) {
			Event::CallInfo& info = e.getCallInfo();
			this->call(info.name, info.value);
			return;
		}
		events.push_back(e);
	}

	void update() {
		for(EventCont::iterator p=events.begin(); p!=events.end();) {
			if(p->delay>0) {
				--p->delay;
				++p;
				continue;
			}
			Event& e = *p;
			current_event = &e;
			switch(e.getType()) {
			case EVT_INIT:      onInit(e);      break;
			case EVT_IDLE:      onIdle(e);      break;
			case EVT_CREATE:    onCreate(e);    break;
			case EVT_UPDATE:    onUpdate(e);    break;
			case EVT_COLLIDE:   onCollide(e);   break;
			case EVT_DAMAGE:    onDamage(e);    break;
			case EVT_DESTROY:   onDestroy(e);   break;
			case EVT_KILL:      onKill(e);      break;
			case EVT_RAISE:     onRaise(e);     break;
			case EVT_ACTION:    onAction(e);    break;
			case EVT_COMMAND:   onCommand(e);   break;
			case EVT_SELECT:    onSelect(e);    break;
			case EVT_DISSELECT: onDisselect(e); break;
			case EVT_BUTTON:    onButton(e);    break;
			case EVT_MOTION:    onMotion(e);    break;
			case EVT_KEY:       onKey(e);       break;
			default: assert(0 && "unknown event"); break;
			}
			events.erase(p++);
			current_event = 0;
		}
	}

	virtual void onInit(     Event&)  {}
	virtual void onIdle(     Event&)  {}
	virtual void onCreate(   Event&)  {}
	virtual void onUpdate(   Event&)  {}
	virtual void onCollide(  Event&)  {}
	virtual void onDamage(   Event&)  {}
	virtual void onDestroy(  Event&)  { clearEventQueue(); this->sendEvent(KillEvent(this->getID(), this->getID())); }
	virtual void onKill(     Event&)  { clearEventQueue(); }
	virtual void onRaise(    Event&)  {}
	virtual void onAction(   Event&)  {}
	virtual void onCommand(  Event& e){ this->call(e.getCommandInfo().name, e.getCommandInfo().value); }
	virtual void onSelect(   Event&)  {}
	virtual void onDisselect(Event&)  {}
	virtual void onButton(   Event&)  {}
	virtual void onMotion(   Event&)  {}
	virtual void onKey(      Event&)  {}

	virtual void clearEventQueue() {
	//	assert(!current_event);
		for(EventCont::iterator p=events.begin(); p!=events.end();) {
			if(&(*p)==current_event || p->getType()==EVT_KILL || p->getType()==EVT_RAISE)
				++p;
			else
				events.erase(p++);
		}
	}

private:
	EventCont events;
	Event *current_event;
};



template<class T>
class HaveID : public T
{
public:
	typedef HaveID BaseClass;

	HaveID() : id(CreateID()) {}
	gobj_id getID() const { return id; }

	void onCreate(Event& e)  {
		T::onCreate(e);
		Event::CreateInfo& info = e.getCreateInfo();
		id = info.id;
	}

private:
	gobj_id id;
};



template<class T>
class HaveDeathFlag : public T
{
public:
	typedef HaveDeathFlag BaseClass;

	HaveDeathFlag() : is_dead(false), is_added(false) {}
	virtual bool isDead() const { return is_dead; }
	virtual bool isAdded() const{ return is_added; }
	virtual void kill()   { is_dead=true; }
	virtual void raise()  { is_dead=false; }

	virtual void onAdded()   { is_added=true; }
	virtual void onRemoved() { is_added=false; }

	virtual void onDestroy(Event& e) {
		this->T::onDestroy(e);
	}
	virtual void onKill(Event& e) {
		kill();
		this->T::onKill(e);
	}
	virtual void onRaise(Event& e) {
		this->T::onRaise(e);
		raise();
	}

private:
	bool is_dead;
	bool is_added;
};


template<class T>
class HaveFrameCounter : public T
{
public:
	typedef HaveFrameCounter BaseClass;

	HaveFrameCounter() : frame_count(0) {}
	int getFrame() const       { return frame_count; }
	void setFrame(int f) { frame_count=f; }

	void onUpdate(Event& e) {
		this->T::onUpdate(e);
		if(frame_count==0)
			onZeroFrame();
		++frame_count;
	}
	void raise() {
		this->T::raise();
		frame_count = 0;
	}

	virtual void onZeroFrame() {}

private:
	int frame_count;
};



template <class T>
class HavePointCollisionModel : public T
{
public:
	typedef HavePointCollisionModel BaseClass;

	float getSelectionRadius() const { return selection_model.getRadius(); }
	float getHitRadius() const       { return hit_model.getRadius(); }
	void setSelectionRadius(float r) { selection_model.setRadius(r); }
	void setHitRadius(float r)       { hit_model.setRadius(r); }
	virtual void setPos(const ist::vector4& v) {
		this->T::setPos(v);
		hit_model.setPos(this->getPos());
		selection_model.setPos(this->getPos());
	}

	const ist::ICollisionModel& getHitModel() const { return hit_model; }
	const ist::ICollisionModel& getSelectionModel() const { return selection_model; }

	bool query(const std::string& name, boost::any& value) const {
		if     (name=="getSelectionRadius")value = getSelectionRadius();
		else if(name=="getHitRadius")      value = getHitRadius();
		else return this->T::query(name, value);
		return true;
	}

protected:
	ist::SimplePointCollisionModel hit_model;
	ist::SimplePointCollisionModel selection_model;
};



template <class T>
class HaveCharacterGridInserter : public T
{
public:
	typedef HaveCharacterGridInserter BaseClass;

	HaveCharacterGridInserter() : is_hidden(false), is_in_character_grid(false) {}

	virtual void setHide(bool h=true) { is_hidden=h; }
	virtual bool isHidden() const { return is_hidden; }
	virtual bool isInCharacterGrid() const { return is_in_character_grid; }

	virtual void onAdded()   {
		T::onAdded();
		this->_insertToCharacterGrid();
		is_in_character_grid = true;
	}

	virtual void onKill(Event& e) {
		EraseFromCharacterGrid(this->getPos(), this);
		this->T::onKill(e);
	}

	virtual void setPos(const ist::vector4& v) {
		if(this->isAdded() && !this->isDead()) {
			ist::vector4 before = this->getPos();
			selectable_ptr s;
			const ist::IPointCollisionModel *pcm = 0;

			if(this->isBuilding() || this->isMaterial()) {
				s = ToSelectable(this);
				const ist::ICollisionModel& cm = s->getHitModel();
				if(cm.getType()==ist::CM_POINT) {
					pcm = &static_cast<const ist::IPointCollisionModel&>(cm);
				}
			}

			if(!!s && !!pcm)
				RemoveCircleCollision(s, pcm->getPos(), pcm->getRadius());

			this->T::setPos(v);

			MoveCharacterGrid(before, this->getPos(), this);
			if(!!s && !!pcm)
				AddCircleCollision(s, pcm->getPos(), pcm->getRadius());
		}
		else {
			this->T::setPos(v);
		}
	}

	virtual bool query(const std::string& name, boost::any& value) const {
		if(name=="isHidden")               value = isHidden();
		else if(name=="isInCharacterGrid") value = isInCharacterGrid();
		else return this->T::query(name, value);
		return true;
	}


private:
	void _eraseFromCharacterGrid() {
		EraseFromCharacterGrid(this->getPos(), this);
	}

	void _insertToCharacterGrid() {
		if(!this->isHidden())
			InsertToCharacterGrid(this->getPos(), this);
	}

	bool is_hidden;
	bool is_in_character_grid;
};

template<class T>
class HaveCollideChecker : public T
{
public:
	typedef HaveCollideChecker BaseClass;

	virtual bool conditionCollide(selectable_ptr p) const {
		return true;
	}

	virtual bool testCollide(gobj_ptr p) {
		if(p.get()==this)
			return false;
		selectable_ptr sp = ToSelectable(p);
		if(!sp || !conditionCollide(sp))
			return false;

		ist::SimplePointCollisionModel pcm;
		const ist::ICollisionModel *cm = 0;
		if(sp->isMaterial() || sp->isBuilding()) {
			pcm.setPos(this->getPos());
			cm = &pcm;
		}
		else {
			cm = &this->getHitModel();
		}

		ist::CollisionDetector cd;
		if(cd.detect(sp->getHitModel(), *cm)) {
			SendCollideEvent(p, this, new Event::CollideInfo(cd.collide_pos, cd.collide_normal, cd.distance));
			return true;
		}
		return false;
	}

	virtual int sendCollideEvent() {
		IGObjIterator& block = GetBlocksByRadius(this->getPos(), 100);
		int num_collide = 0;
		while(block.hasNext()) {
			if(testCollide(block.iterate()))
				++num_collide;
		}
		return num_collide;
	}
};


template<class T>
class HaveSelectChecker : public T
{
public:
	typedef HaveSelectChecker BaseClass;

	HaveSelectChecker() : is_selected(false) {}
	bool isSelected() const { return is_selected; }

	virtual bool isMouseOver() const {
		ist::SimpleLineCollisionModel lcm;
		lcm.setHead( GetCamera().position+GetScreenRay()*GetCamera().z_far );
		lcm.setTail( GetCamera().position );
		ist::CollisionDetector cd;
		if(cd.detect(this->getSelectionModel(), lcm))
			return true;
		return false;
	}

	void onSelect(Event& e) {
		is_selected = true;
		T::onSelect(e);
	}

	void onDisselect(Event& e) {
		is_selected = false;
		T::onDisselect(e);
	}

private:
	bool is_selected;
};


template<class T>
class HaveActionQueue : public T
{
public:
	typedef HaveActionQueue BaseClass;
	typedef std::deque<Event::actioninfo_ptr> ActionQueue;


	void checkMoveRegion(Event::actioninfo_ptr a) const {
		const ist::vector2& mmin = GetMapMin();
		const ist::vector2& mmax = GetMapMax();
		ist::vector4& dst = a->target_pos;
		dst.x = std::max(mmin.x, std::min(mmax.x-1.0f, dst.x));
		dst.z = std::max(mmin.y, std::min(mmax.y-1.0f, dst.z));
	}
	virtual void pushbackAction(Event::actioninfo_ptr a)  {
		checkMoveRegion(a);
		action_queue.push_back(a);
	}
	virtual void pushfrontAction(Event::actioninfo_ptr a) {
		checkMoveRegion(a);
		action_queue.push_front(a);
	}
	virtual void popAction() {
		if(!action_queue.empty())
			action_queue.pop_front();
	}
	virtual void clearAction() { action_queue.clear(); }
	virtual size_t sizeAction() const { return action_queue.size(); }
	virtual Event::actioninfo_ptr getAction(size_t i) const {
		if(i>=action_queue.size())
			return Event::actioninfo_ptr();
		return action_queue[i];
	}
	virtual Event::actioninfo_ptr getCurrentAction() const {
		if(action_queue.empty())
			return Event::actioninfo_ptr();
		return action_queue.front();
	}

	virtual void move(Event::ActionInfo&) { popAction(); }
	virtual void action(Event::ActionInfo&) { popAction(); }
	virtual void idle() {}

	virtual void onChangeCurrentAction(Event::actioninfo_ptr) {}

	virtual void processActionQueue() {
		if(!action_queue.empty()) {
			Event::actioninfo_ptr info = action_queue.front();
			if(current_action!=info) {
				current_action = info;
				onChangeCurrentAction(current_action);
			}
			if(!!info->getObj())
				action(*info);
			else
				move(*info);
		}
		else
			idle();

		if(action_queue.empty() && !!current_action) {
			current_action = Event::actioninfo_ptr();
		}
	}

	void onUpdate(Event& e) {
		this->T::onUpdate(e);
		processActionQueue();
	}

	void onAction(Event& e) {
		this->T::onAction(e);
		Event::ActionInfo& info = e.getActionInfo();
		if(info.flagAppend()) {
			pushbackAction(boost::static_pointer_cast<Event::ActionInfo>(e.info));
		}
		else if(info.flagAppendF()) {
			pushfrontAction(boost::static_pointer_cast<Event::ActionInfo>(e.info));
		}
		else {
			clearAction();
			pushbackAction(boost::static_pointer_cast<Event::ActionInfo>(e.info));
		}
	}

	void onKill(Event& e) {
		clearAction();
		this->T::onKill(e);
	}

	bool call(const std::string& name, boost::any& value) {
		if     (name=="clearAction") clearAction();
		else if(name=="popAction")   popAction();
		else return this->T::call(name, value);
		return true;
	}

private:
	Event::actioninfo_ptr current_action;
	ActionQueue action_queue;
};


template<class T>
class HaveAttackAction : public T
{
public:
	typedef HaveAttackAction BaseClass;

	HaveAttackAction() :
		cool_down(0)
	{}

	virtual int getCoolDown() const { return cool_down; }
	virtual void setCoolDown(int cd) { cool_down=cd; }
	virtual void fire(selectable_ptr p) {}

	virtual void turnToTarget(const ist::vector4& _target) {
		ist::vector4 target = ist::vector4(_target.x, 0.0f, _target.z);
		target.normalize();
		float ry = acos(ist::vector4(0.0f, 0.0f, 1.0f).dot(target));
		if(target.x < 0)
			ry*=-1.0f;
		float add = fmod((ry/ist::radian-this->getRot().y+540.0f), 360.0f)-180.0f;
		float y = fmod(this->getRot().y+(add/7.0f), 360.0f);
		if(y!=y)
			y = 0.0f;

		this->setRot(ist::vector4(0.0f, y, 0.0f));
	}

	virtual bool conditionFire() const {
		return this->getCoolDown()<0;
	}

	virtual void idle() {
		if(this->getFrame()%90==0)
			this->pollTarget();
		if(this->getFrame()%120==0)
			this->pollLink();
	}

	virtual void action(Event::ActionInfo& info) {
		selectable_ptr& s = info.getObj();
		if(!s || s->isDead() || s->getID()==this->getID()) {
			this->popAction();
			if(!this->getCurrentAction())
				this->pollTarget();
			return;
		}

		if(!IsEnemy(this, s)) {
			if(!info.flagPath())
				info.target_pos = s->getPos();
			this->move(info);
		}
		else {
			attackAction(info);
		}
	}

	virtual void attackAction(Event::ActionInfo& info) {
		breakable_ptr p = ToBreakable(info.getObj());
		if(!p)
			return;

		float distace_from_target = (p->getPos()-this->getPos()).setY(0).norm();
		if(distace_from_target < this->getFireRange()) {
			this->turnToTarget(p->getPos()-this->getPos());
			if(conditionFire())
				this->fire(p);
		}
		else if(distace_from_target < this->getSight() || info.flagInsist()) {
			if(!info.flagPath())
				info.target_pos = p->getPos();
			this->move(info);
		}
		else {
			this->popAction();
		}
	}


	virtual bool conditionLink(unit_ptr p) const {
		if(!IsAlly(this->getFaction(), p->getFaction()))
			return false;

		if(Event::actioninfo_ptr l = p->getCurrentAction()) {
			if(selectable_ptr& s = l->getObj()) {
				if(!s->isDead() && IsEnemy(this, s) && (s->getPos()-p->getPos()).setY(0).norm() < p->getFireRange())
					return true;
			}
		}
		return false;
	}

	// t߂ɂUsĂ閡Tĉ
	virtual void pollLink() {
		Event::actioninfo_ptr linked;
		float ally_min_dist = 0;

		IGObjIterator& block = GetBlocksByRadius(this->getPos(), this->getSearchRange());
		while(block.hasNext()) {
			unit_ptr p = ToUnit(block.iterate());
			if(!p)
				continue;
			float dist = (p->getPos()-this->getPos()).setY(0).norm();
			if(dist<this->getSearchRange() && (dist<ally_min_dist || !linked) && conditionLink(p)) {
				linked = p->getCurrentAction();
				ally_min_dist = dist;
			}
		}
		if(!!linked) {
			SendActionEvent(this, this, new Event::ActionInfo(linked->target_pos, linked->getObj(),
				Event::ActionInfo::INSIST | Event::ActionInfo::LINK));
		}
	}

	// t߂ɂGTčUΏۂ
	virtual void pollTarget() {
		breakable_ptr target;
		float enemy_min_dist = 0;

		IGObjIterator& block = GetBlocksByRadius(this->getPos(), this->getSearchRange());
		while(block.hasNext()) {
			breakable_ptr p = ToBreakable(block.iterate());
			if(!p)
				continue;
			if(IsEnemy(this, p) && IsTargetable(p)) {
				float dist = (p->getPos()-this->getPos()).setY(0).norm();
				if(dist<this->getSearchRange() && (!target || dist<enemy_min_dist)) {
					target = p;
					enemy_min_dist = dist;
				}
			}
		}
		if(!!target)
			SendActionEvent(this, this, new Event::ActionInfo(ist::vector4(), target));
	}

	void onUpdate(Event& e) {
		this->T::onUpdate(e);
		--cool_down;
	}

	// ɂȂƂUH甽ɂ
	virtual void counter(selectable_ptr p) {
		if(this->sizeAction()==0 && IsEnemy(this, p) && IsTargetable(p))
			this->pushfrontAction(new Event::ActionInfo(ist::vector4(), p, Event::ActionInfo::INSIST));
	}

	void onDamage(Event& e) {
		this->T::onDamage(e);
		
		selectable_ptr p = ToSelectable(GetGObj(e.from));
		if(!!p)
			counter(p);
	}

	void raise() {
		this->T::raise();
		cool_down=0;
	}

private:
	int cool_down;
};


template<class T>
class HaveMoveAction : public T
{
public:
	typedef HaveMoveAction BaseClass;

	HaveMoveAction() : pre_dist(0), suspender(0) {}
	void pushfrontAction(Event::actioninfo_ptr a) {
		this->T::pushfrontAction(a);
		suspender = 0;
		pre_dist = 0;
	}
	void popAction() {
		this->T::popAction();
		suspender = 0;
		pre_dist = 0;
	}

	virtual float suspendOnCollide() const { return 1.0f; }
	virtual float suspendOnAway() const { return 2.0f; }
	virtual float getMoveSpeed() const { return 0; }

	void move(Event::ActionInfo& info) {
		const ist::vector4& target = info.target_pos;
		const ist::vector4& pos = this->getPos();
		ist::vector4 dist = (target-pos).setY(0);
		float distance_from_target = dist.norm();
		ist::vector4 aim = dist/distance_from_target;


		float speed = 0;
		if(info.flagRank())
			speed = info.speed;
		else
			speed = getMoveSpeed();

		if(distance_from_target<speed) {
			this->setPos(target);
			this->popAction();
		}
		else {
			this->setPos(pos+aim*speed);
			this->turnToTarget(aim);

			if(info.flagRank() && distance_from_target<20.0f) {
				this->popAction();
			}
		}

		float distance_from_target_obj = 0;
		selectable_ptr& s = info.getObj();
		if(s)
			distance_from_target_obj = (s->getPos()-pos).norm();

		if(suspender>40.0f)
			this->popAction(); // 荞 
		else if(suspender>=5.0f && !!s && distance_from_target_obj<this->getFireRange()+50.0f
			&& (s->isBuilding() || s->isMaterial() || IsEnemy(this, s))) {
			this->pushfrontAction(new Event::ActionInfo(
				this->getPos()+(ist::vector4().cross(aim, ist::vector4(0.0, 1.0f, 0.0))-aim/4.0f)*30.0f, selectable_ptr(),
					Event::ActionInfo::PATH));
		}

		if(this->sendCollideEvent()!=0) // ܂sLZ 
			suspender+=suspendOnCollide();
		if(distance_from_target>pre_dist)
			suspender+=suspendOnAway();
		suspender = std::max<float>(0.0f, suspender-1.0f);
		pre_dist = distance_from_target;

		if(info.flagActive() && this->getFrame()%30==0)
			this->pollTarget();
	}

	virtual void onCollideTarget(selectable_ptr) {
		this->popAction();
	}
	void onCollide(Event& e) {
		this->T::onCollide(e);
		if(!!this->getCurrentAction()) {
			selectable_ptr& p = this->getCurrentAction()->getObj();
			selectable_ptr q = ToSelectable(GetGObj(e.from));
			if(!!p && !!q && p==q)
				onCollideTarget(p);
		}
	}

private:
	float pre_dist;
	float suspender;
};




template<class T>
class BelongFaction : public T
{
public:
	typedef BelongFaction BaseClass;

	BelongFaction() {}

	virtual faction_ptr getFaction() const { assert(!!faction); return faction; }
	virtual void setFaction(faction_ptr f) { faction=f; }

	virtual bool isInSight(faction_ptr f) const {
		if(f==getFaction())
			return true;
		else 
			return T::isInSight(f);
	}

	void onKill(Event& e) {
	//	faction = faction_ptr();	//̃IuWFNgQƂ邱Ƃ
		T::onKill(e);
	}

private:
	faction_ptr faction;
};












template <class T>
class HavePolygonModel : public T
{
public:
	typedef HavePolygonModel BaseClass;

	HavePolygonModel() : frame_texture(0) {}

	int getFrameTexture() const { return frame_texture; }
	texture_ptr getTexture() const { return texture; }
	pmodel_ptr getPModel() const { return pmodel; }

	void setFrameTexture(int ft) { frame_texture=ft; }
	void setTexture(const std::string& filename) { setTexture(GetTexture(filename)); }
	void setTexture(texture_ptr arg) { texture=arg; }
	void setPModel(const std::string& filename) { setPModel(GetPModel(filename)); }
	void setPModel(pmodel_ptr arg) { pmodel = arg; }

	virtual void setTexture() const {
		if(texture!=0) {
			glEnable(GL_TEXTURE_2D);
			texture->assign(frame_texture);
		}
	}

	virtual void setLight(ist::Light& light) const {
		glEnable(GL_LIGHTING);
		light.position.y = 10000.0f;
		light.position.x = 10000.0f;
		light.position.z = -10000.0f;
		light.enable();
	}

	bool isInCameraSight() const {
		if(!pmodel || pmodel->getNumChunc()==0)
			return false;
		const ist::Sphere& bsphere = pmodel->getChunc(0)->getBSphere();
		return GetCamera().view_frustum.test(this->getPos()+(bsphere.pos*this->getScl()), bsphere.radius*this->getScl().y)!=0;
	}

	float getLifeGageFloating() const {
		if(!!pmodel)
			return pmodel->getChunc(0)->getBSphere().radius;
		return 0;
	}

	void drawModel() const {
		if(!pmodel)
			return;
		setTexture();
		ist::Light light;
		setLight(light);
		{
			ist::MatrixSaver msaver(GL_MODELVIEW_MATRIX);
			ist::matrix44 mat;
			calcMatrix(mat);
			glMultMatrixf( mat.v );
			pmodel->draw();
		}
		glDisable(GL_LIGHTING);
		glDisable(GL_TEXTURE_2D);
	}

	void raise() {
		this->T::raise();
		frame_texture=0;
	}

private:
	texture_ptr texture;
	pmodel_ptr pmodel;
	int frame_texture;
};





template <class T>
class HaveAPolygonModel : public T
{
public:
	typedef HaveAPolygonModel BaseClass;

	HaveAPolygonModel() :
		frame_texture(0),
		frame_anim(0),
		play_speed(1.0f),
		is_animation_finished(false)
	{}

	texture_ptr getTexture() const { return texture; }
	amodel_ptr getAModel() const { return amodel; }
	int getFrameAnim() const { return frame_anim; }
	float getAnimPlayeSpeed() const { return play_speed; }
	const std::string& getClipName() const { return clip_name; }
	int getFrameTexture() const { return frame_texture; }

	void setTexture(const std::string& filename) { setTexture(GetTexture(filename)); }
	void setTexture(texture_ptr arg) { texture=arg; }
	void setAModel(const std::string& filename) { setAModel(GetAModel(filename)); }
	void setAModel(amodel_ptr arg) { amodel=arg; }
	void setFrameAnim(float f) { frame_anim=f; is_animation_finished = false; }
	void setAnimPlayeSpeed(float f) { play_speed=f; }

	void setClipName(const std::string& cn) {
		clip_name = cn;
		if(!!amodel) {
			setFrameAnim(amodel->beginFrame(clip_name));
		}
	}

	void setFrameTexture(int ft) { frame_texture=ft; }

	bool isAnimationFinished() const { return is_animation_finished; }
	void resetAnimation() {
		if(!!amodel)
			setFrameAnim(amodel->beginFrame(clip_name));
	}




	virtual void setLight(ist::Light& light) const {}

	bool isInCameraSight() const {
		if(!amodel)
			return false;
		ist::Sphere bsphere = amodel->getBSphere();
		return GetCamera().view_frustum.test(this->getPos()+(bsphere.pos*this->getScl()), bsphere.radius*this->getScl().y)!=0;
	}

	float getLifeGageFloating() const {
		if(!!amodel) {
			ist::Sphere bsphere = amodel->getBSphere();
			return bsphere.pos.y*this->getScl().y+bsphere.radius*this->getScl().y;
		}
		return 0;
	}

	void drawModel() const {
		if(!amodel)
			return;
		if(!!texture) {
			glEnable(GL_TEXTURE_2D);
			texture->assign(frame_texture);
		}
		ist::Light light;
		setLight(light);
		{
			ist::MatrixSaver msaver(GL_MODELVIEW_MATRIX);
			ist::matrix44 mat;
			calcMatrix(mat);
			glMultMatrixf( mat.v );
			amodel->draw(clip_name, frame_anim);
		}
		glDisable(GL_LIGHTING);
		glDisable(GL_TEXTURE_2D);
	}

	virtual void onAnimationFinished() {}
	void onUpdate(Event& e) {
		this->T::onUpdate(e);
		if(!!amodel) {
			frame_anim+=play_speed;
			if(frame_anim>amodel->endFrame(clip_name) && !is_animation_finished) {
				is_animation_finished = true;
				onAnimationFinished();
			}
		}
	}

	void raise() {
		this->T::raise();
		frame_texture=0;
		frame_anim=0;
		play_speed=1.0f;
	}

private:
	texture_ptr texture;
	amodel_ptr amodel;
	int frame_texture;
	float frame_anim;
	float play_speed;
	bool is_animation_finished;
	std::string clip_name;
};




template <class T>
class HaveCPolygonModel : public T
{
public:
	typedef HaveCPolygonModel BaseClass;

	HaveCPolygonModel() :
		frame_texture(0),
		frame_anim(0),
		is_animation_finished(false)
	{}

	texture_ptr getTexture() const { return texture; }
	cmodel_ptr getCModel() const { return cmodel; }
	int getFrameTexture() const { return frame_texture; }
	int getFrameAnim() const { return frame_anim; }
	const std::string& getClipName() const { return clip_name; }

	void setTexture(const std::string& filename) { setTexture(GetTexture(filename)); }
	void setTexture(texture_ptr arg) { texture=arg; }
	void setCModel(const std::string& filename) { setCModel(GetCModel(filename)); }
	void setCModel(cmodel_ptr arg) { cmodel=arg; }
	void setFrameTexture(int ft) { frame_texture=ft; }
	void setFrameAnim(int f) { frame_anim = f; is_animation_finished = false; }

	void setClipName(const std::string& cn) {
		clip_name = cn;
		if(!!cmodel) {
			setFrameAnim(cmodel->beginFrame(clip_name));
		}
	}

	bool isAnimationFinished() const { return is_animation_finished; }



	virtual void setLight(ist::Light& light) const {
		glEnable(GL_LIGHTING);
		light.position.y = 10000.0f;
		light.position.x = 10000.0f;
		light.position.z = -10000.0f;
		light.enable();
	}

	bool isInCameraSight() const {
		if(!cmodel)
			return false;
		const ist::Sphere& bsphere = cmodel->getBSphere();
		return GetCamera().view_frustum.test(this->getPos()+bsphere.pos*this->getScl().y, bsphere.radius*this->getScl().y)!=0;
	}

	float getLifeGageFloating() const {
		if(!!cmodel) {
			const ist::Sphere& bsphere = cmodel->getBSphere();
			return bsphere.pos.y*this->getScl().y+bsphere.radius*this->getScl().y;
		}
		return 0;
	}

	void drawModel() const {
		if(!cmodel)
			return;
		if(!!texture) {
			glEnable(GL_TEXTURE_2D);
			texture->assign(frame_texture);
		}
		ist::Light light;
		setLight(light);
		{
			ist::MatrixSaver msaver(GL_MODELVIEW_MATRIX);
			ist::matrix44 mat;
			calcMatrix(mat);
			glMultMatrixf( mat.v );
			cmodel->draw(clip_name, frame_anim);
		}
		glDisable(GL_LIGHTING);
		glDisable(GL_TEXTURE_2D);
	}

	virtual void onAnimationFinished() {}
	void onUpdate(Event& e) {
		this->T::onUpdate(e);
		if(!!cmodel && ++frame_anim>cmodel->endFrame(clip_name) && !is_animation_finished) {
			is_animation_finished = true;
			onAnimationFinished();
		}
	}

	virtual void raise() {
		this->T::raise();
		frame_texture=0;
		frame_anim=0;
	}

private:
	texture_ptr texture;
	cmodel_ptr cmodel;
	int frame_texture;
	int frame_anim;
	bool is_animation_finished;
	std::string clip_name;
};




template<class Iter>
inline void draw_polyline(Iter begin, Iter end, ist::vector4& camera, float width, ist::vector2 t1, ist::vector2 t2)
{
	size_t size = distance(begin, end);
	if(size<2) return;

	glBegin( GL_QUAD_STRIP );

	float tv = t2.y;
	for(Iter p=begin; p!=end; ++p) {
		ist::vector4 tangent;
		if(p==begin)      tangent = *(p+1)-*p;
		else if(p==end-1) tangent = *p-*(p-1);
		else              tangent = *(p+1)-*(p-1);
		tangent.normalize();

		ist::vector4 aim_camera= (camera-(*p)).normalize();
		ist::vector4 distance  = (ist::vector4().cross(tangent, aim_camera))*(width/2);

		ist::vector4 right= *p+distance;
		ist::vector4 left = *p-distance;

		glTexCoord2f(t1.x, tv);
		glVertex3fv( left.v );

		glTexCoord2f(t2.x, tv);
		glVertex3fv( right.v );

		tv-=(t2.y-t1.y)/(size-1);
	}

	glEnd();
}

template <class T>
class HavePolyLine : public T
{
public:
	typedef HavePolyLine BaseClass;

	HavePolyLine() :
		polyline_size(0), polyline_width(1), frame_texture(0) {}

	int getFrameTexture() const { return frame_texture; }
	texture_ptr getTexture() const { return texture; }

	void setFrameTexture(size_t ft) { frame_texture=ft; }
	void setPolylineSize(int s) { polyline_size=s; }
	void setPolylineWidth(float w) { polyline_width=w; }
	void setTexture(const std::string& filename) { setTexture(GetTexture(filename)); }
	void setTexture(texture_ptr arg) { texture=arg; }

	void setPos(const ist::vector4& v) {
		this->T::setPos(v);
		const ist::vector4& pos = this->getPos();
		if(pos_history.empty())
			pos_history.resize(polyline_size, pos);
		else {
			pos_history.push_back(pos);
			pos_history.pop_front();
		}

		ist::vector4 center = pos_history.back()+(pos_history.front()-pos_history.back())/2;
		bsphere.pos = center;
		bsphere.radius = (pos_history.back()-center).norm();
	}

	void drawModel() const {
		if(texture!=0) {
			glEnable(GL_TEXTURE_2D);
			texture->assign(frame_texture);
		}
		glBlendFunc(GL_SRC_ALPHA, GL_ONE);
		glDepthMask(GL_FALSE);
		draw_polyline(pos_history.begin(), pos_history.end(), GetCamera().position,
			polyline_width, ist::vector2(0.0f, 0.0f), ist::vector2(1.0f, 1.0f));
		glDepthMask(GL_TRUE);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glDisable(GL_TEXTURE_2D);
	}

	bool isInCameraSight() const {
		return GetCamera().view_frustum.test(bsphere.pos, bsphere.radius)!=0;
	}

	void raise() {
		this->T::raise();
		frame_texture=0;
		pos_history.clear();
	}

private:
	ist::Sphere bsphere;
	std::deque<ist::vector4> pos_history;
	int polyline_size;
	float polyline_width;
	texture_ptr texture;
	pmodel_ptr pmodel;
	int frame_texture;
};




template<class T>
class HaveShadow : public T
{
public:
	typedef HaveShadow BaseClass;

	HaveShadow() {
		shadow_model = GetPModel("resource/plane.mqo");
		shadow_texture = GetTexture("resource/shadow.png");
		shadow_scl = ist::vector4(10, 10, 10);
	}

	void setShadowScl(const ist::vector4& v) { shadow_scl=v; }

	void drawModel() const {
		this->T::drawModel();
		drawShadow();
	}

	virtual void drawShadow() const {
		glEnable(GL_TEXTURE_2D);
		glDepthMask(GL_FALSE);
		shadow_texture->assign();
		{
			ist::MatrixSaver msaver(GL_MODELVIEW_MATRIX);
			ist::matrix44 mat;
			mat.translate(this->getPos().x, GetHeight(this->getPos())+5.0f, this->getPos().z);
			mat.rotate(0, this->getRot().y, 0);
			mat.scale(shadow_scl.v);
			glMultMatrixf( mat.v );
			shadow_model->draw();
		}
		glDepthMask(GL_TRUE);
		glDisable(GL_TEXTURE_2D);
	}

protected:
	pmodel_ptr shadow_model;
	texture_ptr shadow_texture;
	ist::vector4 shadow_scl;
};





template<class T>
class HaveDrawFrontAttrib : public T
{
public:
	typedef HaveDrawFrontAttrib BaseClass;
	void draw() const {
		glDisable(GL_DEPTH_TEST);
		this->T::draw();
		glEnable(GL_DEPTH_TEST);
	}
};

template<class T>
class HaveNoDepthAttrib : public T
{
public:
	typedef HaveNoDepthAttrib BaseClass;
	void setLight(ist::Light&) const {}
	void draw() const {
		glDepthMask(GL_FALSE);
		this->T::draw();
		glDepthMask(GL_TRUE);
	}
};

template<class T>
class HaveBlendAddAttrib : public T
{
public:
	typedef HaveBlendAddAttrib BaseClass;
	void setLight(ist::Light&) const {}
	void draw() const {
/*
		glBlendEquationEXT(GL_FUNC_REVERSE_SUBTRACT_EXT);
		glActiveTextureARB(GL_TEXTURE0_ARB);
		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV, GL_COMBINE_RGB_ARB);
		glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB_ARB, GL_ADD_SIGNED_ARB);

		glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB_ARB, GL_PREVIOUS_ARB);
		glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB_ARB, GL_SRC_COLOR);

		glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_RGB_ARB, GL_TEXTURE);
		glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND1_RGB_ARB, GL_ONE_MINUS_SRC_COLOR);

		glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA_ARB, GL_REPLACE);
		glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_ALPHA_ARB, GL_PRIMARY_COLOR_ARB);
		glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_ALPHA_ARB, GL_SRC_ALPHA);
*/
		glBlendFunc(GL_SRC_ALPHA, GL_ONE);
		glDepthMask(GL_FALSE);
		this->T::draw();
		glDepthMask(GL_TRUE);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
//		glBlendEquationEXT(GL_FUNC_ADD_EXT);
	}
};

template<class T>
class BillboardAttrib : public T
{
public:
	typedef BillboardAttrib BaseClass;

	void calcMatrix(ist::matrix44& mat) const {
		mat.translate(this->getPos().v);
		mat*=GetAimCameraMatrix();
		mat.rotate(this->getRot().v);
		mat.scale(this->getScl().v);
	}
};






class FrameBuffer
{
private:
	unsigned int tex_name;
	int width, height;

public:

	FrameBuffer(int w, int h) :
		tex_name(0), width(w), height(h)
	{
		glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
		glGenTextures(1, &tex_name);
		glBindTexture(GL_TEXTURE_2D, tex_name);

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
	}

	~FrameBuffer() {
		clear();
	}

	void copy() {
		assign();
		glReadBuffer(GL_FRONT);
		glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 0,0, width, height, 0);
	}

	void assign() {
		glBindTexture(GL_TEXTURE_2D, tex_name);
	}
	
	void clear() {
		glDeleteTextures(1, &tex_name);
		tex_name=0;
	}
};




class SphereMap
{
public:
	SphereMap() : env_mode(GL_ADD) {}
	// env_mode : GL_MODULATE | GL_DECAL | GL_BLEND | GL_ADD

	void setTexture(const std::string& name) {
		tex=GetTexture(name);
	}

	void enable() {
		if(!tex)
			return;
		glEnable(GL_TEXTURE_2D);
		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
		glEnable(GL_TEXTURE_GEN_S);
		glEnable(GL_TEXTURE_GEN_T);
		tex->assign();
		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, env_mode);
	}

	void disable() {
		glDisable(GL_TEXTURE_2D);
		glDisable(GL_TEXTURE_GEN_S);
		glDisable(GL_TEXTURE_GEN_T);
	}

	int env_mode;
	texture_ptr tex;
};


/*
class BSphereMap
{
public:
	BSphereMap() : env_mode(GL_BLEND), gen_mode(GL_EYE_LINEAR) {}
	// env_mode : GL_MODULATE | GL_DECAL | GL_BLEND | GL_ADD
	// gen_mode : GL_OBJECT_LINEAR | GL_EYE_LINEAR | or GL_SPHERE_MAP

	virtual void assignTexture()=0;

	void enable() {
		glEnable(GL_TEXTURE_2D);
		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, gen_mode);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, gen_mode);
		glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, gen_mode);
		glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, gen_mode);
		glEnable(GL_TEXTURE_GEN_S);
		glEnable(GL_TEXTURE_GEN_T);
		glEnable(GL_TEXTURE_GEN_R);
		glEnable(GL_TEXTURE_GEN_Q);
		assignTexture();
		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, env_mode);
	}

	void disable() {
		glDisable(GL_TEXTURE_2D);
		glDisable(GL_TEXTURE_GEN_S);
		glDisable(GL_TEXTURE_GEN_T);
		glDisable(GL_TEXTURE_GEN_R);
		glDisable(GL_TEXTURE_GEN_Q);
	}

	int env_mode;
	int gen_mode;
};

class SphereMap : public BSphereMap
{
public:
	void assignTexture() {
		if(!tex)
			return;
		tex->assign();
	}

	texture_ptr tex;
};

class EnvSphereMap : public BSphereMap
{
public:
	EnvSphereMap() : fb(1024, 1024) {}

	void assignTexture() {
		ist::MatrixSaver msaver(GL_MODELVIEW_MATRIX);
		ist::MatrixSaver psaver(GL_PROJECTION_MATRIX);

		ist::PerspectiveCamera c = GetCamera();
		c.position-=c.target;
		c.target=ist::vector4();
		c.look();
		ist::matrix44 proj;
		ist::matrix44 view;
		glGetFloatv(GL_PROJECTION_MATRIX, proj.v);
		glGetFloatv(GL_MODELVIEW_MATRIX, view.v);
		static const ist::matrix44 bias(
			0.5f, 0.0f, 0.0f, 0.0f,
			0.0f, 0.5f, 0.0f, 0.0f,
			0.0f, 0.0f, 0.5f, 0.0f,
			0.5f, 0.5f, 0.5f, 1.0f);
		ist::matrix44 tex = proj*view;
		glTexGenfv(GL_S, GL_EYE_PLANE, tex.m[0]);
		glTexGenfv(GL_T, GL_EYE_PLANE, tex.m[1]);
		glTexGenfv(GL_R, GL_EYE_PLANE, tex.m[2]);
		glTexGenfv(GL_Q, GL_EYE_PLANE, tex.m[3]);

		
		fb.copy();
		fb.assign();
	}

	FrameBuffer fb;
};


*/









class PathConstructor
{
public:
	void construct(const ist::vector4& pos, Event::actioninfo_ptr a) {
		action.clear();

		IPathFinder& pf = GetPathFinder();
		pf.findPath(pos, a->target_pos, a->getObj());
		int flag = Event::ActionInfo::PATH | Event::ActionInfo::INSIST;
		flag|=a->getFlag();
		for(int i=1; i<int(pf.getPathSize())-1; ++i) {
			ist::vector4 &path = pf.getPath(i);
			path.setY(GetHeight(path));
			Event::actioninfo_ptr ap = new Event::ActionInfo(path, a->getObj(), flag);
			ap->speed = a->speed;
			action.push_back(ap);
		}
	}

	std::vector<Event::actioninfo_ptr> action;
};







inline void SendScopedDamage(gobj_id from, faction_ptr f, const ist::vector4& center, float radius, Event::damageinfo_ptr info) {
	ist::CollisionDetector cd;
	ist::SimplePointCollisionModel cm(center, radius);
	IGObjIterator& block = GetBlocksByRadius(center, radius);

	Event e(EVT_DAMAGE, from, 0, info);
	while(block.hasNext()) {
		breakable_ptr p = ToBreakable(block.iterate());
		if(!!p && IsEnemy(f, p->getFaction()) && cd.detect(p->getHitModel(), cm)) {
			e.to = p->getID();
			p->sendEvent(e);
		}
	}
}

inline void Artillery(gobj_id from, faction_ptr f, const ist::vector4& center, float radius, Event::damageinfo_ptr info) {
	ist::CollisionDetector cd;
	ist::SimplePointCollisionModel cm(center, radius);
	IGObjIterator& block = GetBlocksByRadius(center, radius);

	Event e(EVT_DAMAGE, from, 0, info);
	while(block.hasNext()) {
		breakable_ptr p = ToBreakable(block.iterate());
		if(!!p && IsEnemy(f, p->getFaction()) && IsTargetable(p) && cd.detect(p->getHitModel(), cm)) {
			e.to = p->getID();
			p->sendEvent(e);
		}
	}
}







/*


class IAction
{
public:
	virtual ~IAction() {}
	virtual void update(Event::actioninfo_ptr info)=0;
};


class BuildingAction : public IAction
{
public:
	BuildingAction(cbuilding_ptr _u) : u(_u) {}

	cbuilding_ptr u;
}

class UnitActionActive : public IAction
{
public:
	UnitActionActive(cunit_ptr _u) : u(_u) {}

	virtual void update(Event::actioninfo_ptr info) {
		if(info) {
			if(!!info->getObj())
				action(*info);
			else
				move(*info);
		}
		else
			idle();
	}

	virtual void idle() {
		if(u->getFrame()%90==0)
			this->pollTarget();
		if(u->getFrame()%120==0)
			this->pollLink();
	}

	virtual void action(Event::ActionInfo& info) {
		selectable_ptr& s = info.getObj();
		if(!s || s->isDead() || s->getID()==u->getID()) {
			u->popAction();
			if(!u->getCurrentAction() || !u->getCurrentAction()->getObj())
				this->pollTarget();
			return;
		}

		if(!IsEnemy(u, s)) {
			if(!info.flagPath())
				info.target_pos = s->getPos();
			this->move(info);
		}
		else {
			this->attackAction(info);
		}
	}


	void move(Event::ActionInfo& info) {
		const ist::vector4& target = info.target_pos;
		const ist::vector4& pos = u->getPos();
		ist::vector4 dist = (target-pos).setY(0);
		float distance_from_target = dist.norm();
		ist::vector4 aim = dist/distance_from_target;


		float speed = 0.0f;
		if(info.flagRank())
			speed = info.speed;
		else
			speed = u->getMoveSpeed();

		if(distance_from_target<speed) {
			u->setPos(target);
			u->popAction();
		}
		else {
			u->setPos(pos+aim*speed);
			u->turnToTarget(aim);

			if(info.flagRank() && distance_from_target<20.0f) {
				u->popAction();
			}
		}

		float distance_from_target_obj = 0.0f;
		selectable_ptr& s = info.getObj();
		if(s)
			distance_from_target_obj = (s->getPos()-pos).norm();

		if(suspender>40.0f)
			u->popAction(); // 荞 
		else if(suspender>=5.0f && !!s && distance_from_target_obj<u->getFireRange()+50.0f
			&& (s->isBuilding() || s->isMaterial() || IsEnemy(u, s))) {
			u->pushfrontAction(new Event::ActionInfo(
				u->getPos()+(ist::vector4().cross(aim, ist::vector4(0.0, 1.0f, 0.0))-aim/4.0f)*30.0f, selectable_ptr(),
					Event::ActionInfo::PATH));
		}

		if(this->sendCollideEvent()!=0) // ܂sLZ 
			suspender+=suspendOnCollide();
		if(distance_from_target>pre_dist)
			suspender+=suspendOnAway();
		suspender = std::max<float>(0.0f, suspender-1.0f);
		pre_dist = distance_from_target;

		if(info.flagActive() && u->getFrame()%30==0)
			this->pollTarget();
	}


	virtual bool conditionFire() const {
		return this->getCoolDown()<0;
	}

	virtual void attackAction(Event::ActionInfo& info) {
		breakable_ptr p = ToBreakable(info.getObj());
		if(!p)
			return;

		float distace_from_target = (p->getPos()-this->getPos()).setY(0).norm();
		if(distace_from_target < this->getFireRange()) {
			this->turnToTarget(p->getPos()-this->getPos());
			if(conditionFire())
				this->fire(p);
		}
		else if(distace_from_target < this->getSight() || info.flagInsist()) {
			if(!info.flagPath())
				info.target_pos = p->getPos();
			this->move(info);
		}
		else {
			this->popAction();
		}
	}

	virtual bool conditionLink(unit_ptr p) const {
		if(!IsAlly(u->getFaction(), p->getFaction()))
			return false;

		if(Event::actioninfo_ptr l = p->getCurrentAction()) {
			if(selectable_ptr& s = l->getObj()) {
				if(!s->isDead() && IsEnemy(u, s) && (s->getPos()-p->getPos()).setY(0).norm() < p->getFireRange())
					return true;
			}
		}
		return false;
	}

	// t߂ɂUsĂ閡Tĉ 
	virtual void pollLink() {
		Event::actioninfo_ptr linked;
		float ally_min_dist = 0;

		IGObjIterator& block = GetBlocksByRadius(u->getPos(), u->getSearchRange());
		while(block.hasNext()) {
			unit_ptr p = ToUnit(block.iterate());
			if(!p)
				continue;
			float dist = (p->getPos()-u->getPos()).setY(0).norm();
			if(dist<this->getSearchRange() && (dist<ally_min_dist || !linked) && conditionLink(p)) {
				linked = p->getCurrentAction();
				ally_min_dist = dist;
			}
		}
		if(!!linked) {
			SendActionEvent(u, u, new Event::ActionInfo(linked->target_pos, linked->getObj(),
				Event::ActionInfo::INSIST | Event::ActionInfo::LINK));
		}
	}

	// t߂ɂGTčUΏۂ
	virtual void pollTarget() {
		breakable_ptr target;
		float enemy_min_dist = 0;

		IGObjIterator& block = GetBlocksByRadius(this->getPos(), this->getSearchRange());
		while(block.hasNext()) {
			breakable_ptr p = ToBreakable(block.iterate());
			if(!p)
				continue;
			if(IsEnemy(this, p) && IsTargetable(p)) {
				float dist = (p->getPos()-this->getPos()).setY(0).norm();
				if(dist<this->getSearchRange() && (!target || dist<enemy_min_dist)) {
					target = p;
					enemy_min_dist = dist;
				}
			}
		}
		if(!!target)
			SendActionEvent(this, this, new Event::ActionInfo(ist::vector4(), target));
	}

	// ɂȂƂUH甽ɂ
	virtual void counter(selectable_ptr p) {
		if(this->sizeAction()==0 && IsEnemy(this, p) && IsTargetable(p))
			this->pushfrontAction(new Event::ActionInfo(ist::vector4(), p, Event::ActionInfo::INSIST));
	}


	cunit_ptr u;
};

*/




#endif

