#ifndef MENU_INCLUDE
#define MENU_INCLUDE

#include <fstream>
#include "gobj_parts.h"




class IDrawer
{
public:
	virtual ~IDrawer() {}
	virtual void draw() const=0;
};
typedef boost::scoped_ptr<IDrawer> draw_ptr;






typedef LinearInherit4(HaveDeathFlag, HaveID, HaveEventQueue, IMenu) _MenuRoot;

class Menu : public _MenuRoot
{
public:
	typedef _MenuRoot RootClass;
	typedef Menu BaseClass;

	virtual bool isInSight(faction_ptr) const { return true;}
	void draw() const {}
	bool isMouseOver() const { return false; }
	int getDrawPriority() const { return 10000; }
	category_t getRecieveEvents() const { return RCV_UPDATE; }
	const std::string& getDescription() const {
		static std::string dummy;
		return dummy;
	}
};





/*

class class Widget;
typedef boost::intrusive_ptr<Widget> widget_ptr;

class Widget : public Menu
{
public:
	category_t getRecieveEvents() {
		return (RCV_UPDATE | RCV_BUTTON | RCV_MOTION | RCV_KEY);
	}

	virtual const ist::vector2& getPos()  { return pos; }
	virtual const ist::vector2& getRect() { return rect; }
	virtual widget_ptr getRoot()          { return root; }
	virtual void setPos(const ist::vector2& a)  { pos=a; }
	virtual void setRect(const ist::vector2& a) { rect=a; }
	virtual void setRoot(widget_ptr r)          { root=r; }

	bool isMouseOver() {
		const ist::vector2& mouse = GetMousePointerPos2D();
		if(mouse.x<pos.x+rect.x && mouse.x>pos.x && mouse.y<pos.y+rect.y && mouse.y>pos.y)
			return true;
		return false;
	}

private:
	widget_ptr root;
	ist::vector2 pos;
	ist::vector2 rect;
};


class Window : public Widget
{
public:
	typedef std::vector<widget_ptr> widget_cont;

	~Window() {
		for(widget_cont::iterator p=widgets.begin(); p!=widgets.end(); ++p)
			SendKillEvent(this, *p);
		widgets.clear();
	}
	virtual void addWidget(widget_ptr w) {
		w->setRoot(this);
		widget_cont.push_back(w);
	}
	virtual void setTexture(const std::string& filename) { setTexture(GetTexture(filename)); }
	virtual void setTexture(texture_ptr a) { texture=a; }

	void draw() const {
		if(!!texture_over) {
			glEnable(GL_TEXTURE_2D);
			texture->assign();
		}
		else {
			color.assign();
		}
		DrawRect(pos, pos+rect);
		glDisable(GL_TEXTURE_2D);
		ist::gl::Color().assign();
	}

private:
	widget_cont widgets;
	ist::gl::Color color;
	texture_ptr texture;
};


class StaticString : public Widget
{
public:
	void draw() const {
		ist::Color().assign();
		DrawFont(2, 317, message->c_str());
	}

private:
	std::string str;
};
*/





class Button : public Menu
{
public:
	Button() : hotkey(0), disabled(false) {
		setRect(ist::vector2(25, 25));
	}

	int getDrawPriority() const { return 20000; }
	category_t getRecieveEvents() const {
		return (RCV_UPDATE | RCV_BUTTON | RCV_MOTION | RCV_KEY);
	}

	void setHotKey(int k) { hotkey=k; }
	virtual const ist::vector2& getPos() const  { return pos; }
	virtual const ist::vector2& getRect() const { return rect; }
	virtual void setPos(const ist::vector2& a)  { pos=a; }
	virtual void setRect(const ist::vector2& a) { rect=a; }
	virtual void setTexture(const std::string& filename) { setTexture(GetTexture(filename)); }
	virtual void setTexture(texture_ptr a) { texture=a; }
	virtual void setTextureOver(const std::string& filename) { setTextureOver(GetTexture(filename)); }
	virtual void setTextureOver(texture_ptr a) { texture_over=a; }
	virtual void setColor(const ist::Color& c) { color=c; }
	virtual void setColorOver(const ist::Color& c) { color_over=c; }

	void setDescription(const std::string& d) { description=d; }
	const std::string& getDescription() const { return description; }

	void draw() const {
		if(!!texture) {
			if(!!texture_over && isMouseOver()) {
				glEnable(GL_TEXTURE_2D);
				texture_over->assign();
			}
			else {
				glEnable(GL_TEXTURE_2D);
				texture->assign();
			}
		}
		else {
			if(isMouseOver())
				color_over.assign();
			else
				color.assign();
		}
		DrawRect(pos, pos+rect);
		glDisable(GL_TEXTURE_2D);
		ist::Color().assign();
	}

	bool isMouseOver() const {
		const ist::vector2& mouse = GetMousePointerPos2D();
		if(mouse.x<pos.x+rect.x && mouse.x>pos.x && mouse.y<pos.y+rect.y && mouse.y>pos.y)
			return true;
		return false;
	}

	virtual void action() {}
	void onButton(Event& e) {
		Event::ButtonInfo& info = e.getButtonInfo();
		if(info.button==1 && info.isButtonUp() && isMouseOver())
			action();
	}

	void onKey(Event& e) {
		Event::KeyInfo& info = e.getKeyInfo();
		if(info.isKeyUp() && info.key==hotkey)
			action();
	}

	void disable() { disabled=true; }
	bool isDisabled() const { return disabled; }

private:
	std::string description;
	texture_ptr texture, texture_over;
	ist::Color color, color_over;

	ist::vector2 pos;
	ist::vector2 rect;
	int hotkey;
	bool disabled;
};
typedef boost::intrusive_ptr<Button> button_ptr;


class ButtonSet : public Menu
{
public:
	typedef std::vector<button_ptr> button_cont;

	void addButton(button_ptr b) {
		AddGObj(b);
		buttons.push_back(b);
	}

	void onKill(Event& e) {
		BaseClass::onKill(e);
		for(size_t i=0; i<buttons.size(); ++i)
			SendKillEvent(this, buttons[i]);
		buttons.clear();
	}

	button_cont buttons;
};


class CommandButtonSet : public Menu
{
public:
	typedef std::vector<button_ptr> button_cont;
	typedef std::vector<std::list<button_ptr> > back_cont;

	class AppendInfo
	{
	public:
		AppendInfo(int _x, int _y, button_ptr _b) : x(_x), y(_y), b(_b) {}
		int x;
		int y;
		button_ptr b;
	};


	CommandButtonSet() {
		buttons.resize(maxRow()*maxCol());
		backs.resize(maxRow()*maxCol());
	}


	void setButton(size_t x, size_t y, button_ptr b) {
		if(x>=maxRow() || y>=maxCol())
			return;

		if(!!b) {
			if(isAdded() && !b->isAdded())
				AddGObj(b);

			b->setPos(ist::vector2(75.0f+24.0f*x, 397.0f+24.0f*y));
			b->setRect(ist::vector2(23, 23));
		}

		button_ptr& cel = buttons[maxRow()*y+x];
		if(!!cel && cel!=b)
			SendKillEvent(this, cel);
		cel = b;
	}
	void appendButton(size_t x, size_t y,  button_ptr b) {
		size_t i = maxRow()*y+x;
		if(!buttons[i])
			setButton(x, y, b);
		else
			backs[i].push_back(b);
	}

	size_t maxRow() const { return 7; }
	size_t maxCol() const { return 3; }

	void raise() {
		BaseClass::raise();

		Event e(EVT_RAISE, getID(), 0);
		for(size_t i=0; i<buttons.size(); ++i) {
			if(!!buttons[i]) {
				buttons[i]->sendEvent(e);
				buttons[i]->update();
			}
		}
	}

	void onAdded() {
		BaseClass::onAdded();
		for(size_t i=0; i<buttons.size(); ++i) {
			if(!!buttons[i] && !GetGObj(buttons[i]->getID()))
				AddGObj(buttons[i]);
		}
	}

	void onKill(Event& e) {
		for(size_t i=0; i<buttons.size(); ++i) {
			if(!!buttons[i])
				SendKillEvent(this, buttons[i]);
		}
		BaseClass::onKill(e);
	}

	void onCommand(Event& e) {
		Event::CommandInfo& info = e.getCommandInfo();
		if(info.name=="setButton") {
			if(info.value.type()==typeid(AppendInfo)) {
				const AppendInfo& ai = *boost::any_cast<AppendInfo>(&info.value);
				setButton(ai.x, ai.y, ai.b);
			}
			else if(info.value.type()==typeid(button_ptr)) {
				for(size_t i=0; i<buttons.size(); ++i) {
					if(!buttons[i]) {
						setButton(0, 0, boost::any_cast<button_ptr>(info.value));
						break;
					}
				}
			}
		}
		else if(info.name=="setSender") {
			for(size_t i=0; i<buttons.size(); ++i) {
				if(!!buttons[i])
					SendCommandEvent(this, buttons[i], &info);
				if(!backs[i].empty())
					SendCommandEvent(this, backs[i].front(), &info);
			}
		}
		else Menu::onCommand(e);
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);

		for(size_t i=0; i<buttons.size(); ++i) {
			if(!!buttons[i] && buttons[i]->isDisabled()) {
				setButton(i%maxRow(), i/maxRow(), button_ptr());
			}
			if(!buttons[i] && !backs[i].empty()) {
				setButton(i%maxRow(), i/maxRow(), backs[i].front());
				backs[i].pop_front();
			}
		}
	}

	button_cont buttons;
	back_cont backs;
};


class CommandButton : public Button
{
public:
	CommandButton() {}

	void setCommand(Event::commandinfo_ptr ci) {
		command = ci;
	}

	void action() {
		assert(!!command);
		SendCommandEvent(getSender(), GetPlayerFaction(), command);
	}

	void setSender(gobj_ptr s) { sender = s; }
	gobj_ptr getSender() const { return sender; }

	void onCommand(Event& e) {
		Event::CommandInfo& info = e.getCommandInfo();
		if(info.name=="setSender")
			setSender(ToGObj(info.value));
		else Button::onCommand(e);
	}
	void onKill(Event& e) {
		sender = gobj_ptr();
		Button::onKill(e);
	}

	gobj_ptr sender;
	Event::commandinfo_ptr command;
};


class UnitCommandButton : public CommandButton
{
public:
	void action() {
		assert(!!command);
		IGObjIterator& gi = GetPlayerFaction()->getSelection();
		while(gi.hasNext()) {
			selectable_ptr s = ToSelectable(gi.iterate());
			if(!!s && IsAlly(s->getFaction(), GetPlayerFaction()))
				RouteCommandEvent(getSender(), s, command);
		}
	}
};











class SelectionMenu : public Menu
{
public:
	class ClassImage : public IDrawer
	{
	public:
		ClassImage(selectable_ptr r) : root(r) {}

		void draw() const {
			if(!!root && !!root->getUnitImage()) {
				glEnable(GL_TEXTURE_2D);
				root->getUnitImage()->assign();
				DrawRect(ist::vector2(265, 415), ist::vector2(305, 455));
				glDisable(GL_TEXTURE_2D);
			}
		}

	private:
		selectable_ptr root;
	};

	class LifeGage : public IDrawer
	{
	public:
		LifeGage(selectable_ptr r) : root(r) {}

		void draw() const {
			breakable_ptr p = ToBreakable(root);
			if(!!p) {
				ist::Color(0, 0, 0, 1).assign();
				DrawRect(ist::vector2(265, 456), ist::vector2(305, 459));
				p->getFaction()->getFactionColor().assign();
				DrawRect(ist::vector2(265, 456), ist::vector2(265+40.0f*(p->getLife()/p->getMaxLife()), 459));
				ist::Color().assign();

				char buf[32];
				sprintf(buf, "%.0f/%.0f", p->getLife(), p->getMaxLife());
				DrawFont(265, 465, buf);
			}
		}

	private:
		selectable_ptr root;
	};

	class EnergyGage : public IDrawer
	{
	public:
		EnergyGage(selectable_ptr r) : root(r) {}

		void draw() const {
			boost::any t1, t2;
			if(!root || !root->query("getEnergy", t1) || !root->query("getMaxEnergy", t2))
				return;
			float energy = ToFloat(t1);
			float max_energy = ToFloat(t2);
			if(max_energy==0)
				return;

			ist::Color(0.0f, 0.0f, 0.0f, 1.0f).assign();
			DrawRect(ist::vector2(265, 460), ist::vector2(305, 463));

			ist::Color col = root->getFaction()->getFactionColor();
			col.color+=ist::vector4(0.2f, 0.2f, 0.2f);
			col.assign();
			DrawRect(ist::vector2(265, 460), ist::vector2(265+40.0f*(energy/max_energy), 463));
			ist::Color().assign();

		//	char buf[32];
		//	sprintf(buf, "%.0f/%.0f", p->getLife(), p->getMaxLife());
		//	DrawFont(265, 465, buf);
		}

	private:
		selectable_ptr root;
	};

	class NameDisplayer : public IDrawer
	{
	public:
		NameDisplayer(selectable_ptr r) : root(r) {}

		void draw() const {
			DrawFont(265, 400, root->getUnitName().c_str());
			DrawFont(235, 375, root->getFaction()->getPlayerName().c_str());
			DrawFont(340, 375, root->getFaction()->getFactionName().c_str());
			if(root->getFaction()==GetPlayerFaction()) {
			}
			else if(IsAlly(root->getFaction(), GetPlayerFaction())) {
				ist::Color(0.0f, 0.0f, 1.0f, 1.0f).assign();
				DrawFont(300, 375, "");
				ist::Color().assign();
			}
			else {
				ist::Color(1.0f, 0.0f, 0.0f, 1.0f).assign();
				DrawFont(300, 375, "G");
				ist::Color().assign();
			}
		}

	private:
		selectable_ptr root;
	};

	class ParameterDisplayer : public IDrawer
	{
	public:
		ParameterDisplayer(selectable_ptr r) : root(r) {}

		void draw() const {
			breakable_ptr p = ToBreakable(root);
			if(!!p) {
				char buf[32];

				sprintf(buf, "U: %.1f", p->getFirePower());
				DrawFont(325, 417, buf);

				sprintf(buf, "˒: %.0f", p->getFireRange()/10.0f);
				DrawFont(325, 433, buf);

				sprintf(buf, "he: %.0f%%", p->getAntiBullet()*100.0f);
				DrawFont(325, 449, buf);

				sprintf(buf, "h: %.0f%%", p->getAntiLaser()*100.0f);
				DrawFont(325, 465, buf);
			}
		}

	private:
		selectable_ptr root;
	};




	SelectionMenu(selectable_ptr root) {
		go = root;
		name_displayer.reset(new NameDisplayer(root));
		class_image.reset(new ClassImage(root));
		life_gage.reset(new LifeGage(root));
		en_gage.reset(new EnergyGage(root));
		param.reset(new ParameterDisplayer(root));
	}

	void draw() const {
		name_displayer->draw();
		class_image->draw();
		life_gage->draw();
		en_gage->draw();
		param->draw();
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		if(go->isDead())
			SendKillEvent(this, this);
	}

	void onKill(Event& e) {
		go = selectable_ptr();
		BaseClass::onKill(e);
	}

private:
	selectable_ptr go;
	draw_ptr name_displayer;
	draw_ptr class_image;
	draw_ptr life_gage;
	draw_ptr en_gage;
	draw_ptr param;
};


class MaterialSelectionMenu : public Menu
{
public:
	class ClassImage : public IDrawer
	{
	public:
		ClassImage(selectable_ptr r) : root(r) {}

		void draw() const {
			if(!!root && !!root->getUnitImage()) {
				glEnable(GL_TEXTURE_2D);
				root->getUnitImage()->assign();
				DrawRect(ist::vector2(265, 415), ist::vector2(305, 455));
				glDisable(GL_TEXTURE_2D);
			}
		}

	private:
		selectable_ptr root;
	};

	class NameDisplayer : public IDrawer
	{
	public:
		NameDisplayer(selectable_ptr r) : root(r) {}

		void draw() const {
			DrawFont(265, 400, root->getUnitName().c_str());
			DrawFont(235, 375, root->getFaction()->getPlayerName().c_str());
			DrawFont(340, 375, root->getFaction()->getFactionName().c_str());
		}

	private:
		selectable_ptr root;
	};

	class ParameterDisplayer : public IDrawer
	{
	public:
		ParameterDisplayer(material_ptr r) : root(r) {}

		void draw() const {
			float q = root->getQuantity();
			char buf[32];
			if(root->isTimber())
				sprintf(buf, ": %.0f", q);
			else if(root->isSilicon())
				sprintf(buf, ": %.0f", q);
			else if(root->isIron())
				sprintf(buf, "S: %.0f", q);
			else
				assert(0);
			DrawFont(325, 417, buf);
		}

	private:
		material_ptr root;
	};



	MaterialSelectionMenu(material_ptr root) {
		go = root;
		name_displayer.reset(new NameDisplayer(root));
		class_image.reset(new ClassImage(root));
		param.reset(new ParameterDisplayer(root));
	}

	void draw() const {
		name_displayer->draw();
		class_image->draw();
		param->draw();
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		if(go->isDead())
			SendKillEvent(this, this);
	}

	void onKill(Event& e) {
		go = material_ptr();
		BaseClass::onKill(e);
	}

private:
	material_ptr go;
	draw_ptr name_displayer;
	draw_ptr class_image;
	draw_ptr param;
};



class ExtInfoMenu : public Menu
{
public:
	typedef std::vector<std::string> info_type;
	ExtInfoMenu(selectable_ptr r) : root(r) {}

	void draw() const {
		boost::any dat;
		if(!root->query("getExtInfo", dat) || dat.type()!=typeid(const info_type*))
			return;
		const info_type& info = *boost::any_cast<const info_type*>(dat);
		for(size_t i=0; i<info.size(); ++i) {
			DrawFont(385, 400+i*16, info[i].c_str());
		}
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		if(root->isDead())
			SendKillEvent(this, this);
	}

	void onKill(Event& e) {
		root = selectable_ptr();
		BaseClass::onKill(e);
	}


	std::vector<std::string> info;
	selectable_ptr root;
};



class FactionInfoMenu : public LinearInherit2(BelongFaction, Menu)
{
public:

	class ResourceDisplayer : public IDrawer
	{
	public:
		ResourceDisplayer(faction_ptr f) :
			faction(f)
		{}

		void draw() const {
			char buf[32];

			Resource& r = faction->getResource();
			sprintf(buf, ": %.0f", r.timber);
			DrawFont(9, 397, buf);

			sprintf(buf, ": %.0f", r.silicon);
			DrawFont(9, 414, buf);

			sprintf(buf, "S: %.0f", r.iron);
			DrawFont(9, 431, buf);

			sprintf(buf, "En: %.0f", r.energy);
			DrawFont(9, 448, buf);

			sprintf(buf, "l: %d", faction->getPop());
			DrawFont(9, 465, buf);
		}

	private:
		faction_ptr faction;
	};


	class AgeDisplayer : public IDrawer
	{
	public:
		AgeDisplayer(faction_ptr f) :
			faction(f)
		{}

		void draw() const {
			DrawFont(300, 5, faction->getAgeName().c_str());
		}

	private:
		faction_ptr faction;
	};

	class MessageDisplayer : public IDrawer
	{
	public:
		MessageDisplayer(faction_ptr f) :
			faction(f)
		{}

		void draw() const {
			size_t num_message = faction->getNumMessage();
			for(size_t i=std::max<int>(0, num_message-10); i<num_message; ++i) {
				DrawFont(3, 40+16*i, faction->getMessage(i).c_str());
			}
		}

	private:
		faction_ptr faction;
	};


	FactionInfoMenu(faction_ptr f) {
		setFaction(f);
		resource.reset(new ResourceDisplayer(f));
		age.reset(new AgeDisplayer(f));
		message.reset(new MessageDisplayer(f));
	}

	void draw() const {
		resource->draw();
		age->draw();
		message->draw();
	}

private:
	draw_ptr resource;
	draw_ptr age;
	draw_ptr message;
};


class DescriptionMenu : public LinearInherit2(BelongFaction, Menu)
{
public:
	DescriptionMenu(faction_ptr f) {
		setFaction(f);
	}

	void draw() const {
		gobj_ptr focused = getFaction()->getFocused();
		if(!focused)
			return;

		const std::string *message = 0;
		if(focused->isMenu()) {
			message = &boost::static_pointer_cast<IMenu>(focused)->getDescription();
		}
		else {
			selectable_ptr p = ToSelectable(focused);
			if(!!p)
				message = &p->getDescription();
		}

		if(!message || message->empty())
			return;

		ist::Color(0, 0, 0, 0.3f).assign();
		DrawRect(ist::vector2(0, 315), ist::vector2(235, 390));
		ist::Color().assign();
		DrawFont(2, 317, message->c_str());
	}
};


class MiniMap : public LinearInherit3(BelongFaction, HaveFrameCounter, Menu)
{
public:
	struct Point {
		ist::Color col;
		ist::vector4 pos;
	};

	MiniMap(faction_ptr f) :
		flag_reset(false),
		is_dragging(false),
		rect_min(0, 0),
		rect_max(150, 150)
	{
		setFaction(f);
		texture = GetTexture("resource/minimap.png");
		const ist::vector2& mmin = GetMapMin();
		const ist::vector2& mmax = GetMapMax();
		screen_l = mmin.x/0.707f;
		screen_r = mmax.x/0.707f;
		screen_b = mmin.y/0.707f;
		screen_t = mmax.y/0.707f;
	}

	void setPos(const ist::vector2& p) {
		rect_min+=p;
		rect_max+=p;
		v1 = ist::vector2((rect_min.x+rect_max.x)/2, rect_min.y);
		v2 = ist::vector2(rect_min.x, (rect_min.y+rect_max.y)/2);
		v3 = ist::vector2((rect_min.x+rect_max.x)/2, rect_max.y);
		v4 = ist::vector2(rect_max.x, (rect_min.y+rect_max.y)/2);
	}

	virtual void drawPoints() const {
		float rx = (float)GetWindowWidth()/640.0f;
		float ry = (float)GetWindowHeight()/480.0f;
		glViewport((GLsizei)(rect_min.x*rx), (GLsizei)((480.0f-rect_max.y)*ry),
			(GLsizei)((rect_max.x-rect_min.x)*rx), (GLsizei)((rect_max.y-rect_min.y)*ry));
		ist::OrthographicCamera top_camera;
		top_camera.setScreen(screen_l, screen_r, screen_b, screen_t);
		top_camera.direction = ist::vector3(0, 0, -1.0f);
		top_camera.position = ist::vector4(0, 5000.0f, 0);
		top_camera.z_far = 10000.0f;
		top_camera.look();


		glPointSize(2);
		glVertexPointer(4, GL_FLOAT, 0, &point_pos.front().v);
		glColorPointer(4, GL_FLOAT, 0, &point_col.front().v);
		glEnableClientState(GL_VERTEX_ARRAY);
		glEnableClientState(GL_COLOR_ARRAY);
		{
			ist::MatrixSaver(GL_MODELVIEW_MATRIX);
			glRotatef(-45, 0, 1, 0);
			glDrawArrays(GL_POINTS, 0, point_pos.size());
		}
		glDisableClientState(GL_COLOR_ARRAY);
		glDisableClientState(GL_VERTEX_ARRAY);
		ist::Color().assign();

		glPointSize(5);
		glBegin(GL_POINTS);
		glVertex3fv(GetCamera().target.v);
		glEnd();
		glPointSize(1);
		GetCamera().look();
	}


	void draw() const {
		if(flag_reset) {
			ist::Bitmap bmp(128, 128);
			GetCurrentWorld().getMapTexture(bmp);
			texture->load(bmp);
			resetPoints();
			flag_reset = false;
		}

		glDisable(GL_DEPTH_TEST);

		glEnable(GL_TEXTURE_2D);
		texture->assign();
		DrawQuad(v1, v2, v3, v4);
		glDisable(GL_TEXTURE_2D);
		drawPoints();
		glEnable(GL_DEPTH_TEST);

		glViewport(0, 0, GetWindowWidth(), GetWindowHeight());
	}

	bool isMouseOver() const {
		const ist::vector2& mouse = GetMousePointerPos2D();
		float ang = 0;
		ang+=(v1-mouse).cos(v2-mouse);
		ang+=(v2-mouse).cos(v3-mouse);
		ang+=(v3-mouse).cos(v4-mouse);
		ang+=(v4-mouse).cos(v1-mouse);
		if(ang>359.0f*ist::radian)
			return true;
		return false;
	}

	ist::vector4 getMousePointerPos() const {
		ist::vector2 rel = GetMousePointerPos2D()-rect_min;
		ist::vector2 ratio = rel/(rect_max-rect_min);
		ist::vector2 pos2((screen_r-screen_l)*ratio.x+screen_l, (screen_t-screen_b)*ratio.y+screen_b);
		pos2 = ist::matrix22().rotate(-45)*pos2;
		return ist::vector4(pos2.x, 0, pos2.y);
	}

	void moveCamera() {
		ist::vector4 target = getMousePointerPos();
		SetCameraPos(target);
	}

	void resetPoints() const {
		point_pos.clear();
		point_col.clear();

		const ist::vector2& mmin = GetMapMin();
		const ist::vector2& mmax = GetMapMax();
		IGObjIterator& block = GetBlocksByRadius(ist::vector4(), mmax.x-mmin.x);
		while(block.hasNext()) {
			selectable_ptr p = ToSelectable(block.iterate());
			if(!p || !p->isInSight(GetPlayerFaction()))
				continue;
			point_pos.push_back(p->getPos());
			if(p->isSelected())
				point_col.push_back(ist::vector4(1.0f, 1.0f, 1.0f));
			else {
				if(p->isMaterial()) {
					material_ptr m = boost::static_pointer_cast<IMaterial>(p);
					if(m->isTimber())
						point_col.push_back(ist::vector4(0.0f, 0.4f, 0.0f));
					else if(m->isSilicon())
						point_col.push_back(ist::vector4(0.7f, 0.7f, 0.7f));
					else if(m->isIron())
						point_col.push_back(ist::vector4(0.7f, 0.7f, 0.0f));
					else
						assert(0);
				}
				else if(p->isBuilding())
					point_col.push_back(p->getFaction()->getFactionColor().color - ist::vector4(0.2f, 0.2f, 0.2f));
				else
					point_col.push_back(p->getFaction()->getFactionColor().color);
			}
		}
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		if(getFrame()%30==1) {
			flag_reset = true;
		}
	}

	void onButton(Event& e) {
		Event::ButtonInfo& info = e.getButtonInfo();
		if(info.button==1 && info.isButtonDown() && isMouseOver()) {
			is_dragging = true;
			moveCamera();
		}
		else if(info.button==1 && info.isButtonUp()) {
			is_dragging = false;
		}
	}

	void onMotion(Event&) {
		if(is_dragging && isMouseOver())
			moveCamera();
	}


	mutable bool flag_reset;;
	bool is_dragging;
	texture_ptr texture;
	mutable std::vector<ist::vector4> point_pos;
	mutable std::vector<ist::vector4> point_col;
	ist::vector2 rect_min;
	ist::vector2 rect_max;
	ist::vector2 v1, v2, v3, v4;
	float screen_l;
	float screen_r;
	float screen_b;
	float screen_t;
};




class SystemDaemon : public Menu
{
public:
	SystemDaemon() :
		lock_camera(false),
		arrow_right(false),
		arrow_left(false),
		arrow_top(false),
		arrow_bottom(false),
		move_right(false),
		move_left(false),
		move_top(false),
		move_bottom(false)
	{}

	category_t getRecieveEvents() const { return RCV_KEY | RCV_MOTION | RCV_BUTTON; }

	void update() {
		BaseClass::update();
		if(lock_camera || (!move_top && !move_bottom && !move_right && !move_left))
			return;

		float camera_speed = 10;
		ist::vector4 vertical = GetCamera().position-GetCamera().target;
		vertical.y=0;
		vertical.normalize();
		vertical*=camera_speed;
		ist::vector4 horisontal = ist::matrix44().rotateY(90)*vertical;
		if(move_top)   { GetCamera().position-=vertical; GetCamera().target-=vertical; }
		if(move_bottom){ GetCamera().position+=vertical; GetCamera().target+=vertical; }
		if(move_right) { GetCamera().position+=horisontal; GetCamera().target+=horisontal; }
		if(move_left)  { GetCamera().position-=horisontal; GetCamera().target-=horisontal; }
		GetCamera().setViewFrustum();
	}

	void onKey(Event& e) {
		Event::KeyInfo& info = e.getKeyInfo();
		if(info.state==SDL_KEYDOWN) {
			if(info.key==SDLK_UP)        { arrow_top=move_top=true; }
			else if(info.key==SDLK_DOWN) { arrow_bottom=move_bottom=true; }
			else if(info.key==SDLK_RIGHT){ arrow_right=move_right=true; }
			else if(info.key==SDLK_LEFT) { arrow_left=move_left=true; }
		}
		else if(info.state==SDL_KEYUP) {
			if(info.key==SDLK_ESCAPE) { RequestSetCurrentWorld("title"); }
			else if(info.key==SDLK_UP)   { arrow_top=move_top=false; }
			else if(info.key==SDLK_DOWN) { arrow_bottom=move_bottom=false; }
			else if(info.key==SDLK_RIGHT){ arrow_right=move_right=false; }
			else if(info.key==SDLK_LEFT) { arrow_left=move_left=false; }
		}
	}

	void onButton(Event& e) {
		Event::ButtonInfo& info = e.getButtonInfo();
		if(info.isButtonDown())
			lock_camera = true;
		else if(info.isButtonUp())
			lock_camera = false;
	}

	void onMotion(Event& e) {
		Event::MotionInfo& info = e.getMotionInfo();
		if(info.x<10) { move_left=true; }
		else if(info.x>630) { move_right=true; }
		else if(!arrow_right && !arrow_left) { move_left=move_right=false; }

		if(info.y<10) { move_top=true; }
		else if(info.y>470) { move_bottom=true; }
		else if(!arrow_top && !arrow_bottom) { move_bottom=move_top=false; }
	}

private:
	bool lock_camera;
	bool arrow_right;
	bool arrow_left;
	bool arrow_top;
	bool arrow_bottom;
	bool move_right;
	bool move_left;
	bool move_top;
	bool move_bottom;
};





class BottomFrame : public Menu
{
public:
	BottomFrame() {}

	void draw() const {
		if(!texture)
			return;
		glEnable(GL_TEXTURE_2D);
		texture->assign();
		DrawRect(ist::vector2(0, 0), ist::vector2(640, 480));
		glDisable(GL_TEXTURE_2D);
	}

	void onCommand(Event& e) {
		Event::CommandInfo& info = e.getCommandInfo();
		if(info.name=="setTexture" && info.value.type()==typeid(std::string)) {
			texture = GetTexture(*boost::any_cast<std::string>(&info.value));
		}
		else
			BaseClass::onCommand(e);
	}

private:
	texture_ptr texture;
};


#endif
