#ifndef unit_ability
#define unit_ability

#include "game_object.h"
#include "effect.h"





CommandButtonSet* CreateDefaultUnitMenu() {
	CommandButtonSet *bs = new CommandButtonSet();
	{
		UnitCommandButton *b = new UnitCommandButton();
		b->setTexture("resource/button_stop_up.png");
		b->setCommand(new Event::CommandInfo("clearAction", boost::any()));
		b->setDescription("~(s)");
		b->setHotKey(SDLK_s);
		bs->setButton(6, 2, b);
	}
	return bs;
}





template<class T>
class HaveRapidAbility : public T
{
private:
	bool enable_rapid_abiliry;
	int rapid_level;
	int rapid_timecount;

public:
	typedef HaveRapidAbility BaseClass;

	HaveRapidAbility() : enable_rapid_abiliry(true), rapid_level(1), rapid_timecount(0)
	{}

	virtual void setCoolDown(int _cd) {
		int cd = _cd;
		if(isRapid()) {
			if(rapid_level==1)
				cd = int((float)cd*0.75f);
			else if(rapid_level==2)
				cd = int((float)cd*0.65f);
			else if(rapid_level==3)
				cd = int((float)cd*0.60f);
		}
		T::setCoolDown(cd);
	}

	virtual float getAntiBullet(){
		float a = T::getAntiBullet();
		if(isRapid()) {
			if(rapid_level==1)
				a-=0.15f;
			else if(rapid_level==2)
				a-=0.25f;
			else if(rapid_level==3)
				a-=0.30f;
		}
		return std::max<float>(a, 0);
	}

	virtual float getAntiLaser() {
		float a = T::getAntiLaser();
		if(isRapid()) {
			if(rapid_level==1)
				a-=0.15f;
			else if(rapid_level==2)
				a-=0.25f;
			else if(rapid_level==3)
				a-=0.30f;
		}
		return std::max<float>(a, 0);
	}

	virtual void onSetRapid() {}
	virtual void onUnsetRapid() {}

	bool isRapid() const { return rapid_timecount>0; }
	void enableRapidAbility(bool b=true) { enable_rapid_abiliry=b; }
	void setRapidLevel(int rl) { rapid_level=rl; }
	void setRapid() {
		if(!enable_rapid_abiliry || isRapid() || this->getEnergy()<75.0f)
			return;

		rapid_timecount = 600;
		setEnergy(this->getEnergy()-75.0f);

		SendCommandEvent(this, this->getFaction(), new Event::CommandInfo("refreshMenu", boost::any()));
		onSetRapid();
	}

	virtual void appendRapidButton(menu_ptr bs) {
		if(!enable_rapid_abiliry)
			return;

		UnitCommandButton *b = new UnitCommandButton();
		if(isRapid())
			b->setTexture("resource/button_mode_attack_down.png");
		else
			b->setTexture("resource/button_mode_attack_up.png");
		b->setCommand(new Event::CommandInfo("setRapid", boost::any()));

		if(rapid_level==1)
			b->setDescription("Lv1(r)\n10bԍŮԊu25%ZkA\nh͂15%ቺB");
		else if(rapid_level==2)
			b->setDescription("Lv2(r)\n10bԍŮԊu35%ZkA\nh͂25%ቺB");
		else if(rapid_level==3)
			b->setDescription("Lv3(r)\n10bԍŮԊu40%ZkA\nh͂30%ቺB");
		b->setHotKey(SDLK_r);

		SendCommandEvent(this, bs, new Event::CommandInfo("setButton", button_ptr(b)));
		bs->update();
	}

	menu_ptr getCommandMenu() {
		menu_ptr bs = T::getCommandMenu();
		if(!bs)
			bs = CreateDefaultUnitMenu();
		this->appendRapidButton(bs);
		return bs;
	}


	void onUpdate(Event& e) {
		T::onUpdate(e);
		if(isRapid()) {
			--rapid_timecount;
			if(!isRapid()) {
				onUnsetRapid();
				SendCommandEvent(this, this->getFaction(), new Event::CommandInfo("refreshMenu", boost::any()));
			}
		}
	}

	void onCommand(Event& e) {
		Event::CommandInfo& info = e.getCommandInfo();
		if(info.name=="setRapid")                this->setRapid();
		else if(info.name=="enableRapidAbility") this->enableRapidAbility(ToBool(info.value));
		else T::onCommand(e);
	}
};



template<class T>
class HaveSnipeAbility : public T
{
private:
	bool enable_snipe_abiliry;
	int snipe_level;
	int snipe_timecount;

public:
	typedef HaveSnipeAbility BaseClass;

	HaveSnipeAbility() : enable_snipe_abiliry(true), snipe_level(1), snipe_timecount(0)
	{}

	virtual bool conditionFire() const {
		if(isSnipe())
			return false;
		return T::conditionFire();
	}

	virtual float getFireRange() const {
		float a = T::getFireRange();
		if(isSnipe()) {
			if(snipe_level==1)
				a*=1.30f;
			else if(snipe_level==2)
				a*=1.45f;
			else if(snipe_level==3)
				a*=1.60f;
		}
		return a;
	}

	virtual float getSight() const {
		return std::max<float>(T::getSight(), this->getFireRange());
	}

	virtual float getMoveSpeed() const {
		float a = T::getMoveSpeed();
		if(isSnipe()) {
			if(snipe_level==1)
				a*=0.50f;
			else if(snipe_level==2)
				a*=0.35f;
			else if(snipe_level==3)
				a*=0.20f;
		}
		return a;
	}

	virtual float getAntiBullet() const{
		float a = T::getAntiBullet();
		if(this->isSnipe()) {
			if(snipe_level==1)
				a-=0.15f;
			else if(snipe_level==2)
				a-=0.25f;
			else if(snipe_level==3)
				a-=0.30f;
		}
		return std::max<float>(a, 0);
	}

	virtual float getAntiLaser() const {
		float a = T::getAntiLaser();
		if(isSnipe()) {
			if(snipe_level==1)
				a-=0.15f;
			else if(snipe_level==2)
				a-=0.25f;
			else if(snipe_level==3)
				a-=0.30f;
		}
		return std::max<float>(a, 0);
	}

	virtual void onSetSnipe() {}
	virtual void onUnsetSnipe() {}

	bool isSnipe() const { return snipe_timecount>0; }
	void enableSnipeAbility(bool b=true) { enable_snipe_abiliry=b; }
	void setSnipeLevel(int rl) { snipe_level=rl; }
	void setSnipe() {
		if(!enable_snipe_abiliry || isSnipe() || this->getEnergy()<75.0f)
			return;

		snipe_timecount = 900;
		this->setEnergy(this->getEnergy()-75.0f);
		onSetSnipe();

		SendCommandEvent(this, this->getFaction(), new Event::CommandInfo("refreshMenu", boost::any()));
	}

	virtual void appendSnipeButton(menu_ptr bs) {
		if(!enable_snipe_abiliry)
			return;

		UnitCommandButton *b = new UnitCommandButton();
		if(isSnipe())
			b->setTexture("resource/button_barrier_down.png");
		else
			b->setTexture("resource/button_barrier_up.png");
		b->setCommand(new Event::CommandInfo("setSnipe", boost::any()));

		if(snipe_level==1)
			b->setDescription("_Lv1(n)\n15bԎ˒30%LB\n͈ړxቺ50%B");
		else if(snipe_level==2)
			b->setDescription("_Lv2(n)\n15bԎ˒45%LB\n͈ړxቺ65%B");
		else if(snipe_level==3)
			b->setDescription("_Lv3(n)\n15bԎ˒60%LB\n͈ړxቺ80%B");
		b->setHotKey(SDLK_n);

		SendCommandEvent(this, bs, new Event::CommandInfo("setButton", button_ptr(b)));
		bs->update();
	}

	menu_ptr getCommandMenu() {
		menu_ptr bs = T::getCommandMenu();
		if(!bs)
			bs = CreateDefaultUnitMenu();
		this->appendSnipeButton(bs);
		return bs;
	}


	void onUpdate(Event& e) {
		T::onUpdate(e);
		if(isSnipe()) {
			--snipe_timecount;
			if(!isSnipe()) {
				onUnsetSnipe();
				SendCommandEvent(this, this->getFaction(), new Event::CommandInfo("refreshMenu", boost::any()));
			}
		}
	}

	void onCommand(Event& e) {
		Event::CommandInfo& info = e.getCommandInfo();
		if(info.name=="setSnipe")                this->setSnipe();
		else if(info.name=="enableSnipeAbility") this->enableSnipeAbility(ToBool(info.value));
		else T::onCommand(e);
	}
};


template<class T>
class HaveGuardAbility : public T
{
private:
	bool enable_guard_abiliry;
	int guard_level;
	int guard_timecount;

public:
	typedef HaveGuardAbility BaseClass;

	HaveGuardAbility() : enable_guard_abiliry(true), guard_level(1), guard_timecount(0)
	{}

	virtual bool conditionFire() const {
		if(isGuard())
			return false;
		return T::conditionFire();
	}

	virtual float getMoveSpeed() const {
		float a = T::getMoveSpeed();
		if(isGuard()) {
			if(guard_level==1)
				a*=0.50f;
			else if(guard_level==2)
				a*=0.35f;
			else if(guard_level==3)
				a*=0.20f;
		}
		return a;
	}

	virtual float getAntiBullet() const {
		float a = T::getAntiBullet();
		if(isGuard()) {
			if(guard_level==1)
				a+=0.15f;
			else if(guard_level==2)
				a+=0.25f;
			else if(guard_level==3)
				a+=0.30f;
		}
		return std::min<float>(a, 0.99f);
	}

	virtual float getAntiLaser() const {
		float a = T::getAntiLaser();
		if(isGuard()) {
			if(guard_level==1)
				a+=0.15f;
			else if(guard_level==2)
				a+=0.25f;
			else if(guard_level==3)
				a+=0.30f;
		}
		return std::min<float>(a, 0.99f);
	}

	virtual void onSetGuard() {}
	virtual void onUnsetGuard() {}

	bool isGuard() const { return guard_timecount>0; }
	void enableGuardAbility(bool b=true) { enable_guard_abiliry=b; }
	void setGuardLevel(int rl) { guard_level=rl; }
	void setGuard() {
		if(!enable_guard_abiliry || isGuard() || this->getEnergy()<75.0f)
			return;

		guard_timecount = 600;
		this->setEnergy(this->getEnergy()-75.0f);
		onSetGuard();

		SendCommandEvent(this, this->getFaction(), new Event::CommandInfo("refreshMenu", boost::any()));
	}

	virtual void appendGuardButton(menu_ptr bs) {
		if(!enable_guard_abiliry)
			return;

		UnitCommandButton *b = new UnitCommandButton();
		if(isGuard())
			b->setTexture("resource/button_barrier_down.png");
		else
			b->setTexture("resource/button_barrier_up.png");
		b->setCommand(new Event::CommandInfo("setGuard", boost::any()));

		if(guard_level==1)
			b->setDescription("hLv1(g)\n10bԖh͂15%㏸B\n͍UsAړxቺ50%B");
		else if(guard_level==2)
			b->setDescription("hLv2(g)\n10bԖh͂25%㏸B\n͍UsAړxቺ65%B");
		else if(guard_level==3)
			b->setDescription("hLv3(g)\n10bԖh͂30%㏸B\n͍UsAړxቺ80%B");
		b->setHotKey(SDLK_g);

		SendCommandEvent(this, bs, new Event::CommandInfo("setButton", button_ptr(b)));
		bs->update();
	}

	menu_ptr getCommandMenu() {
		menu_ptr bs = T::getCommandMenu();
		if(!bs)
			bs = CreateDefaultUnitMenu();
		this->appendGuardButton(bs);
		return bs;
	}


	void onUpdate(Event& e) {
		T::onUpdate(e);
		if(isGuard()) {
			--guard_timecount;
			if(!isGuard()) {
				onUnsetGuard();

				SendCommandEvent(this, this->getFaction(), new Event::CommandInfo("refreshMenu", boost::any()));
			}
		}
	}

	void onCommand(Event& e) {
		Event::CommandInfo& info = e.getCommandInfo();
		if(info.name=="setGuard")                this->setGuard();
		else if(info.name=="enableGuardAbility") this->enableGuardAbility(ToBool(info.value));
		else T::onCommand(e);
	}
};






template<class T>
class HaveCloakAbility : public T
{
private:
	bool is_cloaking;
	bool enable_cloak_abiliry;
	float cloak_energy_use;
	int cloak_detect_count;

public:
	typedef HaveCloakAbility BaseClass;

	HaveCloakAbility() :
		is_cloaking(false),
		enable_cloak_abiliry(false),
		cloak_energy_use(100.0f/3600.0f),
		cloak_detect_count(0)
	{}

	virtual bool conditionEnergyRegen() const {
		return T::conditionEnergyRegen() && !isCloaking();
	}

	bool isCloaking() const { return is_cloaking; }
	void enableCloakAbility(bool b=true) { enable_cloak_abiliry=b; }
	void setCloak(bool b) {
		if(!enable_cloak_abiliry || is_cloaking==b)
			return;

		is_cloaking = b;

		SendCommandEvent(this, this->getFaction(), new Event::CommandInfo("refreshMenu", boost::any()));
	}
	void setCloakEnergyUse(float eu) { cloak_energy_use=eu; }



	int getDrawPriority() const {
		if(is_cloaking)
			return T::getDrawPriority()+1;
		return T::getDrawPriority();
	}

	virtual void appendCloakButton(menu_ptr bs) {
		if(!enable_cloak_abiliry)
			return;

		UnitCommandButton *b = new UnitCommandButton();
		if(is_cloaking)
			b->setTexture("resource/button_cloak_down.png");
		else
			b->setTexture("resource/button_cloak_up.png");
		b->setCommand(new Event::CommandInfo("setCloak", bool(!is_cloaking)));
		b->setDescription("N[N(c)\nB\n͎ȖΏۂɂȂȂȂA\nCHȂȂB");
		b->setHotKey(SDLK_c);

		SendCommandEvent(this, bs, new Event::CommandInfo("setButton", button_ptr(b)));
		bs->update();
	}

	menu_ptr getCommandMenu() {
		menu_ptr bs = T::getCommandMenu();
		if(!bs)
			bs = CreateDefaultUnitMenu();
		this->appendCloakButton(bs);
		return bs;
	}

	void assignMaterial(const ist::Material& _mat) const {
		if(isCloaking()) {
			ist::Material mat = _mat;
			mat.diffuse.w = 0.3f;
			T::assignMaterial(mat);
		}
		else
			T::assignMaterial(_mat);
	}

	void draw() const {
		if(!IsAlly(GetPlayerFaction(), this->getFaction())
			&& !(GetPlayerFaction()->getFocused().get()==this)
			&& isCloaking() && cloak_detect_count==0)
			return;
		T::draw();
	}

	void onUpdate(Event& e) {
		T::onUpdate(e);
		if(is_cloaking) {
			this->setEnergy(this->getEnergy()-cloak_energy_use);
			if(this->getEnergy()==0)
				this->setCloak(false);
		}
		if(cloak_detect_count>0)
			--cloak_detect_count;
	}

	void onCommand(Event& e) {
		Event::CommandInfo& info = e.getCommandInfo();
		if(info.name=="setCloak")                this->setCloak(ToBool(info.value));
		else if(info.name=="enableCloakAbility") this->enableCloakAbility(ToBool(info.value));
		else if(info.name=="detectCloak")        cloak_detect_count=11;
		else T::onCommand(e);
	}

	bool query(const std::string& name, boost::any& value) const {
		if(name=="isTargetable") value = (!isCloaking() || cloak_detect_count!=0);
		else return this->T::query(name, value);
		return true;
	}
};


template<class T>
class HaveSpyAbility : public T
{
private:
	bool is_spying;
	bool enable_spy_abiliry;
	float spy_energy_use;
	int spy_detect_count;

public:
	typedef HaveSpyAbility BaseClass;

	HaveSpyAbility() :
		is_spying(false),
		enable_spy_abiliry(false),
		spy_energy_use(100.0f/3600.0f),
		spy_detect_count(0)
	{}

	virtual bool conditionEnergyRegen() const  {
		return T::conditionEnergyRegen() && !isSpying();
	}

	bool isSpying() const { return is_spying; }
	void enableSpyAbility(bool b=true) { enable_spy_abiliry=b; }
	void setSpy(bool b) {
		if(!enable_spy_abiliry || is_spying==b)
			return;

		is_spying = b;

		SendCommandEvent(this, this->getFaction(), new Event::CommandInfo("refreshMenu", boost::any()));
	}
	void setSpyEnergyUse(float eu) { spy_energy_use=eu; }


	int getDrawPriority() const  {
		if(is_spying)
			return T::getDrawPriority()+1;
		return T::getDrawPriority();
	}

	virtual void appendSpyButton(menu_ptr bs) {
		if(!enable_spy_abiliry)
			return;

		UnitCommandButton *b = new UnitCommandButton();
		if(is_spying)
			b->setTexture("resource/no_image.png");
		else
			b->setTexture("resource/no_image.png");
		b->setCommand(new Event::CommandInfo("setSpy", bool(!is_spying)));
		b->setDescription("U(s)\nG͎RƓFɌB\n͎ȖΏۂɂȂȂȂB");
		b->setHotKey(SDLK_s);

		SendCommandEvent(this, bs, new Event::CommandInfo("setButton", button_ptr(b)));
		bs->update();
	}

	menu_ptr getCommandMenu() {
		menu_ptr bs = T::getCommandMenu();
		if(!bs)
			bs = CreateDefaultUnitMenu();
		this->appendSpyButton(bs);
		return bs;
	}

	void assignMaterial(const ist::Material& _mat) const {
		if(isSpying()) {
			if(IsAlly(this->getFaction(), GetPlayerFaction())) {
				ist::Material mat;
				T::assignMaterial(mat);
			}
			else {
				ist::Material mat = GetPlayerFaction()->getFactionMaterial();
				T::assignMaterial(mat);
			}
		}
		else
			T::assignMaterial(_mat);
	}

	void onUpdate(Event& e) {
		T::onUpdate(e);
		if(is_spying) {
			this->setEnergy(this->getEnergy()-spy_energy_use);
			if(this->getEnergy()==0)
				this->setSpy(false);
		}
		if(spy_detect_count>0)
			--spy_detect_count;
	}

	void onCommand(Event& e) {
		Event::CommandInfo& info = e.getCommandInfo();
		if(info.name=="setSpy")                this->setSpy(ToBool(info.value));
		else if(info.name=="enableSpyAbility") this->enableSpyAbility(ToBool(info.value));
		else if(info.name=="detectSpy")        spy_detect_count=11;
		else T::onCommand(e);
	}

	bool query(const std::string& name, boost::any& value) const {
		if(name=="isTargetable") value = (!isSpying() || spy_detect_count!=0);
		else return this->T::query(name, value);
		return true;
	}
};



template<class T>
class HaveSelfDestructAbility : public T
{
private:
	bool enable_sd_abiliry;
	bool is_sd;
	int sd_count;
	float sd_energy_use;
	float sd_range;

public:
	typedef HaveSelfDestructAbility BaseClass;

	HaveSelfDestructAbility() :
		enable_sd_abiliry(false),
		is_sd(false),
		sd_count(0),
		sd_energy_use(50),
		sd_range(100)
	{}


	void enableSDAbility(bool b=true) { enable_sd_abiliry=b; }
	void setSD(bool b) {
		if(!enable_sd_abiliry || is_sd==b)
			return;
		if(b && this->getEnergy()<50.0f)
			return;

		is_sd = b;
		if(!is_sd) {
			sd_count = 0;
		}
		SendCommandEvent(this, this->getFaction(), new Event::CommandInfo("refreshMenu", boost::any()));
	}

	void invokeSD() {
		IGObjIterator& i = GetBlocksByRadius(this->getPos(), sd_range);
		while(i.hasNext()) {
			breakable_ptr s = ToBreakable(i.iterate());
			Event::damageinfo_ptr dp = new Event::DamageInfo(getLife()/4.0f, 0, getLife()/4.0f);
			if(!!s && s.get()!=this && (s->getPos()-this->getPos()).setY(0).norm()<sd_range) {
				SendDamageEvent(this, s, dp);
			}
		}
		if(this->isInCameraSight()) {
			Pillar *p = new Pillar();
			AddGObj(p);
			p->setTexture("resource/pillar2.png");
			p->setScl(ist::vector4(sd_range, 0, sd_range*0.9f));
			p->setPos(this->getPos());

			p = new Pillar();
			AddGObj(p);
			p->setTexture("resource/pillar2.png");
			p->setScl(ist::vector4(sd_range, 0, sd_range*1.0f));
			p->setExpand(ist::vector4(0, 320.0f/40.0f, 0));
			p->setPos(this->getPos());

			p = new Pillar();
			AddGObj(p);
			p->setTexture("resource/pillar2.png");
			p->setScl(ist::vector4(sd_range, 0, sd_range*1.1f));
			p->setExpand(ist::vector4(0, 340.0f/40.0f, 0));
			p->setPos(this->getPos());
		}
		SendDestroyEvent(this, this);
	}


	virtual void appendSDButton(menu_ptr bs) {
		if(!enable_sd_abiliry)
			return;

		UnitCommandButton *b = new UnitCommandButton();
		if(is_sd)
			b->setTexture("resource/no_image.png");
		else
			b->setTexture("resource/no_image.png");
		b->setCommand(new Event::CommandInfo("setSD", bool(!is_sd)));
		b->setDescription("(d)\n3bɎAG֌WȂ͂Ƀ_[W^B\n_[WHPc/4BɓɗLB");
		b->setHotKey(SDLK_d);

		SendCommandEvent(this, bs, new Event::CommandInfo("setButton", button_ptr(b)));
		bs->update();
	}

	menu_ptr getCommandMenu() {
		menu_ptr bs = T::getCommandMenu();
		if(!bs)
			bs = CreateDefaultUnitMenu();
		this->appendSDButton(bs);
		return bs;
	}

	void assignMaterial(const ist::Material& _mat) const {
		if(is_sd) {
			ist::Material mat = GetPlayerFaction()->getFactionMaterial();
			mat.emission = ist::vector4(1.0f, 0.7f, 0.7f, 1.0f)/180.0f*float(sd_count);
			T::assignMaterial(mat);
		}
		else
			T::assignMaterial(_mat);
	}

	void onUpdate(Event& e) {
		T::onUpdate(e);
		if(is_sd && ++sd_count==180) {
			invokeSD();
		}
	}

	void onCommand(Event& e) {
		Event::CommandInfo& info = e.getCommandInfo();
		if(info.name=="setSD")                this->setSD(ToBool(info.value));
		else if(info.name=="invokeSD")        this->invokeSD();
		else if(info.name=="enableSDAbility") this->enableSDAbility(ToBool(info.value));
		else T::onCommand(e);
	}
};

template<class T>
class HaveEMPBarrageAbility : public T
{
public:
	typedef HaveEMPBarrageAbility BaseClass;
};

template<class T>
class HaveRaderJammerAbility : public T
{
public:
	typedef HaveRaderJammerAbility BaseClass;
};


#endif
