#ifndef material
#define material

#include "templates/game_object.h"
#include "templates/menu.h"



material_ptr CreateTimber();
material_ptr CreateSilicon();
material_ptr CreateIron();




class Material : public LinearInherit4(
	HavePolygonModel,
	BelongFaction,
	HaveCharacterGridInserter,
	LinearInherit9(
		HaveSelectChecker,
		HaveFrameCounter,
		HavePointCollisionModel,
		HaveSelectableInfo,
		HaveTransformInfo,
		HaveDeathFlag,
		HaveID,
		HaveEventQueue,
		IMaterial ))
{
public:
	Material() : quantity(10000.0f) {
		setFaction(GetNatureFaction());
	}
	virtual float getQuantity() const { return quantity; }
	virtual void setQuantity(float q) { quantity=q; }
	virtual void decreaseQuantity(float q) { quantity-=q; }

	bool isInSight(faction_ptr f) const {
		if(!f)
			return true;
		return GetCurrentWorld().isExplored(f, this->getPos());
	}

	void setPos(const ist::vector4& p) {
		BaseClass::setPos(ist::vector4(p.x, GetHeight(p), p.z));
	}


	void assignMaterial(const ist::Material& _mat) const {
		bool highlight = this->isAdded() && (this->isSelected() || (GetPlayerFaction()->getFocused().get()==this));
		if(highlight) {
			ist::Material mat = _mat;
			mat.emission = ist::vector4(0.15f, 0.15f, 0.15f);
			mat.assign();
		}
		else if(!GetCurrentWorld().isSighted(GetPlayerFaction(), getPos())) {
			ist::Material mat = _mat;
			mat.diffuse.x*=0.4f;
			mat.diffuse.y*=0.4f;
			mat.diffuse.z*=0.4f;
			mat.assign();
		}
		else {
			_mat.assign();
		}
	}

	void draw() const {
		ist::Material mat;
		mat.diffuse = ist::vector4(1.0f, 1.0f, 1.0f);
		assignMaterial(mat);
		drawModel();
		ist::Material().assign();
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		if(getQuantity()<=0.0f)
			SendDestroyEvent(this, this);
	}

	void onDestroy(Event& e) {
		BaseClass::onDestroy(e);
		Fader *f = new Fader();
		f->setPos(getPos());
		f->setRot(getRot());
		f->setPModel(getPModel());
		f->setTexture(getTexture());
		AddGObj(f);
	}


	category_t getRecieveEvents() const { return RCV_UPDATE; }

	bool isTimber() const { return false; }
	bool isSilicon() const{ return false; }
	bool isIron() const   { return false; }

	float getSight() const { return 0; }
	texture_ptr getUnitImage() const { return texture_ptr(); }
	menu_ptr getInfomationMenu() { return new MaterialSelectionMenu(this); }

private:
	float quantity;
};


class Timber : public Material
{
public:
	Timber() {
		setQuantity(500);
		setPModel("resource/timber.mqo");
		setTexture("resource/timber.png");
		setDrawPriority(250);
		setSelectionRadius(30);
		setHitRadius(30);
	}
	texture_ptr getUnitImage() const { return GetTexture("resource/timber_icon.png"); }

	bool isTimber() const { return true; }
	const std::string& getClassName() const { static std::string s("Timber"); return s; }
	const std::string& getUnitName() const { static std::string s(""); return s; }
	const std::string& getDescription() const { static std::string s("؍ށB"); return s; }
};

class Silicon : public Material
{
public:
	Silicon() {
		setPModel("resource/silicon.mqo");
		setSelectionRadius(45);
		setHitRadius(45);
	}
	texture_ptr getUnitImage() const { return GetTexture("resource/silicon_icon.png"); }

	bool isSilicon() const { return true; }
	const std::string& getClassName() const { static std::string s("Silicon"); return s; }
	const std::string& getUnitName() const { static std::string s(""); return s; }
	const std::string& getDescription() const { static std::string s("΍ށB"); return s; }

	ist::vector4 diffuse;
};

class Iron : public Material
{
public:
	Iron() {
		setPModel("resource/iron.mqo");
		metalic.setTexture("resource/metalic.png");
		setSelectionRadius(45);
		setHitRadius(45);
	}
	texture_ptr getUnitImage() const { return GetTexture("resource/iron_icon.png"); }
	void draw() const {
		ist::Material mat;
		mat.diffuse = ist::vector4(0.5f, 0.5f, 0.5f);
		assignMaterial(mat);
		metalic.enable();
		drawModel();
		metalic.disable();
		ist::Material().assign();
	}

	bool isIron() const { return true; }
	const std::string& getClassName() const { static std::string s("Iron"); return s; }
	const std::string& getUnitName() const { static std::string s("Sz"); return s; }
	const std::string& getDescription() const { static std::string s("SB"); return s; }

	mutable SphereMap metalic;
};


#endif
