#include "tohu_ruler.h"







class tohuEnemyColony : public LinearInherit2(HaveFrameCounter, Menu)
{
public:
	typedef std::list<gobj_id> unit_cont;
	typedef LinearInherit2(HaveFrameCounter, Menu) RootClass;
	typedef tohuEnemyColony BaseClass;


	gobj_id player_faction;
	gobj_id enemy_faction;
	gobj_id base;
	gobj_id target;
	gobj_ptr root;
	size_t supply_interval;
	size_t max_units;
	unit_cont units;
	ist::vector4 center;
	bool flag_active;


	tohuEnemyColony(gobj_id pf, gobj_id ef) :
		player_faction(pf),
		enemy_faction(ef),
		base(0),
		target(0),
		supply_interval(300),
		max_units(20),
		flag_active(false)
	{}

	gobj_id createUnit(const std::string& class_name) {
		gobj_id u = CreateID();
		RouteCreateEvent(new Event::CreateInfo(UNIT, u, class_name, enemy_faction));
		units.push_back(u);
		return u;
	}

	gobj_id createBuilding(const std::string& class_name) {
		gobj_id u = CreateID();
		RouteCreateEvent(new Event::CreateInfo(BUILDING, u, class_name, enemy_faction));
		return u;
	}

	void setTarget(gobj_id t)             { target=t; }
	void setRoot(gobj_ptr r)              { root=r; }
	void setSupplyInterval(size_t si)     { supply_interval=si; }
	void setMaxUnits(size_t mu)           { max_units=mu; }
	void setCenter(const ist::vector4& c) { center=c; }

	virtual void dispose(int num) {
		for(int i=0; i<num; ++i) {
			onSupply();
		}
	}

	virtual void checkLiving() {
		for(unit_cont::iterator p=units.begin(); p!=units.end();) {
			if(!GetGObj((*p)))
				units.erase(p++);
			else
				++p;
		}
	}

	virtual void attack() {
		gobj_id group = CreateID();
		Event::createinfo_ptr info = new Event::CreateInfo(UNITGROUP, group, std::string());
		for(unit_cont::iterator p=units.begin(); p!=units.end(); ++p) {
			if(unit_ptr u = ToUnit(GetGObj(*p)))
				info->group.push_back(u->getID());
		}
		RouteCreateEvent(info);

		Event::actioninfo_ptr act = new Event::ActionInfo(ToSelectable(GetGObj(target))->getPos(),
			ToSelectable(GetGObj(target)), Event::ActionInfo::INSIST | Event::ActionInfo::ACTIVE);
		act->speed = 1.0f;
		RouteActionEvent(0, group, act);
		units.clear();
	}


	virtual void setFlagActive(bool b) { flag_active=b; }
	virtual void setLevel(int i) {}
	virtual void onSupply() {}
	virtual void onBaseDestroyed() { SendKillEvent(this, this); }

	void onUpdate(Event& e) {
		RootClass::onUpdate(e);
		checkLiving();
		if(!!base) {
			if(!GetGObj(base)) {
				base = 0;
				onBaseDestroyed();
			}
			else if(getFrame()%supply_interval==0) {
				if(units.size()<max_units)
					onSupply();
				else if(units.size()==max_units && flag_active && getFrame()>14400)
					attack();
			}
		}
	}

	bool call(const std::string& name, boost::any& value) {
		if(name=="dispose" && value.type()==typeid(int))
			dispose(*boost::any_cast<int>(&value));
		else if(name=="attack")
			attack();
		else if(name=="setLevel" && value.type()==typeid(int))
			setLevel(boost::any_cast<int>(value));
		else if(name=="setFlagActive")
			setFlagActive(ToBool(value));
		else
			return BaseClass::call(name, value);
		return true;
	}
};



class tohuTowerEnemyColony : public tohuEnemyColony
{
public:
	tohuTowerEnemyColony(gobj_id pf, gobj_id ef) :
		tohuEnemyColony(pf, ef)
	{}

	void dispose(int num) {
		tohuEnemyColony::dispose(num);
		if(!base) {
			base = createBuilding("tohuTower");
			RouteCallEvent(0, base, new Event::CallInfo("setPos", center));
		}
	}

	virtual void setLevel(int i) {
		if(!base)
			return;

		if(i==0) {
			setMaxUnits(25);
			setSupplyInterval(300);
			RouteCallEvent(0, base, new Event::CallInfo("setFirePower", float(15)));
			RouteCallEvent(0, base, new Event::CallInfo("setMaxLife", float(750)));
		}
		else if(i==1) {
			setMaxUnits(50);
			setSupplyInterval(220);
			RouteCallEvent(0, base, new Event::CallInfo("setFirePower", float(20)));
			RouteCallEvent(0, base, new Event::CallInfo("setMaxLife", float(1500)));
		}
		else if(i==2) {
			setMaxUnits(80);
			setSupplyInterval(180);
			RouteCallEvent(0, base, new Event::CallInfo("setFirePower", float(30)));
			RouteCallEvent(0, base, new Event::CallInfo("setMaxLife", float(3000)));
		}
		else {
			setMaxUnits(120);
			setSupplyInterval(150);
			RouteCallEvent(0, base, new Event::CallInfo("setFirePower", float(40)));
			RouteCallEvent(0, base, new Event::CallInfo("setMaxLife", float(5000)));
		}
	}

	void sendUpgrade() {
		std::string uginfoS[3] = {
			"smallLevel1",
			"smallLevel2",
			"smallLevel3",
		};
		std::string uginfoM[2] = {
			"mediumLevel1",
			"mediumLevel2",
		};
		boost::any query;
		for(int i=0; i<3; ++i) {
			query = uginfoS[i];
			ToGObj(player_faction)->query("isUpgraded", query);
			if(!ToBool(query)) {
				RouteCallEvent(0, player_faction, new Event::CallInfo("upgrade", uginfoS[i]));
				RouteCallEvent(0, enemy_faction, new Event::CallInfo("upgrade", uginfoS[i]));
				break;
			}
		}
		for(int i=0; i<2; ++i) {
			query = uginfoM[i];
			ToGObj(player_faction)->query("isUpgraded", query);
			if(!ToBool(query)) {
				RouteCallEvent(0, player_faction, new Event::CallInfo("upgrade", uginfoM[i]));
				RouteCallEvent(0, enemy_faction, new Event::CallInfo("upgrade", uginfoM[i]));
				break;
			}
		}
		RouteCallEvent(0, player_faction, new Event::CallInfo("pushMessage",
			std::string("jbgAbvO[h܂B")));
	}

	void onBaseDestroyed() {
		sendUpgrade();
		SendCallEvent(this, root, new Event::CallInfo("changeActive", boost::any()));
		SendKillEvent(this, this);
	}
};
typedef boost::intrusive_ptr<tohuEnemyColony> colony_ptr;


class tohuAssasinEnemyColony : public tohuTowerEnemyColony
{
public:
	tohuAssasinEnemyColony(gobj_id pf, gobj_id ef) :
		tohuTowerEnemyColony(pf, ef)
	{}

	void dispose(int num) {
		tohuEnemyColony::dispose(num);
		if(!base) {
			base = createBuilding("tohuTower");
			RouteCallEvent(0, base, new Event::CallInfo("setPos", center));
		}
	}


	void onSupply() {
		ist::vector4 u_pos = center;
		boost::any hradius;
		gobj_ptr tw = GetGObj(base);
		selectable_ptr tg = ToSelectable(GetGObj(target));
		if(!!tw && !!tg && tw->query("getHitRadius", hradius))
			u_pos+=(tg->getPos()-center).normalize()*ToFloat(hradius);

		float f = GetRand();
		if(f<0.3f) {
			gobj_id u = createUnit("tohuSoldier1");
			RouteCallEvent(0, u, new Event::CallInfo("setPos", u_pos));
			RouteActionEvent(0, u,
				new Event::ActionInfo(center+RandomAim()*80+ist::vector4(RandomAim()).normalize()*170, selectable_ptr()));
		}
		else if(f<0.75f) {
			gobj_id u = createUnit("tohuAssasin");
			RouteCallEvent(0, u, new Event::CallInfo("setPos", u_pos));
			RouteCallEvent(0, u, new Event::CallInfo("setMaxEnergy", float(25.0f)));
			RouteCallEvent(0, u, new Event::CallInfo("setCloak", true));
			RouteActionEvent(0, u,
				new Event::ActionInfo(center+RandomAim()*100+ist::vector4(RandomAim()).normalize()*200, selectable_ptr()));
		}
		else if(f<0.95f) {
			gobj_id u = createUnit("tohuBomber1");
			RouteCallEvent(0, u, new Event::CallInfo("setPos", u_pos));
			RouteActionEvent(0, u,
				new Event::ActionInfo(center+RandomAim()*50+ist::vector4(RandomAim()).normalize()*100, selectable_ptr()));
		}
		else {
			gobj_id u = createUnit("tohuDetector");
			RouteCallEvent(0, u, new Event::CallInfo("setPos", u_pos));
			RouteActionEvent(0, u,
				new Event::ActionInfo(center+RandomAim()*50+ist::vector4(RandomAim()).normalize()*50, selectable_ptr()));
		}
	}
};



class tohuSoldierEnemyColony : public tohuTowerEnemyColony
{
public:
	tohuSoldierEnemyColony(gobj_id pf, gobj_id ef) :
		tohuTowerEnemyColony(pf, ef)
	{}

	void dispose(int num) {
		tohuEnemyColony::dispose(num);
		if(!base) {
			base = createBuilding("tohuTower");
			RouteCallEvent(0, base, new Event::CallInfo("setPos", center));
		}
	}


	void onSupply() {
		ist::vector4 u_pos=center;
		boost::any hradius;
		gobj_ptr tw = GetGObj(base);
		selectable_ptr tg = ToSelectable(GetGObj(target));
		if(!!tw && !!tg && tw->query("getHitRadius", hradius))
			u_pos+=(tg->getPos()-center).normalize()*ToFloat(hradius);

		float f = GetRand();
		if(f<0.4f) {
			gobj_id u = createUnit("tohuSoldier1");
			RouteCallEvent(0, u, new Event::CallInfo("setPos", u_pos));
			RouteActionEvent(0, u,
				new Event::ActionInfo(center+RandomAim()*80+ist::vector4(RandomAim()).normalize()*170, selectable_ptr()));
		}
		else if(f<0.6f) {
			gobj_id u = createUnit("tohuArmor1");
			RouteCallEvent(0, u, new Event::CallInfo("setPos", u_pos));
			RouteActionEvent(0, u,
				new Event::ActionInfo(center+RandomAim()*50+ist::vector4(RandomAim()).normalize()*250, selectable_ptr()));
		}
		else if(f<0.85f) {
			gobj_id u = createUnit("tohuLaser1");
			RouteCallEvent(0, u, new Event::CallInfo("setPos", u_pos));
			RouteActionEvent(0, u,
				new Event::ActionInfo(center+RandomAim()*50+ist::vector4(RandomAim()).normalize()*80, selectable_ptr()));
		}
		else {
			gobj_id u = createUnit("tohuBomber1");
			RouteCallEvent(0, u, new Event::CallInfo("setPos", u_pos));
			RouteActionEvent(0, u,
				new Event::ActionInfo(center+RandomAim()*50+ist::vector4(RandomAim()).normalize()*100, selectable_ptr()));
		}
	}
};




class tohuEmperorEnemyColony : public tohuTowerEnemyColony
{
public:
	tohuEmperorEnemyColony(gobj_id pf, gobj_id ef) :
		tohuTowerEnemyColony(pf, ef), emperor(0)
	{}

	void dispose(int num) {
		emperor = createUnit("tohuEmperor");
		RouteCallEvent(0, emperor, new Event::CallInfo("setPos", center));
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		if(!!emperor && !GetGObj(emperor)) {
			SendKillEvent(this, this);
		}
	}

	gobj_id emperor;
};



class tohuCastleEnemyColony : public tohuEnemyColony
{
public:
	tohuCastleEnemyColony(gobj_id pf, gobj_id ef) :
		tohuEnemyColony(pf, ef)
	{}

	virtual void setLevel(int i) {
		if(!base)
			return;

		setMaxUnits(70);
		setSupplyInterval(130);
		if(i==0) {
			RouteCallEvent(0, base, new Event::CallInfo("setFirePower", float(40)));
			RouteCallEvent(0, base, new Event::CallInfo("setMaxLife", float(7000)));
		}
		else if(i==1) {
			RouteCallEvent(0, base, new Event::CallInfo("setFirePower", float(45)));
			RouteCallEvent(0, base, new Event::CallInfo("setMaxLife", float(9000)));
		}
		else if(i==2) {
			RouteCallEvent(0, base, new Event::CallInfo("setFirePower", float(50)));
			RouteCallEvent(0, base, new Event::CallInfo("setMaxLife", float(12000)));
		}
		else {
			RouteCallEvent(0, base, new Event::CallInfo("setFirePower", float(60)));
			RouteCallEvent(0, base, new Event::CallInfo("setMaxLife", float(15000)));
		}
	}

	virtual gobj_id createUnit(const std::string& name) {
		gobj_id u = tohuEnemyColony::createUnit(name);
		RouteCallEvent(0, u, new Event::CallInfo("upgrade", std::string("smallLevel3")));
		RouteCallEvent(0, u, new Event::CallInfo("upgrade", std::string("mediumLevel2")));
		return u;
	}

	void dispose(int num) {
		tohuEnemyColony::dispose(num);
		if(!base) {
			base = CreateID();
			RouteCreateEvent(new Event::CreateInfo(BUILDING, base, "tohuCastle", enemy_faction));
			RouteCallEvent(0, base, new Event::CallInfo("setPos", center));
		}
	}

	void onSupply() {
		ist::vector4 u_pos=center;
		ist::vector4 random_aim = ist::vector4(RandomAim()).normalize();
		boost::any hradius;
		gobj_ptr tw = GetGObj(base);
		selectable_ptr tg = ToSelectable(GetGObj(target));
		if(!!tw && !!tg && tw->query("getHitRadius", hradius))
			u_pos+=(tg->getPos()-center).normalize()*ToFloat(hradius);
		
		float f = GetRand();
		if(f<0.4f) {
			gobj_id u = createUnit("tohuArbalest");
			RouteCallEvent(0, u, new Event::CallInfo("setPos", u_pos));

			ist::vector4 pos = center+RandomAim()*50+random_aim*175;
			ist::vector4 aim = (ToSelectable(GetGObj(target))->getPos()-center).normalize();
			RouteActionEvent(0, u, new Event::ActionInfo(pos, selectable_ptr()));
			RouteActionEvent(0, u, new Event::ActionInfo(pos+aim*10, selectable_ptr(), Event::ActionInfo::APPEND));
			RouteCommandEvent(0, u, new Event::CommandInfo("setModeAttack", true), 700);
		}
		else if(f<0.7f) {
			gobj_id u = createUnit("tohuAssasin");
			RouteCallEvent(0, u, new Event::CallInfo("setPos", u_pos));
			RouteCallEvent(0, u, new Event::CallInfo("setMaxEnergy", float(25.0f)));
			RouteCallEvent(0, u, new Event::CallInfo("setCloak", true));
			RouteActionEvent(0, u,
				new Event::ActionInfo(center+RandomAim()*100+random_aim*200, selectable_ptr()));
		}
		else {
			gobj_id u = createUnit("tohuArmor1");
			RouteCallEvent(0, u, new Event::CallInfo("setPos", u_pos));
			RouteActionEvent(0, u,
				new Event::ActionInfo(center+RandomAim()*75+random_aim*250, selectable_ptr()));
		}
	}

	void onBaseDestroyed() {
		BaseClass::onBaseDestroyed();
		tohuEmperorEnemyColony *boss = new tohuEmperorEnemyColony(player_faction, enemy_faction);
		AddGObj(boss);
		boss->setCenter(center);
		SendCommandEvent(this, boss, new Event::CommandInfo("dispose", int(0)));
		SendCommandEvent(this, root, new Event::CommandInfo("setBossColony", colony_ptr(boss)));
	}
};







class tohuRuler : public LinearInherit2(HaveFrameCounter, Menu)
{
public:
	typedef std::list<colony_ptr> group_cont;

	tohuRuler() : level(0) {}
	void draw() const {
		char buf[64];
		sprintf(buf, "%d:%d", getFrame()/60/60, (getFrame()/60)%60);
		DrawFont(3, 20, buf);
	}


	void dispose() {
		SetCurrentWorld("tohu_kingdom");
		AddGObj(new SystemDaemon());

		GetCurrentWorld().loadGround("resource/ground.hgt");
		RouteSystemEvent(new Event::SystemInfo("loadGround", std::string("resource/ground.hgt")));
		RouteSystemEvent(new Event::SystemInfo("loadGroundTexture", std::string("resource/ground.png")));
		RouteSystemEvent(new Event::SystemInfo("enableGroundLighting", true));
		RouteSystemEvent(new Event::SystemInfo("enableGroundFog", false));


		ist::vector2 mmin = GetMapMin();
		ist::vector2 mmax = GetMapMax();
		ist::vector2 cell = (mmax-mmin)/3;
		Grid grid;
		grid.resize(3*3);
		for(int i=0; i<3; ++i) {
			for(int j=0; j<3; ++j) {
				ist::vector2 center = ist::vector2(cell.x*j, cell.y*i)+cell/2+mmin;
				grid[i*3+j].first = ist::vector4(center.x, 0, center.y);
			}
		}


		{
			nature_faction = CreateID();
			RouteCreateEvent(new Event::CreateInfo(FACTION, nature_faction, "tohuFaction"));
			RouteCallEvent(0, nature_faction, new Event::CallInfo("setPlayerName", std::string("nature")));

			{
				std::vector<ist::vector4> poses;
				for(int i=0; i<15; ++i) {
					ist::vector4 center = RandomPos();
					for(int i=0; i<40; ++i) {
						ist::vector4 pos = center+RandomVector()*200;
						if(pos.x<=GetMapMin().x || pos.x>=GetMapMax().x || pos.z<=GetMapMin().y || pos.z>=GetMapMax().y)
							continue;
						poses.push_back(pos.setY(GetHeight(pos)));
					}
				}
				std::sort(poses.begin(), poses.end(), greater_vec4<ist::vector4>());
				for(size_t i=0; i<poses.size(); ++i) {
					gobj_id u = CreateID();
					RouteCreateEvent(new Event::CreateInfo(MATERIAL, u, "tohuTimber", nature_faction));
					RouteCallEvent(0, u, new Event::CallInfo("setPos", poses[i]));
				}
			}
			for(int i=0; i<25; ++i) {
				gobj_id u = CreateID();
				RouteCreateEvent(new Event::CreateInfo(MATERIAL, u, "tohuSilicon", nature_faction));
				RouteCallEvent(0, u, new Event::CallInfo("setPos", RandomPos()));
			}
			for(int i=0; i<20; ++i) {
				gobj_id u = CreateID();
				RouteCreateEvent(new Event::CreateInfo(MATERIAL, u, "tohuIron", nature_faction));
				RouteCallEvent(0, u, new Event::CallInfo("setPos", RandomPos()));
			}
		}
		{
			player_faction = CreateID();
			RouteCreateEvent(new Event::CreateInfo(FACTION, player_faction, "tohuFaction"));
			RouteCallEvent(0, player_faction, new Event::CallInfo("setPlayerName", std::string("player")));
			RouteCallEvent(0, player_faction, new Event::CallInfo("setTimber", float(1000)));
			RouteCallEvent(0, player_faction, new Event::CallInfo("setSilicon", float(1000)));
			RouteCallEvent(0, player_faction, new Event::CallInfo("setIron", float(1000)));
			RouteCallEvent(0, player_faction, new Event::CallInfo("setEnergy", float(1000)));
			SetPlayerFaction(ToFaction(GetGObj(player_faction)));
		//	RouteSystemEvent(new Event::SystemInfo("setPlayerFaction", player_faction));

			player_center = getEmptyGrid(grid);
			SetCameraPos(player_center);
			{
				player_town = CreateID();
				RouteCreateEvent(new Event::CreateInfo(BUILDING, player_town, "tohuTownCenter", player_faction));
				RouteCallEvent(0, player_town, new Event::CallInfo("setPos", player_center));
			}
			for(int i=0; i<8; ++i) {
				static ist::vector4 ps[8] = {
					ist::vector4(75, 0, 150),
					ist::vector4(-75, 0, 150),
					ist::vector4(150, 0, 300),
					ist::vector4(-150, 0, 300),
					ist::vector4(100, 0, -150),
					ist::vector4(-100, 0, -150),
					ist::vector4(250, 0, 0),
					ist::vector4(-250, 0, 0),
				};

				gobj_id u = CreateID();
				if(i<2)
					RouteCreateEvent(new Event::CreateInfo(BUILDING, u, "tohuFactoryS", player_faction));
				else if(i<4)
					RouteCreateEvent(new Event::CreateInfo(BUILDING, u, "tohuFactoryM", player_faction));
				else if(i<6)
					RouteCreateEvent(new Event::CreateInfo(BUILDING, u, "tohuPowerPlant", player_faction));
				else if(i<8)
					RouteCreateEvent(new Event::CreateInfo(BUILDING, u, "tohuTower", player_faction));
				RouteCallEvent(0, u, new Event::CallInfo("setPos", player_center+ps[i]));
			}
			for(int i=0; i<5; ++i) {
				gobj_id u = CreateID();
				RouteCreateEvent(new Event::CreateInfo(UNIT, u, "tohuWorker", player_faction));
				RouteCallEvent(0, u, new Event::CallInfo("setPos", player_center));
				RouteActionEvent(0, u, new Event::ActionInfo(player_center+ist::vector4(0, 0, -150), selectable_ptr()));
			}

			RouteCallEvent(0, player_faction,
				new Event::CallInfo("pushMessage", std::string("FG̏j󂵁Ac|")));
			RouteCallEvent(0, player_faction,
				new Event::CallInfo("pushMessage", std::string("skFR̎s̊ח")));
		}

		{
			enemy_faction = CreateID();
			RouteCreateEvent(new Event::CreateInfo(FACTION, enemy_faction, "tohuFaction"));
			RouteCallEvent(0, enemy_faction, new Event::CallInfo("setPlayerName", std::string("enemy")));
			for(int i=0; i<4; ++i) {
				colony_ptr m;
				if(i<2) m = new tohuAssasinEnemyColony(player_faction, enemy_faction);
				else    m = new tohuSoldierEnemyColony(player_faction, enemy_faction);
				AddGObj(m);
				ist::vector4 c = getEmptyGrid(grid);
				c.x+=(cell.x*GetRand()-cell.x/2)*0.7f;
				c.z+=(cell.y*GetRand()-cell.y/2)*0.7f;
				m->setCenter(c);
				m->setTarget(player_town);
				m->setRoot(this);
				groups.push_back(m);
				SendCommandEvent(this, m, new Event::CallInfo("dispose", int(0)));
			}
			SendCommandEvent(this, groups.front(), new Event::CallInfo("setFlagActive", true));
			{
				boss_group = new tohuCastleEnemyColony(player_faction, enemy_faction);
				AddGObj(boss_group);
				boss_group->setCenter(getEmptyGrid(grid));
				boss_group->setTarget(player_town);
				boss_group->setRoot(this);
				SendCommandEvent(this, boss_group, new Event::CallInfo("dispose", int(0)));
			}
		}

		setLevel(0);
	}

	void disposeMuddy() {
		dispose();

		for(group_cont::iterator p=groups.begin(); p!=groups.end(); ++p) {
			SendCommandEvent(this, (*p), new Event::CommandInfo("dispose", int(50)));
		}
		SendCommandEvent(this, boss_group, new Event::CommandInfo("dispose", int(30)));

		for(int i=0; i<200; ++i) {
			gobj_id u = CreateID();
			RouteCreateEvent(new Event::CreateInfo(UNIT, u, "tohuSoldier1", player_faction));
			RouteCallEvent(0, u, new Event::CallInfo("setPos", RandomPos()));
		}
		for(int i=0; i<175; ++i) {
			gobj_id u = CreateID();
			RouteCreateEvent(new Event::CreateInfo(UNIT, u, "tohuArmor1", player_faction));
			RouteCallEvent(0, u, new Event::CallInfo("setPos", RandomPos()));
		}
		for(int i=0; i<200; ++i) {
			gobj_id u = CreateID();
			RouteCreateEvent(new Event::CreateInfo(UNIT, u, "tohuLaser1", player_faction));
			RouteCallEvent(0, u, new Event::CallInfo("setPos", RandomPos()));
		}
		for(int i=0; i<175; ++i) {
			gobj_id u = CreateID();
			RouteCreateEvent(new Event::CreateInfo(UNIT, u, "tohuAssasin", player_faction));
			RouteCallEvent(0, u, new Event::CallInfo("setPos", RandomPos()));
		}
		for(int i=0; i<100; ++i) {
			gobj_id u = CreateID();
			RouteCreateEvent(new Event::CreateInfo(UNIT, u, "tohuBomber1", player_faction));
			RouteCallEvent(0, u, new Event::CallInfo("setPos", RandomPos()));
		}
	}

	void versusEmperor() {
		SetCurrentWorld("tohu_kingdom");
		AddGObj(new SystemDaemon());

		GetCurrentWorld().loadGround("resource/ground.hgt");
		RouteSystemEvent(new Event::SystemInfo("loadGround", std::string("resource/ground.hgt")));
		RouteSystemEvent(new Event::SystemInfo("loadGroundTexture", std::string("resource/ground.png")));
		RouteSystemEvent(new Event::SystemInfo("enableGroundLighting", true));
		RouteSystemEvent(new Event::SystemInfo("enableGroundFog", false));
	}


	bool call(const std::string& name, boost::any& value) {
		if(name=="dispose")
			dispose();
		else if(name=="disposeMuddy")
			disposeMuddy();
		else if(name=="versusEmperor")
			versusEmperor();
		else if(name=="victory") {
			RouteCallEvent(0, player_faction, new Event::CallInfo("pushMessage", std::string("܂I")));
			StopSendingUpdateEvent();
		}
		else if(name=="defeat") {
			RouteCallEvent(0, player_faction, new Event::CallInfo("pushMessage", std::string("sk܂I")));
			StopSendingUpdateEvent();
		}
		else if(name=="setBossColony" && value.type()==typeid(colony_ptr)) {
			boss_group = boost::any_cast<colony_ptr>(value);
		}
		else if(name=="changeActive") {
			for(group_cont::iterator p=groups.begin(); p!=groups.end(); ++p) {
				if(!(*p)->isDead()) {
					SendCallEvent(this, *p, new Event::CallInfo("setFlagActive", true));
					break;
				}
			}
		}
		else {
			return BaseClass::call(name, value);
		}
		return true;
	}


	void tohuRuler::onUpdate(Event& e) {
		BaseClass::onUpdate(e);

		if(!!boss_group && boss_group->isDead()) {
			boss_group = colony_ptr();
			SendCommandEvent(this, this, new Event::CommandInfo("victory", boost::any()), 180);
		}
		else if(!!player_town && !GetGObj(player_town)) {
			player_town = 0;
			SendCommandEvent(this, this, new Event::CommandInfo("defeat", boost::any()), 180);
		}

		for(group_cont::iterator p=groups.begin(); p!=groups.end();) {
			if((*p)->isDead()) {
				groups.erase(p++);
				setLevel(getLevel()+1);
			}
			else
				++p;
		}
	}

	int getLevel() const { return level; }
	void setLevel(int l) {
		level = l;
		for(group_cont::iterator p=groups.begin(); p!=groups.end(); ++p)
			SendCommandEvent(this, *p, new Event::CommandInfo("setLevel", int(level)));
		if(!!boss_group)
			SendCommandEvent(this, boss_group, new Event::CommandInfo("setLevel", int(level)));
	}

private:
	gobj_id nature_faction;
	gobj_id player_faction;
	gobj_id player_town;
	ist::vector4 player_center;

	gobj_id enemy_faction;
	group_cont groups;
	colony_ptr boss_group;
	int level;
};










class DirectoryList : public std::vector<std::string>
{
public:
	DirectoryList() {}
	DirectoryList(const std::string& path) { get(path); }
	bool get(const std::string& path) {
#ifdef WIN32
		WIN32_FIND_DATA wfdata;
		HANDLE handle = FindFirstFile((path+"/*").c_str(), &wfdata);
		if(handle!=INVALID_HANDLE_VALUE) {
			do {
				if(strcmp(wfdata.cFileName, ".")!=0 && strcmp(wfdata.cFileName, "..")!=0)
					push_back(path+"/"+wfdata.cFileName);
			} while(FindNextFile(handle, &wfdata));
			FindClose(handle);
		}
		else {
			return false;
		}
#else
		DIR *dir = opendir(path.c_str());
		if(dir!=0) {
			dirent *dr;
			while((dr = readdir(dir))!=0) {
				if(strcmp(dr->d_name, ".")!=0 && strcmp(dr->d_name, "..")!=0)
					push_back(path+"/"+dr->d_name);
			}
			closedir(dir);
		}
		else {
			return false;
		}
#endif
		return true;
	}
};




class ReplayMenu : public LinearInherit2(HaveFrameCounter, Menu)
{
public:
	class FactionButton : public Button
	{
	public:
		FactionButton(faction_ptr f) : faction(f) {
			setColorOver(ist::Color(1, 1, 1, 0.5f));
			setRect(ist::vector2(60, 16));
		}
		void draw() const {
		//	if(GetPlayerFaction()==faction)
		//		setColor(ist::Color(1, 1, 1, 0.5f));
		//	else
		//		setColor(ist::Color(0, 0, 0, 0.5f));
			Button::draw();
			const ist::vector2& pos = getPos();
			faction->getFactionColor().assign();
			DrawFont((int)pos.x+5, (int)pos.y+3, faction->getPlayerName().c_str());
			ist::Color().assign();
		}
		void action() {
			SetPlayerFaction(faction);
		}

		faction_ptr faction;
	};

	class SightButton : public Button
	{
	public:
		SightButton() : sight(true) {
			setColor(ist::Color(0, 0, 0, 0.5f));
			setColorOver(ist::Color(1, 1, 1, 0.5f));
			setRect(ist::vector2(40, 16));
		}
		void draw() const {
		//	if(sight)
		//		setColor(ist::Color(0, 0, 0, 0.5f));
		//	else
		//		setColor(ist::Color(1, 1, 1, 0.5f));
			Button::draw();
			const ist::vector2& pos = getPos();
			if(sight)
				DrawFont((int)pos.x+3, (int)pos.y+3, "EOff");
			else
				DrawFont((int)pos.x+3, (int)pos.y+3, "EOn");
		}
		void action() {
			sight = !sight;
			GetCurrentWorld().enableSight(sight);
		}

		bool sight;
	};

	class NoWaitButton : public Button
	{
	public:
		NoWaitButton() : nowait(false) {
			setColor(ist::Color(0, 0, 0, 0.5f));
			setColorOver(ist::Color(1, 1, 1, 0.5f));
			setRect(ist::vector2(50, 16));
		}
		void draw() const {
		//	if(nowait)
		//		setColor(ist::Color(1, 1, 1, 0.5f));
		//	else
		//		setColor(ist::Color(0, 0, 0, 0.5f));
			Button::draw();
			const ist::vector2& pos = getPos();
			if(nowait)
				DrawFont((int)pos.x+3, (int)pos.y+3, "Off");
			else
				DrawFont((int)pos.x+3, (int)pos.y+3, "On");
		}
		void action() {
			nowait = !nowait;
			GetIGame().enableNoWait(nowait);
		}

		bool nowait;
	};


	ReplayMenu() {}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		if(getFrame()==5)
			init();
	}

	void init() {
		{
			Button *s = new SightButton();
			AddGObj(s);
			s->setPos(ist::vector2(585, 15));
		}
		{
			Button *w = new NoWaitButton();
			AddGObj(w);
			w->setPos(ist::vector2(580, 33));
		}
		int i = 0;
		IGObjIterator& iter = GetFactions();
		while(iter.hasNext()) {
			gobj_ptr p = iter.iterate();
			++i;
			if(i==1)
				continue; // 厩Rfaction 
			Button *b = new FactionButton(ToFaction(p));
			AddGObj(b);
			b->setPos(ist::vector2(575, 55+float(i-2)*20));
		}
	}
};









class NormalBattleButton : public Button
{
public:
	NormalBattleButton() {
		setTexture("resource/button_up.png");
		setTextureOver("resource/button_over.png");
		setRect(ist::vector2(120, 30));
	}
	void draw() const {
		Button::draw();
		const ist::vector2& pos = getPos();
		DrawFont((int)pos.x+45, (int)pos.y+10, "ʏ퓬");
	}
	void action() {
		SetCurrentWorld("tohu_kingdom");
		menu_ptr r = new tohuRuler();
		AddGObj(r);
		SendCommandEvent(this, r, new Event::CommandInfo("dispose", boost::any()));
	}
};

class MuddyBattleButton : public Button
{
public:
	MuddyBattleButton() {
		setTexture("resource/button_up.png");
		setTextureOver("resource/button_over.png");
		setRect(ist::vector2(120, 30));
	}
	void draw() const {
		Button::draw();
		const ist::vector2& pos = getPos();
		DrawFont((int)pos.x+45, (int)pos.y+10, "");
	}
	void action() {
		SetCurrentWorld("tohu_kingdom");
		menu_ptr r = new tohuRuler();
		AddGObj(r);
		SendCommandEvent(this, r, new Event::CommandInfo("disposeMuddy", boost::any()));
	}
};

class VSEmperorButton : public Button
{
public:
	VSEmperorButton() {
		setTexture("resource/button_up.png");
		setTextureOver("resource/button_over.png");
		setRect(ist::vector2(120, 30));
	}
	void draw() const {
		Button::draw();
		const ist::vector2& pos = getPos();
		DrawFont((int)pos.x+45, (int)pos.y+10, "΍c");
	}
	void action() {
		SetCurrentWorld("tohu_kingdom");
		menu_ptr r = new tohuRuler();
		AddGObj(r);
		SendCommandEvent(this, r, new Event::CommandInfo("versusEmperor", boost::any()));
	}
};


class ReplayButton : public Button
{
public:
	class SelectButton : public Button
	{
	public:
		SelectButton(const std::string p) : path(p) {
			setTexture("resource/button_up.png");
			setTextureOver("resource/button_over.png");
			setRect(ist::vector2(120, 30));
		}
		void draw() const {
			Button::draw();
			const ist::vector2& pos = getPos();
			DrawFont((int)pos.x+5, (int)pos.y+10, path.c_str());
		}
		void action() {
			SetCurrentWorld("tohu_kingdom");
			AddGObj(new SystemDaemon());
			AddGObj(new ReplayMenu());
			SetWorldMode(REPLAY);
			if(!LoadReplay(path)) {
				assert(0 && "loading replay file failed");
			}
		}

		std::string path;
	};


	ReplayButton() {
		setTexture("resource/button_up.png");
		setTextureOver("resource/button_over.png");
		setRect(ist::vector2(120, 30));
	}
	void draw() const {
		Button::draw();
		const ist::vector2& pos = getPos();
		DrawFont((int)pos.x+25, (int)pos.y+10, "L^ꂽQ[");
	}
	void action() {
		SetCurrentWorld("replay_list");
		DirectoryList dir("savegame");
		for(size_t i=0; i<dir.size(); ++i) {
			Button *b = new SelectButton(dir[i]);
			AddGObj(b);
			b->setPos(ist::vector2(10+float(i/11)*140, 10+float(i%11)*40));
		}
	}
};



button_ptr CreateNormalBattleButton() { return new NormalBattleButton(); }
button_ptr CreateMuddyBattleButton()  { return new MuddyBattleButton(); }
button_ptr CreateReplayButton()     { return new ReplayButton(); }

