#include <sstream>
#include "tohu_ruler.h"




class NetRuler : public LinearInherit2(HaveFrameCounter, Menu)
{
public:
	NetRuler() {
		SetCurrentWorld("tohu_kingdom");
		AddGObj(new SystemDaemon());
		GetCurrentWorld().loadGround("resource/ground.hgt");
	}

	void draw() const {
		char buf[64];
		sprintf(buf, "%d:%d", getFrame()/60/60, (getFrame()/60)%60);
		DrawFont(300, 25, buf);
	}
};

class ServerRuler : public NetRuler
{
public:
	ServerRuler() {
		SetWorldMode(SERVER);
	}

	void onInit(Event& e) {
		BaseClass::onInit(e);

		RouteSystemEvent(new Event::SystemInfo("loadGround", std::string("resource/ground.hgt")));
		RouteSystemEvent(new Event::SystemInfo("loadGroundTexture", std::string("resource/ground.png")));
		RouteSystemEvent(new Event::SystemInfo("enableGroundLighting", true));
		RouteSystemEvent(new Event::SystemInfo("enableGroundFog", false));

		gobj_id nature = CreateID();
		RouteCreateEvent(new Event::CreateInfo(FACTION, nature, "tohuFaction"));
		{
			{
				std::vector<ist::vector4> poses;
				for(int i=0; i<20; ++i) {
					ist::vector4 center = RandomPos();
					for(int i=0; i<20; ++i) {
						ist::vector4 pos = center+RandomVector()*150;
						if(pos.x<=GetMapMin().x || pos.x>=GetMapMax().x || pos.z<=GetMapMin().y || pos.z>=GetMapMax().y)
							continue;
						poses.push_back(pos.setY(GetHeight(pos)));
					}
				}
				std::sort(poses.begin(), poses.end(), greater_vec4<ist::vector4>());
				for(size_t i=0; i<poses.size(); ++i) {
					gobj_id u = CreateID();
					RouteCreateEvent(new Event::CreateInfo(MATERIAL, u, "tohuTimber", nature));
					RouteCallEvent(0, u, new Event::CallInfo("setPos", poses[i]));
				}
			}
			for(int i=0; i<25; ++i) {
				gobj_id u = CreateID();
				RouteCreateEvent(new Event::CreateInfo(MATERIAL, u, "tohuSilicon", nature));
				RouteCallEvent(0, u, new Event::CallInfo("setPos", RandomPos()));
			}
			for(int i=0; i<20; ++i) {
				gobj_id u = CreateID();
				RouteCreateEvent(new Event::CreateInfo(MATERIAL, u, "tohuIron", nature));
				RouteCallEvent(0, u, new Event::CallInfo("setPos", RandomPos()));
			}
		}

		ist::vector4 tcpos[8] = {
			ist::vector4(1000,0,0),
			ist::vector4(-1000,0,0),
			ist::vector4(0,0,1000),
			ist::vector4(0,0,-1000),
			ist::vector4(1000,0,1000),
			ist::vector4(-1000,0,-1000),
			ist::vector4(1000,0,-1000),
			ist::vector4(-1000,0,1000),
		};

		gobj_id f[8];
		for(int i=0; i<8; ++i) {
			f[i] = CreateID();
			RouteCreateEvent(new Event::CreateInfo(FACTION, f[i], "tohuFaction"));
			RouteCallEvent(0, f[i], new Event::CallInfo("setPlayerName", std::string("no player")));
			RouteCallEvent(0, f[i], new Event::CallInfo("setTimber", float(500)));
			RouteCallEvent(0, f[i], new Event::CallInfo("setSilicon", float(300)));
			RouteCallEvent(0, f[i], new Event::CallInfo("setIron", float(300)));
			RouteCallEvent(0, f[i], new Event::CallInfo("setEnergy", float(0)));
			if(i==0)
				RouteSystemEvent(new Event::SystemInfo("setPlayerFaction", f[i]));

			gobj_id tc = CreateID();
			RouteCreateEvent(new Event::CreateInfo(BUILDING, tc, "tohuTownCenter", f[i]));
			RouteCallEvent(0, tc, new Event::CallInfo("setPos", tcpos[i]));

			ist::vector4 fpos[3] = {
				ist::vector4(-100,0,100),
				ist::vector4(-100,0,-100),
				ist::vector4(150,0,0),
			};
			std::string bname[3] = {
				std::string("tohuPowerPlant"),
				std::string("tohuFactoryS"),
				std::string("tohuTower"),
			};
			for(int j=0; j<3; ++j) {
				gobj_id b = CreateID();
				RouteCreateEvent(new Event::CreateInfo(BUILDING, b, bname[j], f[i]));
				RouteCallEvent(0, b, new Event::CallInfo("setPos", tcpos[i]+fpos[j]));
			}
		}
		for(int i=0; i<8; ++i) {
			for(int j=0; j<8; ++j) {
				if(i!=j && ((i%2==0 && j%2==0) || (i%2==1 && j%2==1)))
					RouteCallEvent(0, f[i], new Event::CallInfo("addAlly", f[j]));
			}
		}
	}
};


class ClientRuler : public NetRuler
{
public:
	ClientRuler() {
		SetWorldMode(CLIENT);
	}
};





class WarawaraColony : public LinearInherit2(HaveFrameCounter, Menu)
{
public:
	WarawaraColony(gobj_id f, const ist::vector4& p) :
		pos(p), faction(f), castle(0), supply_interval(130), max_units(80) {}

	void onInit(Event& e) {
		BaseClass::onInit(e);

		{
			castle = CreateID();
			RouteCreateEvent(new Event::CreateInfo(BUILDING, castle, "tohuCastle", faction));
			RouteCallEvent(0, castle, new Event::CallInfo("setPos", pos));
		}
	}

	void supply() {
		static std::vector<std::string> classname;
		if(classname.empty()) {
			classname.push_back("tohuSoldier1");
			classname.push_back("tohuSoldier1");
			classname.push_back("tohuSoldier1");
			classname.push_back("tohuLaser1");
			classname.push_back("tohuLaser1");
			classname.push_back("tohuLaser1");
			classname.push_back("tohuArmor1");
			classname.push_back("tohuArmor1");
			classname.push_back("tohuArmor1");
			classname.push_back("tohuBomber1");
			classname.push_back("tohuAssasin");
			classname.push_back("tohuDetector");
			classname.push_back("tohuSoldier1");
			classname.push_back("tohuLaser1");
			classname.push_back("tohuArmor1");
			classname.push_back("tohuArmor3");
		}

		gobj_id u = CreateID();
		RouteCreateEvent(new Event::CreateInfo(UNIT, u, classname[rand()%classname.size()], faction));
		RouteCallEvent(0, u, new Event::CallInfo("setPos", pos));
		RouteActionEvent(0, u,
			new Event::ActionInfo(pos+ist::vector4((GetRand()-0.5f)*900, 0, (GetRand()-0.5f)*900), selectable_ptr()));
		units.push_back(u);
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);

		building_ptr c = ToBuilding(GetGObj(castle));
		if(!c) {
			SendKillEvent(this, this);
			return;
		}

		for(gobj_cont::iterator p=units.begin(); p!=units.end();) {
			if(!GetGObj(*p))
				units.erase(p++);
			else
				++p;
		}

		if(units.size()<max_units && getFrame()%supply_interval==0) { // 莞Ԓuspawn 
			supply();
		}
		else if(!units.empty() && getFrame()>28800 && getFrame()%180==0) { // 8ȏoĂUjbg쐬 
			gobj_id g = units.front();
			units.pop_front();
			if(gobj_ptr u = GetGObj(g)) {
				const std::string& cn = ToUnit(u)->getClassName();
				if(getFrame()<50400) { // ʃjbg͂ƌɂȂȂƏoȂ 
					if(cn=="tohuArmor3" || cn=="tohuArmor2" || cn=="tohuLaser2" || cn=="tohuSoldier2")
						return;
				}
				ist::vector4 pos = ToUnit(u)->getPos();
				pos.z+=1500;
				RouteActionEvent(0, g, new Event::ActionInfo(pos, selectable_ptr(), Event::ActionInfo::ACTIVE));
			}
		}
	}

	typedef std::list<gobj_id> gobj_cont;
	ist::vector4 pos;
	gobj_id faction;
	gobj_id castle;
	gobj_cont units;
	size_t supply_interval;
	size_t max_units;
};

class WarawaraBossColony : public Menu
{
public:
	class ServantInfo
	{
	public:
		ServantInfo(const std::string& cn) : sv_id(0), wait(0), classname(cn) {}
		gobj_id sv_id;
		int wait;
		std::string classname;
	};


	WarawaraBossColony(gobj_id f, const ist::vector4& p) : pos(p), faction(f), emperor(0) {}

	void onInit(Event& e) {
	//	emperor = CreateID();
	//	RouteCreateEvent(new Event::CreateInfo(UNIT, emperor, "tohuEmperor", faction));
	//	RouteCallEvent(0, emperor, new Event::CallInfo("setPos", pos));

		ist::vector4 upos[4] = {
			ist::vector4(50,0,50), ist::vector4(-50,0,50),
			ist::vector4(120,0,0), ist::vector4(-120,0,0),
		};
		std::string classname[4] = {
			"tohuSoldier3", "tohuSoldier3",
			"tohuArmor3", "tohuArmor3"
		};
		for(int i=0; i<4; ++i) {
			gobj_id u = CreateID();
			RouteCreateEvent(new Event::CreateInfo(UNIT, u, classname[i], faction));
			RouteCallEvent(0, u, new Event::CallInfo("setPos", pos+upos[i]));
			RouteCallEvent(0, u, new Event::CallInfo("setMaxLife", float(20000)));

			servants.push_back(ServantInfo(classname[i]));
			servants.back().sv_id = u;
		}
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		SendKillEvent(this, this);
/*
		unit_ptr em = ToUnit(GetGObj(emperor));
		if(!em) {
			SendKillEvent(this, this);
			return;
		}

		for(std::vector<ServantInfo>::iterator p=servants.begin(); p!=servants.end(); ++p) {
			gobj_ptr gp = GetGObj(p->sv_id);
			if(!gp && ++p->wait==1200) {
				p->wait = 0;
				p->sv_id = CreateID();

				RouteCreateEvent(new Event::CreateInfo(UNIT, p->sv_id, p->classname, faction));
				RouteCallEvent(0, p->sv_id, new Event::CallInfo("setPos", em->getPos()));
			}
		}
*/
	}


	ist::vector4 pos;
	gobj_id faction;
	gobj_id emperor;
	std::vector<ServantInfo> servants;
};

class ServerWarawararRuler : public NetRuler
{
public:
	ServerWarawararRuler() {
		SetWorldMode(SERVER);
	}

	void onInit(Event& e) {
		BaseClass::onInit(e);

		RouteSystemEvent(new Event::SystemInfo("loadGround", std::string("resource/ground.hgt")));
		RouteSystemEvent(new Event::SystemInfo("loadGroundTexture", std::string("resource/ground.png")));
		RouteSystemEvent(new Event::SystemInfo("enableGroundLighting", true));
		RouteSystemEvent(new Event::SystemInfo("enableGroundFog", false));

		// ʒu 
		gobj_id nature = CreateID();
		RouteCreateEvent(new Event::CreateInfo(FACTION, nature, "tohuFaction"));
		{
			// X 
			{
				std::vector<ist::vector4> poses;
				for(int i=0; i<20; ++i) {
					ist::vector4 center = RandomPos();
					for(int i=0; i<25; ++i) {
						ist::vector4 pos = center+RandomVector()*150;
						if(pos.x<=GetMapMin().x || pos.x>=GetMapMax().x || pos.z<=GetMapMin().y || pos.z>=GetMapMax().y)
							continue;
						poses.push_back(pos.setY(GetHeight(pos)));
					}
				}
				std::sort(poses.begin(), poses.end(), greater_vec4<ist::vector4>());
				for(size_t i=0; i<poses.size(); ++i) {
					gobj_id u = CreateID();
					RouteCreateEvent(new Event::CreateInfo(MATERIAL, u, "tohuTimber", nature));
					RouteCallEvent(0, u, new Event::CallInfo("setPos", poses[i]));
				}
			}
			// ΂u 
			for(int i=0; i<40; ++i) {
				gobj_id u = CreateID();
				RouteCreateEvent(new Event::CreateInfo(MATERIAL, u, "tohuSilicon", nature));
				RouteCallEvent(0, u, new Event::CallInfo("setPos", RandomPos()));
			}
			// Su 
			for(int i=0; i<30; ++i) {
				gobj_id u = CreateID();
				RouteCreateEvent(new Event::CreateInfo(MATERIAL, u, "tohuIron", nature));
				RouteCallEvent(0, u, new Event::CallInfo("setPos", RandomPos()));
			}
		}

		// u 
		ist::vector4 tcpos[5] = {
			ist::vector4(1000, 0,1000),
			ist::vector4(500,  0,1000),
			ist::vector4(0,    0,1000),
			ist::vector4(-500, 0,1000),
			ist::vector4(-1000,0,1000),
		};
		gobj_id f[5];
		for(int i=0; i<5; ++i) {
			f[i] = CreateID();
			RouteCreateEvent(new Event::CreateInfo(FACTION, f[i], "tohuFaction"));
			RouteCallEvent(0, f[i], new Event::CallInfo("setPlayerName", std::string("no player")));
			RouteCallEvent(0, f[i], new Event::CallInfo("setTimber", float(500)));
			RouteCallEvent(0, f[i], new Event::CallInfo("setSilicon", float(300)));
			RouteCallEvent(0, f[i], new Event::CallInfo("setIron", float(300)));
		//	RouteCallEvent(0, f[i], new Event::CallInfo("setTimber", float(100000)));
		//	RouteCallEvent(0, f[i], new Event::CallInfo("setSilicon", float(100000)));
		//	RouteCallEvent(0, f[i], new Event::CallInfo("setIron", float(100000)));
			RouteCallEvent(0, f[i], new Event::CallInfo("setEnergy", float(0)));
			if(i==0)
				RouteSystemEvent(new Event::SystemInfo("setPlayerFaction", f[i]));

			gobj_id tc = CreateID();
			RouteCreateEvent(new Event::CreateInfo(BUILDING, tc, "tohuTownCenter", f[i]));
			RouteCallEvent(0, tc, new Event::CallInfo("setPos", tcpos[i]));

			ist::vector4 fpos[3] = {
				ist::vector4(-100,0,100),
				ist::vector4(-100,0,-100),
				ist::vector4(150,0,0),
			};
			std::string bname[3] = {
				std::string("tohuPowerPlant"),
				std::string("tohuFactoryS"),
				std::string("tohuTower"),
			};
			for(int j=0; j<3; ++j) {
				gobj_id b = CreateID();
				RouteCreateEvent(new Event::CreateInfo(BUILDING, b, bname[j], f[i]));
				RouteCallEvent(0, b, new Event::CallInfo("setPos", tcpos[i]+fpos[j]));
			}
		}
		for(int i=0; i<5; ++i) {
			for(int j=0; j<5; ++j) {
				if(i!=j)
					RouteCallEvent(0, f[i], new Event::CallInfo("addAlly", f[j]));
			}
		}

		// NPCGu 
		gobj_id enemy = CreateID();
		RouteCreateEvent(new Event::CreateInfo(FACTION, enemy, "tohuFaction"));
		RouteCallEvent(0, enemy, new Event::CallInfo("setPlayerName", std::string("enemy")));
		AddGObj(new WarawaraColony(enemy, ist::vector4(1000, 0,-800))); // T[o[݂̂ɔzu 
		AddGObj(new WarawaraColony(enemy, ist::vector4(334,  0,-800)));
		AddGObj(new WarawaraColony(enemy, ist::vector4(-330, 0,-800)));
		AddGObj(new WarawaraColony(enemy, ist::vector4(-1000,0,-800)));
	//	AddGObj(new WarawaraBossColony(enemy, ist::vector4(0,0,-1000)));
	}
};





class ServerBattleButton : public Button
{
public:
	ServerBattleButton() {
		setTexture("resource/button_up.png");
		setTextureOver("resource/button_over.png");
		setRect(ist::vector2(120, 30));
	}
	void draw() const {
		Button::draw();
		const ist::vector2& pos = getPos();
		DrawFont((int)pos.x+45, (int)pos.y+10, "ΐ");
	}

	void action() {
		AddGObj(new ServerRuler());
	}
};

class ServerWarawararButton : public Button
{
public:
	ServerWarawararButton() {
		setTexture("resource/button_up.png");
		setTextureOver("resource/button_over.png");
		setRect(ist::vector2(120, 30));
	}
	void draw() const {
		Button::draw();
		const ist::vector2& pos = getPos();
		DrawFont((int)pos.x+45, (int)pos.y+10, "COM");
	}

	void action() {
		AddGObj(new ServerWarawararRuler());
	}
};



class ServerSelectButton : public Button
{
public:
	class SelectButton : public Button
	{
	public:
		SelectButton(const std::string n, const std::string& a, unsigned short p) : name(n), address(a), port(p) {
			setTexture("resource/button_up.png");
			setTextureOver("resource/button_over.png");
			setRect(ist::vector2(120, 30));
		}
		void draw() const {
			Button::draw();
			const ist::vector2& pos = getPos();
			DrawFont((int)pos.x+5, (int)pos.y+10, name.c_str());
		}
		void action() {
			SetServerAddress(address);
			SetServerPort(port);
			AddGObj(new ClientRuler());
		}

		std::string name;
		std::string address;
		unsigned short port;
	};


	ServerSelectButton() {
		setTexture("resource/button_up.png");
		setTextureOver("resource/button_over.png");
		setRect(ist::vector2(120, 30));
	}
	void draw() const {
		Button::draw();
		const ist::vector2& pos = getPos();
		DrawFont((int)pos.x+35, (int)pos.y+10, "T[o[ꗗ");
	}
	void action() {
		SetCurrentWorld("server_list");

		ist::HTTP h;
		h.post(GetServerDatabase(), "mode=list");
		if(!!h) {
			std::stringstream ss(h.getContent().str());
			std::string l;
			int i = 0;
			while(std::getline(ss, l)) {
				char name[128], address[64];
				int port = 0;
				if(sscanf(l.c_str(), "%[^,], %[^,], %d", name, address, &port)==3) {
					++i;
					Button *b = new SelectButton(name, address, unsigned short(port));
					AddGObj(b);
					b->setPos(ist::vector2(10+float(i/11)*140, 10+float(i%11)*40));
				}
			}
		}
	}
};





class ServerButton : public Button
{
public:
	ServerButton() {
		setTexture("resource/button_up.png");
		setTextureOver("resource/button_over.png");
		setRect(ist::vector2(120, 30));
	}
	void draw() const {
		Button::draw();
		const ist::vector2& pos = getPos();
		DrawFont((int)pos.x+45, (int)pos.y+10, "T[o[");
	}
	void action() {
		SetCurrentWorld("multiplayscenario");

		button_ptr b = new ServerBattleButton();
		b->setPos(ist::vector2(260, 300));
		AddGObj(b);

		b = new ServerWarawararButton();
		b->setPos(ist::vector2(260, 360));
		AddGObj(b);
	}
};

class ClientButton : public Button
{
public:
	ClientButton() {
		setTexture("resource/button_up.png");
		setTextureOver("resource/button_over.png");
		setRect(ist::vector2(120, 30));
	}
	void draw() const {
		Button::draw();
		const ist::vector2& pos = getPos();
		DrawFont((int)pos.x+45, (int)pos.y+10, "NCAg");
	}
	void action() {
		AddGObj(new ClientRuler());
	}
};



button_ptr CreateServerButton() { return new ServerButton(); }
button_ptr CreateClientButton() { return new ClientButton(); }
button_ptr CreateServerSelectButton() { return new ServerSelectButton(); }

