#include "tohu_unit_common.h"






class Explosion : public BBullet
{
public:
	class Dome : public LinearInherit3(HavePolygonModel, HaveFrameCounter, Particle)
	{
	public:
		Dome() : opa(0.08f), fade(0), ext(0.0125f, 0.0125f, 0.0125f) {
			setPModel("resource/sphere.mqo");
		}

		void onUpdate(Event& e) {
			BaseClass::onUpdate(e);

			setScl(getScl()+ext);
			opa-=fade;
			if(opa<=0)
				SendKillEvent(this, this);
		}

		void setLight(ist::Light&) const {}

		void draw() const {
			ist::Color col;
			col.color = ist::vector4(1.0f, 0.25f, 0.25f, opa);
			col.assign();
			glBlendFunc(GL_SRC_ALPHA, GL_ONE);
			glDepthMask(GL_FALSE);
			ist::vector4 s = getScl();
			BaseClass::draw();
		//	setScl(s+ist::vector4(0.1f, 0.1f, 0.1f));
		//	BaseClass::draw();
		//	setScl(s+ist::vector4(0.2f, 0.2f, 0.2f));
		//	BaseClass::draw();
		//	setScl(s);
			glDepthMask(GL_TRUE);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			ist::Color().assign();
		}

		float opa;
		float fade;
		ist::vector4 ext;
	};

	struct Fragment : public SimplePModel
	{
		ist::vector4 pos, vel, ax;
		ist::vector4 diffuse;
		float rot, rot_speed;
		int frame, lifetime;

		Fragment(const ist::vector4& p, const ist::vector4& v) : pos(p), vel(v), rot(0), rot_speed(0), frame(0) {
			pmodel = GetPModel("resource/rock.mqo");
			ax = RandomVector().normalize();
			diffuse = ist::vector4(0.8f, 0.6f, 0.4f);
			rot_speed = GetRand()*3+1;
			lifetime = 60+int(GetRand()*60);
			pos.y = GetHeight(pos)-30;
		}

		void calcMatrix(ist::matrix44& mat) const {
			mat.translate(pos.v);
			mat.rotateA(ax, rot);
		}

		bool isDead() const {
			return diffuse.w<=0;
		}

		void update() {
			++frame;
			rot+=rot_speed;
			pos+=vel;

			if(frame>lifetime)
				diffuse.w-=0.02f;
		}

		void draw() const {
			glMaterialfv(GL_FRONT, GL_DIFFUSE,  diffuse.v);
			drawModel();
			ist::Material().assign();
		}
	};


	Explosion(const ist::vector4& pos) {
		setPos(pos);
		dome = new Dome();
		AddGObj(dome);
		dome->setPos(pos);
		dome->setScl(ist::vector4(0, 0, 0));

		blur = new Blur();
		AddGObj(blur);
		blur->setPos(pos);
		blur->w = 1.0f;
		blur->h = 1.0f;

		f = new Fragments<Fragment>();
		AddGObj(f);
		f->setPos(pos);
		f->setDrawPriority(100);
	}
	

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);

		if(getFrame()%30==1) {
			SendScopedDamage(getRoot()->getID(), getFaction(), getPos(), float(getFrame())*1.25f, getDamage());
			if(isInCameraSight()) {
				CameraShaker *mp = new CameraShaker(30+getFrame()/5, 2.0f+float(getFrame())/30.0f);
				AddGObj(mp);

				for(int i=0; i<5; ++i) {
					ist::vector4 aim = RandomVector().setY(0).normalize();
					ist::vector4 pos = getPos()+aim*float(getFrame())*1.25f - aim*15*GetRand();
					f->add(Fragment(pos, ist::vector4(0, GetRand()*1.0f+0.8f, 0)));
				}
			}
		}
		if(getFrame()==300) {
			SendKillEvent(this, this);
			blur->fade = true;
			dome->fade = 0.001f;
		}
	}

	boost::intrusive_ptr<Dome> dome;
	boost::intrusive_ptr<Blur> blur;
	boost::intrusive_ptr< Fragments<Fragment> > f;
};



class HyuganatsuBomb : public BBullet
{
public:
	class Bomb : public LinearInherit3(HavePolygonModel, HaveFrameCounter, Particle)
	{
	public:
		ist::vector4 vel, ax;
		ist::vector4 diffuse;
		float rot, rot_speed;
		bool stoped;

		Bomb(const ist::vector4& p, const ist::vector4& v) : vel(v), rot(0), rot_speed(0), stoped(false) {
			setPos(p);
			setPModel("resource/rock.mqo");
			ax = RandomVector().normalize();
			diffuse = ist::vector4(0.8f, 0.6f, 0.4f);
			rot_speed = GetRand()*3+1;
		}

		void calcMatrix(ist::matrix44& mat) {
			mat.translate(getPos().v);
			mat.rotateA(ax, rot);
		}
		void draw() const {
			glMaterialfv(GL_FRONT, GL_DIFFUSE,  diffuse.v);
			drawModel();
			ist::Material().assign();
		}

		void onUpdate(Event& e) {
			BaseClass::onUpdate(e);

			const ist::vector4& pos = getPos();
			rot+=rot_speed;
			setPos(pos+vel);
			vel.y-=0.15f;
			
			float h=GetHeight(pos);
			if(pos.y<h) {
				setPos(ist::vector4(pos.x, h, pos.z));
				vel.y*=0.4f;
				vel.y=fabs(vel.y);
				if(vel.y<0.5f)
					stoped=true;
			}
			if(stoped) {
				diffuse.w-=0.02f;
				if(diffuse.w<=0)
					SendKillEvent(this, this);
			}
			else if(getFrame()>300) {
				SendKillEvent(this, this);
			}
		}
	};

	HyuganatsuBomb(const ist::vector4 target, const ist::vector4 p) {
		setPos(p);
		shoot(target);
	}

	void shoot(const ist::vector4 target) {
		float d = (target-getPos()).setY(0).norm();
		float hd = target.y-getPos().y;
		float g=0.15f;
		float speed = d/60.0f;
		float ang = asin(d*g/(speed*speed))/2.0f;
		ist::vector4 vel=(target-getPos()).setY(0).normalize()*speed;
		vel.y = sin(ang)*speed;
		float c = cos(ang);
		vel.x*=c;
		vel.z*=c;
	/*
		Bomb *bomb = new Bomb(getPos(), vel);
	*/
		Bomb *bomb = new Bomb(getPos(), (target-getPos()).normalize().setY(1)*speed);
		AddGObj(bomb);

		SendKillEvent(this, this);
	}
};


class Grave : public BBullet
{
public:
	class Spear : public LinearInherit3(HavePolygonModel, HaveFrameCounter, Particle)
	{
	public:
		Spear(const ist::vector4& pos, const ist::vector4& _aim) {
			aim = _aim;
			pos_final = pos;
			setPos(pos-aim*(300/aim.y));
			setRot(ist::vector4(0, 360*GetRand(), 0));
			setPModel("resource/grave.mqo");
			diffuse = ist::vector4(0.8f, 0.6f, 0.4f);
			setDrawPriority(100);
		}

		void onUpdate(Event& e) {
			BaseClass::onUpdate(e);

			if(getFrame()<150) {
				const ::ist::vector4& p = getPos();
				setPos(p+(pos_final-p)*0.1f);
			}
			else {
				diffuse.w-=0.01f;
				if(diffuse.w<=0)
					SendKillEvent(this, this);
			}
		}

		void draw() const {
			glMaterialfv(GL_FRONT, GL_DIFFUSE,  diffuse.v);
			drawModel();
			ist::Material().assign();
		}

		void calcMatrix(ist::matrix44& mat) const {
			mat.translate(this->getPos().v);
			mat*=ist::matrix44().aimVector(aim);
			mat.rotate(this->getRot().v);
		}

		ist::vector4 diffuse;
		ist::vector4 pos_final;
		ist::vector4 aim;
	};


	struct Fragment : public SimplePModel
	{
		ist::vector4 pos, vel, ax;
		ist::vector4 diffuse;
		float rot, rot_speed;
		bool stoped;
		
		Fragment(const ist::vector4& p, const ist::vector4& v) : pos(p), vel(v), rot(0), rot_speed(0), stoped(false) {
			pos.y = GetHeight(pos);
			pmodel = GetPModel("resource/rock.mqo");
			ax = RandomVector().normalize();
			diffuse = ist::vector4(0.8f, 0.6f, 0.4f);
			rot_speed = GetRand()*3+1;
		}

		void calcMatrix(ist::matrix44& mat) const {
			mat.translate(pos.v);
			mat.rotateA(ax, rot);
		}

		bool isDead() const {
			return diffuse.w<=0;
		}

		void update() {
			float g=0.15f;

			rot+=rot_speed;
			pos+=vel;
			vel.y-=g;
			
			float h=GetHeight(pos);
			if(pos.y<h) {
				pos.y=h;
				rot_speed*=0.4f;
				vel*=0.4f;
				vel.y=fabs(vel.y);
				if(vel.y<0.5f)
					stoped=true;
			}
			if(stoped)
				diffuse.w-=0.02f;
		}

		void draw() const {
			glMaterialfv(GL_FRONT, GL_DIFFUSE,  diffuse.v);
			drawModel();
			ist::Material().assign();
		/*	
			glPointSize(5);
			glColor3f(1, 0, 0);
			glBegin(GL_POINTS);
			glVertex3fv(pos.v);
			glEnd();
			glColor3f(1, 1, 1);

			ist::MatrixSaver msaver(GL_MODELVIEW_MATRIX);
			ist::MatrixSaver psaver(GL_PROJECTION_MATRIX);
			ist::ScreenMatrixIV smatrix;

			ist::PerspectiveCamera& camera = GetCamera();
			ist::matrix44 prj;
			float aspect = float(GetWindowWidth())/float(GetWindowHeight());
			prj.m[0][0] = 1.0f/tan(camera.fovy*aspect*0.5f);
			prj.m[1][1] = 1.0f/tan(camera.fovy*0.5f);
			prj.m[2][2] = camera.z_far/(camera.z_far-camera.z_near);
			prj.m[3][2] = -prj.m[2][2]*camera.z_near;
			prj.m[2][3] = 1.0f;

			ist::matrix44 screen;
			screen.m[0][0] = float(GetWindowWidth())/2;
			screen.m[0][3] = float(GetWindowWidth())/2;
			screen.m[1][1] = float(GetWindowHeight())/2;
			screen.m[1][3] = float(GetWindowHeight())/2;
			screen.m[2][2] = 1.0f;
			screen.m[3][3] = 1.0f;
			
			ist::vector4 point = screen*(prj*pos);
			glBegin(GL_POINTS);
			glVertex3fv(point.v);
			glEnd();
			glPointSize(1);
		*/
		}
	};



	Grave(const ist::vector4& pos, const ist::vector4& _aim) {
		setPos(pos);
		aim = _aim;
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);

		bool attack=false;
		ist::vector4 atk_pos;
		ist::vector4 atk_aim;
		if(getFrame()==10) {
			attack=true;
			atk_pos = getPos()-aim*50 +ist::vector4(GetRand()-0.5f, GetRand()-0.5f, GetRand()-0.5f)*100;
		}
		else if(getFrame()==40) {
			attack=true;
			atk_pos = getPos()+aim*100 +ist::vector4(GetRand()-0.5f, GetRand()-0.5f, GetRand()-0.5f)*80;
		}
		else if(getFrame()==70) {
			attack=true;
			atk_pos = getPos()+aim*250 +ist::vector4(GetRand()-0.5f, GetRand()-0.5f, GetRand()-0.5f)*60;
		}
		else if(getFrame()==100) {
			attack=true;
			atk_pos = getPos()+aim*400 +ist::vector4(GetRand()-0.5f, GetRand()-0.5f, GetRand()-0.5f)*40;
			SendKillEvent(this, this);
		}

		if(attack) {
			SendScopedDamage(getRoot()->getID(), getFaction(), atk_pos, 175, getDamage());

			atk_aim = ist::vector4(aim.x, GetRand()+1.25f, aim.z).normalize();
			atk_pos.y = GetHeight(atk_pos);

			Spear *g = new Spear(atk_pos, atk_aim);
			AddGObj(g);

			if(isInCameraSight()) {
				CameraShaker *mp = new CameraShaker(90, 5);
				AddGObj(mp);

				Fragments<Fragment> *f = new Fragments<Fragment>();
				AddGObj(f);
				f->setPos(atk_pos);
				f->setDrawPriority(100);
				for(int i=0; i<15; ++i) {
					ist::vector4 vel = RandomVector()*2.0f;
					vel.y=fabs(vel.y)*4;
					f->add(Fragment(atk_pos, vel));
				}
			}
		}
	}

	ist::vector4 aim;
};



class ThunderBolt : public BBullet
{
public:
};


class MeteorSwarm : public BBullet
{
public:
	class Shield : public LinearInherit2(HavePolygonModel, Particle)
	{
	public:
		Shield() {
			setPModel("resource/meteor_shield.mqo");
		}

		void setLight(ist::Light&) const {}

		void draw() const {
			ist::Color col;
			col.color = ist::vector4(1.0f, 0.3f, 0.3f);
			col.assign();
			glBlendFunc(GL_SRC_ALPHA, GL_ONE);
			glDepthMask(GL_FALSE);
			BaseClass::draw();
			glDepthMask(GL_TRUE);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			ist::Color().assign();
		}
	};
	
	class Meteor : public LinearInherit3(HaveShadow, HavePolygonModel, Bullet)
	{
	public:
		Meteor() {
			setLifeTime(300);
			setShadowScl(ist::vector4(50, 50, 50));
			setVel(ist::vector4(0, -15, 0));
			setPModel("resource/meteor.mqo");
			shield = new Shield();
			AddGObj(shield);
		}

		void setPos(const ist::vector4& p) {
			BaseClass::setPos(p);
			if(!!shield)
				shield->setPos(getPos());
		}

		void explode() {
			if(!isInCameraSight())
				return;

			HaveDrawFrontAttrib<Flasher> *p = new HaveDrawFrontAttrib<Flasher>();
			AddGObj(p);
			p->setScl(ist::vector4(300, 300, 300));
			p->setPos(getPos());

			CameraShaker *cs = new CameraShaker(90, 3);
			AddGObj(cs);

			Fragments<Grave::Fragment> *f = new Fragments<Grave::Fragment>();
			AddGObj(f);
			f->setPos(getPos());
			f->setDrawPriority(100);
			for(int i=0; i<10; ++i) {
				ist::vector4 vel = RandomVector()*2.0f;
				vel.y=fabs(vel.y)*4;
				f->add(Grave::Fragment(getPos(), vel));
			}
		}


		void onUpdate(Event& e) {
			BaseClass::onUpdate(e);

			const ist::vector4& pos = getPos();
			if(pos.y<GetHeight(pos)) {
				explode();
				SendScopedDamage(getRoot()->getID(), getFaction(), getPos(), 150, getDamage());
				SendKillEvent(this, this);
			}
		}

		void onKill(Event& e) {
			BaseClass::onKill(e);
			SendKillEvent(this, shield);
			shield = particle_ptr();
		}

	private:
		particle_ptr shield;
	};


	MeteorSwarm(const ist::vector4& pos) {
		setPos(pos);
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);

		ist::vector4 atk_pos;
		ist::vector4 atk_aim;
		if(getFrame()==10 || getFrame()==40 || getFrame()==70 || getFrame()==100 || getFrame()==130) {
			Meteor *meteor = new Meteor();
			const ist::vector4& pos = getPos();
			meteor->setRoot(getRoot());
			meteor->setPos(ist::vector4(pos.x+(GetRand()-0.5f)*250, GetHeight(pos)+500.0f, pos.z+(GetRand()-0.5f)*250));
			meteor->setDamage(getDamage());
			AddGObj(meteor);
		}
		if(getFrame()==130) {
			SendKillEvent(this, this);
		}
	}

};
