#include "tohu_unit_common.h"


class tohuBuildingWreck : public  LinearInherit4(HavePolygonModel, HaveFrameCounter, BelongFaction, Particle)
{
public:
	tohuBuildingWreck() {
		setDrawPriority(100);
	}

	void draw() const {
		this->getFaction()->getFactionMaterial().assign();
		this->drawModel();
		ist::Material().assign();
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		ist::vector4 pos = getPos();
		pos.y-=2.0f;
		setPos(pos);
		setRot(ist::vector4(GetRand()*1.5f, 0, GetRand()*1.5f));

		float pradius = 0;
		if(!!getPModel())
			pradius = getPModel()->getChunc(0)->getBSphere().radius;

		if(isInCameraSight()) {
			ExplodeParticle *p = new ExplodeParticle();
			AddGObj(p);
			p->setVel(ist::vector4(GetRand()-0.5f, GetRand(), GetRand()-0.5f)*2.0f);
			ist::vector4 aim = ist::vector4(GetRand()-0.5f, 0, GetRand()-0.5f)*2.0f;
			ist::vector4 ppos = getPos()+aim*radius;
			ppos.y = (GetHeight(ppos));
			p->setPos(ppos);
		}

		float dist = GetHeight(pos)-pos.y;
		if(dist>pradius*2.0f) {
			SendKillEvent(this, this);
			return;
		}
	}

	void setRadius(float r) { radius=r; }

private:
	float radius;
};


template<class T>
class tohuBuilding : public T
{
public:
	typedef tohuBuilding BaseClass;

	void onBuildComplete() {
		this->getFaction()->pushMessage(getUnitName()+" ݂̌܂B");
	}

	void draw() const {
		if(this->getBuildStatus()<1.0f) {
			ist::MatrixSaver msaver(GL_MODELVIEW_MATRIX);
			ist::matrix44 mat;
			mat.translate(0.0f, -this->getLifeGageFloating()*(1.0f-this->getBuildStatus()), 0.0f);
			glMultMatrixf(mat.v);
			T::draw();
		}
		else
			T::draw();
	}

	virtual void onDestroy(Event& e) {
		tohuBuildingWreck *wreck = new tohuBuildingWreck();
		AddGObj(wreck);
		wreck->setFaction(this->getFaction());
		wreck->setRot(this->getRot());
		wreck->setPModel(this->getPModel());
		wreck->setRadius(this->getHitRadius());
		{
			const ist::vector4& p = this->getPos();
			if(this->getBuildStatus()<1.0f)
				wreck->setPos(ist::vector4(p.x, p.y-this->getLifeGageFloating()*(1.0f-this->getBuildStatus()), p.z));
			else
				wreck->setPos(p);
		}
		this->T::onDestroy(e);
	}

	bool conditionCreate() {
		if(this->getBuildStatus()<1.0f)
			return false;
		return true;
	}

	const std::string& getUnitName() const {
		if(this->getBuildStatus()<1.0f) {
			static std::string n;
			n = GetUnitName(this->getClassName());
			char buf[32];
			sprintf(buf, "(%.0f%%)", this->getBuildStatus()*100.0f);
			n+=buf;
			return n;
		}
		return GetUnitName(this->getClassName());
	}
};



class tohuTownCenter : public LinearInherit5(tohuBuilding, tohuBreakable, HaveUnitCreator, HavePolygonModel, Building)
{
public:
	tohuTownCenter(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(65);
		setHitRadius(65);
		setMaxLife(4000);
		setAntiBullet(0.7f);
		setAntiLaser(0.75f);
		setSight(150);
		setBuildSpeed(2.0f);
		setPModel("resource/tohu_towncenter.mqo");
	}

	menu_ptr _getCommandMenu() {
		if(conditionCreate())
			return getFaction()->createCommandMenu("TownCenter");
		return menu_ptr();
	}

	bool isSettlement() const { return true; }
	const std::string& getClassName() const { static std::string cname("tohuTownCenter"); return cname; }
};



class tohuSettlement : public LinearInherit5(tohuBuilding, tohuBreakable, HaveUnitCreator, HavePolygonModel, Building)
{
public:
	tohuSettlement(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(40);
		setHitRadius(40);
		setMaxLife(200);
		setAntiBullet(0.5f);
		setAntiLaser(0.7f);
		setSight(150);
		setPModel("resource/tohu_settlement.mqo");
	}

	menu_ptr _getCommandMenu() {
		if(conditionCreate())
			return getFaction()->createCommandMenu("Settlement");
		return menu_ptr();
	}
	bool isSettlement() const { return true; }
	const std::string& getClassName() const { static std::string cname("tohuSettlement"); return cname; }
};


class tohuPowerPlant : public LinearInherit4(tohuBuilding, tohuBreakable, HavePolygonModel, Building)
{
public:
	tohuPowerPlant(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(40);
		setHitRadius(40);
		setMaxLife(350);
		setAntiBullet(0.5f);
		setAntiLaser(0.7f);
		setSight(150);
		setPModel("resource/tohu_power_plant.mqo");
	}

	void idle() {
		faction_ptr f = getFaction();
		Resource& r = getFaction()->getResource();
		r.energy+=0.04f;
	}

	const std::string& getClassName() const { static std::string cname("tohuPowerPlant"); return cname; }
};




class tohuFactoryS : public LinearInherit5(tohuBuilding, tohuBreakable, HaveUnitCreator, HavePolygonModel, Building)
{
public:
	tohuFactoryS(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(50);
		setHitRadius(50);
		setMaxLife(800);
		setAntiBullet(0.50f);
		setAntiLaser(0.60f);
		setSight(150);
		setBuildSpeed(1.4f);
		setPModel("resource/tohu_factory.mqo");
	}

	menu_ptr _getCommandMenu() {
		if(conditionCreate())
			return getFaction()->createCommandMenu("FactoryS");
		return menu_ptr();
	}

	const std::string& getClassName() const { static std::string cname("tohuFactoryS"); return cname; }
};


class tohuFactoryM : public LinearInherit5(tohuBuilding, tohuBreakable, HaveUnitCreator, HavePolygonModel, Building)
{
public:
	tohuFactoryM(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(70);
		setHitRadius(70);
		setMaxLife(1000);
		setAntiBullet(0.50f);
		setAntiLaser(0.60f);
		setSight(150);
		setBuildSpeed(1.4f);
		setPModel("resource/tohu_factory_l.mqo");
	}

	menu_ptr _getCommandMenu() {
		if(conditionCreate())
			return getFaction()->createCommandMenu("FactoryM");
		return menu_ptr();
	}

	const std::string& getClassName() const { static std::string cname("tohuFactoryM"); return cname; }
};


class tohuFactoryL : public LinearInherit5(tohuBuilding, tohuBreakable, HaveUnitCreator, HavePolygonModel, Building)
{
public:
	tohuFactoryL(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(70);
		setHitRadius(70);
		setMaxLife(2500);
		setAntiBullet(0.50f);
		setAntiLaser(0.60f);
		setSight(150);
		setBuildSpeed(1.6f);
		setPModel("resource/tohu_factory_l.mqo");
	}

	menu_ptr _getCommandMenu() {
		if(conditionCreate())
			return getFaction()->createCommandMenu("FactoryL");
		return menu_ptr();
	}

	const std::string& getClassName() const { static std::string cname("tohuFactoryL"); return cname; }
};


class tohuLaboratory : public LinearInherit5(tohuBuilding, tohuBreakable, HaveUnitCreator, HavePolygonModel, Building)
{
public:
	tohuLaboratory(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(70);
		setHitRadius(70);
		setMaxLife(1000);
		setAntiBullet(0.50f);
		setAntiLaser(0.60f);
		setSight(150);
		setBuildSpeed(1.3f);
		setPModel("resource/tohu_factory_l.mqo");
	}

	menu_ptr _getCommandMenu() {
		if(conditionCreate())
			return getFaction()->createCommandMenu(getClassName());
		return menu_ptr();
	}

	const std::string& getClassName() const { static std::string cname("tohuLaboratory"); return cname; }
};


class tohuTower : public LinearInherit4(tohuBuilding, tohuBreakable, HavePolygonModel, Building)
{
public:
	tohuTower(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(30);
		setHitRadius(30);
		setMaxLife(1500);
		setAntiBullet(0.5f);
		setAntiLaser(0.75f);

		setSight(450);
		setSearchRange(350);
		setFireRange(350);

		setFirePower(15);
		setBuildSpeed(0.75f);
		setPModel("resource/tohu_tower.mqo");
	}

	void fire(selectable_ptr target) {
		int cool_down = 120;
		float rate = (float)cool_down/60.0f;

		SendScopedDamage(getID(), getFaction(), target->getPos(), 25,
			new Event::DamageInfo(getFirePower()*rate, 0, getFirePower()*rate));
		setCoolDown(cool_down);

		if(isInCameraSight()) {
			ist::vector4 fire_pos = getPos()+ist::vector4(0, 150, 0);
			ist::vector4 aim_target = (target->getPos()-fire_pos).normalize();
			ExplodeParticle *p = new ExplodeParticle();
			AddGObj(p);
			p->setTexture("resource/explode0.png");
			p->setVel(aim_target);
			p->setPos(fire_pos+aim_target*25);
		}
		if(target->isInCameraSight()) {
			ExplodeParticle *p = new ExplodeParticle();
			AddGObj(p);
			p->setTexture("resource/explode0.png");
			p->setPos(target->getPos());
		}
	}

	const std::string& getClassName() const { static std::string cname("tohuTower"); return cname; }
};

class tohuCastle : public LinearInherit4(tohuBuilding, tohuBreakable, HavePolygonModel, Building)
{
public:
	tohuCastle(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(65);
		setHitRadius(65);
		setMaxLife(4000);
		setAntiBullet(0.65f);
		setAntiLaser(0.80f);

		setSight(400);
		setSearchRange(350);
		setFireRange(350);

		setFirePower(30);
		setBuildSpeed(1.5f);
		setPModel("resource/tohu_castle.mqo");
	}

	void fire(selectable_ptr target) {
		int cool_down = 120;
		float rate = (float)cool_down/60.0f;

		SendScopedDamage(getID(), getFaction(), target->getPos(), 25,
			new Event::DamageInfo(0, getFirePower()*rate, getFirePower()*rate*0.5f));
		setCoolDown(cool_down);

		if(isInCameraSight()) {
			ist::vector4 fire_pos = getPos()+ist::vector4(0, 150, 0);
			ist::vector4 aim_target = (target->getPos()-fire_pos).normalize();
			Flasher *p = new Flasher();
			AddGObj(p);
			p->setPos(fire_pos+aim_target*25);
			p->setTexture("resource/flareb.png");
		}
		if(target->isInCameraSight()) {
			Flasher *p = new Flasher();
			AddGObj(p);
			p->setPos(target->getPos());
			p->setTexture("resource/flareb.png");
		}
	}

	const std::string& getClassName() const { static std::string cname("tohuCastle"); return cname; }
};





building_ptr tohuCreateTownCenter(faction_ptr f) { return new tohuTownCenter(f); }
building_ptr tohuCreateSettlement(faction_ptr f) { return new tohuSettlement(f); }
building_ptr tohuCreatePowerPlant(faction_ptr f) { return new tohuPowerPlant(f); }
building_ptr tohuCreateFactoryS(faction_ptr f)   { return new tohuFactoryS(f); }
building_ptr tohuCreateFactoryM(faction_ptr f)   { return new tohuFactoryM(f); }
building_ptr tohuCreateFactoryL(faction_ptr f)   { return new tohuFactoryL(f); }
building_ptr tohuCreateLaboratory(faction_ptr f) { return new tohuLaboratory(f); }
building_ptr tohuCreateTower(faction_ptr f)      { return new tohuTower(f); }
building_ptr tohuCreateCastle(faction_ptr f)     { return new tohuCastle(f); }



