#include "ist/dso_setup.h"
#include "tohu.h"
#include <fstream>






const std::string& createDescription(productinfo_ptr pi) {
	static std::string description;
	description = GetUnitName(pi->getProductName());
	description+=" ";
	char buf[256];
	Resource& cost = pi->getCost();
	if(cost.timber>0) {
		sprintf(buf, ":%.0f ", cost.timber);
		description+=buf;
	}
	if(cost.silicon>0) {
		sprintf(buf, ":%.0f ", cost.silicon);
		description+=buf;
	}
	if(cost.iron>0) {
		sprintf(buf, "S:%.0f ", cost.iron);
		description+=buf;
	}
	if(cost.energy>0) {
		sprintf(buf, "En:%.0f ", cost.energy);
		description+=buf;
	}
	description+="\n";
	description+=GetDescription(pi->getProductName());
	return description;
}


UnitCommandButton* tohuUnitButton(productinfo_ptr pi) {
	UnitCommandButton *b = new UnitCommandButton();
	b->setCommand(new Event::CommandInfo("requestCreateUnit", pi->getProductName()));
	b->setTexture(GetUnitImage(pi->getProductName()));
	b->setDescription(createDescription(pi));
	pi->setButton(b);
	return b;
}

CommandButton* tohuBuildingButton(productinfo_ptr pi) {
	CommandButton *b = new CommandButton();
	b->setCommand(new Event::CommandInfo("createBuilding", pi->getProductName()));
	b->setTexture(GetUnitImage(pi->getProductName()));
	b->setDescription(createDescription(pi));
	pi->setButton(b);
	return b;
}

UnitCommandButton* tohuReserchButton(productinfo_ptr pi) {
	UnitCommandButton *b = new UnitCommandButton();
	b->setCommand(new Event::CommandInfo("requestCreateUnit", pi->getProductName()));
	b->setTexture(GetUnitImage(pi->getProductName()));
	b->setDescription(createDescription(pi));
	pi->setButton(b);
	return b;
}





class BuildTarget : public  LinearInherit3(HavePolygonModel, BelongFaction, Particle)
{
public:
	BuildTarget(faction_ptr f, const std::string& fn) {
		setFaction(f);
		setPModel(fn);
		setDrawPriority(100);
	}

	void draw() const {
		this->getFaction()->getFactionMaterial().assign();
		this->drawModel();
		ist::Material().assign();
	}
};


class tohuFaction : public Faction
{
public:
	typedef boost::intrusive_ptr<BuildTarget> btarget_ptr;
	typedef Faction BaseClass;
	typedef boost::intrusive_ptr<CommandButtonSet> cbs_ptr;
	typedef std::map<std::string, cbs_ptr> commandmenu_table;

	commandmenu_table command_menus;
	btarget_ptr build_target;
	int nth_age;

	static faction_ptr create() {
		return new tohuFaction();
	}

	productinfo_ptr getProductInfo(const std::string& name) {
		productinfo_ptr& pi = prod_table[name];
		if(!pi) {
			pi = new ProductInfo();
			pi->setFaction(this);
			pi->setProductName(name);
		}
		return pi;
	}

	tohuFaction() :
		nth_age(0)
	{
		{
			getProductInfo("tohuWorker")->toLimited(50).setCost( Resource(50, 0, 0, 0), 12);

			getProductInfo("tohuSoldier1")->setCost( Resource(30, 50, 0, 10), 6);
			getProductInfo("tohuSoldier2")->setCost( Resource(90, 150, 0, 30), 12);
			getProductInfo("tohuSoldier3")->setCost( Resource(360, 600, 0, 90), 24);
			getProductInfo("tohuArmor1")->setCost( Resource(0, 80, 50, 15), 8);
			getProductInfo("tohuArmor2")->setCost( Resource(0, 320, 200, 45), 16);
			getProductInfo("tohuArmor3")->setCost( Resource(0, 960, 600, 135), 32);
			getProductInfo("tohuLaser1")->setCost( Resource(80, 0, 0, 18), 7);
			getProductInfo("tohuLaser2")->setCost( Resource(320, 0, 0, 54), 14);
			getProductInfo("tohuLaser3")->setCost( Resource(960, 0, 0, 162), 28);
			getProductInfo("tohuAssasin")->setCost( Resource(0, 30, 60, 20), 7);
			getProductInfo("tohuHealer")->setCost( Resource(70, 0, 0, 20), 9);

			getProductInfo("tohuArbalest")->setCost( Resource(0, 50, 120, 30), 12);
			getProductInfo("tohuTransporter")->setCost( Resource(0, 100, 150, 40), 15);
			getProductInfo("tohuMine")->setCost( Resource(0, 30, 50, 20), 9);
			getProductInfo("tohuBomber1")->setCost( Resource(0, 100, 100, 25), 12);
			getProductInfo("tohuDetector")->setCost( Resource(150, 100, 0, 30), 15);

			getProductInfo("tohuDestroyer")->setCost( Resource(1000, 250, 500, 200), 30);
			getProductInfo("tohuCruiser")->setCost( Resource(250, 500, 1000, 200), 30);
			getProductInfo("tohuMovilFort")->setCost( Resource(500, 1000, 750, 250), 40);

			getProductInfo("tohuEmperor")->setCost( Resource(1000, 1000, 1000, 300), 60);
		}
		{
			getProductInfo("tohuTownCenter")->setCost( Resource(500, 500, 500, 250));
			getProductInfo("tohuSettlement")->setCost( Resource(100, 0, 0, 0));
			getProductInfo("tohuPowerPlant")->toLimited(10).setCost( Resource(0, 150, 150, 0));

			getProductInfo("tohuFactoryS")->setCost( Resource(200, 100, 0, 30));
			getProductInfo("tohuFactoryM")->setCost( Resource(300, 0, 150, 40));
			getProductInfo("tohuFactoryL")->setCost( Resource(0, 400, 400, 80));
			getProductInfo("tohuLaboratory")->setCost( Resource(0, 200, 200, 40));

			getProductInfo("tohuTower")->setCost( Resource(0, 500, 250, 75));
			getProductInfo("tohuCastle")->setCost( Resource(0, 750, 1000, 150));
		}
		{
			getProductInfo("tohuAgeOfTonyu")->toReserch().setCost(Resource(200, 200, 200, 200), 40);
			getProductInfo("tohuAgeOfTohu")->toReserch().setCost(Resource(400, 400, 400, 400), 60);
		//	getProductInfo("tohuAgeOfTonyu")->toReserch().setCost(Resource(0, 0, 0, 0), 5);
		//	getProductInfo("tohuAgeOfTohu")->toReserch().setCost(Resource(0, 0, 0, 0), 5);

			getProductInfo("Gather1")->toReserch().setCost(Resource(50, 50, 50, 100), 20);
			getProductInfo("Gather2")->toReserch().setCost(Resource(150, 150, 150, 200), 20);
			getProductInfo("Gather3")->toReserch().setCost(Resource(250, 250, 250, 300), 20);
			getProductInfo("Carry1")->toReserch().setCost(Resource(0, 100, 100, 250), 20);
			getProductInfo("Carry2")->toReserch().setCost(Resource(0, 200, 200, 350), 20);

			getProductInfo("Cloaking")->toReserch().setCost(Resource(0, 0, 100, 150), 20);
			getProductInfo("Spying")->toReserch().setCost(Resource(0, 0, 100, 200), 20);
		}
	}


	bool addReserch(const std::string& name) {
		bool res = true;

		if(name=="tohuAgeOfTonyu") {
			setNthAge(1);
			pushMessage(getAgeName()+"֐i܂B");
		}
		else if(name=="tohuAgeOfTohu") {
			setNthAge(2);
			pushMessage(getAgeName()+"֐i܂B");
		}
		else if(name=="Gather1" || name=="Gather2" || name=="Gather3"
		|| name=="Carry1" || name=="Carry2"
		|| name=="Cloaking" || name=="Spying") {
			upgrade(name);
		}
		else {
			res = false;
		}

		if(res) {
			prod_table.erase(name);
			reserchinfo.insert(name);
		}
		return res;
	}

	cbs_ptr getCommandMenu(const std::string& name) {
		cbs_ptr& cbs = command_menus[name];
		if(!cbs)
			cbs = new CommandButtonSet();
		return cbs;
	}


	void setNthAge(int n) {
		nth_age = n;
		cbs_ptr bs;
		if(n==0) {
			bs = getCommandMenu("TownCenter");
			bs->appendButton(0, 0, tohuUnitButton( getProductInfo("tohuWorker") ));
			bs->appendButton(0, 1, tohuReserchButton( getProductInfo("tohuAgeOfTonyu") ));

			bs = getCommandMenu("Settlement");
			bs->appendButton(0, 0, tohuReserchButton( getProductInfo("Gather1") ));

			bs = getCommandMenu("FactoryS");
			bs->appendButton(0, 0, tohuUnitButton( getProductInfo("tohuSoldier1") ));
			bs->appendButton(1, 0, tohuUnitButton( getProductInfo("tohuArmor1") ));
			bs->appendButton(2, 0, tohuUnitButton( getProductInfo("tohuLaser1") ));

			bs = getCommandMenu("Worker");
			bs->appendButton(1, 0, tohuBuildingButton( getProductInfo("tohuSettlement") ));
			bs->appendButton(2, 0, tohuBuildingButton( getProductInfo("tohuPowerPlant") ));
			bs->appendButton(3, 0, tohuBuildingButton( getProductInfo("tohuFactoryS") ));
			bs->appendButton(0, 1, tohuBuildingButton( getProductInfo("tohuTower") ));

		}
		else if(n==1) {
			bs = getCommandMenu("TownCenter");
			bs->appendButton(0, 1, tohuReserchButton( getProductInfo("tohuAgeOfTohu") ));

			bs = getCommandMenu("Settlement");
			bs->appendButton(0, 0, tohuReserchButton( getProductInfo("Gather2") ));
			bs->appendButton(0, 1, tohuReserchButton( getProductInfo("Carry1") ));

			bs = getCommandMenu("FactoryS");
			bs->appendButton(0, 1, tohuUnitButton( getProductInfo("tohuSoldier2") ));
			bs->appendButton(1, 1, tohuUnitButton( getProductInfo("tohuArmor2") ));
			bs->appendButton(2, 1, tohuUnitButton( getProductInfo("tohuLaser2") ));
			bs->appendButton(3, 0, tohuUnitButton( getProductInfo("tohuAssasin") ));
			bs->appendButton(4, 0, tohuUnitButton( getProductInfo("tohuHealer") ));

			bs = getCommandMenu("FactoryM");
			bs->appendButton(0, 0, tohuUnitButton( getProductInfo("tohuArbalest") ));
			bs->appendButton(1, 0, tohuUnitButton( getProductInfo("tohuTransporter") ));
			bs->appendButton(2, 0, tohuUnitButton( getProductInfo("tohuMine") ));
			bs->appendButton(3, 0, tohuUnitButton( getProductInfo("tohuBomber1") ));
			bs->appendButton(4, 0, tohuUnitButton( getProductInfo("tohuDetector") ));

			bs = getCommandMenu("tohuLaboratory");
			bs->appendButton(0, 0, tohuReserchButton( getProductInfo("Cloaking") ));

			bs = getCommandMenu("Worker");
			bs->appendButton(0, 0, tohuBuildingButton( getProductInfo("tohuTownCenter") ));
			bs->appendButton(4, 0, tohuBuildingButton( getProductInfo("tohuFactoryM") ));
			bs->appendButton(5, 0, tohuBuildingButton( getProductInfo("tohuLaboratory") ));
			bs->appendButton(1, 1, tohuBuildingButton( getProductInfo("tohuCastle") ));
		}
		else if(n==2) {
			bs = getCommandMenu("Settlement");
			bs->appendButton(0, 0, tohuReserchButton( getProductInfo("Gather3") ));
			bs->appendButton(0, 1, tohuReserchButton( getProductInfo("Carry2") ));

			bs = getCommandMenu("FactoryS");
			bs->appendButton(0, 2, tohuUnitButton( getProductInfo("tohuSoldier3") ));
			bs->appendButton(1, 2, tohuUnitButton( getProductInfo("tohuArmor3") ));
			bs->appendButton(2, 2, tohuUnitButton( getProductInfo("tohuLaser3") ));

			bs = getCommandMenu("FactoryL");
			bs->appendButton(0, 0, tohuUnitButton( getProductInfo("tohuDestroyer") ));
			bs->appendButton(1, 0, tohuUnitButton( getProductInfo("tohuCruiser") ));
			bs->appendButton(2, 0, tohuUnitButton( getProductInfo("tohuMovilFort") ));

			bs = getCommandMenu("tohuLaboratory");
			bs->appendButton(0, 1, tohuReserchButton( getProductInfo("Spying") ));

			bs = getCommandMenu("Worker");
			bs->appendButton(6, 0, tohuBuildingButton( getProductInfo("tohuFactoryL") ));
		}
	}

	const std::string& getFactionName() const {
		static std::string faction_name = "哤鍑";
		return faction_name;
	}

	const std::string& getAgeName() const {
		static std::string age[3] = {
			std::string("哤̎"),
			std::string("̎"),
			std::string("̎"),
		};
		static std::string dummy = "H";
		if(nth_age>=0 && nth_age<3)
			return age[nth_age];
		return dummy;
	}

	menu_ptr createCommandMenu(const std::string& name) {
		commandmenu_table::iterator res = command_menus.find(name);
		if(res!=command_menus.end() && !!res->second) {
			commandmenu_table::mapped_type& m = res->second;
			m->update();
			m->sendEvent(Event(EVT_RAISE, getID(), m->getID()));
			m->update();
			return m;
		}
		return menu_ptr();
	}

	building_ptr createBuilding(const std::string& class_name) {
		building_ptr u = Faction::createBuilding(class_name);
		productinfo_ptr pi = prod_table[class_name];
		if(!!pi) {
			SendCommandEvent(this, u, new Event::CommandInfo("setCost", Resource(pi->getCost())));
		}
		else {
			assert(0);
		}
		return u;
	}

	void sendCommandEvent(Event::commandinfo_ptr ip, const std::string& class_name) {
		IGObjIterator& iter = getUnitTable(class_name);
		while(iter.hasNext()) {
			selectable_ptr p = ToSelectable(iter.iterate());
			if(!!p && p->getFaction().get()==this) {
				SendCommandEvent(this, p, ip);
			}
		}
	}


	void eraseBuildTarget() {
		if(!!build_target) {
			SendKillEvent(this, build_target);
			build_target = btarget_ptr();
		}
	}

	void executeCommand(Event::ButtonInfo&, Event::CommandInfo& command) {
		if(command.name=="createBuilding") {
			productinfo_ptr pi = prod_table[ToString(command.value)];
			if(!!pi && !!build_target) {
				if(getResource()<pi->getCost()) {
					pushMessage("sĂ܂B");
					return;
				}
				{
					building_ptr b = CreateBuildingR(pi->getProductName(), this);
					b->setPos(build_target->getPos());
					b->setRot(build_target->getRot());

					ist::CollisionDetector cd;
					IGObjIterator& it = GetBlocksByRadius(b->getPos(), 200);
					while(it.hasNext()) {
						selectable_ptr s = ToSelectable(it.iterate());
						if(!!s && !s->isUnit() && cd.detect(s->getHitModel(), b->getHitModel())) {
							pushMessage("Q܂B");
							return;
						}
					}
				}

				gobj_id b = CreateID();
				RouteCreateEvent(new Event::CreateInfo(BUILDING, b, pi->getProductName(), this->getID()));
				RouteCallEvent(this->getID(), b, new Event::CallInfo("setPos", build_target->getPos()));
				RouteCallEvent(this->getID(), b, new Event::CallInfo("setRot", build_target->getRot()));
				RouteCallEvent(this->getID(), b, new Event::CallInfo("initBuildStatus", boost::any()));
				getResource()-=pi->getCost();

				// I𒆂̃jbg֌݂̎w

				IGObjIterator& selection = getSelection();
				while(selection.hasNext()) {
					selectable_ptr p = ToSelectable(selection.iterate());
					if(!!p && p->getFaction()==GetPlayerFaction()) {
						Event::actioninfo_ptr action_info = new Event::ActionInfo(ist::vector4(), 0, 0, b);
						action_info->flag|=Event::ActionInfo::INSIST;
						if(IsShiftKeyDown())
							action_info->flag|=Event::ActionInfo::APPEND;
						RouteActionEvent(this, p, action_info);
					}
				}

				eraseBuildTarget();
			}
		}
	}

	void killBuildTarget() {
		if(build_target) {
			SendKillEvent(this, build_target);
			build_target = btarget_ptr();
		}
	}
	
	void clearCommand() {
		Event::commandinfo_ptr c = getCurrentCommand();
		if(!!c && c->name=="createBuilding" && c->value.type()==typeid(std::string)) {
			eraseBuildTarget();
		}
		BaseClass::clearCommand();
	}
	void onCommandChange() {
		Event::commandinfo_ptr c = getCurrentCommand();
		if(!!c && c->name=="createBuilding" && c->value.type()==typeid(std::string)) {
			productinfo_ptr pi = prod_table[ToString(c->value)];
			if(!!pi) {
				killBuildTarget();
				build_target = new BuildTarget(this, GetUnitModel(pi->getProductName()));
				AddGObj(build_target);
			}
		}
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		if(getFrame()==1)
			setNthAge(nth_age);

		Resource& r = getResource();
		r.energy = std::min(getUnitTable()["tohuPowerPlant"].size()*100.0f, r.energy);

		if(!!build_target) {
			build_target->setPos(GetMousePointerPos3D());
		}
	}

	void onButton(Event& e) {
		BaseClass::onButton(e);
		Event::ButtonInfo& info = e.getButtonInfo();
		if(!!build_target) { // zC[ŌݑΏۂ]
			ist::vector4 r = build_target->getRot();
			if(info.button==4)
				r.y+=4.0f;
			else if(info.button==5)
				r.y-=4.0f;
			build_target->setRot(r);
		}
	}

	void onSelect(Event& e) {
		BaseClass::onSelect(e);
		setFrameTexture("resource/frame_bottom.png");
	}


	void onKill(Event& e) {
		killBuildTarget();
		BaseClass::onKill(e);
	}
};






























namespace {
	IGame *igame=0;

	typedef std::map<std::string, unit_creator> unit_table;
	typedef std::map<std::string, building_creator> building_table;
	typedef std::map<std::string, material_creator> material_table;
	unit_table ut;
	building_table bt;
	material_table mt;

	void init_creator_table() {
		ut["tohuWorker"] = tohuCreateWorker;
		ut["tohuSoldier1"] = tohuCreateSoldier1;
		ut["tohuSoldier2"] = tohuCreateSoldier2;
		ut["tohuSoldier3"] = tohuCreateSoldier3;
		ut["tohuArmor1"] = tohuCreateArmor1;
		ut["tohuArmor2"] = tohuCreateArmor2;
		ut["tohuArmor3"] = tohuCreateArmor3;
		ut["tohuLaser1"] = tohuCreateLaser1;
		ut["tohuLaser2"] = tohuCreateLaser2;
		ut["tohuLaser3"] = tohuCreateLaser3;
		ut["tohuHealer"] = tohuCreateHealer;
		ut["tohuAssasin"] = tohuCreateAssasin;
		ut["tohuEmperor"] = tohuCreateEmperor;

		ut["tohuArbalest"] = tohuCreateArbalest;
		ut["tohuMine"] = tohuCreateMine;
		ut["tohuDetector"] = tohuCreateDetector;
		ut["tohuTransporter"]=tohuCreateTransporter;
		ut["tohuBomber1"] = tohuCreateBomber1;

		ut["tohuDestroyer"] = tohuCreateDestroyer;
		ut["tohuCruiser"] = tohuCreateCruiser;
		ut["tohuMovilFort"] = tohuCreateMovilFort;


		bt["tohuTownCenter"] = tohuCreateTownCenter;
		bt["tohuSettlement"] = tohuCreateSettlement;
		bt["tohuPowerPlant"] = tohuCreatePowerPlant;
		bt["tohuFactoryS"] = tohuCreateFactoryS;
		bt["tohuFactoryM"] = tohuCreateFactoryM;
		bt["tohuFactoryL"] = tohuCreateFactoryL;
		bt["tohuLaboratory"] = tohuCreateLaboratory;
		bt["tohuTower"] = tohuCreateTower;
		bt["tohuCastle"] = tohuCreateCastle;


		mt["tohuTimber"] = CreateTimber;
		mt["tohuSilicon"] = CreateSilicon;
		mt["tohuIron"] = CreateIron;
		

		AddFactionCreator("tohuFaction", &tohuFaction::create);
		for(unit_table::iterator p=ut.begin(); p!=ut.end(); ++p)
			AddUnitCreator(p->first, p->second);

		for(building_table::iterator p=bt.begin(); p!=bt.end(); ++p)
			AddBuildingCreator(p->first, p->second);

		for(material_table::iterator p=mt.begin(); p!=mt.end(); ++p)
			AddMaterialCreator(p->first, p->second);
	}
}
IGame& _GetIGame() { return *igame; }

DECLARE_DSO_ENTRYPOINT();

EXPORT void OnLoad(IGame& ig)
{
	igame= &ig;

	texture_ptr t;
	t = CreateTexture("resource/explode0.png");
	if(!(*t))
		t->load("resource/explode0.png", 32, 32);

	t = CreateTexture("resource/explode1.png");
	if(!(*t))
		t->load("resource/explode1.png", 32, 32);

	t = CreateTexture("resource/explode2.png");
	if(!(*t))
		t->load("resource/explode2.png", 32, 32);

	init_creator_table();
}

