#include "templates/unit_ability.h"
#include "special_attack.h"








template<class T>
class tohuUnit : public tohuBreakable<T>
{
public:
	typedef tohuBreakable<T> RootClass;
	typedef tohuUnit BaseClass;

	tohuUnit() : grade(0), en_use(0.0f) {}

	bool category(const std::string& c) const {
		return (c=="Tohu") ? true : RootClass::category(c);
	}

	float getEnUse() const { return en_use; }
	void setEnUse(float e) { en_use=e; }

	menu_ptr getCommandMenu() { return CreateDefaultUnitMenu(); }

	virtual const std::string& getUnitName() const {
		return GetUnitName(this->getClassName());
	}

	void onSelect(Event& e) {
		RootClass::onSelect(e);
		int i = rand()%3;
		if(i==0)      PlaySE("resource/wav/select1.wav", SE_VOICE);
		else if(i==1) PlaySE("resource/wav/select2.wav", SE_VOICE);
		else if(i==2) PlaySE("resource/wav/select3.wav", SE_VOICE);
	}

	void onUpdate(Event& e) {
		RootClass::onUpdate(e);
		Resource& r = getFaction()->getResource();
		r.energy = std::max(0.0f, r.energy-en_use);
	}


	void setGrade(int g) { grade = g; }

private:
	int grade;
	float en_use;
};








// Small Unit



class tohuWorker : public LinearInherit4(tohuUnit, HaveShadow, HavePolygonModel, Unit)
{
public:
	tohuWorker(faction_ptr arg) : build_speed(0), gather_timber(0), gather_silicon(0), gather_iron(0),  max_resource(0) {
		setFaction(arg);
		setSelectionRadius(8);
		setHitRadius(3);
		setFloat(20);
	//	setFloat(10);
		setShadowScl(ist::vector4(20, 20, 20));

		setMaxLife(80);
		setAntiBullet(0.25f);
		setAntiLaser(0.20f);
		setAntiBomb(0.95f);
		setMoveSpeed(1.5f);

		setSight(200);

		setPModel("resource/tohu_worker.mqo");
	//	setPModel("resource/natsuko.mqo");
	//	setTexture("resource/natsuko.png");

		addBuildSpeed(5.0f);
		addMaxResource(20.0f);
		addGatherTimber(2.0f);
		addGatherSilicon(2.0f);
		addGatherIron(2.0f);
	}

	bool category(const std::string& c) const {
		return (c=="Small" || c=="Worker") ? true : BaseClass::category(c);
	}

	void addMaxResource(float f) { max_resource+=f; }
	void addBuildSpeed(float f) { build_speed+=f/60.0f; }
	void addGatherTimber(float f) { gather_timber+=f/60.0f; }
	void addGatherSilicon(float f) { gather_silicon+=f/60.0f; }
	void addGatherIron(float f) { gather_iron+=f/60.0f; }


//	void setLight(ist::Light&){}

	 const info_type& getExtInfo() const {
		static info_type info;
	 	info = BaseClass::getExtInfo();
		char buf[256];
		if(res.timber>0) {
			sprintf(buf, ": %.0f", res.timber);
			info.push_back(std::string(buf));
		}
		if(res.silicon>0) {
			sprintf(buf, ": %.0f", res.silicon);
			info.push_back(std::string(buf));
		}
		if(res.iron>0) {
			sprintf(buf, "S: %.0f", res.iron);
			info.push_back(std::string(buf));
		}
		return info;
	}

	void counter(selectable_ptr p) {
		ist::vector4 away = getPos()+(getPos()-p->getPos()).normalize()*100;
		SendActionEvent(this, this, new Event::ActionInfo(away, selectable_ptr(), Event::ActionInfo::PATH));
	}

	void idle() {
		if(this->getFrame()%120==0)
			this->pollLink();
	}

	void pollLink() {
		Event::actioninfo_ptr linked;
		float ally_min_dist = 0;

		IGObjIterator& block = GetBlocksByRadius(this->getPos(), this->getSight());
		while(block.hasNext()) {
			unit_ptr p = ToUnit(block.iterate());
			if(!p)
				continue;
			float dist = (p->getPos()-this->getPos()).norm();
			if(IsAlly(this, p) && dist<this->getSight()/2.0f && (dist<ally_min_dist || !linked)) {
				if(Event::actioninfo_ptr l = p->getCurrentAction()) {
					selectable_ptr& s = l->getObj();
					if(s && !s->isDead() && s->isMaterial() && (s->getPos()-p->getPos()).norm()<p->getSight()) {
						linked = l;
						ally_min_dist = dist;
					}
				}
			}
		}
		if(!!linked)
			SendActionEvent(this, this,
				new Event::ActionInfo(linked->target_pos, linked->getObj(), Event::ActionInfo::INSIST));
	}

	bool searchSettlement() {
		settlement = building_ptr();
		float min_dist = 0;
		static std::vector<std::string> settlement_type;
		if(settlement_type.empty()) {
			settlement_type.push_back("tohuTownCenter");
			settlement_type.push_back("tohuSettlement");
		}
		for(size_t i=0; i<settlement_type.size(); ++i) {
			IGObjIterator& it = getFaction()->getUnitTable(settlement_type[i]);
			while(it.hasNext()) {
				building_ptr b=ToBuilding(it.iterate());
				float dist=(b->getPos()-getPos()).square();
				if(!settlement || dist<min_dist) {
					min_dist = dist;
					settlement=b;
				}
			}
		}
		return !!settlement;
	}

	bool searchMaterial(material_ptr old) {
		material_ptr nearest;
		float dist = 0.0f;

		IGObjIterator& it = GetBlocksByRadius(getPos(), getSight());
		while(it.hasNext()) {
			material_ptr m=ToMaterial(it.iterate());
			if(!m)
				continue;
			if(!old
			|| (old->isTimber() && m->isTimber())
			|| (old->isSilicon()&& m->isSilicon())
			|| (old->isIron()   && m->isIron())) {
				float dist2 = (m->getPos()-getPos()).norm();
				if(!nearest || (dist2<dist && dist2<getSight())) {
					dist=dist2;
					nearest=m;
				}
			}
		}
		if(!!nearest) {
			int flag = Event::ActionInfo::INSIST;
			if(dist<500.0f)
				flag|=Event::ActionInfo::PATH;
			SendActionEvent(this, this, new Event::ActionInfo(nearest->getPos(), nearest, flag));
			return true;
		}
		return false;
	}

	void onCling(selectable_ptr p) {
		if(building_ptr b = ToBuilding(p)) {
			if(b->isBuilt() && b->isSettlement()
			&& (res.timber>0 || res.silicon>0 || res.iron>0)) { // ^
				Resource& r = getFaction()->getResource();
				r+=res;
				res = Resource();
			}
			else if(b->getLife()<b->getMaxLife()) { // C
				b->build(build_speed);
				const ist::vector4& rot = getRot();
				setRot(ist::vector4(rot.x, rot.y-4.0f, rot.z));
			}
			else {
				popAction();
				if(sizeAction()==0 && b->isSettlement())
					searchMaterial(material_ptr());
			}
		}
		else if(material_ptr b = ToMaterial(p)) {
			if((res.timber+res.silicon+res.iron)>=max_resource) {
				popAction();
				searchSettlement();
				if(!!settlement) {
					int flag = Event::ActionInfo::INSIST | Event::ActionInfo::APPENDF;
					if((settlement->getPos()-getPos()).norm()<500.0f)
						flag|=Event::ActionInfo::PATH;
					SendActionEvent(this, this, new Event::ActionInfo(p->getPos(), p, flag));
					SendActionEvent(this, this, new Event::ActionInfo(settlement->getPos(), settlement, flag));
				}
				return;
			}
			if(b->isTimber()) {
				b->decreaseQuantity(gather_timber);
				res.timber+=std::min(b->getQuantity(), gather_timber);
			}
			else if(b->isSilicon()) {
				b->decreaseQuantity(gather_silicon);
				res.silicon+=std::min(b->getQuantity(), gather_silicon);
			}
			else if(b->isIron()) {
				b->decreaseQuantity(gather_iron);
				res.iron+=std::min(b->getQuantity(), gather_iron);
			}

			const ist::vector4& rot = getRot();
			setRot(ist::vector4(rot.x, rot.y+4.0f, rot.z));
		}
	}

	void action(Event::ActionInfo& info) {
		selectable_ptr& p = info.getObj();
		if(!p || p.get()==this) {
			popAction();
			return;
		}
		else if(p->isDead()) {
			if(p->isMaterial() && sizeAction()==1) {
				searchMaterial(ToMaterial(p));
			}
			popAction();
			return;
		}

		bool cling=false;
		if(p->isMaterial() || (p->isBuilding() && p->getFaction()==getFaction())) { // Ă邩
			ist::CollisionDetector cd;
			ist::SimplePointCollisionModel pcm(getPos(), getHitRadius()+getMoveSpeed());
			if(cd.detect(pcm, p->getHitModel())) {
				cling=true;
				onCling(p);
			}
		}
		if(!cling) {
			if(!info.flagPath())
				info.target_pos = p->getPos();
			move(info);
		}
	}

	void upgrade(const std::string& ugname) {
		if(ugname=="Gather1") {
			addGatherTimber(0.5f);
			addGatherSilicon(0.5f);
			addGatherIron(0.5f);
		}
		else if(ugname=="Gather2") {
			addGatherTimber(0.5f);
			addGatherSilicon(0.5f);
			addGatherIron(0.5f);
		}
		else if(ugname=="Gather3") {
			addGatherTimber(0.5f);
			addGatherSilicon(0.5f);
			addGatherIron(0.5f);
		}
		else if(ugname=="Carry1") {
			addMaxResource(10.0f);
		}
		else if(ugname=="Carry2") {
			addMaxResource(10.0f);
		}
		else if(ugname=="WorkerProtector") {
			setMaxLife(getMaxLife()+40);
			setAntiBullet(getAntiBullet()+0.15f);
			setAntiLaser(getAntiLaser()+0.15f);
		}
	}

	void onCollideTarget(selectable_ptr p) {
		if(!(p->isBuilding() && p->getFaction()==getFaction()))
			popAction();
	}

	menu_ptr getCommandMenu() {
		return getFaction()->createCommandMenu("Worker");
	}

	void onKill(Event& e) {
		settlement = building_ptr();
		BaseClass::onKill(e);
	}

	void explode() { SmallExplode(this); }
	const std::string& getClassName() const { static std::string cname("tohuWorker"); return cname; }

private:
	float build_speed;
	float gather_timber, gather_silicon, gather_iron;

	building_ptr settlement;
	Resource res;
	float max_resource;
};










//class tohuEmperor : public LinearInherit4(tohuUnit, HaveShadow, HavePolygonModel, Unit)
class tohuEmperor : public LinearInherit4(tohuUnit, HaveShadow, HaveCPolygonModel, Unit)
{
public:
	int satk_count;

	tohuEmperor(faction_ptr arg) : satk_count(0) {
		setFaction(arg);
		setSelectionRadius(45);
		setHitRadius(30);
		setFloat(70);
		setMaxLife(99999);
		setMaxEnergy(100);
		setAntiBullet(0.0f);
		setAntiLaser(0.0f);
		setAntiBomb(0.9f);
		setMoveSpeed(1.5f);
		setFirePower(75);

		setSight(400);
		setSearchRange(400);
		setFireRange(400);

		setShadowScl(ist::vector4(70, 70, 70));
		setCModel("resource/tohu_emperor.avx");
		setClipName("wait");
	//	setPModel("resource/natsuko.mqo");
	//	setTexture("resource/natsuko.png");
	}

	bool category(const std::string& c) const {
		return (c=="Small" || c=="Emperor") ? true : BaseClass::category(c);
	}

//	void setLight(ist::Light&){}


	void fire(selectable_ptr target) {
		if(getEnergy()==getMaxEnergy()) {
			setCoolDown(150);
			setClipName("attack");
			setEnergy(0);
			SendCommandEvent(this, this, new Event::CommandInfo("specialAttack", target), 45);
			return;
		}

		int cool_down = 10;
		float rate = (float)cool_down/60.0f;

		SendDamageEvent(this, target, new Event::DamageInfo(getFirePower()*rate, 0, getFirePower()*rate*0.5f));
		setCoolDown(cool_down);

		if(target->isInCameraSight()) {
			ExplodeParticle *p = new ExplodeParticle();
			AddGObj(p);
			p->setTexture("resource/explode0.png");
			p->setPos(target->getPos());
		}
	}

	void commandSpecialAttack(selectable_ptr target) {
	/*	
	*/
	/*
		HyuganatsuBomb *g = new HyuganatsuBomb(target->getPos(), getPos());
		AddGObj(g);
		g->setRoot(this);
		g->setDamage(new Event::DamageInfo(25, 0, 50, 1.0f));
	*/

		++satk_count;
		int pattern = satk_count%3;
		if(pattern==0) {
			Grave *g = new Grave(target->getPos(), (target->getPos()-getPos()).setY(0).normalize());
			AddGObj(g);
			g->setRoot(this);
			g->setDamage(new Event::DamageInfo(25, 0, 50, 1.0f));
		}
		else if(pattern==1) {
			MeteorSwarm *m = new MeteorSwarm(target->getPos());
			AddGObj(m);
			m->setRoot(this);
			m->setDamage(new Event::DamageInfo(25, 0, 50, 1.0f));
		}
		else {
			Explosion *g = new Explosion(target->getPos());
			AddGObj(g);
			g->setRoot(this);
			g->setDamage(new Event::DamageInfo(10, 0, 20, 1.0f));
		}
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		setEnergy(getEnergy()+(100.0f/900.0f));
	}

	void explode() { MediumExplode(this); }
	const std::string& getClassName() const { static std::string cname("tohuEmperor"); return cname; }
};





class tohuHealer : public LinearInherit4(tohuUnit, HaveShadow, HaveCPolygonModel, Unit)
{
public:
	tohuHealer(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(10);
		setHitRadius(5);
		setFloat(20);
		setMaxLife(70);
		setAntiBullet(0.0f);
		setAntiLaser(0.0f);
		setAntiBomb(0.95f);
		setMoveSpeed(1.0f);
		setFirePower(2.0f);
		setEnUse(0.001f);

		setSight(200);
		setSearchRange(150);
		setFireRange(100);

		setCModel("resource/tohu_healer.avx");
		setClipName("wait");
	}

	bool category(const std::string& c) const {
		return (c=="Small" || c=="Healer") ? true : BaseClass::category(c);
	}

	virtual void pollTarget() {
		breakable_ptr target;
		float min_dist = 0;

		IGObjIterator& block = GetBlocksByRadius(this->getPos(), this->getSight());
		while(block.hasNext()) {
			breakable_ptr p = ToBreakable(block.iterate());
			if(!!p && p.get()!=this && p->isUnit() && IsAlly(this, p) && p->getLife()<p->getMaxLife()) {
				float dist = (p->getPos()-this->getPos()).setY(0).norm();
				if(dist<this->getSight() && (!target || dist<min_dist)) {
					target = p;
					min_dist = dist;
				}
			}
		}
		if(!!target)
			pushfrontAction(new Event::ActionInfo(ist::vector4(), target));
	}
	void pollLink() {}

	void counter(selectable_ptr p) {}

	void action(Event::ActionInfo& info) {
		breakable_ptr p = ToBreakable(info.getObj());
		if(!p || p->isDead() || p.get()==this || (IsAlly(this, p) && p->getLife()==p->getMaxLife())) {
			this->popAction();
			if(sizeAction()==0 || !getCurrentAction()->getObj())
				this->pollTarget();
			return;
		}

		if(!IsAlly(this, p)) {
			info.target_pos = p->getPos();
			this->move(info);
			return;
		}

		float distace_from_target = (p->getPos()-this->getPos()).setY(0).norm();
		if(distace_from_target < this->getFireRange()) {
			this->turnToTarget(p->getPos()-this->getPos());
			if(this->getCoolDown()<0)
				this->fire(p);
		}
		else if(distace_from_target < this->getSight() || info.flagInsist()) {
			info.target_pos = p->getPos();
			this->move(info);
		}
		else {
			this->popAction();
		}
	}

	void fire(selectable_ptr t) {
		int cool_down = 180;
		float rate = (float)cool_down/60.0f;

		IGObjIterator& i = GetBlocksByRadius(t->getPos(), 70);
		while(i.hasNext()) {
			breakable_ptr s = ToBreakable(i.iterate());
			Event::commandinfo_ptr cp = new Event::CommandInfo("recover", float(getFirePower()*rate));
			if(!!s && s->isUnit() && s.get()!=this && IsAlly(s, this) && (s->getPos()-t->getPos()).setY(0).norm()<70) {
				SendCommandEvent(this, s, cp);
			}
		}
		if(t->isInCameraSight()) {
			Pillar *p = new Pillar();
			AddGObj(p);
			p->setScl(ist::vector4(70, 0, 70));
			p->setPos(t->getPos());
		}
		setCoolDown(cool_down);
	}

	void explode() { SmallExplode(this); }
	const std::string& getClassName() const { static std::string cname("tohuHealer"); return cname; }
};


class tohuSoldier1 : public LinearInherit5(HaveSpyAbility, tohuUnit, HaveShadow, HaveCPolygonModel, Unit)
{
public:
	tohuSoldier1(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(10);
		setHitRadius(5);
		setFloat(20);
		setAntiBomb(0.95f);

		setSight(230);
		setSearchRange(200);
		setFireRange(160);
		setEnUse(0.001f);

		setCModel("resource/tohu_soldier.avx");
		setClipName("wait");

		upgrade("smallLevel0");
	}

	bool category(const std::string& c) const {
		return (c=="Small" || c=="Infantry") ? true : BaseClass::category(c);
	}

	void upgrade(const std::string& ugname) {
		if(ugname=="smallLevel0") {
			setMaxLife(100);
			setMaxEnergy(75);
			setAntiBullet(0.30f);
			setAntiLaser(0.10f);
			setFirePower(5.0f);
			setMoveSpeed(1.5f);
			setGrade(0);
		}
		else if(ugname=="smallLevel1") {
			setMaxLife(116);
			setAntiBullet(0.34f);
			setAntiLaser(0.13f);
			setFirePower(6.0f);
			setMoveSpeed(1.6f);
			setGrade(1);
		}
		else if(ugname=="smallLevel2") {
			setMaxLife(134);
			setAntiBullet(0.38f);
			setAntiLaser(0.16f);
			setFirePower(7.0f);
			setMoveSpeed(1.7f);
			setGrade(2);
		}
		else if(ugname=="smallLevel3") {
			setMaxLife(150);
			setAntiBullet(0.42f);
			setAntiLaser(0.19f);
			setFirePower(7.5f);
			setMoveSpeed(1.8f);
			setGrade(3);
		}
		else if(ugname=="Spying") {
			enableSpyAbility(true);
		}
	}

	void fire(selectable_ptr target) {
		int cool_down = 40;
		float rate = (float)cool_down/60.0f;

		tohuGunBullet *bul = new tohuGunBullet();
		AddGObj(bul);
		bul->setRoot(this);
		bul->setTarget(target);
		bul->setPos(getPos());
		bul->setVel((target->getPos()-getPos()).normalize()*15);
		bul->setDamage(new Event::DamageInfo(getFirePower()*rate, 0, 0));
		setCoolDown(cool_down);
		setClipName("attack");
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		if(!isSpying()) {
			setEnergy(getEnergy()+0.025f);
		}
	}

	void explode() { SmallExplode(this); }
	const std::string& getClassName() const { static std::string cname("tohuSoldier1"); return cname; }
};

class tohuSoldier2 : public LinearInherit4(tohuUnit, HaveShadow, HavePolygonModel, Unit)
{
public:
	tohuSoldier2(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(15);
		setHitRadius(6);
		setFloat(20);
		setAntiBomb(0.95f);

		setSight(230);
		setSearchRange(200);
		setFireRange(160);
		setEnUse(0.003f);

		setPModel("resource/tohu_soldier2.mqo");
	//	setCModel("resource/tohu_soldier.avx");
	//	setClipName("wait");

		upgrade("smallLevel0");
	}

	bool category(const std::string& c) const {
		return (c=="Small" || c=="Infantry") ? true : BaseClass::category(c);
	}

	void upgrade(const std::string& ugname) {
		if(ugname=="smallLevel0") {
			setMaxLife(300);
			setAntiBullet(0.25f);
			setAntiLaser(0.10f);
			setFirePower(15.0f);
			setFireRange(200);
			setMoveSpeed(1.5f);
			setGrade(0);
		}
	}

	void fire(selectable_ptr target) {
		int cool_down = 40;
		float rate = (float)cool_down/60.0f;

		tohuGunBullet *bul = new tohuGunBullet();
		AddGObj(bul);
		bul->setRoot(this);
		bul->setTarget(target);
		bul->setPos(getPos());
		bul->setVel((target->getPos()-getPos()).normalize()*15);
		bul->setDamage(new Event::DamageInfo(getFirePower()*rate, 0, 0));
		setCoolDown(cool_down);
	//	setClipName("attack");
	}

	void explode() { SmallExplode(this); }
	const std::string& getClassName() const { static std::string cname("tohuSoldier2"); return cname; }
};

class tohuSoldier3 : public LinearInherit5(HaveRapidAbility, tohuUnit, HaveShadow, HavePolygonModel, Unit)
{
public:
	tohuSoldier3(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(20);
		setHitRadius(7);
		setFloat(20);
		setAntiBomb(0.95f);

		setSight(230);
		setSearchRange(200);
		setFireRange(160);
		setEnUse(0.009f);

		setMaxEnergy(100);
		setRapidLevel(3);
		setPModel("resource/tohu_soldier3.mqo");
	//	setCModel("resource/tohu_soldier.avx");
	//	setClipName("wait");

		upgrade("smallLevel0");
	}

	bool category(const std::string& c) const {
		return (c=="Small" || c=="Infantry") ? true : BaseClass::category(c);
	}

	void upgrade(const std::string& ugname) {
		if(ugname=="smallLevel0") {
			setMaxLife(900.0f);
			setAntiBullet(0.25f);
			setAntiLaser(0.10f);
			setFirePower(45.0f);
			setFireRange(200.0f);
			setMoveSpeed(1.5f);
			setGrade(0);
		}
	}

	void fire(selectable_ptr target) {
		int cool_down = 40;
		float rate = (float)cool_down/60.0f;

		tohuGunBullet *bul = new tohuGunBullet();
		AddGObj(bul);
		bul->setRoot(this);
		bul->setTarget(target);
		bul->setPos(getPos());
		bul->setVel((target->getPos()-getPos()).normalize()*15);
		bul->setDamage(new Event::DamageInfo(getFirePower()*rate, 0, 0));
		setCoolDown(cool_down);
	//	setClipName("attack");
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		if(!isRapid()) {
			setEnergy(getEnergy()+0.03f);
		}
	}

	void explode() { SmallExplode(this); }
	const std::string& getClassName() const { static std::string cname("tohuSoldier3"); return cname; }
};




class tohuArmor1 : public LinearInherit4(tohuUnit, HaveShadow, HaveCPolygonModel, Unit)
{
public:
	int fire_count;

	tohuArmor1(faction_ptr arg) :
		fire_count(0)
	{
		setFaction(arg);
		setSelectionRadius(14);
		setHitRadius(7);
		setFloat(20);
		setAntiBomb(0.75f);

		setSight(230);
		setSearchRange(150);
		setFireRange(120);
		setEnUse(0.001f);

		setMoveSpeed(0.8f);
		setCModel("resource/tohu_armor.avx");
		setClipName("wait");

		upgrade("smallLevel0");
	}

	bool category(const std::string& c) const {
		return (c=="Small" || c=="Infantry" || c=="HeavyInfantry") ? true : BaseClass::category(c);
	}

	void upgrade(const std::string& ugname) {
		if(ugname=="smallLevel0") {
			setMaxLife(160);
			setAntiBullet(0.25f);
			setAntiLaser(0.70f);
			setFirePower(3.5f);
			setGrade(0);
		}
		else if(ugname=="smallLevel1") {
			setMaxLife(186);
			setAntiBullet(0.28f);
			setAntiLaser(0.73f);
			setFirePower(4.1f);
			setGrade(1);
		}
		else if(ugname=="smallLevel2") {
			setMaxLife(213);
			setAntiBullet(0.31f);
			setAntiLaser(0.76f);
			setFirePower(4.7f);
			setGrade(2);
		}
		else if(ugname=="smallLevel3") {
			setMaxLife(240);
			setAntiBullet(0.34f);
			setAntiLaser(0.79f);
			setFirePower(5.3f);
			setGrade(3);
		}
	}


	void fire(selectable_ptr target) {
		int cool_down = 30;
		float rate = (float)cool_down/60.0f;

		ist::matrix44 mat;
		calcMatrix(mat);
		ist::vector4 fire_pos;
		if(fire_count%2==0) {
			setClipName("attackR");
			fire_pos = mat*ist::vector4(22, 4, 4);
		}
		else {
			setClipName("attackL");
			fire_pos = mat*ist::vector4(-22, 4, 4);
		}

		tohuGunBullet *bul = new tohuGunBullet();
		AddGObj(bul);
		bul->setRoot(this);
		bul->setTarget(target);
		bul->setPos(fire_pos);
		bul->setVel((target->getPos()-fire_pos).normalize()*15);
		bul->setDamage(new Event::DamageInfo(getFirePower()*rate, 0, getFirePower()*rate*0.5f));
		setCoolDown(cool_down);
		++fire_count;
	}

	void explode() { SmallExplode(this); }
	const std::string& getClassName() const { static std::string cname("tohuArmor1"); return cname; }
};


class tohuArmor2 : public LinearInherit4(tohuUnit, HaveShadow, HavePolygonModel, Unit)
{
public:
	tohuArmor2(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(16);
		setHitRadius(8);
		setFloat(20);
		setAntiBomb(0.75f);

		setSight(230);
		setSearchRange(150);
		setFireRange(120);
		setEnUse(0.003f);

		setMoveSpeed(0.8f);
		setPModel("resource/tohu_armor2.mqo");
	//	setCModel("resource/tohu_armor.avx");
	//	setClipName("wait");

		upgrade("smallLevel0");
	}

	bool category(const std::string& c) const {
		return (c=="Small" || c=="Infantry" || c=="HeavyInfantry") ? true : BaseClass::category(c);
	}

	void upgrade(const std::string& ugname) {
		if(ugname=="smallLevel0") {
			setMaxLife(480);
			setAntiBullet(0.35f);
			setAntiLaser(0.50f);
			setFirePower(10.5f);
			setFireRange(130);
			setGrade(0);
		}
	}


	void fire(selectable_ptr target) {
		int cool_down = 30;
		float rate = (float)cool_down/60.0f;

		tohuGunBullet *bul = new tohuGunBullet();
		AddGObj(bul);
		bul->setRoot(this);
		bul->setTarget(target);
		bul->setPos(getPos());
		bul->setVel((target->getPos()-getPos()).normalize()*15);
		bul->setDamage(new Event::DamageInfo(getFirePower()*rate, 0, getFirePower()*rate*0.5f));
		setCoolDown(cool_down);
	}

	void explode() { SmallExplode(this); }
	const std::string& getClassName() const { static std::string cname("tohuArmor2"); return cname; }
};

class tohuArmor3 : public LinearInherit5(HaveGuardAbility, tohuUnit, HaveShadow, HavePolygonModel, Unit)
{
public:
	tohuArmor3(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(18);
		setHitRadius(9);
		setFloat(20);
		setAntiBomb(0.75f);

		setSight(230);
		setSearchRange(150);
		setFireRange(120);
		setEnUse(0.009f);

		setMoveSpeed(0.8f);
		setMaxEnergy(100);
		setGuardLevel(3);
		setPModel("resource/tohu_armor3.mqo");
	//	setCModel("resource/tohu_armor.avx");
	//	setClipName("wait");

		upgrade("smallLevel0");
	}

	bool category(const std::string& c) const {
		return (c=="Small" || c=="Infantry" || c=="HeavyInfantry") ? true : BaseClass::category(c);
	}

	void upgrade(const std::string& ugname) {
		if(ugname=="smallLevel0") {
			setMaxLife(1440);
			setAntiBullet(0.35f);
			setAntiLaser(0.50f);
			setFirePower(31.5f);
			setFireRange(130);
			setGrade(0);
		}
	}


	void fire(selectable_ptr target) {
		int cool_down = 30;
		float rate = (float)cool_down/60.0f;

		tohuGunBullet *bul = new tohuGunBullet();
		AddGObj(bul);
		bul->setRoot(this);
		bul->setTarget(target);
		bul->setPos(getPos());
		bul->setVel((target->getPos()-getPos()).normalize()*15);
		bul->setDamage(new Event::DamageInfo(getFirePower()*rate, 0, getFirePower()*rate*0.5f));
		setCoolDown(cool_down);
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		if(!isGuard()) {
			setEnergy(getEnergy()+0.03f);
		}
	}

	void explode() { SmallExplode(this); }
	const std::string& getClassName() const { static std::string cname("tohuArmor3"); return cname; }
};



class tohuLaser1 : public LinearInherit4(tohuUnit, HaveShadow, HaveCPolygonModel, Unit)
{
public:
	tohuLaser1(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(8);
		setHitRadius(4);
		setFloat(20);
		setAntiBomb(0.95f);

		setSight(275);
		setSearchRange(250);
		setFireRange(250);
		setEnUse(0.001f);

		setCModel("resource/tohu_laser.avx");
		setClipName("wait");

		upgrade("smallLevel0");
	}

	bool category(const std::string& c) const {
		return (c=="Small" || c=="Archer") ? true : BaseClass::category(c);
	}

	void upgrade(const std::string& ugname) {
		if(ugname=="smallLevel0") {
			setMaxLife(40.0f);
			setFirePower(8.0f);
			setMoveSpeed(1.2f);
			setGrade(0);
		}
		else if(ugname=="smallLevel1") {
			setMaxLife(45.0f);
			setFirePower(9.5f);
			setMoveSpeed(1.3f);
			setGrade(1);
		}
		else if(ugname=="smallLevel2") {
			setMaxLife(50.0f);
			setFirePower(11.0f);
			setMoveSpeed(1.4f);
			setGrade(2);
		}
		else if(ugname=="smallLevel3") {
			setMaxLife(55.0f);
			setFirePower(12.5f);
			setMoveSpeed(1.5f);
			setGrade(3);
		}
	}

	void fire(selectable_ptr target) {
		int cool_down = 90;
		float rate = (float)cool_down/60.0f;

		tohuRayBullet *bul = new tohuRayBullet();
		AddGObj(bul);
		bul->setRoot(this);
		bul->setTarget(target);
		bul->setPos(getPos());
		bul->setVel( ((target->getPos()-getPos()).normalize()*-25).setY(15) );
		bul->setDamage(new Event::DamageInfo(0, getFirePower()*rate, 0));
		setCoolDown(cool_down);
		setClipName("attack");
	}

	void explode() { SmallExplode(this); }
	const std::string& getClassName() const { static std::string cname("tohuLaser1"); return cname; }
};

class tohuLaser2 : public LinearInherit4(tohuUnit, HaveShadow, HavePolygonModel, Unit)
{
public:
	tohuLaser2(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(10);
		setHitRadius(5);
		setFloat(20);
		setAntiBomb(0.95f);

		setSight(275);
		setSearchRange(250);
		setFireRange(250);
		setEnUse(0.003f);

		setPModel("resource/tohu_laser2.mqo");
	//	setCModel("resource/tohu_laser.avx");
	//	setClipName("wait");

		upgrade("smallLevel0");
	}

	bool category(const std::string& c) const {
		return (c=="Small" || c=="Archer") ? true : BaseClass::category(c);
	}

	void upgrade(const std::string& ugname) {
		if(ugname=="smallLevel0") {
			setMaxLife(120.0f);
			setFirePower(21.0f);
			setFireRange(300.0f);
			setMoveSpeed(1.2f);
			setGrade(0);
		}
	}

	void fire(selectable_ptr target) {
		int cool_down = 90;
		float rate = (float)cool_down/60.0f;

		tohuRayBullet *bul = new tohuRayBullet();
		AddGObj(bul);
		bul->setRoot(this);
		bul->setTarget(target);
		bul->setPos(getPos());
		bul->setVel( ((target->getPos()-getPos()).normalize()*-25).setY(15) );
		bul->setDamage(new Event::DamageInfo(0, getFirePower()*rate, 0));
		setCoolDown(cool_down);
	//	setClipName("attack");
	}

	void explode() { SmallExplode(this); }
	const std::string& getClassName() const { static std::string cname("tohuLaser2"); return cname; }
};

class tohuLaser3 : public LinearInherit4(tohuUnit, HaveShadow, HavePolygonModel, Unit)
{
public:
	tohuLaser3(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(8);
		setHitRadius(6);
		setFloat(20);
		setAntiBomb(0.95f);

		setSight(275);
		setSearchRange(250);
		setFireRange(250);
		setEnUse(0.009f);

		setPModel("resource/tohu_laser3.mqo");
	//	setCModel("resource/tohu_laser.avx");
	//	setClipName("wait");

		upgrade("smallLevel0");
	}

	bool category(const std::string& c) const {
		return (c=="Small" || c=="Archer") ? true : BaseClass::category(c);
	}

	void upgrade(const std::string& ugname) {
		if(ugname=="smallLevel0") {
			setMaxLife(360.0f);
			setFirePower(63.0f);
			setFireRange(300.0f);
			setMoveSpeed(1.2f);
			setGrade(0);
		}
	}

	void fire(selectable_ptr target) {
		int cool_down = 90;
		float rate = (float)cool_down/60.0f;

		tohuRayBullet *bul = new tohuRayBullet();
		AddGObj(bul);
		bul->setRoot(this);
		bul->setTarget(target);
		bul->setPos(getPos());
		bul->setVel( ((target->getPos()-getPos()).normalize()*-25).setY(15) );
		bul->setDamage(new Event::DamageInfo(0, getFirePower()*rate, 0));
		setCoolDown(cool_down);
	//	setClipName("attack");
	}

	void explode() { SmallExplode(this); }
	const std::string& getClassName() const { static std::string cname("tohuLaser3"); return cname; }
};



class tohuAssasin : public LinearInherit5(HaveCloakAbility, tohuUnit, HaveShadow, HaveCPolygonModel, Unit)
{
public:
	tohuAssasin(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(12);
		setHitRadius(4);
		setFloat(20);
		setMaxLife(70);
		setAntiBomb(0.95f);

		setSight(250);
		setSearchRange(200);
		setFireRange(120);
		setEnUse(0.001f);


		setCModel("resource/tohu_assasin.avx");
		setClipName("wait");

		upgrade("smallLevel0");
	}

	bool category(const std::string& c) const {
		return (c=="Small" || c=="Infantry" || c=="Assasin") ? true : BaseClass::category(c);
	}

	void upgrade(const std::string& ugname) {
		if(ugname=="smallLevel0") {
			setMaxLife(70);
			setMaxEnergy(100);
			setAntiBullet(0.15f);
			setAntiLaser(0.35f);
			setFirePower(4.0f);
			setMoveSpeed(2.2f);
			setGrade(0);
		}
		else if(ugname=="smallLevel1") {
			setMaxLife(80);
			setMaxEnergy(120);
			setAntiBullet(0.18f);
			setAntiLaser(0.38f);
			setFirePower(4.5f);
			setMoveSpeed(2.3f);
			setGrade(1);
		}
		else if(ugname=="smallLevel2") {
			setMaxLife(90);
			setMaxEnergy(140);
			setAntiBullet(0.21f);
			setAntiLaser(0.41f);
			setFirePower(5.0f);
			setMoveSpeed(2.4f);
			setGrade(2);
		}
		else if(ugname=="smallLevel3") {
			setMaxLife(100);
			setMaxEnergy(160);
			setAntiBullet(0.24f);
			setAntiLaser(0.44f);
			setFirePower(5.5f);
			setMoveSpeed(2.5f);
			setGrade(3);
		}
		else if(ugname=="Cloaking") {
			enableCloakAbility(true);
		}
	}

/*
	void move(Event::ActionInfo& info) {
		float d = (info.target_pos-getPos()).setY(0).norm();
		if(!!info.getObj())
			d-=getFireRange();
		float enuse = (100.0f/5000.0f)*d;

		if(d>1000 && getEnergy()>enuse) {
			if(isInCameraSight()) {
				Pillar *p = new Pillar();
				AddGObj(p);
				p->setScl(ist::vector4(30, 0, 30));
				p->setPos(getPos());
			}

			ist::vector4 aim = info.target_pos-getPos();
			aim.y=0;
			aim.normalize();
			setPos(getPos()+aim*d);
			setEnergy(getEnergy()-enuse);

			if(isInCameraSight()) {
				Pillar *p = new Pillar();
				AddGObj(p);
				p->setScl(ist::vector4(30, 300, 30));
				p->setExpand(ist::vector4(0, -p->getExpand().y, 0));
				p->setPos(getPos());
			}
		}
		else {
			BaseClass::move(info);
		}
	}
*/
	void fire(selectable_ptr target) {
		int cool_down = 60;
		float rate = (float)cool_down/60.0f;

		ist::matrix44 mat;
		calcMatrix(mat);
		ist::vector4 fire_pos = mat*ist::vector4(0, 0, 20);

		tohuEnergyBullet *bul = new tohuEnergyBullet();
		AddGObj(bul);
		bul->setRoot(this);
		bul->setTarget(target);
		bul->setPos(fire_pos);
		bul->setVel((target->getPos()-fire_pos).normalize()*10);
		bul->setDamage(new Event::DamageInfo(getFirePower()*rate, 0, 0, 1.0f));
		setCoolDown(cool_down);
		setClipName("attack");
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		if(!isCloaking()) {
			setEnergy(getEnergy()+0.03f);
		}
	}

	void explode() { SmallExplode(this); }
	const std::string& getClassName() const { static std::string cname("tohuAssasin"); return cname; }
};








// Medium Unit



class tohuTargetMarker : public LinearInherit4(HaveNoDepthAttrib, HavePolygonModel, HaveFrameCounter, Particle)
{
public:
	float marker_radius;

	tohuTargetMarker() : marker_radius(0)
	{
		setPModel("resource/plane.mqo");
		setTexture("resource/artillery_marker.png");
	}

	int getDrawPriority() const { return 10; }

	void draw() const {
		static std::vector<ist::vector4> vertex;
		if(vertex.empty()) {
			vertex.resize(32);
			for(size_t i=0; i<vertex.size(); ++i) {
				vertex[i].x = cosf((360.0f/vertex.size())*i*ist::radian);
				vertex[i].z = sinf((360.0f/vertex.size())*i*ist::radian);
			}
		}

		glLineWidth(3);
		glDepthMask(GL_FALSE);
		glDisable(GL_DEPTH_TEST);
		ist::Color(1.0f, 1.0f, 1.0f, 0.25f).assign();
		{
			ist::MatrixSaver msaver(GL_MODELVIEW_MATRIX);
			ist::matrix44 mat;
			calcMatrix(mat);
			glMultMatrixf(mat.v);
			glVertexPointer(4, GL_FLOAT, 0, vertex[0].v);
			glEnableClientState(GL_VERTEX_ARRAY);
			glDrawArrays(GL_LINE_LOOP, 0, vertex.size());
			glDisableClientState(GL_VERTEX_ARRAY);
		}
		ist::Color().assign();
		glEnable(GL_DEPTH_TEST);
		glDepthMask(GL_TRUE);
		glLineWidth(1);
	}
	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		float s = getMarkerRadius()-sinf(float(getFrame())*0.1f)*2.0f;
		setScl(ist::vector4(s, s, s));
	}

	void setMarkerRadius(float f) { marker_radius=f; }
	float getMarkerRadius() const { return marker_radius; }
};

class tohuArbalest : public LinearInherit4(tohuUnit, HaveShadow, HaveCPolygonModel, Unit)
{
public:
	float fire_radius;
	bool mode_attack;
	particle_ptr marker;

	tohuArbalest(faction_ptr arg) : fire_radius(0), mode_attack(false)
	{
		setFaction(arg);
		setSelectionRadius(15);
		setHitRadius(10);
		setFloat(20);
		setAntiBullet(0.0f);
		setAntiLaser(0.0f);
		setAntiBomb(0.90f);
		setMoveSpeed(0.3f);

		setSight(150);
		setFireRange(500);
		setEnUse(0.002f);

		setCModel("resource/tohu_arbalest.avx");
		setClipName("wait");

		upgrade("mediumLevel0");
	}

	bool category(const std::string& c) const {
		return (c=="Medium" || c=="Siege" || c=="Arbalest") ? true : BaseClass::category(c);
	}

	void setFireRadius(float f) { fire_radius=f; }
	float getFireRadius() const { return fire_radius; }

	void upgrade(const std::string& ugname) {
		if(ugname=="mediumLevel0") {
			setMaxLife(100);
			setFirePower(2.6f);
			setFireRadius(100);
		}
		else if(ugname=="mediumLevel1") {
			setMaxLife(110);
			setFirePower(2.8f);
			setFireRadius(115);
		}
		else if(ugname=="mediumLevel2") {
			setMaxLife(120);
			setFirePower(3.0f);
			setFireRadius(130);
		}
	}

	void move(Event::ActionInfo& info) {
		if(!mode_attack)
			BaseClass::move(info);
		else
			popAction();
	}

	void action(Event::ActionInfo& info) {
		info.target_pos = info.getObj()->getPos();
		this->move(info);
	}

	void fire() {
		int cool_down = 120;
		float rate = (float)cool_down/60.0f;

		if(!marker)
			return;
		const ist::vector4& attack_pos = marker->getPos();
		Artillery(getID(), getFaction(), attack_pos, getFireRadius(),
			new Event::DamageInfo(getFirePower()*rate, 0, getFirePower()*rate*0.5f));
		setCoolDown(cool_down);
		setClipName("attack");

		if(isInCameraSight()) {
			ist::matrix44 mat;
			calcMatrix(mat);
			ist::vector4 fire_pos = mat*ist::vector4(0, 18, 28);

			ist::vector4 vel = (attack_pos-getPos()).setY(0).normalize()*0.5f;
			ExplodeParticle *p = new ExplodeParticle();
			AddGObj(p);
			p->setTexture("resource/explode0.png");
			p->setVel(vel);
			p->setPos(fire_pos);
			p->setScl(ist::vector4(15, 15, 15));
		}
		if(IsInCameraSight(attack_pos, 50)) {
			for(int i=0; i<2; ++i) {
				ist::vector4 vel = (attack_pos-getPos()).setY(0).normalize()*2.0f;
				vel.x+=(GetRand()-0.5f)*2.0f;
				vel.z+=(GetRand()-0.5f)*2.0f;
				vel.y+=2.0f+(GetRand()-0.5f)*2.0f;

				ist::vector4 pos = attack_pos;
				pos.x+=(GetRand()-0.5f)*40;
				pos.z+=(GetRand()-0.5f)*40;
				pos.y-=20.0f;

				ExplodeParticle *p = new ExplodeParticle();
				AddGObj(p);
				p->setTexture("resource/explode0.png");
				p->setVel(vel);
				p->setPos(attack_pos);
				p->setScl(ist::vector4(100, 100, 100));
			}
		}
	}

	void counter(selectable_ptr p) {}

	void setModeAttack(bool b) {
		if(mode_attack==b)
			return;
		mode_attack = b;
		if(mode_attack) {
			ist::matrix44 mat;
			mat.rotate(0, getRot().y, 0);
			ist::vector4 marker_pos = getPos()+mat*ist::vector4(0, 0, getFireRange());
			marker_pos.y = GetHeight(marker_pos);

			releaseMarker();
			tohuTargetMarker *m = new tohuTargetMarker();
			AddGObj(m);
			m->setMarkerRadius(getFireRadius());
			m->setPos(marker_pos);
			marker = m;
			setClipName("set");
		}
		else {
			releaseMarker();
			setClipName("unset");
		}

		setCoolDown(60);
		SendCommandEvent(this, getFaction(), new Event::CommandInfo("refreshMenu", boost::any()));
	}

	menu_ptr getCommandMenu() {
		CommandButtonSet *bs = CreateDefaultUnitMenu();
		UnitCommandButton *b = new UnitCommandButton();
		if(mode_attack)
			b->setTexture("resource/button_mode_attack_down.png");
		else
			b->setTexture("resource/button_mode_attack_up.png");
		b->setCommand(new Event::CommandInfo("setModeAttack", bool(!mode_attack)));
		b->setDescription("U[h(a)");
		b->setHotKey(SDLK_a);
		bs->setButton(0, 0, b);
		return bs;
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		if(mode_attack && getCoolDown()<0)
			fire();
	}

	bool call(const std::string& n, boost::any& v) {
		if(n=="setModeAttack") setModeAttack(ToBool(v));
		else if(n=="setPos" && mode_attack) {
			static std::string mes("U[h͓]ł܂B");
			if(getFaction()->getNumMessage()==0 || getFaction()->getMessage(getFaction()->getNumMessage()-1)!=mes)
				getFaction()->pushMessage(mes);
		}
		else return BaseClass::call(n, v);
		return true;
	}

	void onKill(Event& e) {
		releaseMarker();
		BaseClass::onKill(e);
	}


	void explode() { SmallExplode(this); }
	const std::string& getClassName() const { static std::string cname("tohuArbalest"); return cname; }

	void releaseMarker() {
		if(!!marker) {
			SendKillEvent(this, marker);
			marker = particle_ptr();
		}
	}
};


class tohuTransporter : public LinearInherit4(tohuUnit, HaveShadow, HaveCPolygonModel, Unit)
{
public:
	tohuTransporter(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(14);
		setHitRadius(8);
		setFloat(70);
		setMaxLife(250);
		setMaxEnergy(100);
		setAntiBullet(0.2f);
		setAntiLaser(0.4f);
		setAntiBomb(0.95f);
		setMoveSpeed(1.5f);
		setFirePower(0.0f);


		setSight(150);
		setSearchRange(0);
		setFireRange(0);
		setEnUse(0.003f);

		setCModel("resource/tohu_transporter.avx");
		setClipName("wait");
	}

	bool category(const std::string& c) const {
		return (c=="Medium" || c=="Transport") ? true : BaseClass::category(c);
	}

	void idle() {
		BaseClass::idle();
		setEnergy(getEnergy()+0.1f);
	}

	void counter(selectable_ptr p) {}

	void action(Event::ActionInfo& info) {
		selectable_ptr s = info.getObj();
		if(IsAlly(s, this)) {
			if(getEnergy()>40.0f) {
				setEnergy(getEnergy()-40.0f);
				ist::vector4 base_pos = s->getPos();
				IGObjIterator& i = GetBlocksByRadius(base_pos, 150);
				while(i.hasNext()) {
					breakable_ptr s = ToBreakable(i.iterate());
					if(!!s && s->isUnit() && s.get()!=this && IsAlly(s, this) && (s->getPos()-base_pos).setY(0).norm()<150) {
						Event::commandinfo_ptr info = new Event::CommandInfo("setPos", getPos());
						SendCommandEvent(this, s, info);
					}
				}
				if(s->isInCameraSight()) {
					Pillar *p = new Pillar();
					AddGObj(p);
					p->setScl(ist::vector4(150, 0, 150));
					p->setPos(s->getPos());
				}
				if(isInCameraSight()) {
					Pillar *p = new Pillar();
					AddGObj(p);
					p->setScl(ist::vector4(150, 300, 150));
					p->setExpand(ist::vector4(0, -p->getExpand().y, 0));
					p->setPos(getPos());
				}
			}
			popAction();
		}
		else
			BaseClass::action(info);
		setEnergy(getEnergy()+0.01f);
	}

	void explode() { SmallExplode(this); }
	const std::string& getClassName() const { static std::string cname("tohuTransporter"); return cname; }
};


class tohuMine : public LinearInherit5(HaveSelfDestructAbility, tohuUnit, HaveShadow, HaveCPolygonModel, Unit)
{
public:
	bool barrier;

	tohuMine(faction_ptr arg) : barrier(false)
	{
		setFaction(arg);
		setSelectionRadius(15);
		setHitRadius(10);
		setFloat(40);
		setMaxLife(100);
		setMaxEnergy(100);
		setAntiBullet(0.0f);
		setAntiLaser(0.0f);
		setAntiBomb(0.90f);
		setMoveSpeed(0.8f);
		setFirePower(0.0f);

		setSight(175);
		setSearchRange(0);
		setFireRange(0);
		setEnUse(0.002f);

		setCModel("resource/tohu_shield.avx");
		setClipName("wait");

		enableSDAbility(true);
	}

	bool category(const std::string& c) const {
		return (c=="Medium" || c=="Mine") ? true : BaseClass::category(c);
	}

	float getAntiBullet() const { return isBarriered() ? 0.95f : BaseClass::getAntiBullet(); }
	float getAntiLaser() const  { return isBarriered() ? 0.95f : BaseClass::getAntiLaser(); }
	float getAntiBomb() const   { return isBarriered() ? 0.95f : BaseClass::getAntiBomb(); }
	void setBarrier(bool b) {
		barrier = b;
		SendCommandEvent(this, getFaction(), new Event::CommandInfo("refreshMenu", boost::any()));
	}

	bool isBarriered() const { return barrier; }

	menu_ptr getCommandMenu() {
		CommandButtonSet *bs = CreateDefaultUnitMenu();
		UnitCommandButton *b = new UnitCommandButton();
		if(barrier)
			b->setTexture("resource/button_barrier_down.png");
		else
			b->setTexture("resource/button_barrier_up.png");
		b->setCommand(new Event::CommandInfo("setBarrier", bool(!barrier)));
		b->setDescription("G(b)");
		b->setHotKey(SDLK_b);
		bs->setButton(0, 0, b);
		return bs;
	}

	void draw() const {
		BaseClass::draw();
		if(barrier) {
			glBlendFunc(GL_SRC_ALPHA, GL_ONE);
			glDepthMask(GL_FALSE);
			glEnable(GL_TEXTURE_2D);
			GetTexture("resource/tohu_barrier.png")->assign();
			{
				ist::MatrixSaver msaver(GL_MODELVIEW_MATRIX);
				ist::matrix44 mat;
				calcMatrix(mat);
				if(getFrame()%2==0)
					mat.scale(1.0f, 1.5f, 1.0f);
				glMultMatrixf( mat.v );
				GetPModel("resource/tohu_shield_barrier.mqo")->draw();
			}
			glDisable(GL_TEXTURE_2D);
			glDepthMask(GL_TRUE);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		}
	}

	void counter(selectable_ptr p) {}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		if(barrier) {
			setEnergy(getEnergy()-(100.0f/1200.0f));
			if(getEnergy()==0)
				setBarrier(false);
		}
		else {
			setEnergy(getEnergy()+(100.0f/1800.0f));
		}
	}

	void onDamage(Event& e) {
		if(!barrier && getEnergy()==getMaxEnergy())
			setBarrier(true);
		BaseClass::onDamage(e);
	}

	void onCommand(Event& e) {
		Event::CommandInfo& info = e.getCommandInfo();
		if(info.name=="setBarrier") setBarrier(ToBool(info.value));
		else BaseClass::onCommand(e);
	}

	void explode() { SmallExplode(this); }
	const std::string& getClassName() const { static std::string cname("tohuMine"); return cname; }
};



class tohuBomber1 : public LinearInherit4(tohuUnit, HaveShadow, HaveCPolygonModel, Unit)
{
public:
	tohuBomber1(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(15);
		setHitRadius(12);
		setFloat(50);
		setAntiBomb(0.8f);

		setSight(200);
		setSearchRange(150);
		setFireRange(130);
		setEnUse(0.002f);

		setMoveSpeed(1.0f);
		setShadowScl(ist::vector4(20, 20, 20));
		setCModel("resource/tohu_bomber.avx");
		setClipName("wait");

		upgrade("mediumLevel0");
	}

	bool category(const std::string& c) const {
		return (c=="Medium" || c=="Siege" || c=="Bomber") ? true : BaseClass::category(c);
	}

	void upgrade(const std::string& ugname) {
		if(ugname=="mediumLevel0") {
			setMaxLife(250);
			setAntiBullet(0.20f);
			setAntiLaser(0.80f);
			setFirePower(25);
			setFloat(70);
			setGrade(0);
		}
		else if(ugname=="mediumLevel1") {
			setMaxLife(300);
			setAntiBullet(0.23f);
			setAntiLaser(0.82f);
			setFirePower(30);
			setFloat(75);
			setGrade(1);
		}
		else if(ugname=="mediumLevel2") {
			setMaxLife(350);
			setAntiBullet(0.26f);
			setAntiLaser(0.84f);
			setFirePower(35);
			setFloat(80);
			setGrade(2);
		}
	}


	void fire(selectable_ptr target) {
		int cool_down = 180;
		SendCommandEvent(this, this, new Event::CommandInfo("attack", target), 45);
		setCoolDown(cool_down);
		setClipName("attack");
	}

	void commandAttack(selectable_ptr target) {
		int cool_down = 180;
		float rate = (float)cool_down/60.0f;

		tohuBombBullet *b = new tohuBombBullet();
		AddGObj(b);
		b->setRoot(this);
		b->setTarget(target);
		b->setPos(getPos());
		b->setVel((target->getPos()-getPos()).normalize()*5);
		b->setDamage(new Event::DamageInfo(0, 0, getFirePower()*rate));
	}

	void explode() { SmallExplode(this); }
	const std::string& getClassName() const { static std::string cname("tohuBomber1"); return cname; }
};





class tohuDetector : public LinearInherit4(tohuUnit, HaveShadow, HaveCPolygonModel, Unit)
{
public:
	tohuDetector(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(14);
		setHitRadius(8);
		setFloat(70);
		setMaxLife(250);
		setMaxEnergy(100);
		setAntiBullet(0.2f);
		setAntiLaser(0.4f);
		setAntiBomb(0.95f);
		setMoveSpeed(1.5f);
		setFirePower(0.0f);

		setSight(350);
		setSearchRange(0);
		setFireRange(0);
		setEnUse(0.003f);

		setCModel("resource/tohu_transporter.avx");
		setClipName("wait");
	}

	bool category(const std::string& c) const {
		return (c=="Medium" || c=="Detector") ? true : BaseClass::category(c);
	}

	void counter(selectable_ptr p) {}

	void detect() {
		IGObjIterator& block = GetBlocksByRadius(getPos(), 350);
		Event::commandinfo_ptr c = new Event::CommandInfo("detectCloak", boost::any());
		while(block.hasNext()) {
			selectable_ptr s = ToSelectable(block.iterate());
			if(!!s && IsEnemy(s, this) && (s->getPos()-getPos()).setY(0).norm()<400) {
				SendCommandEvent(this, s, c);
			}
		}
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		if(getFrame()%10==0)
			detect();
	}

	void explode() { SmallExplode(this); }
	const std::string& getClassName() const { static std::string cname("tohuDetector"); return cname; }
};













// Large Unit



class _FortRoot : public LinearInherit3(
	HaveAttackAction,
	HaveCharacterGridInserter,
	LinearInherit9(
		HaveFrameCounter,
		HaveActionQueue,
		BelongFaction,
		HaveBreakableInfo,
		HaveTransformInfo,
		HaveDeathFlag,
		HaveID,
		HaveEventQueue,
		IBullet))
{};

class Fort : public _FortRoot
{
private:
	unit_ptr root;
	ist::vector4 rel_pos;

public:
	Fort() {}

	int getDrawPriority() const { return 100; }
	category_t getRecieveEvents() const { return RCV_UPDATE; }

	virtual unit_ptr getRoot() { return root; }
	virtual const ist::vector4& getRPos() const { return rel_pos; }

	virtual void setRoot(unit_ptr v) {
		root = v;
		setFaction(root->getFaction());
		setSight(root->getSight());
		setFireRange(root->getFireRange());
		setSearchRange(getFireRange());
	}
	virtual void setRPos(const ist::vector4& v) { rel_pos=v; }



	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		
		unit_ptr r = getRoot();
		if(!r) {
			SendKillEvent(this, this);
			return;
		}
		ist::matrix44 mat;
		mat.translate(r->getPos().v);
		mat.rotate(r->getRot().v);
		setPos(mat*getRPos());

		if(getFrame()%10==0) {
			Event::actioninfo_ptr ap = r->getCurrentAction();
			if(!!ap && !!ap->getObj()) {
				clearAction();
				pushfrontAction(ap);
			}
		}
	}

	void onKill(Event& e) {
		root = unit_ptr();
		BaseClass::onKill(e);
	}

	virtual void drawModel() const {}

	void draw() const {
		getFaction()->getFactionMaterial().assign();
		this->drawModel();
		ist::Material().assign();
	}
};


class tohuBit1 : public LinearInherit2(HavePolygonModel, Fort)
{
public:
	ist::vector4 aim;

	tohuBit1(unit_ptr r) {
		setRoot(r);
		setPModel("resource/tohu_bit1.mqo");
	//	setClipName("wait");
		aim = ist::vector4(ist::vector4(0, 0, 1.0f));
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);

		Event::actioninfo_ptr ap = getCurrentAction();
		if(!!ap && !!ap->getObj()) {
			const ist::vector4& t_pos = ap->getObj()->getPos();
			aim = (aim*10+(t_pos-getPos()).normalize()).normalize();
		}
	}

	void calcMatrix(ist::matrix44& mat) const {
		mat.translate(getPos().v);
		mat.aimVector(aim);
	}

	void fire(selectable_ptr target) {
		int cool_down = 30;
		float rate = (float)cool_down/60.0f;

		tohuGunBullet *bul = new tohuGunBullet();
		AddGObj(bul);
		bul->setRoot(getRoot());
		bul->setTarget(target);
		bul->setPos(getPos());
		bul->setVel((target->getPos()-getPos()).normalize()*15);
		bul->setDamage(new Event::DamageInfo(getFirePower()*rate, 0, 0));
		setCoolDown(cool_down);
	}
};

class tohuBit2 : public tohuBit1
{
public:
	tohuBit2(unit_ptr r) : tohuBit1(r) {
		setPModel("resource/tohu_bit2.mqo");
	}

	void fire(selectable_ptr target) {
		int cool_down = 30;
		float rate = (float)cool_down/60.0f;

		tohuRayBullet *bul = new tohuRayBullet();
		AddGObj(bul);
		bul->setRoot(getRoot());
		bul->setTarget(target);
		bul->setPos(getPos());
		bul->setVel( ((target->getPos()-getPos()).normalize()*-25).setY(15) );
		bul->setDamage(new Event::DamageInfo(0, getFirePower()*rate, 0));
		setCoolDown(cool_down);
	}

};



template<class T>
class NotTurnTarget : public T
{
public:
	typedef NotTurnTarget BaseClass;
	void attackAction(Event::ActionInfo& info) {
		selectable_ptr& p = info.getObj();

		float distace_from_target = (p->getPos()-this->getPos()).setY(0).norm();
		if(distace_from_target < this->getFireRange()) {}
		else if(distace_from_target < this->getSight() || info.flagInsist()) {
			info.target_pos = p->getPos();
			this->move(info);
		}
		else {
			this->popAction();
		}
	}
};

template<class T>
class HaveServantObject : public T
{
public:
	typedef T BaseClass;
	typedef std::vector<gobj_ptr> servant_cont;
	servant_cont servants;

	void addServant(gobj_ptr p) {
		servants.push_back(p);
	}

	void draw() const {
		T::draw();
		for(servant_cont::const_iterator p=servants.begin(); p!=servants.end(); ++p)
			(*p)->draw();
	}

	void update() {
		T::update();
		for(servant_cont::iterator p=servants.begin(); p!=servants.end(); ++p)
			(*p)->update();
	}

	void onUpdate(Event& e) {
		T::onUpdate(e);
		for(servant_cont::iterator p=servants.begin(); p!=servants.end(); ++p)
			(*p)->sendEvent(e);
	}

	void onKill(Event& e) {
		T::onKill(e);
		for(servant_cont::iterator p=servants.begin(); p!=servants.end(); ++p)
			(*p)->sendEvent(e);
	}

	void onDestroy(Event& e) {
		T::onDestroy(e);
		for(servant_cont::iterator p=servants.begin(); p!=servants.end(); ++p)
			(*p)->sendEvent(e);
	}
};


class tohuDestroyer : public LinearInherit6(HaveServantObject, NotTurnTarget, tohuUnit, HaveShadow, HavePolygonModel, Unit)
{
public:
	tohuDestroyer(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(8);
		setHitRadius(6);
		setFloat(50);
		setAntiBomb(0.95f);
		setPModel("resource/tohu_destroyer.mqo");
	//	setCModel("resource/tohu_laser.avx");
	//	setClipName("wait");

		setMaxLife(3000);
		setAntiBullet(0.60f);
		setAntiLaser(0.50f);
		setAntiBomb(0.50f);
		setFirePower(60);
		setMoveSpeed(0.95f);

		setSight(350);
		setSearchRange(275);
		setFireRange(275);
		setEnUse(0.01f);

		setForts();
	}

	bool category(const std::string& c) const {
		return (c=="Large" || c=="Ship" || c=="BattleShip") ? true : BaseClass::category(c);
	}

	void setForts() {
		static ist::vector4 fort_pos[4] = {
			ist::vector4(55, -10, 19),
			ist::vector4(30, 32, 25),
			ist::vector4(-30, 32, 25),
			ist::vector4(-55, -10, 19),
		};
		for(int i=0; i<4; ++i) {
			tohuBit1 *f = new tohuBit1(this);
			f->setRPos(fort_pos[i]);
			f->setFirePower(getFirePower()/4.0f);
			f->setFireRange(300);
			addServant(f);
		}
	}

	void counter(selectable_ptr p) {}
	void fire(selectable_ptr target) {}

	void explode() { SmallExplode(this); }
	const std::string& getClassName() const { static std::string cname("tohuDestroyer"); return cname; }
};


class tohuCruiser : public LinearInherit6(HaveServantObject, NotTurnTarget, tohuUnit, HaveShadow, HavePolygonModel, Unit)
{
public:
	class tohuCruiserBit1 : public tohuBit1
	{
	public:
		tohuCruiserBit1(unit_ptr r) : tohuBit1(r) {}
		void fire(selectable_ptr target) {
			if(target->category("Building") || target->category("Ship")) {
				int cool_down = 30;
				float rate = (float)cool_down/60.0f;

				tohuGunBullet *bul = new tohuGunBullet();
				AddGObj(bul);
				bul->setRoot(getRoot());
				bul->setTarget(target);
				bul->setPos(getPos());
				bul->setVel((target->getPos()-getPos()).normalize()*15);
				bul->setDamage(new Event::DamageInfo(getFirePower()*rate*4.0f, 0, 0));
				bul->setTexture("resource/tohu_bullet2.png");
				setScl( ist::vector4(18.0f, 18.0f, 18.0f) );
				setCoolDown(cool_down);
			}
			else
				tohuBit1::fire(target);
		}
	};
	class tohuCruiserBit2 : public tohuBit2
	{
	public:
		tohuCruiserBit2(unit_ptr r) : tohuBit2(r) {}
		void fire(selectable_ptr target) {
			if(target->category("Building") || target->category("Ship")) {
				int cool_down = 30;
				float rate = (float)cool_down/60.0f;

				tohuRayBullet *bul = new tohuRayBullet();
				AddGObj(bul);
				bul->setRoot(getRoot());
				bul->setTarget(target);
				bul->setPos(getPos());
				bul->setVel( ((target->getPos()-getPos()).normalize()*-25).setY(15) );
				bul->setDamage(new Event::DamageInfo(0, getFirePower()*rate*4.0f, 0));
				bul->setTexture("resource/hl2.png");
				setCoolDown(cool_down);
			}
			else
				tohuBit2::fire(target);
		}
	};

	tohuCruiser(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(8);
		setHitRadius(6);
		setFloat(60);
		setAntiBomb(0.95f);
		setPModel("resource/tohu_destroyer.mqo");
	//	setCModel("resource/tohu_laser.avx");
	//	setClipName("wait");

		setMaxLife(3500);
		setAntiBullet(0.60f);
		setAntiLaser(0.70f);
		setAntiBomb(0.35f);
		setFirePower(30);
		setMoveSpeed(0.8f);

		setSight(350);
		setSearchRange(300);
		setFireRange(300);
		setEnUse(0.01f);

		setForts();
	}

	bool category(const std::string& c) const {
		return (c=="Large" || c=="Ship" || c=="BattleShip") ? true : BaseClass::category(c);
	}


	void setForts() {
		static ist::vector4 fort_pos[3] = {
			ist::vector4(55, -10, 19),
			ist::vector4(-55, -10, 19),
			ist::vector4(0, 32, 50),
		};
		for(int i=0; i<2; ++i) {
			tohuBit1 *f = new tohuCruiserBit1(this);
			f->setRPos(fort_pos[i]);
			f->setFirePower(getFirePower()/3.0f);
			f->setFireRange(300);
			addServant(f);
		}
		{
			tohuBit2 *f = new tohuCruiserBit2(this);
			f->setRPos(fort_pos[2]);
			f->setFirePower(getFirePower()/3.0f);
			f->setFireRange(350);
			addServant(f);
		}
	}

	void counter(selectable_ptr p) {}
	void fire(selectable_ptr target) {}

	void explode() { SmallExplode(this); }
	const std::string& getClassName() const { static std::string cname("tohuCruiser"); return cname; }
};

class tohuMovilFort : public LinearInherit5(NotTurnTarget, tohuUnit, HaveShadow, HavePolygonModel, Unit)
{
public:
	tohuMovilFort(faction_ptr arg) {
		setFaction(arg);
		setSelectionRadius(8);
		setHitRadius(6);
		setFloat(60);
		setAntiBomb(0.95f);
		setPModel("resource/tohu_destroyer.mqo");
	//	setCModel("resource/tohu_laser.avx");
	//	setClipName("wait");

		setMaxLife(8000);
		setAntiBullet(0.65f);
		setAntiLaser(0.65f);
		setAntiBomb(0.05f);
		setFirePower(200);
		setMoveSpeed(0.6f);

		setSight(350);
		setSearchRange(250);
		setFireRange(250);
		setEnUse(0.01f);
	}

	bool category(const std::string& c) const {
		return (c=="Large" || c=="Ship" || c=="MotherShip") ? true : BaseClass::category(c);
	}

	void fire(selectable_ptr target) {
		int cool_down = 90;
		float rate = (float)cool_down/60.0f;

		tohuRayBullet *bul = new tohuRayBullet();
		AddGObj(bul);
		bul->setRoot(this);
		bul->setTarget(target);
		bul->setPos(getPos());
		bul->setVel( ((target->getPos()-getPos()).normalize()*-25).setY(15) );
		bul->setDamage(new Event::DamageInfo(0, getFirePower()*rate, 0));
		setCoolDown(cool_down);
	//	setClipName("attack");
	}

	void explode() { SmallExplode(this); }
	const std::string& getClassName() const { static std::string cname("tohuMovilFort"); return cname; }
};







unit_ptr tohuCreateWorker(faction_ptr f)   { return new tohuWorker(f); }
unit_ptr tohuCreateSoldier1(faction_ptr f) { return new tohuSoldier1(f); }
unit_ptr tohuCreateSoldier2(faction_ptr f) { return new tohuSoldier2(f); }
unit_ptr tohuCreateSoldier3(faction_ptr f) { return new tohuSoldier3(f); }
unit_ptr tohuCreateArmor1(faction_ptr f)   { return new tohuArmor1(f); }
unit_ptr tohuCreateArmor2(faction_ptr f)   { return new tohuArmor2(f); }
unit_ptr tohuCreateArmor3(faction_ptr f)   { return new tohuArmor3(f); }
unit_ptr tohuCreateLaser1(faction_ptr f)   { return new tohuLaser1(f); }
unit_ptr tohuCreateLaser2(faction_ptr f)   { return new tohuLaser2(f); }
unit_ptr tohuCreateLaser3(faction_ptr f)   { return new tohuLaser3(f); }
unit_ptr tohuCreateBomber1(faction_ptr f)  { return new tohuBomber1(f); }
unit_ptr tohuCreateEmperor(faction_ptr f)  { return new tohuEmperor(f); }

unit_ptr tohuCreateHealer(faction_ptr f)      { return new tohuHealer(f); }
unit_ptr tohuCreateArbalest(faction_ptr f)    { return new tohuArbalest(f); }
unit_ptr tohuCreateTransporter(faction_ptr f) { return new tohuTransporter(f); }
unit_ptr tohuCreateMine(faction_ptr f)      { return new tohuMine(f);  }
unit_ptr tohuCreateAssasin(faction_ptr f)     { return new tohuAssasin(f);  }
unit_ptr tohuCreateDetector(faction_ptr f)    { return new tohuDetector(f);  }

unit_ptr tohuCreateDestroyer(faction_ptr f) { return new tohuDestroyer(f);  }
unit_ptr tohuCreateCruiser(faction_ptr f)   { return new tohuCruiser(f);  }
unit_ptr tohuCreateMovilFort(faction_ptr f) { return new tohuMovilFort(f);  }
