#ifndef tohu_unit_common
#define tohu_unit_common

#include "tohu.h"





template<class T>
class HaveTarget : public T
{
public:
	typedef HaveTarget BaseClass;

	void setTarget(selectable_ptr t) { target=t; }
	selectable_ptr getTarget() { return target; }

	void onUpdate(Event& e) {
		this->T::onUpdate(e);

		if(!target)
			return;

		if(target->isDead()) {
			setTarget(selectable_ptr());
		}
		else {
			ist::CollisionDetector cd;
			if(cd.detect(this->getHitModel(), target->getHitModel())) {
				SendCollideEvent(target, this, new Event::CollideInfo(cd.collide_pos, cd.collide_normal, cd.distance));
			}
		}
	}

	void onKill(Event& e) {
		target = selectable_ptr();
		this->T::onKill(e);
	}

private:
	selectable_ptr target;
};











class tohuGunBullet : public LinearInherit4(HaveTarget, HaveBlendAddAttrib, HavePolygonModel, Bullet)
{
public:
	mutable bool is_matrix_set;
	mutable ist::matrix44 matrix;

	tohuGunBullet() :
		is_matrix_set(false)
	{
		setLifeTime(20);
		setSelectionRadius(5);
		setHitRadius(5);
		setPModel("resource/plane.mqo");
		setTexture("resource/tohu_bullet.png");
		setScl( ist::vector4(10.0f, 10.0f, 10.0f) );
	}

	void calcMatrix(ist::matrix44& mat) const {
		if(!is_matrix_set) {
			ist::vector4 unit_vel = ist::vector4(getVel()).normalize();
			ist::vector4 ax = ist::vector4().cross(unit_vel, ist::vector4(0, -1, 0));
			ist::vector4 up = ist::vector4().cross(unit_vel, ax);
			matrix.aimVector(up);
			if(unit_vel.y<0)
				matrix.rotateY(180);
			matrix.scale(getScl().v);
			is_matrix_set = true;
		}
		mat.translate(getPos().v);
		mat*=matrix;
	}

	void onCollide(Event& e) {
		SendKillEvent(this, this);
		gobj_ptr target = GetGObj(e.from);
		if(!!target)
			SendDamageEvent(getRoot(), target, getDamage());
		if(isInCameraSight()) {
			Flasher *p = new Flasher();
			AddGObj(p);
			p->setPos(getPos());
			p->setScl(ist::vector4(20, 20, 20));
		}
	}
};


class tohuEnergyBullet : public LinearInherit5(HaveTarget, BillboardAttrib, HaveNoDepthAttrib, HavePolygonModel, Bullet)
{
public:
	tohuEnergyBullet() {
		setLifeTime(30);
		setSelectionRadius(5);
		setHitRadius(5);
		setPModel("resource/plane.mqo");
		setTexture("resource/tohu_ebullet.png");
		setScl( ist::vector4(8.0f, 8.0f, 8.0f) );
	}

	void onCollide(Event& e) {
		SendKillEvent(this, this);
		gobj_ptr target = GetGObj(e.from);
		if(!!target)
			SendDamageEvent(getRoot(), target, getDamage());

		if(isInCameraSight()) {
			Flasher *p = new Flasher();
			AddGObj(p);
			p->setPos(getPos());
			p->setScl(ist::vector4(25, 25, 25));
		}
	}
};


class tohuRayBullet : public LinearInherit4(HaveTarget, HaveBlendAddAttrib, HavePolyLine, Bullet)
{
public:
	tohuRayBullet() : is_target_alive(true) {
		setLifeTime(120);
		setSelectionRadius(9);
		setHitRadius(9);
		setPolylineSize(10);
		setPolylineWidth(13);
		setTexture("resource/hl.png");
	}

	void raise() {
		BaseClass::raise();
		is_target_alive = true;
	}

	void searchTarget() {
		breakable_ptr target;
		float min_dist = 0;

		IGObjIterator& block = GetBlocksByRadius(getPos(), 400);
		while(block.hasNext()) {
			breakable_ptr p = ToBreakable(block.iterate());
			if(!!p && IsEnemy(getFaction(), p->getFaction()) && IsTargetable(p)) {
				float dist = (p->getPos()-this->getPos()).setY(0).norm();
				if(dist<300.0f && !target || dist<min_dist) {
					target = p;
					min_dist = dist;
				}
			}
		}
		if(!!target)
			setTarget(target);
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);

		selectable_ptr target = getTarget();
		if(!!target) {
			if(!target->isDead()) {
				setVel((getVel()*0.93f) + (target->getPos()-getPos()).normalize()*1.2f);
				is_target_alive = true;
			}
		}
		else if(is_target_alive) {
			is_target_alive = false;
			searchTarget();
		}
	}

	void onCollide(Event& e) {
		SendKillEvent(this, this);
		gobj_ptr target = GetGObj(e.from);
		if(!!target)
			SendDamageEvent(getRoot(), target, getDamage());

		if(isInCameraSight()) {
			Flasher *p = new Flasher();
			AddGObj(p);
			p->setPos(getPos());
			p->setScl(ist::vector4(40, 40, 40));
		}
	}

private:
	bool is_target_alive;
};

class tohuBombBullet : public LinearInherit5(HaveTarget, BillboardAttrib, HaveNoDepthAttrib, HavePolygonModel, Bullet)
{
public:
	tohuBombBullet() {
		setLifeTime(120);
		setSelectionRadius(10);
		setHitRadius(10);
		setPModel("resource/plane.mqo");
		setTexture("resource/tohu_ebulletr.png");
		setScl( ist::vector4(13.0f, 13.0f, 13.0f) );
	}

	void onCollide(Event& e) {
		SendKillEvent(this, this);
		gobj_ptr target = GetGObj(e.from);
		if(!!target)
			SendDamageEvent(getRoot(), target, getDamage());

		if(isInCameraSight()) {
			for(int i=0; i<3; ++i) {
				ExplodeParticle *p = new ExplodeParticle();
				AddGObj(p);
				p->setTexture("resource/explode1.png");
				p->setPos(getPos());
			}
			{
				Flasher *p = new Flasher();
				AddGObj(p);
				p->setPos(getPos());
				p->setScl(ist::vector4(20, 20, 20));
			}
		}
	}
};








inline void SmallExplode(breakable_ptr bp) {
	if(!bp->isInCameraSight())
		return;

	PlaySE("resource/wav/destroy1.wav", SE_DESTROY);
	{
		Flasher *p = new Flasher();
		AddGObj(p);
		p->setPos(bp->getPos());
	}
	{
		Ring *p = new Ring();
		AddGObj(p);
		p->setPos(bp->getPos());
	}
	for(int i=0; i<3; ++i) {
		ExplodeParticle *p = new ExplodeParticle();
		AddGObj(p);
		p->setPos(bp->getPos());
	}
}


inline void MediumExplode(breakable_ptr bp) {
	if(!bp->isInCameraSight())
		return;

	{
		Flasher *p = new Flasher();
		AddGObj(p);
		p->setPos(bp->getPos());
		p->setScl( ist::vector4(200.0f, 200.0f, 200.0f) );
	}
	for(int i=0; i<40; ++i) {
		ExplodeParticle *p = new ExplodeParticle();
		AddGObj(p);
		p->setPos(bp->getPos());
		p->setVel( ist::vector4((GetRand()-0.5f)*2.0f, (GetRand()-0.5f)*2.0f, (GetRand()-0.5f))*2.0f );
	}
}













/*
class KindSelectionButton : public Button
{
public:
	KindSelectionButton(IGObjIterator& i) : iter(i) {
		setRect(ist::vector2(18, 18));
		if(!!request)
			setTexture(GetUnitImage(request->getName()));
	}

	void draw() const {
		Button::draw();
		char buf[32];
		sprintf(buf, "%.0f%%", request->getProgress());
		DrawFont((int)getPos().x, (int)getPos().y, buf);
	}

	void action() {
		request->cancel();
		SendKillEvent(this, this);
	}

	void onUpdate(Event& e) {
		Button::onUpdate(e);
		if(!request || request->isCanceled() || request->isComplete())
			SendKillEvent(this, this);
	}

	const std::string& getDescription() {
		static std::string mes = "NbNƂ̎ނ̃jbgI܂B";
		return mes;
	}

	IGObjIterator& iter;
};
*/



/*
	:warning:
	쐬LZƓ̂ŃLZs\ɂĂB΍B
*/

class ProductProgressButton : public Button
{
public:
	ProductProgressButton(productr_ptr r) : request(r) {
		setRect(ist::vector2(18, 18));
		if(!!request)
			setTexture(GetUnitImage(request->getName()));
	}

	void draw() const {
		Button::draw();
		char buf[32];
		sprintf(buf, "%.0f%%", request->getProgress());
		DrawFont((int)getPos().x, (int)getPos().y, buf);
	}

	void action() {
	//	request->cancel();
	//	SendKillEvent(this, this);
	}

	void onUpdate(Event& e) {
		Button::onUpdate(e);
		if(!request || request->isCanceled() || request->isComplete())
			SendKillEvent(this, this);
	}

	const std::string& getDescription() {
	//	static std::string mes = "NbNƂ̃jbg̐Y𒆎~܂B";
		static std::string mes = "";
		return mes;
	}

	productr_ptr request;
};

class ProgressMenu : public Menu
{
public:
	typedef boost::intrusive_ptr<Button> button_ptr;
	typedef std::list<button_ptr> button_cont;

	void align() {
		ist::vector2 base_pos(388, 400);
		int loop_count=0;
		for(button_cont::iterator p=buttons.begin(); p!=buttons.end(); ++p) {
			(*p)->setPos(base_pos);
			++loop_count;
			if(loop_count/4<3) {
				if(loop_count%4==0) {
					base_pos.x=388;
					base_pos.y+=20;
				}
				else {
					base_pos.x+=20;
				}
			}
			else {
				base_pos = ist::vector2(1000, 1000);
			}
		}
	}

	void addButton(button_ptr b) {
		AddGObj(b);
		buttons.push_back(b);
		align();
	}

	void eraseDeadObject() {
		bool exist_dead = false;
		for(button_cont::iterator p=buttons.begin(); p!=buttons.end();) {
			if((*p)->isDead()) {
				buttons.erase(p++);
				exist_dead = true;
			}
			else {
				++p;
			}
		}
		if(exist_dead)
			align();
	}

	void onUpdate(Event& e) {
		BaseClass::onUpdate(e);
		eraseDeadObject();
	}

	void onKill(Event& e) {
		BaseClass::onKill(e);
		for(button_cont::iterator p=buttons.begin(); p!=buttons.end(); ++p)
			SendKillEvent(this, *p);
		buttons.clear();
	}

	button_cont buttons;
};


template <class T>
class HaveRoot : public T
{
public:
	typedef HaveRoot BaseClass;

	void setRoot(selectable_ptr r) { root = r; }
	selectable_ptr getRoot() { return root; }

	virtual void onRootDead() {
		SendKillEvent(this, this);
	}

	void onUpdate(Event& e) {
		this->T::onUpdate(e);
		if(root->isDead())
			onRootDead();
	}

	void onKill(Event& e) {
		root = selectable_ptr();
		this->T::onKill(e);
	}

private:
	selectable_ptr root;
};


template<class T>
class HaveUnitCreator : public T
{
public:
	typedef HaveUnitCreator BaseClass;

	HaveUnitCreator() : mode_continue(false) {}
	const ist::vector4& getTargetPos() const { return target_pos; }
	void setPos(const ist::vector4& p) {
		T::setPos(p);
		target_pos = this->getPos();
		target_pos.z+=30;
	}

	void onAction(Event& e) {
		T::onAction(e);
		Event::ActionInfo& info = e.getActionInfo();
		target_pos = info.target_pos;
		target_obj = info.getObj();
	}

	virtual bool conditionCreate() const { return true; }

	virtual void appendSequenceButton(menu_ptr bs) {
		UnitCommandButton *ucb = new UnitCommandButton();
		ucb->setCommand(new Event::CommandInfo("setModeContinue", !mode_continue));
		if(mode_continue)
			ucb->setTexture("resource/button_continue_down.png");
		else
			ucb->setTexture("resource/button_continue_up.png");
		ucb->setDescription("AY");

		SendCommandEvent(this, bs, new Event::CommandInfo("setButton", CommandButtonSet::AppendInfo(6, 2, ucb)));
		bs->update();
	}

	virtual menu_ptr _getCommandMenu()=0;
	menu_ptr getCommandMenu() {
		menu_ptr mp = _getCommandMenu();
		if(!!mp) {
			appendSequenceButton(mp);
			SendCommandEvent(this, mp, new Event::CommandInfo("setSender", gobj_ptr(this)));
			mp->update();
		}
		return mp;
	}

	menu_ptr getExtraMenu() {
		if(!!current_request || !requests.empty()) {
			ProgressMenu *progress_menu = new ProgressMenu();
			if(!!current_request)
				progress_menu->addButton(new ProductProgressButton(current_request));
			for(RequestCont::iterator p=requests.begin(); p!=requests.end(); ++p)
				progress_menu->addButton(new ProductProgressButton(*p));
			return progress_menu;
		}
		return menu_ptr();
	}

	virtual void setModeContinue(bool b) {
		if(conditionCreate()) {
			mode_continue = b;
			SendCommandEvent(this, GetPlayerFaction(), new Event::CommandInfo("refreshMenu", boost::any()));
		}
	}


	void requestCreateUnit(const std::string& cname) {
		if(conditionCreate()) {
			requests.push_back(new ProductRequest(cname));
			SendCommandEvent(this, GetPlayerFaction(), new Event::CommandInfo("refreshMenu", boost::any()));
		}
	}

	virtual void createUnit(const std::string& class_name) {
		gobj_id u = CreateID();
		RouteCreateEvent(new Event::CreateInfo(UNIT, u, class_name, this->getFaction()->getID()));
		ist::vector4 u_pos = this->getPos();
		if(target_pos==this->getPos())
			u_pos+=ist::vector4(0, 0, 1)*this->getHitRadius();
		else
			u_pos+=(target_pos-this->getPos()).normalize()*this->getHitRadius();
		RouteCallEvent(this->getID(), u, new Event::CallInfo("setPos", u_pos));

		if(!!target_obj && target_obj->isDead())
			target_obj = selectable_ptr();
		RouteActionEvent(this->getID(), u, new Event::ActionInfo(target_pos, target_obj));

	/*
		unit_ptr u = this->getFaction()->createUnit(class_name);
		if(!!u) {
			AddGObj(u);
			this->getFaction()->pushMessage(u->getUnitName()+" 쐬܂B");
			ist::vector4 u_pos = this->getPos();
			if(target_pos==this->getPos())
				u_pos+=ist::vector4(0, 0, 1)*this->getHitRadius();
			else
				u_pos+=(target_pos-this->getPos()).normalize()*this->getHitRadius();
			RouteCallEvent(this, u, new Event::CallInfo("setPos", u_pos));

			if(!!target_obj && target_obj->isDead())
				target_obj=selectable_ptr();
			RouteActionEvent(this, u, new Event::ActionInfo(target_pos, target_obj));
		}
	*/
	}

	virtual void completeRequest() {
		if(!this->getFaction()->addReserch(current_request->getName()) && GetWorldMode()!=CLIENT)
			SendCommandEvent(this, this, new Event::CommandInfo("createUnit", current_request->getName()));
		if(mode_continue && GetWorldMode()!=CLIENT)
			RouteCommandEvent(this, this, new Event::CommandInfo("requestCreateUnit", current_request->getName()));
		current_request = productr_ptr();
	}

	virtual void selectNextRequest() {
		current_request = requests.front();
		if(!this->getFaction()->pushRequest(current_request)) {
			current_request->cancel();
			current_request = productr_ptr();
		}
		requests.pop_front();
		SendCommandEvent(this, GetPlayerFaction(), new Event::CommandInfo("refreshMenu", boost::any()));
	}

	void onUpdate(Event& e) {
		T::onUpdate(e);
		for(RequestCont::iterator p=requests.begin(); p!=requests.end(); ) {
			if((*p)->isCanceled())
				requests.erase(p++);
			else
				++p;
		}

		if(!!current_request) {
			if(current_request->isCanceled())
				current_request = productr_ptr();
			else if(current_request->isComplete())
				completeRequest();
		}
		if(!current_request && !requests.empty()) {
			selectNextRequest();
		}
	}

	void onKill(Event& e) {
		T::onKill(e);
		target_obj = selectable_ptr();
	}

	virtual bool call(const std::string& n, boost::any& v) {
		if(n=="setModeContinue") setModeContinue(ToBool(v));
		else if(n=="createUnit") createUnit(ToString(v));
		else return T::call(n, v);
		return true;
	}

	void onCommand(Event& e) {
		Event::CommandInfo& info = e.getCommandInfo();
		if(info.name=="requestCreateUnit") {
			selectable_ptr sender = ToSelectable(GetGObj(e.from));
			if(!!sender && sender->getClassName()==this->getClassName())
				requestCreateUnit(ToString(info.value));
		}
		else
			T::onCommand(e);
	}

private:
	typedef std::list<productr_ptr> RequestCont;
	typedef boost::intrusive_ptr<ProgressMenu> pmenu_ptr;

	RequestCont requests;
	productr_ptr current_request;
	ist::vector4 target_pos;
	selectable_ptr target_obj;
	bool mode_continue;
};



template<class T>
class tohuBreakable : public T
{
public:
	typedef tohuBreakable BaseClass;
	virtual const std::string& getClassName() const=0;
	virtual const std::string& getUnitName() const { return GetUnitName(getClassName()); }
	virtual const std::string& getDescription() const {
		static std::string mes;
		mes = this->getUnitName();
		mes+="\n";
		mes+=GetDescription(getClassName());
		return mes;
	}
	virtual texture_ptr getUnitImage() const { return GetTexture(GetUnitImage(getClassName())); }
};

#endif
